Package: CellNOptR
Type: Package
Title: Training of boolean logic models of signalling networks using
        prior knowledge networks and perturbation data
Version: 1.56.0
Date: 2022-03-16
Authors@R: c(
    person(given = "Thomas",
           family = "Cokelaer",
           role = c("aut")),
    person(given = "Federica",
           family = "Eduati",
           role = "aut"),
    person(given = "Aidan",
           family = "MacNamara",
           role = "aut"),
    person(given = "S",
           family = "Schrier",
           role = "ctb"),
    person(given = "Camille",
           family = "Terfve",
           role = "aut"),
    person(given = "Enio",
           family = "Gjerga",
           role = "ctb"),
    person(given = "Attila",
           family = "Gabor",
           role = "cre",
           email = "attila.gabor@uni-heidelberg.de")
           )
Depends: R (>= 4.0.0), RBGL, graph, methods, RCurl, Rgraphviz, XML,
        ggplot2, rmarkdown
Imports: igraph, stringi, stringr
Suggests: data.table, dplyr, tidyr, readr, knitr, RUnit, BiocGenerics,
Enhances: doParallel, foreach
VignetteBuilder: knitr
biocViews: CellBasedAssays, CellBiology, Proteomics, Pathways, Network,
        TimeCourse, ImmunoOncology
Description: This package does optimisation of boolean logic networks of
    signalling pathways based on a previous knowledge network and a set of data upon
    perturbation of the nodes in the network.
License: GPL-3
LazyLoad: yes
SystemRequirements: Graphviz version >= 2.2
RoxygenNote: 7.1.2
git_url: https://git.bioconductor.org/packages/CellNOptR
git_branch: RELEASE_3_22
git_last_commit: 2244640
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.22
Date/Publication: 2025-10-29
NeedsCompilation: yes
Packaged: 2025-10-30 03:01:32 UTC; biocbuild
Author: Thomas Cokelaer [aut],
  Federica Eduati [aut],
  Aidan MacNamara [aut],
  S Schrier [ctb],
  Camille Terfve [aut],
  Enio Gjerga [ctb],
  Attila Gabor [cre]
Maintainer: Attila Gabor <attila.gabor@uni-heidelberg.de>

import("graph", "RBGL", "ggplot2")
import("XML", except="addNode") # conflict with graph::addNode; neither used

useDynLib(CellNOptR)

importFrom("methods","is","new")
importFrom("RCurl", "getURL")
importFrom("Rgraphviz", "layoutGraph", "renderGraph", "toDot","tail","lines")

importFrom("stringr","str_sub","%>%")

export(
    "buildBitString",
    "checkSignals",
    "exhaustive",
    "CNOdata",
    "CNORwrap",
    "CNORbool",
    "CNOlist",
    "compressModel",
    "computeScoreT1",
    "computeScoreTN",
    "crossInhibitedData",
    "cutAndPlot",
    "cutAndPlotResultsT1",
    "cutAndPlotResultsTN",
    "cutCNOlist",
    "cutNONC",
    "cutModel",
    "cutSimList",
    "defaultParameters",
    "expandGates",
    "findNONC",
    "gaBinaryT1",
    "gaBinaryTN",
    "graph2sif",
    "getFit",
    "indexFinder",
    "mapBack",
    "makeCNOlist",
    "model2sif",
    "model2igraph",
    "normaliseCNOlist",
    "plotCNOlist",
    "plotCNOlist2",
    "plotCNOlistLarge",
    "plotCNOlistLargePDF",
    "plotCNOlistPDF",
    "plotFit",
    "plotOptimResults",
    "plotOptimResultsPan",
    "plotOptimResultsPDF",
    "prep4sim",
    "prep4Sim",
    "preprocessing",
    "randomizeCNOlist",
    "readSBMLQual",
    "readMIDAS",
    "readSIF",
    "readSif",
    "readBND",
    "readBNET",
    "residualError",
    "sif2graph",
    "simulateT1",
    "simulateTN",
    "simulatorT0",
    "simulatorT1",
    "simulatorTN",
    "toSBML",
    "writeDot",
    "writeMIDAS",
    "writeSIF",
    "writeNetwork",
    "writeReport",
    "writeScaffold",
    "plotModel",
    "ilpBinaryTN",
    "ilpBinaryT2",
    "ilpBinaryT1",
    "crossvalidateBoolean")

# CNOlist class
exportClasses(CNOlist)

# CNOlist class methods:
exportMethods("plot", "randomize", "length", "compatCNOlist",  "readErrors", "writeErrors")
exportMethods("getCues", "getSignals", "getStimuli", "getInhibitors", "getVariances", "getTimepoints")
exportMethods("setSignals<-")

# -*- python -*-
#
#  This file is part of the CNO software
#
#  Copyright (c) 2011-2012 - EMBL - European Bioinformatics Institute
#
#  File author(s): Thomas Cokelaer <cokelaer@ebi.ac.uk>
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  CNO website: http://www.cellnopt.org
#
##############################################################################
# $Id$
readSIF<-function(sifFile){

    # Read the sif file expected 3 columns all over the file
    status = tryCatch({
            sif<-read.table(sifFile)
            sif<-as.matrix(sif)
            status = TRUE},
        error = function(e) { status = FALSE })

    # if the sif file has more than 3 columns e.g.:
    # A 1 B C
    # B 1 D
    # then another parser is required:
    if (status == FALSE){
        data = readLines(sifFile)
        # replace all tabs by spaces
        data = gsub("\t", " ",data)
        # replace all long whitespaces by a single space character
        data = gsub("  *", " ",data)
        sif = c()
        # scan all lines and extract the reactions
        for (line in data){
            items = unlist(strsplit(line, " "))
            LHS = items[1]
            edge = items[2]
            RHSs = items[-c(1,2)] # let us look at the RHS elements
            for (RHS in RHSs){
                sif = c(sif, LHS)
                sif = c(sif, edge)
                sif = c(sif, RHS)
            }
        }
        sif = matrix(sif, ncol=3, byrow=TRUE)
    }

	sif <- unique(sif)   # we only select the unique interactions

    #Create the vector of names of species
    namesSpecies<-unique(c(as.character(sif[,1]), as.character(sif[,3])))

    #Create the vector of names or reactions

    createReacID<-function(x){
        neg<-ifelse(test=(x[2] == "-1"), TRUE, FALSE)
        ID<-ifelse(
            neg,
            paste("!",x[1],"=",x[3],sep=""),
            paste(x[1],"=",x[3],sep=""))
        return(ID)
        }

    reacID<-apply(sif,1,createReacID)

    # reaction with and gates can not have "and -1 specy" reaction: edge must be
    # 1
    if (any(substring(reacID, 1,4)=="!and")==TRUE){
        stop("Found an and gate with not edge. This is not possible check your SIF file. Must be 'and 1 specy', not 'and -1 specy' !!")
    }
    if (any(substring(reacID, 1,4)=="!AND")==TRUE){
        stop("Found an and gate with not edge. This is not possible check your SIF file. Must be 'and 1 specy', not 'and -1 specy' !!")
    }
#Create empty interMat and notMat
    interMat<-matrix(data=0,nrow=length(namesSpecies), ncol=dim(sif)[1])
    rownames(interMat)<-namesSpecies
    colnames(interMat)<-reacID
    notMat<-matrix(data=0,nrow=length(namesSpecies), ncol=dim(sif)[1])
    rownames(notMat)<-namesSpecies
    colnames(notMat)<-reacID


#Fill in interMat and notMat
    for(r in 1:length(reacID)){

        source<-match(sif[r,1],namesSpecies)
        target<-match(sif[r,3],namesSpecies)
        neg<-ifelse(test=(sif[r,2] == "-1"), TRUE, FALSE)
        interMat[source,r]<-(-1)
        interMat[target,r]<-1
        if(neg) notMat[source,r]<-1

        }

#Take care of the "and" nodes
#browser()
#1.detect them
    AndNodes<-grep(
        pattern="(^[a,A][n,N][d,D]\\d+$)",namesSpecies,perl=TRUE,ignore.case=FALSE)
    AndNodesV<-grep(
        pattern="(^[a,A][n,N][d,D]\\d+$)",namesSpecies,perl=TRUE,ignore.case=FALSE,value=TRUE)


    if(length(AndNodes) != 0){

#2.go through them
        for(i in 1:length(AndNodes)){

#3.store the name of the node, and find the associated reactions
            node<-AndNodesV[i]
            AndsReacs<-c(
                grep(pattern=paste("^",node,"=",sep=""),reacID,ignore.case=FALSE),
                grep(pattern=paste(node,"$",sep=""),perl=TRUE,reacID,ignore.case=FALSE))

#4.create the new column of interMat and notMat
            newReac<-rowSums(interMat[,AndsReacs])
            newReacNot<-rowSums(notMat[,AndsReacs])

#5.create the new reacID

            #First case: I have an "and" with two inputs only

            if(length(AndsReacs) == 3){

                output<-grep(pattern=paste("^",node,"=",sep=""),reacID,
                    ignore.case=FALSE,value=TRUE)
                inputs<-grep(pattern=paste(node,"$",sep=""),perl=TRUE,
                    reacID,ignore.case=FALSE,value=TRUE)
                output<-sub(pattern=node,x=output,replacement="",ignore.case=FALSE)
                inputs<-sub(pattern=paste("=",node,"$",sep=""),x=inputs,
                    replacement="+",ignore.case=FALSE)
                inputs<-paste(inputs,collapse='')
                newReacID<-paste(inputs,output,sep="")
                newReacID<-sub(pattern=paste("\\W{1}",output,sep=""),
                    x=newReacID,replacement=output,ignore.case=FALSE,perl=TRUE)

                }else{

            #Second case: I have an 'and' with more than 2 inputs
                    output<-grep(pattern=paste("^",node,"=",sep=""),reacID,
                        ignore.case=FALSE,value=TRUE)

                    if(length(output) == 1){

                        output<-sub(pattern=node,x=output,replacement="",ignore.case=FALSE)
                        inputs<-grep(pattern=paste(node,"$",sep=""),perl=TRUE,
                            reacID,ignore.case=FALSE,value=TRUE)
                        inputs<-sub(pattern=paste("=",node,"$",sep=""),x=inputs,
                            replacement="+",ignore.case=FALSE)
                        inputs<-paste(inputs,collapse='')
                        newReacID<-paste(inputs,output,sep="")
                        newReacID<-sub(pattern=paste("\\W{1}",output,sep=""),
                            x=newReacID,replacement=output,ignore.case=FALSE,perl=TRUE)

                        }else{

                            output<-sub(pattern=paste("^",node,"=",sep=""),x=output,
                                replacement="",ignore.case=FALSE)
                            inputs<-grep(pattern=paste(node,"$",sep=""),perl=TRUE,
                                reacID,ignore.case=FALSE,value=TRUE)
                            inputs<-sub(pattern=paste("=",node,"$",sep=""),x=inputs,
                                replacement="+",ignore.case=FALSE)
                            inputs<-paste(inputs,collapse='')
                            inputs<-sub(pattern="\\W{1}$",x=inputs,replacement="",
                                ignore.case=FALSE,perl=TRUE)
                            newReacID<-paste(inputs,output,sep="=")

                            }
                    }

#5b.if there are more than one outputs, then the interMat and notMat created above are not correct
            output<-grep(pattern=paste("^",node,"=",sep=""),reacID,
                ignore.case=FALSE,value=FALSE)
            inputs<-grep(pattern=paste(node,"$",sep=""),perl=TRUE,
                reacID,ignore.case=FALSE,value=FALSE)

            if(length(output) != 1){

                newReac<-rowSums(interMat[,c(inputs,output[1])])
                newReacNot<-rowSums(notMat[,c(inputs,output[1])])

                for(n in 2:length(output)){
                    newReac<-cbind(newReac,rowSums(interMat[,c(inputs,output[n])]))
                    newReacNot<-cbind(newReacNot,rowSums(notMat[,c(inputs,output[n])]))
                    }

                }

#6. remove the old reactions and replace them by the new ones
            namesSpecies<-namesSpecies[-which(namesSpecies == AndNodesV[i])]
            reacID<-reacID[-AndsReacs]
            interMat<-interMat[,-AndsReacs,drop=FALSE]
            notMat<-notMat[,-AndsReacs,drop=FALSE]
            interMat<-cbind(interMat,newReac)
            notMat<-cbind(notMat,newReacNot)

            reacID<-c(reacID,newReacID)
            interMat<- as.matrix(interMat[-which(rownames(interMat) == AndNodesV[i]),], drop=FALSE)
            notMat <- as.matrix(notMat[-which(rownames(notMat) == AndNodesV[i]),], drop=FALSE)


            if(length(output) == 1){

                colnames(interMat)[dim(interMat)[2]]<-newReacID
                colnames(notMat)[dim(notMat)[2]]<-newReacID

                }else{

                    colnames(interMat)[(dim(interMat)[2]-length(newReacID)+1):dim(interMat)[2]]<-newReacID
                    colnames(notMat)[(dim(notMat)[2]-length(newReacID)+1):dim(notMat)[2]]<-newReacID

                }
            }
        }

    model<-list(
        reacID=reacID,
        namesSpecies=namesSpecies,
        interMat=interMat,
        notMat=notMat)
    return(model)
    }

#
#  This file is part of the CNO software
#
#  Copyright (c) 2011-2012 - EMBL - European Bioinformatics Institute
#
#  File author(s): CNO developers (cno-dev@ebi.ac.uk)
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  CNO website: http://www.cellnopt.org
#
##############################################################################
# $Id: writeScaffold.R 3155 2013-01-09 15:24:58Z cokelaer $
#the global function will still be called writeScaffold and will still need the same arguments
#(ModelComprExpanded,optimResT1,optimResT2,ModelOriginal=ToyModel,CNOlist), but it will be divided into a function writeScaffoldW that does
#the actual writing to files,
#a function that gets the info for the sif file,
#and a function that gets the additional info for the dot file
writeScaffold<-function(
    modelComprExpanded,
    optimResT1,
    optimResT2,
    modelOriginal,
    CNOlist,
    tag=NULL){

#get the stuff that I need for the sif file
    sif<-getSifInfo(modelComprExpanded=modelComprExpanded,
        optimResT1=optimResT1,
        optimResT2=optimResT2,
        modelOriginal=modelOriginal)

#get the stuff that I need for the dot file
    dot<-getDotInfo(
        modelComprExpanded=modelComprExpanded,
        modelOriginal=modelOriginal,
        CNOlist=CNOlist,
        sifFile=sif$sifFile)

#this bit writes the dot, the sif and the sif edges attributes
    writeScaffoldW(
        dN=dot$dN,
        dM=dot$dM,
        modelComprExpanded=modelComprExpanded,
        sifFile=sif$sifFile,
        EApresent=sif$EApresent,
        EAweights=sif$EAweights,
        tag=tag)

    }


###########################################################################
#######these are the functions used above
###########################################################################
#this function writes the sif file, the edge attributes, and the dot file
writeScaffoldW<-function(
    dN,
    dM,
    modelComprExpanded,
    sifFile,
    EApresent,
    EAweights,
    tag=NULL){

    create_filename<-function(x, tag=NULL){
        if (is.null(tag)){
            return(x)
        }
        else{
            return(paste(c(tag, "_", x), collapse=""))
        }
    }

    writeDot(
        dotNodes=dN,
        dotMatrix=dM,
        model=modelComprExpanded,
        filename=create_filename("Scaffold.dot", tag=tag))

    write.table(
        sifFile[,1:3],
        file=create_filename("Scaffold.sif", tag=tag),
        row.names=FALSE,col.names=FALSE,quote=FALSE,sep="\t")

    write.table(
        EApresent,
        file=create_filename("TimesScaffold.EA", tag=tag),
        row.names=FALSE,col.names="Times",quote=FALSE,sep="\t")

    write.table(
        EAweights,
        file=create_filename("weightsScaffold.EA", tag=tag),
        row.names=FALSE,col.names="Weights",quote=FALSE,sep="\t")

    }

######
#this function computes the stuff that is needed for the dot file

getDotInfo<-function(modelComprExpanded,modelOriginal,CNOlist,sifFile){

    dM<-sifFile
    nodesCompr<-modelComprExpanded$speciesCompressed
    indexes<-indexFinder(CNOlist,modelOriginal)
    nodesSig<-modelOriginal$namesSpecies[indexes$signals]
    nodesInh<-modelOriginal$namesSpecies[indexes$inhibited]
    nodesStim<-modelOriginal$namesSpecies[indexes$stimulated]
    nodesNCNO<-findNONC(modelOriginal,indexes)
    nodesAttrNames<-nodesSig
    nodesAttr<-rep("signal",length(nodesSig))

    if(length(nodesInh) != 0){
        nodesAttrNames<-c(nodesAttrNames,nodesInh)
        nodesAttr<-c(nodesAttr,rep("inhibited",length(nodesInh)))
    }

    if(length(nodesStim) != 0){
        nodesAttrNames<-c(nodesAttrNames,nodesStim)
        nodesAttr<-c(nodesAttr,rep("stimulated",length(nodesStim)))
    }

    if(length(nodesNCNO) != 0){
        nodesAttrNames<-c(nodesAttrNames,nodesNCNO)
        nodesAttr<-c(nodesAttr,rep("ncno",length(nodesNCNO)))
    }

    if(length(nodesCompr) != 0){
        nodesAttrNames<-c(nodesAttrNames,nodesCompr)
        nodesAttr<-c(nodesAttr,rep("compressed",length(nodesCompr)))
    }

    dN<-cbind(nodesAttrNames,nodesAttr)
    return(list(dN=dN,dM=dM))
    }

#####
#this function computes the stuff that is needed for the sif file
getSifInfo<-function(modelComprExpanded,
    optimResT1,
    optimResT2,
    modelOriginal){

    bString1<-optimResT1$bString

    if(is.na(optimResT2[1])){
        bString2<-optimResT1$bString[which(optimResT1$bString == 0)]
        }else{
            bString2<-optimResT2$bString
            }

#BStimes is a string containing 0,1 and 2 depending on whether the interaction is
#absent, present at t1 or present at t2
    BStimes<-bString1
    BStimes[which(BStimes == 0)]<-bString2*2

#create: weightsE is a string that holds the weights of the interactions

    if(!is.null(dim(optimResT1$stringsTol))){
        bW1<-apply(optimResT1$stringsTol,2,mean)
        }else{
            bW1<-bString1
            }

    if(!is.na(optimResT2[1])){

        if(!is.null(dim(optimResT2$stringsTol))){
            bW2<-apply(optimResT2$stringsTol,2,mean)
            }else{
                bW2<-bString2
                }

            weightsE<-bW1
            weightsE[which(optimResT1$bString == 0)]<-weightsE[which(optimResT1$bString == 0)]+bW2

            }else{
                weightsE<-bW1
                }

#These mini functions are used to find the inputs and output of reactions
    findOutput<-function(x){
        sp<-which(x == 1)
        sp<-modelComprExpanded$namesSpecies[sp]
        }

    reacOutput<-apply(modelComprExpanded$interMat,2,findOutput)

    findInput<-function(x){
        sp<-which(x == -1)
        sp<-modelComprExpanded$namesSpecies[sp]
        }

    reacInput<-apply(modelComprExpanded$interMat,2,findInput)

#This mini function is used to create a reaction label as used in a cystoscape edge attribute file
    createReac<-function(x){
        r<-paste(x[1]," (",x[2],") ",x[3],sep="")
        return(r)
        }

#if the class of reacInput is not a list, then there are no AND gates
    if(is(reacInput,"list")){
        isNeg<-function(x){
            isNegI<-any(x == 1)
            return(isNegI)
        }
        inpSign<-apply(modelComprExpanded$notMat,2,isNeg)
        inpSign<-!inpSign
        inpSign[inpSign]<-1
        inpSign[!inpSign]<--1
        sifFile<-cbind(reacInput,inpSign,reacOutput)
        EApresent<-apply(sifFile,1,createReac)
        EApresent<-cbind(EApresent,BStimes)
        EAweights<-cbind(EApresent,weightsE)

        # add a fourth and fifth column as expected in writeDot (bug report 39)
        sifFile<-cbind(sifFile,BStimes)
        sifFile<-cbind(sifFile,weightsE)
        }
    else{
        #in this case there are AND gates and so we need to create dummy "and#" nodes
        sifFile<-matrix(0,nrow=4*length(reacOutput),ncol=5)
        nR<-1
        nANDs<-1

        for(i in 1:length(reacOutput)){
            if(length(reacInput[[i]]) == 1){
                sifFile[nR,1]<-reacInput[[i]]
                sifFile[nR,3]<-reacOutput[i]
                sifFile[nR,2]<-ifelse(any(modelComprExpanded$notMat[,i] == 1),-1,1)
                sifFile[nR,4]<-BStimes[i]
                sifFile[nR,5]<-weightsE[i]
                nR<-nR+1
                }
            else{
                for(inp in 1:length(reacInput[[i]])){
                    sifFile[nR,1]<-reacInput[[i]][inp]
                    sifFile[nR,3]<-paste("and",nANDs,sep="")
                    temp_indices = which(reacInput[[i]][inp]==rownames(modelComprExpanded$notMat))
                    sifFile[nR,2]<-ifelse(modelComprExpanded$notMat[temp_indices, i]==1,-1,1)
                    sifFile[nR,4]<-BStimes[i]
                    sifFile[nR,5]<-weightsE[i]
                    nR<-nR+1
                }
                sifFile[nR,1]<-paste("and",nANDs,sep="")
                sifFile[nR,3]<-reacOutput[i]
                sifFile[nR,2]<-1
                sifFile[nR,4]<-BStimes[i]
                sifFile[nR,5]<-weightsE[i]
                nANDs<-nANDs+1
                nR<-nR+1
                }
            }
        sifFile<-sifFile[1:(nR-1),]
        EApresent<-apply(sifFile[,1:3],1,createReac)
        EAweights<-cbind(EApresent,sifFile[,5])
        EApresent<-cbind(EApresent,sifFile[,4])
    }
    #this mini function makes edge attributes in the format e1 (sign) e2 = attr

    makeEA<-function(x){
        ea<-paste(x[1],"=",x[2])
        return(ea)
        }
    EApresent<-apply(EApresent,1,makeEA)
    EAweights<-apply(EAweights,1,makeEA)

#this is the edge attribute matrix that contains, for each edge, whether it is
#absent from the model (0), present at t1(1) or present at t2(2)
    return(list(EApresent=EApresent,EAweights=EAweights,sifFile=sifFile))
    }

#
#  This file is part of the CNO software
#
#  Copyright (c) 2011-2012 - EMBL - European Bioinformatics Institute
#
#  File author(s): CNO developers (cno-dev@ebi.ac.uk)
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  CNO website: http://www.cellnopt.org
#
##############################################################################
# $Id$
#' @export
exhaustive<-function(
    CNOlist,
    model,
    shuffle=FALSE,
    Nmax=NULL,
    verbose=TRUE,
    sizeFac = 0.0001,
    NAFac = 1,
    relTol=0.1,
    timeIndex=2){

    if (!is(CNOlist,"CNOlist")){
        CNOlist = CellNOptR::CNOlist(CNOlist)
    }

    # should be after CNOlist conversion
    if (timeIndex<2){ stop("timeIndex must be >=2")}
    if (timeIndex>length(CNOlist@timepoints)){
        stop(paste("timeIndex must be <= ", length(CNOlist@timepoints), sep=" "))
    }

    bLength = length(model$reacID)
    if (bLength>20){
        print(paste("You will compute ", 2**bLength, "iterations. It may take a
            while..."), sep="")
    }
    if (bLength == 0){
        stop("Nothing to optimise...bitstring length is zero.")
    }
    # boolean case but could be extended easily to fuzzy by changing
    # rep(list(0,1)) to rep(list(0, nTF))
    bitstrings = expand.grid(rep(list(0:1), bLength))

    # do we want to shuffle the order ?
    if (shuffle==TRUE){
        bitstrings = bitstrings[sample.int(nrow(bitstrings)),]
    }

    # compute these variables once for all
    simList = prep4sim(model)
    indexList = indexFinder(CNOlist, model)

    bestScore = 1e6
    bitstring = bitstrings[1]
    N = 2**bLength

    # if the user request a Nmax value, let us use it
    if (is.null(Nmax)==FALSE){
        if (Nmax<=0){stop("Nmax must be positive stricly")}
        N = min(Nmax, N)
    } else{
        Nmax=N
    }

    # NA are removed later on
    PopTol<-rep(NA,bLength)
    PopTolScores<-NA

    all_scores = c()
    t1 = Sys.time()
    for (x in seq(1:Nmax)){

        bitstring = as.double(bitstrings[x,])
        #if (sum(bitstring)==0){next}
        score = computeScoreT1(CNOlist, model, bitstring, simList, indexList,
			sizeFac, NAFac, timeIndex)
        all_scores[x] = score
        changed = FALSE
        if (score <= bestScore){
            if (verbose==TRUE){
                print(paste("--Found a new best score=", score, " at iteration ", x, sep=""))
                print(bitstring)
            }
            bestScore = score
            bestbit = bitstring
            changed = TRUE
        }

        if (verbose == TRUE){
            if ((x%%1000)==0){
                print(paste(x,"/",N,sep=""))
            }
        }

        # Check if the current score/bitstring is within tolerance of the best
        # score
        if( score <=(1+relTol)*bestScore){

            PopTol<-rbind(PopTol, bitstring)
            PopTolScores<-c(PopTolScores, score)
        }

        # if the best score changed, there are maybe scores to be removed now
        if (changed==TRUE){
            indices = PopTolScores <= bestScore* (1+relTol)
            PopTol = PopTol[indices,]
            PopTolScores = PopTolScores[indices]
        }

    }
    t2 = Sys.time()
    if (verbose==TRUE){
        print(t2-t1)
    }

    # remove names of the matrix row and column
    PopTol = matrix(PopTol, ncol=bLength,
        dimnames=list(rep("", dim(PopTol)[1]),rep("",bLength)))

    # remove first NA element. The NA element has the good property that if
    # there is only one score found within the tol, there are 2 elements hence
    # we are still dealing with a matrix. So, we remove the NA only at the end.

    # removing the two rows.
    PopTol = matrix(PopTol[-1,], ncol=bLength)
    PopTolScores = matrix(PopTolScores[-1])

    # a dummy matrix
    res<-c(1, bestScore, toString(bestbit), 1, bestScore, 1, toString(bestbit),
            as.numeric(1, units="secs"))

    names(res)<-c("Generation","Best_score","Best_bitString","Stall_Generation",
        "Avg_Score_Gen","Best_score_Gen","Best_bit_Gen","Iter_time")
    res<-rbind(res,res)



    return(list(
        bString=bestbit,
        bScore=bestScore,
        results=res,
        stringsTol=PopTol,
        stringsTolScores=PopTolScores,
        scores=all_scores))
    }

#
#  This file is part of the CNO software
#
#  Copyright (c) 2011-2012 - EMBL - European Bioinformatics Institute
#
#  File author(s): CNO developers (cno-dev@ebi.ac.uk)
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  CNO website: http://www.cellnopt.org
#
##############################################################################
# $Id: plotCNOlistPDF.R 3155 2013-01-09 15:24:58Z cokelaer $
plotCNOlistPDF <-
function(CNOlist,filename){
	pdf(file=filename,width=14,height=7)
	plotCNOlist(CNOlist)
	dev.off()
	}

#
#  This file is part of the CNO software
#
#  Copyright (c) 2011-2012 - EMBL - European Bioinformatics Institute
#
#  File author(s): CNO developers (cno-dev@ebi.ac.uk)
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  CNO website: http://www.cellnopt.org
#
##############################################################################
# $Id$
mapBack <-
  function(model, PKN, bString){
    #the mapback for each link A->B in the compressed model is done looking
    #at the PKN as a graph and considering a subgraph of it including only
    #node A, node B and compressed nodes. All paths going from A to B in this
    #subnetwork are marked as active in the PKN.

    bStringPKN<-rep(0, length(PKN$reacID))

    #consider the PKN as a graph
    PKNgraph<-sif2graph(model2sif(model=PKN))

    #want to map back only selected links (1 in the bString)
    links2map<-model$reacID[as.logical(bString)]

    #function to search all paths connecting 2 nodes (from StartNode to EndNode)
    #in a given graph (gg)
    SearchPath<-function(gg,StartNode,EndNode){

      nod<-nodes(gg)
      edg<-edges(gg)

      #PathLis: list where all Paths starting from StartNode will e saved
      PathList<-list()
      ix<-1
      PathList[[ix]]<-StartNode

      #the search is implemented using a queue
      #initialization of the queue with all nodes reachale from the StartNode
      queue<-edg[[which(nod==StartNode)]]
      AllQueue<-queue  #this is just a backup al the whole queue for check
      #new visited paths are added
      for (k in 1:length(queue)){
        PathList[[ix+k]]<-c(PathList[[ix]],queue[k])
      }
      ix<-ix+1 #this index points always to the path walked to reach the current first element of the queue

      # remove nodes already visited to avoid problems with cyclic graphs
      gg<-removeNode(StartNode,gg)
      nod<-nodes(gg)
      edg<-edges(gg)

      # untill the queue is not empty
      while (length(queue)>0){
        if (any(which(nod==queue[1]))){
          #save in tmp nodes that can be reached from the first element of the queue
          tmp<-edg[[which(nod==queue[1])]]

          #update the queue with the new nodes at the end and save the corresponding paths walked to reach them
          if (length(tmp)>0){
            queue<-c(queue, tmp)
            AllQueue<-c(AllQueue, tmp)
            for (k in 1:length(tmp)){
              PathList[[length(PathList)+1]]<-as.vector(c(PathList[[ix]],tmp[k]))
            }
          }

          # already visited nodes are removed from the graph to avoid problems with cyclic graphs
          gg<-removeNode(queue[1],gg)
          nod<-nodes(gg)
          edg<-edges(gg)
        }
        # and the first element is removed from the queue
        queue<-queue[-1]
        ix<-ix+1
      }
      PathOK<-PathList[which( sapply(PathList,tail,1)==EndNode)]
      return(PathOK)
    }

    #for each hyperedge in the optimised model...
    for (i in 1:length(links2map)){
      x<-links2map[i]
      x<-gsub("!", "", x)
      x<-unlist(strsplit(x, "=", fixed=TRUE))
      EndNode<-x[2]
      # can contain more than 1 element in case of AND nodes
      StartNodes<-unlist(strsplit(x[1], "+", fixed=TRUE))
      for (j in 1:length(StartNodes)){
        StartNode<-StartNodes[j]

        #consider a subgraph including only StartNode, EndNode and compressed nodes
        okNodes<-c(model$speciesCompressed, EndNode, StartNode)
        noNodes<-setdiff(nodes(PKNgraph), okNodes)

        #this is the subgraph
        gg<-removeNode(noNodes, PKNgraph)

        #search this subgraph for paths between StartNode and EndNode
        PathOK<-SearchPath(gg=gg,StartNode=StartNode,EndNode=EndNode)

        for (j1 in 1:length(PathOK)){
          for (j2 in 2:length(PathOK[[j1]])){
            node1<-PathOK[[j1]][j2-1]
            node2<-PathOK[[j1]][j2]
            reacPKN<-paste(node1,node2,sep="=")
            bStringPKN[grep(reacPKN,PKN$reacID)]<-1
          }
        }

      }
    }


    findOutput<-function(x){
      sp<-which(x == 1)
      sp<-PKN$namesSpecies[sp]
    }
    reacOutput<-apply(PKN$interMat,2,findOutput)
    findInput<-function(x){
      sp<-which(x == -1)
      sp<-PKN$namesSpecies[sp]
    }
    reacInput<-apply(PKN$interMat,2,findInput)

    isNeg<-function(x){
      isNegI<-any(x == 1)
      return(isNegI)
    }
    inpSign<-apply(PKN$notMat,2,isNeg)
    inpSign<-!inpSign
    inpSign[inpSign]<-1
    inpSign[!inpSign]<--1

    sifFile<-cbind(reacInput,inpSign,reacOutput)
    createReac<-function(x){
      r<-paste(x[1]," (",x[2],") ",x[3],sep="")
      return(r)
    }
    EApresent<-apply(sifFile,1,createReac)
    EApresent<-cbind(EApresent,bStringPKN)

    makeEA<-function(x){
      ea<-paste(x[1],"=",x[2])
      return(ea)
    }
    EApresent<-apply(EApresent,1,makeEA)

    write.table(
      EApresent,
      file="weightsPKN.EA",
      row.names=FALSE,col.names="Weights",quote=FALSE,sep="\t")

    return(bStringPKN)
  }

#
#  This file is part of the CNO software
#
#  Copyright (c) 2011-2012 - EBI
#
#  File author(s): CNO developers (cno-dev@ebi.ac.uk)
#
#  Distributed under the GPLv2 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-2.0.html
#
#  CNO website: http://www.ebi.ac.uk/saezrodriguez/cno
#
##############################################################################
# $Id$

# This is a test of the ToyModel and gaBinaryT1
# 

library(CellNOptR)

pknmodel = readSIF(system.file("ToyModelT3/ToyModelT3.sif", package="CellNOptR"))
cnolist = CNOlist(system.file("ToyModelT3/ToyDataT3.csv", package="CellNOptR"))
model = preprocessing(cnolist, pknmodel, verbose=FALSE)

# expected values
bestBS = c(1, 1, 1, 0, 1, 1, 1, 1, 0, 1, 1, 1, 0, 0, 0, 0,0)
bestBS2 <- c(0,0,0,0,0,0,1)
bestBS3 <- c(0,0,1,0,0,0)

# just to check that the simulateTN function works 
SimT1<-simulateTN(CNOlist=cnolist, model=model, bString=list(bestBS))
SimT2<-simulateTN(CNOlist=cnolist, model=model, bStrings=list(bestBS,bestBS2))
SimT3<-simulateTN(CNOlist=cnolist, model=model, bStrings=list(bestBS, bestBS2, bestBS3))


# again, just to check that gaBinary works
# run T1 first, 
T1opt<-gaBinaryT1(CNOlist=cnolist,model=model,verbose=FALSE)
# run T2
T2opt<-gaBinaryTN(CNOlist=cnolist,model=model,bStrings=list(T1opt$bString),verbose=FALSE)
# run T3
T3opt<-gaBinaryTN(CNOlist=cnolist,model=model,bStrings=list(bestBS, bestBS2),verbose=FALSE)

print( T1opt$bScore)
print( T2opt$bScore)
print( T3opt$bScore)
# no using the hardcoded parameters, we can check the output of te scores that
# must be tiny.
score1 = computeScoreT1(cnolist, model, bString=bestBS)
score2 = computeScoreTN(cnolist, model, bStrings=list(bestBS,bestBS2))
score3 = computeScoreTN(cnolist, model, bStrings=list(bestBS,bestBS2, bestBS3))


print(score1)
print(score2)
print(score3)
if (score1>0.01 || score2>0.24 || score3>0.1){
   # ideally, the score should all be close to 0. In practice, it's about 1e-5
   # However, in the  T3 case, once in while, the score is 0.0953 hence the
   # score3>0.1
   stop("errore")
}
cutAndPlot(cnolist, model,bStrings=list(bestBS),plotPDF=TRUE, tag="test1")
cutAndPlot(cnolist, model,bStrings=list(bestBS,bestBS2),plotPDF=TRUE, tag="test2")
cutAndPlot(cnolist, model, bStrings=list(bestBS,bestBS2,bestBS3),plotPDF=TRUE, tag="test3")



warnings()

#
#  This file is part of the CNO software
#
#  Copyright (c) 2011-2013 - EBI
#
#  File author(s): CNO developers (cno-dev@ebi.ac.uk)
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  CNO website: http://www.cellnopt.org
#
##############################################################################
# $Id$

#' @export
buildBitString <- function(bStrings){
    # example:
    # buildBitString(list(c(1,1,0,1,0,0), c(0,1,0), c(0,1)))
    #$bs
    #[1] 1 1 0 1 1 1
    #$bsTimes
    #[1] 1 1 0 1 2 3

    # build bitstrings from different steady states
    if (is.list(bStrings)==FALSE){
        if (is.vector(bStrings, mode="numeric")==TRUE){
            bStrings = list(bStrings)
        } else{ 
            stop("CellNOpt Error: argument bString must be a list of bitstring or a single bitstring. Each bitstring must be a vector. See manual for details.")

        }
    }
    N = length(bStrings)

    if (N < 1 ){
        stop("CellNOpt Error: bStrings argument must be a list of length greater or equal to 1")
    }

    if (N>=1){
        bs = bStrings[[1]]
        bsTimes = bStrings[[1]]
    }

    if (N>1){
        for (i in 2:N){
            bs[which(bs==0)] <- bStrings[[i]]
            bsTimes[which(bsTimes==0)]<-bStrings[[i]]*(i)
        }
    }

    return(list(bs=bs, bsTimes=bsTimes))
}

#
#  This file is part of the CNO software
#
#  Copyright (c) 2011-2012 - EMBL - European Bioinformatics Institute
#
#  File author(s): CNO developers (cno-dev@ebi.ac.uk)
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  CNO website: http://www.cellnopt.org
#
##############################################################################
# $Id$

cSimulator <- function(CNOlist, model, simList, indexList, mode=1) {

    if (!is(CNOlist,"CNOlist")){
        CNOlist = CellNOptR::CNOlist(CNOlist)
    }
	
	# check the structures
	if(is.null(CNOlist@stimuli) || is.null(CNOlist@inhibitors)) {
		stop("This function needs 'valueStimuli' and 'valueInhibitors' in CNOlist")
	}
	
	if(is.null(model$reacID) || is.null(model$namesSpecies)) {
		stop("This function needs 'reacID' and 'namesSpecies' in model")
	}
	
	# variables
	nStimuli <- as.integer(length(indexList$stimulated))
	nInhibitors <- as.integer(length(indexList$inhibited))
	nCond <- as.integer(dim(CNOlist@stimuli)[1])
	nReacs <- as.integer(length(model$reacID))
	nSpecies <- as.integer(length(model$namesSpecies))
	nMaxInputs <- as.integer(dim(simList$finalCube)[2])
	
	# simList
	# used to be 
    # >>> finalCube = as.integer(as.vector(t(simList$finalCube))-1)
    # but as.vector(t is slow and can be replaced by just as.integer albeit
    # appropriate C modifications
	finalCube = as.integer(simList$finalCube-1)
	ixNeg = as.integer(simList$ixNeg)
	ignoreCube = as.integer(simList$ignoreCube)
	maxIx = as.integer(simList$maxIx-1)
	
	# index
	indexSignals <- as.integer(indexList$signals-1)
	indexStimuli <- as.integer(indexList$stimulated-1)
	indexInhibitors <- as.integer(indexList$inhibited-1)
    nSignals <- length(indexSignals)

	# cnolist
	valueInhibitors <- as.integer(CNOlist@inhibitors)
	valueStimuli <- as.integer(CNOlist@stimuli)

	res = .Call("simulatorT1",
		# variables	
		nStimuli,
		nInhibitors,
		nCond,
		nReacs,
		nSpecies,
        nSignals,
		nMaxInputs,
		# simList
		finalCube,
		ixNeg,
		ignoreCube,
		maxIx,		
		# index
		indexSignals, 
		indexStimuli, 
		indexInhibitors,
		# cnolist
		valueInhibitors,
		valueStimuli,
        as.integer(mode)
	)
# should not be cut because it is used in simulateTN as an input 
#    res = res[,indexList$signals]
	return(res)
}

#  Copyright (c) 2019 - SaezLab - Heidelberg Universit
#
#  File author(s): CNO developers
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  CNO website: http://www.cellnopt.org
#
##############################################################################

write_constraints <- function(model,
                              midasExperimentPart,
                              midasTreatmentPart,
                              reaction_sets,
                              y_vector, 
                              midas, 
                              binary_variables){
  
  ## constraint numbers refer to Mitsos et al., PloS Comp Biol. 2009
  ## in th .lp file, the constraint numbers are written for debugging
  
  constraintsVector <- writeConstraint3(model, midasExperimentPart)
  
  # constraints 4, 5 & 7
  reducedMidas <- midasTreatmentPart
  counter <- length(constraintsVector)+1
  for(i in 1:dim(reducedMidas)[1]){
    for(j in 1:length(model$reacID)){
      y_index <- j
      if(length(reaction_sets[[j]][[1]])!=0){ ## constraint 4
        for(k in 1:length(reaction_sets[[j]][[1]])){
          z_index <- length(model$reacID) + 
            length(model$namesSpecies)*dim(reducedMidas)[1] + 
            (i-1)*length(model$reacID) + j 
          x_index <- length(model$reacID) + 
            (i-1)*length(model$namesSpecies) + 
            which(model$namesSpecies==reaction_sets[[j]][[1]][k])
          newConstraint <- paste0("c", counter, ":\t", "xb",z_index, 
                                 " - xb",x_index, " <= 0 \t \t \\ c4")
          constraintsVector[counter] <- newConstraint
          counter= counter+1
        }
      }
      if(length(reaction_sets[[j]][[2]])!=0){ ## constraint 5
        for(k in 1:length(reaction_sets[[j]][[2]])){
          z_index <- length(model$reacID) + 
            length(model$namesSpecies)*dim(reducedMidas)[1] + 
            (i-1)*length(model$reacID) + j 
          x_index <- length(model$reacID) + (i-1)*length(model$namesSpecies) + 
            which(model$namesSpecies==reaction_sets[[j]][[2]][k])
          newConstraint <- paste0("c", counter, ":\t", "xb",z_index, " + xb",
                                 x_index, " <= 1\t \t \\ c5")
          constraintsVector[counter] <- newConstraint
          counter= counter+1
        }
      }
      if(length(reaction_sets[[j]][[3]])!=0){ ## constraint 7
        for(k in 1:length(reaction_sets[[j]][[3]])){
          speciesIsInhibitedInThisExperiment <- FALSE
          if(length(which(colnames(reducedMidas) ==
                          paste0("TR:",reaction_sets[[j]][[3]][k], "i")))==1){
            speciesIsInhibitedInThisExperiment <- 
              (reducedMidas[i,which(colnames(reducedMidas) ==
                                      paste0("TR:",reaction_sets[[j]][[3]][k], 
                                             "i"))] == 1)
          }
          
          if(!speciesIsInhibitedInThisExperiment){
            z_index <- length(model$reacID) + 
              length(model$namesSpecies)*dim(reducedMidas)[1] + 
              (i-1)*length(model$reacID) + j
            x_index <- length(model$reacID) + 
              (i-1)*length(model$namesSpecies) + 
              which(model$namesSpecies==reaction_sets[[j]][[3]][k])
            newConstraint <- paste0("c", counter, ":\t", "xb",z_index, " - xb",
                                   x_index, " <= 0 \t \t \\ c7")
            constraintsVector[counter] <- newConstraint
            counter= counter+1
          }
          
        }
      }
      
      
      #constraint 6
      z_index <- length(model$reacID) + 
        length(model$namesSpecies)*dim(reducedMidas)[1]+ 
        length(model$reacID)*(i-1) + j
      x_reactant_index <- c()
      x_inhibitor_index <- c()
      if(length(reaction_sets[[j]][[1]])!=0){## reactant set is not empty
        for(k in 1:length(reaction_sets[[j]][[1]])){
          x_reactant_index <- append(x_reactant_index, length(model$reacID) + 
                                      (i-1)*length(model$namesSpecies) +
                                      which(model$namesSpecies==
                                              reaction_sets[[j]][[1]][k]))
        }
      }
      
      if(length(reaction_sets[[j]][[2]])!=0){## inhibitors set is not empty
        for(k in 1:length(reaction_sets[[j]][[2]])){
          x_inhibitor_index  <- append(x_inhibitor_index, 
                                      length(model$reacID) + 
                                        (i-1)*length(model$namesSpecies) + 
                                        which(model$namesSpecies==
                                                reaction_sets[[j]][[2]][k]))
        }
      }
      
      newConstraint <- paste0("xb", y_index, " - ", "xb", z_index)
      reactants_sum <- paste0(" <= ", length(x_reactant_index))
      inhibitors_sum <- ""
      if(!is.null(x_reactant_index)){
        binary_sum <- ""
        for(l in 1:length(x_reactant_index)){
          binary_sum <- paste0(binary_sum, " + xb", x_reactant_index[l])
        }
        reactants_sum <- paste0( binary_sum, " <= " , length(x_reactant_index))
      }
      if(!is.null(x_inhibitor_index)){
        binary_sum <- ""
        for(l in 1:length(x_inhibitor_index)){
          binary_sum <- paste0(binary_sum, " - xb", x_inhibitor_index[l])
        }
        inhibitors_sum <- binary_sum
      }
      
      newConstraint <- paste0("c", counter,":\t", newConstraint, 
                             inhibitors_sum, reactants_sum, " \t \t \\ c6")
      constraintsVector[counter] <- newConstraint
      counter= counter+1
      ## --- End Constraint 6
    }
    
    #new constraint 8
    for(j in 1:length(model$namesSpecies)){
      flag_vector <- c()
      newConstraint <- paste0("xb", 
                             length(y_vector)+
                               (i-1)*length(model$namesSpecies)+j)
      for(k in 1:length(model$reacID)){
        
        if(reaction_sets[[k]][3] == model$namesSpecies[j]){
          flag_vector <- c(flag_vector,k)
          #}
        }
      }
      added_reactions <- ""
      if(!is.null(flag_vector)){
        for(k in 1:length(flag_vector)){
          added_reactions <- paste0(added_reactions,
                                   "- xb", 
                                   length(y_vector) + 
                                     (length(model$namesSpecies))*
                                     dim(reducedMidas)[1] + 
                                     (i-1)*length(model$reacID) + 
                                     flag_vector[k])
        }
        newConstraint <- paste0("c", counter, ": ", newConstraint, 
                               added_reactions, " <= 0" ," \t \t \\ c 8 new")
        constraintsVector[counter] <- newConstraint
        counter= counter+1  
      }
    }##end new 8
  }
  
  return(constraintsVector)
}

#
#  This file is part of the CNO software
#
#  Copyright (c) 2011-2013 - EBI
#
#  File author(s): CNO developers (cno-dev@ebi.ac.uk)
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  CNO website: http://www.cellnopt.org
#
##############################################################################
# $Id$

model2igraph <-function(model){

    # The input is a model returned by readSIF()
    # need to convert it back to a sif and then a graphNEL object
    g = sif2graph(model2sif(model))

    # now we can get a igraph object
    g2 = igraph::igraph.from.graphNEL(g)

    # so it can be saved in many different format.
    return(g2)
}

#  Copyright (c) 2019 - SaezLab - Heidelberg University
#
#  File author(s): CNO developers
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  CNO website: http://www.cellnopt.org
#
##############################################################################

cleanupILP <- function(){
  
  if(file.exists("testFile.lp")){
    file.remove("testFile.lp")
  }
  
  if(file.exists("cplexSolution.txt")){
    file.remove("cplexSolution.txt")
  }
  
  if(file.exists("cplex.log")){
    file.remove("cplex.log")
  }
  
  if(file.exists("cplexCommand.txt")){
    file.remove("cplexCommand.txt")
  }
  
  AllFiles <- list.files()
  CloneFiles <- which(grepl(pattern = "clone",x = AllFiles,fixed = TRUE))
  if (length(CloneFiles)>0) {
    for (counter in 1:length(CloneFiles)) {
      file.remove(AllFiles[CloneFiles[counter]])
    }
  }
  
}

#  Copyright (c) 2019 - SaezLab - Heidelberg Universit
#
#  File author(s): CNO developers
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  CNO website: http://www.cellnopt.org
#
##############################################################################

writeConstraint3 <- function(model, midasExperimentPart){
  
  reducedMidas = midasExperimentPart
  constraints = c()
  counter = 1
  for(i in 1:dim(reducedMidas)[1]){
    for(j in 1:length(model$reacID)){
      newConstraint = paste0("c", counter, ":\t", 
                             "xb", 
                             (length(model$reacID)+
                                dim(reducedMidas)[1]*
                                length(model$namesSpecies)+
                                length(model$reacID)*(i-1)+j), 
                             " - xb" , j, " <= 0")
      constraints[counter] =newConstraint
      counter= counter+1
    }
  }
  
  return(constraints)
  
}

#
#  This file is part of the CNO software
#
#  Copyright (c) 2011-2012 - EMBL - European Bioinformatics Institute
#
#  File author(s): CNO developers (cno-dev@ebi.ac.uk)
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  CNO website: http://www.cellnopt.org
#
##############################################################################
# $Id: simulatorT1.R 3155 2013-01-09 15:24:58Z cokelaer $

simulatorT1 <- function(CNOlist,model,simList,indexList, mode=1) {


	#simRes = rSimulatorT1(CNOlist, model, simList, indexList)
	simRes = cSimulator(CNOlist, model, simList, indexList, mode=mode)
	return(simRes)
}

## ----installPackage, eval=FALSE-----------------------------------------------
# if (!requireNamespace("BiocManager", quietly=TRUE))
#     install.packages("BiocManager")
# BiocManager::install("CellNOptR")

## ----installPackage2, eval=FALSE----------------------------------------------
# if (!requireNamespace("remotes", quietly=TRUE))
# 	install.packages("remotes")
# remotes::install_github("saezlab/CellNOptR")

## ----Ropts, include=FALSE-------------------------------------------
options(width=70)

## ----loadLib, eval=TRUE, message=FALSE, warning=FALSE---------------
library(CellNOptR)

## ----newDir, eval=FALSE---------------------------------------------
# dir.create("CNOR_analysis")

## ----quickstart, eval=FALSE-----------------------------------------
# # ---------------------- load the library and get a SIF and MIDAS file
# library(CellNOptR)
# 
# # ---------------------- examples are provided in CellNOptR
# data("ToyModel", package="CellNOptR")
# data("CNOlistToy", package="CellNOptR")
# pknmodel = ToyModel
# cnolist = CNOlist(CNOlistToy)
# 
# # ---------------------- alternatively you can read your own files:
# # pknmodel = readSIF("ToyModel.sif")
# # cnolist = CNOlist("ToyDataMMB.csv")
# 
# # ---------------------- preprocess the network
# model <- preprocessing(cnolist, pknmodel)
# 
# # ---------------------- perform the analysis
# res <- gaBinaryT1(CNOlist = cnolist,model =  model, verbose=FALSE)
# 
# # ---------------------- plot the results
# cutAndPlot(cnolist, model, list(res$bString))

## ----directory, eval=TRUE, include=FALSE----------------------------
cpfile <- dir(system.file("ToyModel", package="CellNOptR"), full=TRUE)
file.copy(from=cpfile, to=getwd(), overwrite=TRUE)

## ----getData, eval=TRUE---------------------------------------------
dataToy <- readMIDAS("ToyDataMMB.csv", verbose=FALSE)
CNOlistToy <- makeCNOlist(dataToy, subfield=FALSE, verbose=FALSE)

## ----getData2, eval=TRUE--------------------------------------------
data(CNOlistToy,package="CellNOptR", verbose=FALSE)

## ----cnolistClass, eval=TRUE, include=FALSE-------------------------
CNOlistToy = CNOlist("ToyDataMMB.csv")

## ----cnolistClass2, eval=FALSE--------------------------------------
# data(CNOlistToy,package="CellNOptR")
# CNOlistToy = CNOlist(CNOlistToy)

## ----showCNO, eval=TRUE---------------------------------------------
CNOlistToy

## ----plotCNO, fig.width=7, fig.height=7, fig.cap="Figure 1: CNOlist data shown by plotting function (either *plot* or *plotCNOlist*)"----
plot(CNOlistToy)

## ----ploCNOPDF, eval=FALSE, include=TRUE----------------------------
# plotCNOlistPDF(CNOlist=CNOlistToy,filename="ToyModelGraph.pdf")

## ----getModel, eval=TRUE--------------------------------------------
pknmodel<-readSIF("ToyPKNMMB.sif")

## ----getModel2, eval=TRUE-------------------------------------------
data(ToyModel,package="CellNOptR")

## ----indices, eval=TRUE---------------------------------------------
checkSignals(CNOlistToy,pknmodel)

## ----plotModel, eval=TRUE, fig.cap="Figure 2: Prior knowledge network (original SIF file visualised by *plotModel*) for the Toy Model example."----
plotModel(pknmodel, CNOlistToy)

## ----NONC, eval=TRUE------------------------------------------------
indicesToy<-indexFinder(CNOlistToy,pknmodel,verbose=TRUE)
ToyNCNOindices<-findNONC(pknmodel,indicesToy,verbose=TRUE)
ToyNCNOcut<-cutNONC(pknmodel,ToyNCNOindices)
indicesToyNCNOcut<-indexFinder(CNOlistToy,ToyNCNOcut)

## ----compress, eval=TRUE--------------------------------------------
ToyNCNOcutComp<-compressModel(ToyNCNOcut,indicesToyNCNOcut)
indicesToyNCNOcutComp<-indexFinder(CNOlistToy,ToyNCNOcutComp)

## ----expand, eval=TRUE----------------------------------------------
model <- expandGates(ToyNCNOcutComp, maxInputsPerGate=3)

## ----expand2, eval=TRUE---------------------------------------------
model <- preprocessing(CNOlistToy, pknmodel, expansion=TRUE,
    compression=TRUE, cutNONC=TRUE, verbose=FALSE)

## ----resError, eval=TRUE--------------------------------------------
resECNOlistToy <- residualError(CNOlistToy)

## ----initbs, eval=TRUE----------------------------------------------
initBstring <- rep(1,length(model$reacID))

## ----optim, eval=TRUE-----------------------------------------------
ToyT1opt <- gaBinaryT1(CNOlist=CNOlistToy, model=model,
    initBstring=initBstring, verbose=FALSE)

## ----resSim,fig.width=7, fig.height=7, fig.cap="Figure 3: Results of the cutAndPlot function on the Toy Model example."----
cutAndPlot(model=model, bStrings=list(ToyT1opt$bString),
    CNOlist=CNOlistToy,plotPDF=TRUE)

## ----plotFit,fig.width=7, fig.height=7, eval=TRUE, fig.cap="Figure 4: Results of the cutAndPlot function on the Toy Model example"----
plotFit(optRes=ToyT1opt)

## ----simFitPDF, include=TRUE, eval=FALSE----------------------------
# cutAndPlot(
#     model=model,
#     bStrings=list(ToyT1opt$bString),
#     CNOlist=CNOlistToy,
#     plotPDF=TRUE)
# pdf("evolFitToyT1.pdf")
# plotFit(optRes=ToyT1opt)
# dev.off()

## ----plotModelToy1, eval=T,fig.cap="Figure 4a: Processed model. "----
plotModel(model, CNOlistToy, bString=ToyT1opt$bString)

## ----plotModelToy2, eval=TRUE, fig=F, include=TRUE, fig.cap="Figure 4b: The edges are on (black or red) or off (grey or pink) according to the best set of parameters found during the optimisation (the best bit string). To obtain the right hand side model, the mapBack function has been used."----

bs = mapBack(model, pknmodel, ToyT1opt$bString)
plotModel(pknmodel, CNOlistToy, bs, compressed=model$speciesCompressed)

## ----eval=FALSE-----------------------------------------------------
# writeScaffold(
#     modelComprExpanded=model,
#     optimResT1=ToyT1opt,
#     optimResT2=NA,
#     modelOriginal=pknmodel,
#     CNOlist=CNOlistToy)
# 
# writeNetwork(
#     modelOriginal=pknmodel,
#     modelComprExpanded=model,
#     optimResT1=ToyT1opt,
#     optimResT2=NA,
#     CNOlist=CNOlistToy)
# 
# namesFilesToy<-list(
#     dataPlot="ToyModelGraph.pdf",
#     evolFitT1="evolFitToyT1.pdf",
#     evolFitT2=NA,
#     simResultsT1="SimResultsT1_1.pdf",
#     simResultsT2=NA,
#     scaffold="Scaffold.sif",
#     scaffoldDot="Scaffold.dot",
#     tscaffold="TimesScaffold.EA",
#     wscaffold="weightsScaffold.EA",
#     PKN="PKN.sif",
#     PKNdot="PKN.dot",
#     wPKN="TimesPKN.EA",
#     nPKN="nodesPKN.NA")
# 
# writeReport(
#     modelOriginal=pknmodel,
#     modelOpt=model,
#     optimResT1=ToyT1opt,
#     optimResT2=NA,
#     CNOlist=CNOlistToy,
#     directory="testToy",
#     namesFiles=namesFilesToy,
#     namesData=list(CNOlist="Toy",model="ToyModel"))

## ----eval=TRUE, echo=FALSE------------------------------------------
unlink("testToy",recursive=TRUE)

## ----eval=TRUE------------------------------------------------------
dataToy <- readMIDAS("ToyDataMMB.csv")
CNOlistToy <- makeCNOlist(dataToy,subfield=FALSE)
pknmodel <- readSIF("ToyPKNMMB.sif")

## ----wrap1, eval=FALSE, include=TRUE--------------------------------
# res <- CNORwrap(
#     paramsList=NA,
#     name="Toy",
#     namesData=list(CNOlist="ToyData",model="ToyModel"),
#     data=CNOlistToy,
#     model=pknmodel)

## ----eraseToyDir, eval=TRUE, include=FALSE--------------------------
unlink("Toy",recursive=TRUE)

## ----wrap2, eval=FALSE----------------------------------------------
# pList<-defaultParameters(CNOlistToy, pknmodel)
# #pList$data = CNOlistToy
# #pList$model = ToyModel
# res <- CNORwrap(
#     paramsList=pList,
#     name="Toy1Step",
#     namesData=list(CNOlist="ToyData",model="ToyModel"))

## ----eraseData, eval=TRUE, echo=FALSE-------------------------------
unlink("ToyDataMMB.csv")
unlink("ToyPKNMMB.sif")
unlink("Toy1Step",recursive=TRUE)
unlink("ToyModelMMB2.sif")

## ----getDREAM, eval=TRUE--------------------------------------------
#Option 1: copy the SIF and MIDAS files (followed by readMIDAS, makeCNOlist and readSIF)
cpfile<-dir(system.file("DREAMModel",package="CellNOptR"),full=TRUE)
file.copy(from=cpfile,to=getwd(),overwrite=TRUE)
#Option 2: load the CNOlist and model objects
data(CNOlistDREAM,package="CellNOptR")
data(DreamModel,package="CellNOptR")

## ----DREAMAnalysis, eval=FALSE--------------------------------------
# model = preprocessing(CNOlistDREAM, DreamModel, verbose=FALSE)
# res = gaBinaryT1(CNOlistDREAM, model, verbose=FALSE, maxTime=10)
# cutAndPlot(CNOlistDREAM, model, bStrings=list(res$bString), plotPDF=TRUE,
#     plotParams=list(maxrow=25, margin=0.1, width=20, height=20))
# 
# 

## ----t2load, eval=TRUE----------------------------------------------
data(CNOlistToy2,package="CellNOptR")
data(ToyModel2,package="CellNOptR")
pknmodel = ToyModel2
cnolist = CNOlist(CNOlistToy2)

## ----t2plotCNOlist, fig.width=7, eval=TRUE--------------------------
plot(cnolist)

## ----t2Opt1, eval=TRUE----------------------------------------------
model = preprocessing(cnolist, pknmodel, verbose=FALSE)
T1opt <- gaBinaryT1(cnolist, model, stallGenMax=10, maxTime=60, verbose=FALSE)

## ----t2OptT1plot, eval=TRUE, fig.width=7----------------------------
sim <- cutAndPlot(model=model, bStrings=list(T1opt$bString),
    CNOlist=cnolist, plotPDF=TRUE)

## ----eval=TRUE, fig.width=6 , fig.height=6--------------------------
plotFit(optRes=T1opt)

## ----t2OptT2, eval=TRUE---------------------------------------------
T2opt<-gaBinaryTN(cnolist, model,  bStrings=list(T1opt$bString),
    stallGenMax=10, maxTime=60, verbose=FALSE)

## ----resSimT2, eval=FALSE-------------------------------------------
# res <- cutAndPlot(
#     model=model,
#     bStrings=list(T1opt$bString, T2opt$bString),
#     CNOlist=cnolist,
#     plotPDF=TRUE, plotParams=list(cex=0.8, cmap_scale=0.5, margin=0.2))

## ----eval=FALSE-----------------------------------------------------
# # ---------------------- load the library and get a SIF and MIDAS file
# library(CellNOptR)
# library(stringr)
# #
# # ---------------------- examples are provided in CellNOptR
# data("ToyModel", package="CellNOptR")
# data("CNOlistToy", package="CellNOptR")
# pknmodel = ToyModel
# cnolist = CNOlist(CNOlistToy)
# #
# # ---------------------- alternatively you can read your own files:
# # pknmodel = readSIF("ToyModel.sif")
# # cnolist = CNOlist("ToyDataMMB.csv")
# #
# # ---------------------- preprocess the network
# model = preprocessing(cnolist, pknmodel)
# #
# # ---------------------- perform the analysis
# cplexPath = "path/to/cplex"
# resILP = ilpBinaryT1(cnolist = cnolist, model = model,
#                      numSolutions = 3, relGap = 0.05,
#                      cplexPath = cplexPath) # asking to retrieve 3
#                                             # equivalent solutions with
#                                             # tolerance gap relGap=0.05
# #
# # ---------------------- plot the results (optimized models + fits)
# cutAndPlot(CNOlist = cnolist, model = model, bStrings = list(resILP$bitstringILP[[1]]))
# plotModel(model, cnolist, bString=resILP$bitstringILP[[1]])
# 
# cutAndPlot(CNOlist = cnolist, model = model, bStrings = list(resILP$bitstringILP[[2]]))
# plotModel(model, cnolist, bString=resILP$bitstringILP[[2]])
# 
# cutAndPlot(CNOlist = cnolist, model = model, bStrings = list(resILP$bitstringILP[[3]]))
# plotModel(model, cnolist, bString=resILP$bitstringILP[[3]])

## ----eval=FALSE-----------------------------------------------------
# # ---------------------- load the library and get a SIF and MIDAS file
# library(CellNOptR)
# #
# # ---------------------- examples are provided in CellNOptR
# data(PKN_ToyPB, package="CellNOptR")
# data(CNOlist_ToyPB, package="CellNOptR")
# #
# # ---------------------- preprocess the network
# model = preprocessing(data = cnodata,model = pknmodel,compression = T,expansion = T)
# plotModel(model,cnodata)
# #
# #------------------------ original CNOlist contains many timepoints, we use only a subset
# plot(cnodata)
# selectedTime = c(0,10)
# cnodata_prep = cutCNOlist(cnodata ,model = model,
#                           cutTimeIndices = which(!cnodata@timepoints %in% selectedTime))
# plot(cnodata_prep)
# #
# # ---------------------- perform the analysis
# opt = gaBinaryT1(CNOlist = cnodata_prep,model = model)
# #
# # ---------------------- 10-fold cross-validation procedure using T1 data
# library(doParallel)
# doParallel::registerDoParallel(cores=3)
# system.time({R1=crossvalidateBoolean(CNOlist = cnodata_prep, model = model,
#                                      type="datapoint", nfolds=10, parallel = TRUE)})
# system.time({R2=crossvalidateBoolean(CNOlist = cnodata_prep, model=model,
#                                      type="experiment", nfolds=10, parallel = TRUE)})
# system.time({R3=crossvalidateBoolean(CNOlist = cnodata_prep, model=model,
#                                      type="observable", nfolds=10, parallel = TRUE)})
# system.time({R4=crossvalidateBoolean(CNOlist = cnodata_prep, model=model,
#                                      type="datapoint", nfolds=10, parallel = FALSE)})
# system.time({R5=crossvalidateBoolean(CNOlist = cnodata_prep, model=model,
#                                      type="experiment", nfolds=10, parallel = FALSE)})
# system.time({R6=crossvalidateBoolean(CNOlist = cnodata_prep, model=model,
#                                      type="observable", nfolds=10, parallel = FALSE)})

#
#  This file is part of the CNO software
#
#  Copyright (c) 2011-2013 - EBI
#
#  File author(s): CNO developers (cno-dev@ebi.ac.uk)
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  CNO website: http://www.cellnopt.org
#
##############################################################################
# $Id$

# randomize a cnolist by adding noise to the signal matrices at all times except time=0
# The random noise is a gaussian distribution, which standard deviation can be
# changed.
# The random noise can also be a uniform noise between 0 and 1, in which case
# the data is not used anymore.
randomizeCNOlist <- function(cnolist, sd=0.1, minValue=0, maxValue=1, mode="gaussian"){

    if (mode %in% list("gaussian", "uniform", "shuffle")==FALSE){
        stop("mode can be only 'gaussian', 'uniform', or 'shuffle'")
    }

    cnames = colnames(cnolist@signals$`0`)
    dimensions = dim(cnolist@signals$`0`)
    times = cnolist@timepoints


    for (time in 2:length(times)){
        if (mode=="gaussian"){
            cnolist@signals[[time]] = rnorm(cnolist@signals[[time]], sd=sd)
        }
        if (mode=="uniform"){
            cnolist@signals[[time]] = runif(cnolist@signals[[time]])
        }
        if (mode=="shuffle"){
            cnolist@signals[[time]] = sample(getSignals(c)[[time]])

        }
        # make sure that values are still in the expected range
        cnolist@signals[[time]][cnolist@signals[[time]]<minValue] <- minValue
        cnolist@signals[[time]][cnolist@signals[[time]]>maxValue] <- maxValue

        # set back the column names that are lost
        dim(cnolist@signals[[time]])<-dimensions
        colnames(cnolist@signals[[time]])<-cnames
    }

     return(cnolist)
 }

#
#  This file is part of the CNO software
#
#  Copyright (c) 2011-2012 - EBI
#
#  File author(s): CNO developers (cno-dev@ebi.ac.uk)
#
#  Distributed under the GPLv2 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-2.0.html
#
#  CNO website: http://www.ebi.ac.uk/saezrodriguez/cno
#
##############################################################################
# $Id$

# This is a test of the ToyModel and gaBinaryT1
library(CellNOptR)

# read data
pknmodel = readSIF(system.file("ToyModel/ToyPKNMMB.sif", package="CellNOptR"))
data = readMIDAS(system.file("ToyModel/ToyDataMMB.csv", package="CellNOptR"))
cnolist = makeCNOlist(data, subfield=FALSE)

# preprocessing
model = preprocessing(cnolist, pknmodel, verbose=FALSE)

# optimisation
T1opt<-gaBinaryT1(CNOlist=cnolist,model=model,verbose=FALSE)

truebs =c(1, 1, 0, 1, 1, 0, 1, 1, 0, 0, 1, 1, 1, 0, 0, 0)
print(truebs)
print(T1opt$bString)

# testing valid output
if (dist(rbind(T1opt$bString, truebs))>2){
stop("something wrong going on")
}

# extra call to simulateTN
SimT1<-simulateTN(CNOlist=cnolist,model=model, bStrings=list(truebs))

#
#  This file is part of the CNO software
#
#  Copyright (c) 2011-2012 - EMBL - European Bioinformatics Institute
#
#  File author(s): CNO developers (cno-dev@ebi.ac.uk)
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  CNO website: http://www.cellnopt.org
#
##############################################################################
# $Id$
normaliseCNOlist <- function(
    CNOlist,
    EC50Data=0.5,
    HillCoef=2,
    EC50Noise=0.,
    detection=0,
    saturation=Inf,
    changeTh=0,
    norm2TorCtrl=NULL,
    mode="time", options=list(rescale_negative=TRUE),
    verbose=FALSE){


    if (!is(CNOlist,"CNOlist")){
         CNOlist = CellNOptR::CNOlist(CNOlist)
     }

    if (is.null(norm2TorCtrl)==FALSE){
        stop("'norm2TorCtrl' argument is deprecated. Please use 'mode=' instead")
    }

    #Check that the parameters have the right format
    if(!is(EC50Data,"numeric") | length(EC50Data) != 1)
        warning("The parameter 'EC50Data' should be a single numeric value")
    if(!is(EC50Noise,"numeric")  | length(EC50Noise) != 1)
        warning("The parameter 'EC50Noise' should be a single numeric value")
    if(!is(HillCoef,"numeric") | length(HillCoef) != 1)
        stop("The parameter 'HillCoef' should be a single numeric value")
    if(!is(detection,"numeric") | length(detection) != 1)
        stop("The parameter 'detection' should be a single numeric value")
    if(!is(changeTh,"numeric") | length(changeTh) != 1)
        stop("The parameter 'changeTh' should be a single numeric value")
    if(!is(saturation,"numeric") | length(saturation) != 1)
        stop("The parameter 'Saturation' should be a single numeric value")
    if(mode %in% c("time","ctrl", "raw") == FALSE)
        stop("The parameter 'mode' should be either 'time' or 'ctrl' or 'raw'")


    if (verbose==TRUE){
        cat("Normalisation mode is: ", mode, "\n", sep="")
    }
    # Check which values are out of the dynamic range of the machine: create a list
    # of matrices, one matrix for each time point (i.e. each element of the field
    # CNOlist$valueSignals) filled with FALSE if the measurement is in the dynamic
    # range, and true if it isn't (i.e. it should be replaced by NA at the end) i.e.
    # if Detection=0 and saturation=Inf, then all the matrices in NaNsList should be
    # filled with 0s this function also tags as NAs all the field that don't have a
    # value (should be imported as NA)

    NaNsList <- CNOlist@signals

    DynamicRange <- function(x){
        if (verbose==TRUE){
            data_sat = x[which(x > saturation )]
            cat("found ", length(data_sat), " data points above saturation (", saturation, ").\n", sep="")
        }
        x[which(x > saturation )] <- Inf

        if (verbose==TRUE){
            data = x[which(x < detection)]
            cat("found ", length(data), " data points below detection (",detection, ").\n", sep="")
        }
        x[which(x < detection )] <- Inf

        x[which(is.na(x))] <- Inf
        x[is.finite(x)] <- FALSE
        x[is.infinite(x)] <- TRUE
        return(x)
        }

    NaNsList <- lapply(NaNsList, DynamicRange )
    # Calculate relative change

    # the following list will contain true for the fold changes that are negatives
    # and significant,and false for the other ones.
    negList <- CNOlist@signals
    negList <- lapply(negList,
                    function(x) x <- matrix(data=FALSE,
                    nrow=dim(x)[1],ncol=dim(x)[2])
                )

    # the following list will contain true for the fold changes that are positive
    # and significant, and false for the other ones.
    posList <- CNOlist@signals
    posList <- lapply(posList,
                    function(x) x <- matrix(data=FALSE,
                    nrow=dim(x)[1],ncol=dim(x)[2])
                )

    # the following matrix will contain the actual fold change values
    FoldChangeList <- CNOlist@signals

    # if mode="raw" data may be negative
    if(mode == "raw"){
        for(i in 2:length(FoldChangeList)){
            negList[[i]][which((CNOlist@signals[[i]] - CNOlist@signals[[1]]) < (-1*changeTh))] <- TRUE
            posList[[i]][which((CNOlist@signals[[i]] - CNOlist@signals[[1]]) > changeTh)] <- TRUE
        }
    }

    # if mode="time" then the relative change is simply matrix t1 - matrix
    # t0 (ie each measurement is compared to the exact same condition at time 0).

    if(mode == "time"){
        for(i in 2:length(FoldChangeList)){
            FoldChangeList[[i]] <- abs(CNOlist@signals[[i]] - CNOlist@signals[[1]])/CNOlist@signals[[1]]


            negList[[i]][which((CNOlist@signals[[i]] - CNOlist@signals[[1]]) < (-1*changeTh))] <- TRUE
            posList[[i]][which((CNOlist@signals[[i]] - CNOlist@signals[[1]]) > changeTh)] <- TRUE
        }

        FoldChangeList[[1]] <- matrix(
            0,
            ncol=dim(FoldChangeList[[1]])[2],
            nrow=dim(FoldChangeList[[1]])[1])

        }

    # else
    if(mode == "ctrl"){

    # if mode="ctrl" then the relative change is computed relative to the
    # ctrl at the same time the ctrl is the case without stimuli, but with the same
    # inhibitors.   In our case this still means that at t0 we are going to have
    # zero everywhere since only two measurements were made: with and without the
    # inhibitor(s) and these measurements have been copied across corresponding
    # position this last bit makes sense because we assume that the inhibitors are
    # already present at time 0 when we add the stimuli to find the right row to
    # normalise, we look for a row in the valueStimuli that has only 0's but where
    # the corresponding row in the inhibitor matrix has the same status as the row
    # that we are trying to normalise.
        for(i in 2:length(FoldChangeList)){
            for(n in 1:dim(FoldChangeList[[i]])[1]){
                #First I treat the rows that are not ctrls
                if(sum(CNOlist@stimuli[n,]) != min(apply(CNOlist@stimuli,MARGIN=1,sum))){
                    ctrlRow <- intersect(
                        which(
                            apply(CNOlist@stimuli,MARGIN=1,sum) ==
                                min(apply(CNOlist@stimuli,MARGIN=1,sum))),
                        which(
                            apply(CNOlist@inhibitors,MARGIN=1,
                                function(x) all(x == CNOlist@inhibitors[n,]))))
                    FoldChangeList[[i]][n,] <- abs(CNOlist@signals[[i]][n,] - CNOlist@signals[[i]][ctrlRow,])/CNOlist@signals[[i]][ctrlRow,]
                    negList[[i]][n,which((CNOlist@signals[[i]][n,] - CNOlist@signals[[i]][ctrlRow,]) < (-1*changeTh) )] <- TRUE
                    posList[[i]][n,which((CNOlist@signals[[i]][n,] - CNOlist@signals[[i]][ctrlRow,]) > changeTh )] <- TRUE

                    }else{

                        #Then I set to 0 all the rows that are ctrls
                        FoldChangeList[[i]][n,] <- rep(0,dim(FoldChangeList[[i]])[2])
                    }

                }
            }

            FoldChangeList[[1]] <- matrix(
                0,
                ncol=dim(FoldChangeList[[1]])[2],
                nrow=dim(FoldChangeList[[1]])[1])

        }
    # Now I compute the penalty for being noisy, which is calculated for each
    # measurement as the measurement divided by the max measurement across all
    # conditions and time for that particular signal (excluding values out of the
    # dynamic range).

    #1.This small function computes the max across all measurements for each signal,
    #excluding the values out of the dynamic range
    SignalMax <- function(signals,NaNsList){

        for(i in 1:length(signals)){
            signals[[i]][which(NaNsList[[i]] == 1)] <- 0
            }

        for(i in 2:length(signals)){
            signals[[i]] <- rbind(signals[[i-1]],signals[[i]])
            }

        signals <- signals[[length(signals)]]
        if (mode=="raw"){ # in raw mode, negative values are possible, need to take abs
            signalABS <- apply(signals,MARGIN=2,abs)
            signalMax <- apply(signalABS,MARGIN=2,max)
        }else{
            signalMax <- apply(signals,MARGIN=2,max)
        }
        return(signalMax)
        }

    signalMax <- SignalMax(signals=CNOlist@signals,NaNsList=NaNsList)
    # 2.This bit takes a list of matrices and goes into each matrix
    # and divides each row by the vector containing the max values
    PenaltyList <- CNOlist@signals
    PenaltyList <- lapply(
        PenaltyList,
        function(x){
            x <- apply(x,MARGIN=1,function(y){y <- y/signalMax});
            return(t(x))
        })

    #3.Now I can make this list of values go through the saturation function
    if (mode=="raw"){
        # in raw mode, negative values are possible. Just take the absolute values
        SatPenalty <- lapply(PenaltyList,abs)
    }else{
        SatPenalty <- PenaltyList
     }
    SatPenalty <- lapply(SatPenalty,function(x) {x/(EC50Noise+x)} )

    # Now I make the data go through the Hill function
    HillData <- FoldChangeList
    HillData <- lapply(
        HillData,
        function(x) {x^HillCoef/((EC50Data^HillCoef)+(x^HillCoef))} )

    # Now I can multiply HillData and SatPenalty, matrix by matrix and element by
    # element.
    NormData <- HillData
    for(i in 1:length(NormData)){
        NormData[[i]] <- HillData[[i]]*SatPenalty[[i]]
        NormData[[i]][which(negList[[i]] == 1)] <- NormData[[i]][which(negList[[i]] == 1)]*-1
        if(mode != "raw"){ # in raw mode, neg+pos can leaad to zero. time 0 is also zero.
            NormData[[i]][which((negList[[i]] + posList[[i]]) == 0)] <- 0
        }
    }

    # Now let us set to NaN all the values that have been tagged in the beginning as
    # out of the dynamic range.
    for(i in 1:length(NormData)){
        NormData[[i]][which(NaNsList[[i]] == 1)] <- NaN
    }

    # rescale the columns that have negative values. T0 is untouched.
    if (options$rescale_negative == TRUE && 1==0){ # this loop does not work. added 1==0 temporary so that we do not enter in the loop
        for (x in colnames(NormData[[1]])){
            for (i in 2:length(NormData)){ #there is always a Time 0 so were are guarantee to have at least 2 time points
                m = min(NormData[[i]][,x], na.rm=TRUE)
                M = max(NormData[[i]][,x], na.rm=TRUE)
                if (m<0 && m!=M ){
                    NormData[[i]][,x] = (NormData[[i]][,x]-m)/ (M-m)
                }
            }
        }
    }

    # The following code is ONE way of dealing with negative values
    # !!! each experiment/specy is treated separately.
    # 0,-0.5,-1 will be rescaled in 1,0.5,0
    # !!! 0,-0.01,0.5 is also rescaled because there is 1 negative value
    if (options$rescale_negative == TRUE){
        # prepare a more convenient data set to manipulate

        # rescale each column independently
        for (x in colnames(NormData[[1]])){
            # extract only data related to the specy x
            c = NormData[[1]][,x]
            for (i in 2:length(NormData)){
                c = rbind(c, NormData[[i]][,x])
            }
            # get the min and max over time (apply) for each experiment
            min_vector = apply(c, 2, min, na.rm=TRUE)
            max_vector = apply(c, 2, max, na.rm=TRUE)

            # rescale negative values if needed for each experiment
            for (i in seq_along(min_vector)){
                m = min_vector[i]
                M = max_vector[i]
                if (m<0 && m!=M){
                    c[,i] = (c[,i]-m)/(M-m)
                    # Finally, save the new values in NormData
                    for (itime in 1:length(NormData)){
                        NormData[[itime]][i,x] = c[itime,i]
                    }
                }
            }

        }
    }


    CNOlist@signals <- NormData

    # Variance should be zero
    for (i in seq_along(CNOlist@timepoints)){
        CNOlist@variances[[i]] = CNOlist@variances[[i]] * 0
    }

    return(CNOlist)
}

#
#  This file is part of the CNO software
#
#  Copyright (c) 2019 - SaezLab - Heidelberg Universit
#
#  File author(s): CNO developers
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  CNO website: http://www.cellnopt.org
#
##############################################################################

create_binary_variables_for_y_vector <- function(y_vector){
  
  variables = c()
  for(i in 1:length(y_vector)){
    variables[i] <- paste0("y_",i)
  }
  binary_variables_list <- list(1:(length(y_vector)),
                               variables, 
                               paste("reaction",y_vector))
  return(binary_variables_list)
  
}

#
#  This file is part of the CNO software
#
#  Copyright (c) 2011-2012 - EMBL - European Bioinformatics Institute
#
#  File author(s): CNO developers (cno-dev@ebi.ac.uk)
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  CNO website: http://www.cellnopt.org
#
##############################################################################
# $Id$

simulatorTN <-function(simResultsPrev, CNOlist, model, simList, indexList, timeIndex=3){
    #timeIndex=3 correspond to T2

    if (!is(CNOlist,"CNOlist")){
        CNOlist = CellNOptR::CNOlist(CNOlist)
    }
    # check the structures
	if(is.null(CNOlist@stimuli) || is.null(CNOlist@inhibitors)) {
		stop("This function needs 'valueStimuli' and 'valueInhibitors' in CNOlist")
	}
	
	if(is.null(model$reacID) || is.null(model$namesSpecies)) {
		stop("This function needs 'reacID' and 'namesSpecies' in model")
	}

    # variables
    nStimuli <- as.integer(length(indexList$stimulated))
    nInhibitors <- as.integer(length(indexList$inhibited))
    nCond <- as.integer(dim(CNOlist@stimuli)[1])
    nReacs <- as.integer(length(model$reacID))
    nSpecies <- as.integer(length(model$namesSpecies))
    nMaxInputs <- as.integer(dim(simList$finalCube)[2])
    timeIndex <- as.integer(t(timeIndex))
    nTimes <- as.integer(length(model$times))

    # model (times vector and interMat)
    times <- as.integer(t(model$times))
    interMat <- as.integer(t(model$interMat))

    # simList
    finalCube = as.integer(as.vector(t(simList$finalCube))-1)
    ixNeg = as.integer(as.vector(t(simList$ixNeg)))
    ignoreCube = as.integer(as.vector(t(simList$ignoreCube)))
    maxIx = as.integer(simList$maxIx-1)

    # index
    indexSignals <- as.integer(as.vector(indexList$signals)-1)
    indexStimuli <- as.integer(as.vector(indexList$stimulated)-1)
    indexInhibitors <- as.integer(as.vector(indexList$inhibited)-1)
    nSignals <- length(indexSignals)

    # cnolist
    valueInhibitors <- as.integer(CNOlist@inhibitors)
    valueStimuli <- as.integer(CNOlist@stimuli)

    # simResults

    valueSimResults = as.integer(t(simResultsPrev))


    # Calling the C simulator !!
    res = .Call("simulatorTN",
        # variables
        nStimuli,
        nInhibitors,
        nCond,
        nReacs,
        nSpecies,
        nSignals,
        nMaxInputs,
        nTimes,

        # model related related
        timeIndex,
        times,
        interMat,

        # simresults
        valueSimResults, 
        # simList
        finalCube,
        ixNeg,
        ignoreCube,
        maxIx,
        # index
        indexSignals,
        indexStimuli,
        indexInhibitors,
        # cnolist
        valueInhibitors,
        valueStimuli
    )

    return(res)
}

#
#  This file is part of the CNO software
#
#  Copyright (c) 2011-2012 - EMBL - European Bioinformatics Institute
#
#  File author(s): CNO developers (cno-dev@ebi.ac.uk)
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  CNO website: http://www.ebi.ac.uk/saezrodriguez/cno
#
##############################################################################
# $Id: plotCNOlistLargePDF.R 3155 2013-01-09 15:24:58Z cokelaer $
plotCNOlistLargePDF <-
function(CNOlist,filename,nsplit, width=14, height=7){
    pdf(file=filename,width=width,height=height)
    plotCNOlistLarge(CNOlist,nsplit=nsplit,newDevice=FALSE)
    dev.off()
    }

#
#  This file is part of the CNO software
#
#  Copyright (c) 2011-2012 - EMBL - European Bioinformatics Institute
#
#  File author(s): CNO developers (cno-dev@ebi.ac.uk)
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  CNO website: http://www.cellnopt.org
#
##############################################################################
# $Id: readMIDAS.R 4105 2013-11-06 15:49:33Z cokelaer $
readMIDAS<-function(MIDASfile, verbose=TRUE){

    # Read the data.
    # First, let us store the length of each row by counting the column
    ncols = count.fields(MIDASfile, sep = ",")

    # check that there is at least a header and one row of data (i.e. 2 rows)
    if (length(ncols) == 1){
        stop("Your input file has a wrong format (only 1 column)")
    }
    # if at least one row of data, all of them must have the same length
    if (length(unique(ncols[2:length(ncols)]))>1){
        print(unique(ncols[2:length(ncols)]))
        stop("Your input file has a wrong format (rows have different lengths)")
    }

    # Second, let us check the consistency between row and header lengths
    maxncol <- max(ncols)
    ncol_header = ncols[1]
    if (ncol_header == maxncol){ #everything seems right, let us use read.csv
        data<-read.csv(file=MIDASfile,header=TRUE,sep=',',fill=TRUE,as.is=TRUE,check.names=FALSE)
    }
    else{
        # too much columns but we can still proceed. This is useful when extra empty
        # commas are used. We cannot use read.csv that expect length of each row
        # to correspond to the length of the header.
        if (ncol_header < maxncol){
            warning("Some rows have more columns than the header. Extra columns will be removed. Check and fix your MIDAS file.")

            # read the data (except header) and convert to data.frame
            data <- data.frame(read.table(MIDASfile,  skip=1,sep=",",flush=TRUE))

            # remove the extra columns as compared to the header
            data <- data[1:nrow(data), 1:ncol_header]
            # finally set the dataframe column names to be those found in the header
            # use read.csv setting nrow=0 . it reads all the data (not optimal)
            # but do not care about discrepancy between columns length.
            # Note also that the check.names is required to keep track of the special keyword :
            # (otherwise replaced by a dot .)
            header_column_names = read.csv(MIDASfile,sep=",", nrow=0, check.names=FALSE)

            colnames(data) <- colnames(header_column_names)

            #data<-read.csv(file=MIDASfile,sep=',',fill=TRUE,as.is=TRUE,col.names=paste("V",seq_len(ncol_header), sep = ""))
        }
        else {
            # not enough data as compared to the header, so let us stop. could
            # fill with NA but there is surely a mistake in such case.
            stop("Some rows have less columns than the header. Fix your MIDAS file.")
        }
    }

    # Determine which are the informative columns (i.e. columns with useful data info and values)
    TRcol<-grep(pattern="TR:",x=colnames(data),ignore.case=FALSE)
    DAcol<-grep(pattern="DA:",x=colnames(data),ignore.case=FALSE)
    DVcol<-grep(pattern="DV:",x=colnames(data),ignore.case=FALSE)
    #data<-data[,c(TRcol,DAcol,DVcol)]

    # Print information about the data set
    if (verbose){
        print(paste(
            "Your data set comprises ", nrow(data),
            "conditions (i.e. combinations of time point and treatment)"))
    }

    # Check that the right number of columns are present


    # The MIDAS may contain the DA:ALL special time
    if(length(grep("DA:ALL",colnames(data))) != 0 && length(DAcol) == 1){
        if (verbose){
            print("Your data comprises only a DA:ALL column;  all readouts are assumed to have been acquired at the same time.")
        }

        # Make the additional columns
        newDA<-matrix(rep(data[,DAcol],length(DVcol)),byrow=FALSE,ncol=length(DVcol))
        newDAnames<-colnames(data)[DVcol]
        colnames(newDA)<-sub(pattern="DV:",x=newDAnames,replacement="DA:",ignore.case=FALSE)
        data<-cbind(data,newDA)
        data<-data[,-DAcol]
    } else {
        if(length(DAcol) != length(DVcol)){
            warning("DA columns and DV columns do not match.")
        }
    }

    if (verbose){
        print(paste("Your data set comprises measurements on ", length(DVcol)," different species"))
    }

    # first, replace all CellType string into CellLine.
    CellType<-grep(pattern="(TR:\\w*:CellType)",
        x=colnames(data),ignore.case=TRUE,perl=TRUE,value=TRUE)
    for (x in CellType){
        names(data)[names(data)==x] <- sub(":CellType", ":CellLine",x)
    }

    # then look at all CellLine strings before removing them.
    CellLine<-grep(pattern="(TR:\\w*:CellLine)",
        x=colnames(data),ignore.case=TRUE,perl=TRUE,value=TRUE)


    if(length(CellLine) != 0){

        CellLine<-sub(pattern="TR:",x=CellLine,replacement="",ignore.case=FALSE)
        CellLine<-sub(pattern=":CellLine",x=CellLine,replacement="",ignore.case=TRUE)

        if (verbose){
            print(paste(
                "Your data set comprises ", (length(TRcol)-length(CellLine)),
                "stimuli/inhibitors and", length(CellLine),"cell line(s) (",CellLine,")" ))
        }

        data<-data[,-grep(pattern="(TR:\\w*:CellLine)",x=colnames(data),ignore.case=TRUE,perl=TRUE,value=FALSE)]
    }
    else{

        stop("There is no cell line information. You must provide one. Please indicate it in your file by naming them 'TR:name:CellLine' (you may use TR:name:CellType' as well.")
    }

    if (verbose){
        print("Please be aware that CNO only handles measurements on one cell line at this time.")
    }

    # data has been changed so we need to extract columns indices again
    TRcol<-grep(pattern="TR:",x=colnames(data),ignore.case=FALSE)
    DAcol<-grep(pattern="DA:",x=colnames(data),ignore.case=FALSE)
    DVcol<-grep(pattern="DV:",x=colnames(data),ignore.case=FALSE)
    data<-data[,c(TRcol,DAcol,DVcol)]


    # replace NaN character by NA. as.matrix is required to scan all columns AND rows
    # Note that on some older R version, the is.nan does not seem to work well, hence the
    # try catch (see Changelog 0.99.6
    conversion <- tryCatch({
        if (any(is.nan(as.matrix(data)))){
            data[is.nan(as.matrix(data))] <- NA
        }
        conversion = TRUE},
        error=function(e) {return(FALSE)},
        finally={
            if (verbose){
                print("Your data file contained 'NaN'. We have assumed that these were missing values and replaced them by NAs.")
            }}
    )

    if ( conversion == FALSE){
        if(any(as.matrix(data == "NaN"))){
            for(c in 1:dim(data)[2]){
                for(r in 1:dim(data)[1]){
                     if(data[r,c] == NA) {
                         data[r,c]<-NA
                     }
                }
            }
            if (verbose){
                print("Your data file contained 'NaN'. We have assumed that these were missing values and replaced them by NAs.")
            }
        }
    }



    return(list(
        dataMatrix=data,
        TRcol=grep(pattern="TR:",x=colnames(data),ignore.case=FALSE),
        DAcol=grep(pattern="DA:",x=colnames(data),ignore.case=FALSE),
        DVcol=grep(pattern="DV:",x=colnames(data),ignore.case=FALSE)))

    }

#
#  This file is part of the CNO software
#
#  Copyright (c) 2019 - SaezLab - Heidelberg Universit
#
#  File author(s): CNO developers
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  CNO website: http://www.cellnopt.org
#
##############################################################################

writeBoundsForEdges_y_i<- function(y_vector){
  y_bound_vector <- c()
  for(i in 1:length(y_vector)){
    y_bound_vector[i] <- paste0("0 <= xb", i, " <= 1")
  }
  return(y_bound_vector)
}

# library(data.table)
# library(dplyr)
# library(tidyr)
# library(readr)

#' Read network from BNET file
#' 
#' @param filename BNET file. The file is a tab delimited file with two columns,
#' `targets` and `factors`. `factors` contains the logic rule and `targets` the node
#' activated by the logic rule.
#'
#' @return CellNOpt network
#'
#' @author Luis Tobalina
readBNET <- function(filename){
	

	if (!requireNamespace("dplyr", quietly = TRUE)) {
		stop("Package \"dplyr\" needed for this function to work. Please install it.",
			 call. = FALSE)
	}
	if (!requireNamespace("tidyr", quietly = TRUE)) {
		stop("Package \"tidyr\" needed for this function to work. Please install it.",
			 call. = FALSE)
	}
	if (!requireNamespace("readr", quietly = TRUE)) {
		stop("Package \"readr\" needed for this function to work. Please install it.",
			 call. = FALSE)
	}
		
	
	
	bnet <- data.table::fread(filename, header=TRUE)
	
	# count number of `and` gates in each logic rule
	bnet <- bnet %>% dplyr::rowwise() %>% 
		mutate(i_and_gates = (nchar(factors) - nchar(gsub("&", "", factors, perl=TRUE))))
	# count how many `and` gates have appeared up to the current rule
	bnet <- bnet %>% transform(i_and_gates = c(0, i_and_gates[-nrow(bnet)])) %>% 
		mutate(i_and_gates = cumsum(i_and_gates))
	# parse each logic rule
	sif <- bnet %>% dplyr::rowwise() %>% 
		dplyr::do(sif_df = build_sif_table_from_rule(.$factors, .$targets, last_and_num=.$i_and_gates)$sif_str) %>% 
		tidyr::unnest(cols = sif_df) %>% 
		dplyr::ungroup() 
	
	# use `readSIF()` to get the network ready for CellNOpt
	fh <- tempfile()
	write.table(sif$sif_df, file=fh,
				row.names=FALSE, col.names=FALSE, quote=FALSE, sep="\t")
	model <- readSIF(fh)
	
	return(model)
}

#' Read network from BND file
#' 
#' BND is a file format used by MaBoSS to store the boolean network definition.
#' The reader works if the logic for the activation of the node is stated withing the parameter `logic` of the node.
#' An example file can be found in `https://maboss.curie.fr/pub/example.bnd`.
#' 
#' @param filename BND file. 
#' 
#' @examples
#' \dontrun{
#' model = readBND("https://maboss.curie.fr/pub/example.bnd")
#' }
#'
#' @return CellNOpt network
#' 
#' @author Luis Tobalina
readBND <- function(filename){
	required_pcks = list("dplyr","tidyr","readr")
	
	if(!all(unlist(lapply(required_pcks,function(str) {require(str,character.only = TRUE)})))){
		print("the following packages need to be installed to use readBND:")	
		print(unlist(required_pcks))
		print("Please, install the packages manually for this feature.")
		return(-1)
	}
	
	
	bnd <- readr::read_file(filename)
	# remove all line breaks
	bnd <- gsub("\\n", "", bnd, perl=TRUE)
	# remove parenthesis enclosing a single node (which is composed of alphanumeric characters and "_")
	bnd <- gsub("\\((!?[[:alnum:]|_]*)\\)", "\\1", bnd, perl=TRUE)
	# get the information in bnet format 
	targets <- regmatches(bnd, gregexpr("(?<=Node ).*?(?= {)", bnd, perl = TRUE))[[1]]
	factors <- regmatches(bnd, gregexpr("(?<=logic = ).*?(?=;)", bnd, perl = TRUE))[[1]]
	bnet <- data.frame(targets, factors)
	
	# use `readBNET()` to get the network ready for CellNOpt
	fh <- tempfile()
	write.table(bnet, file=fh,
				row.names=FALSE, col.names=TRUE, quote=FALSE, sep="\t")
	model <- readBNET(fh)
}


# logic rule parser written following explanation of boolean expression evaluator in:
# https://unnikked.ga/how-to-build-a-boolean-expression-evaluator-518e9e068a65
# also of interest:
# https://www.strchr.com/expression_evaluator
# Unless the boolean expression is written in Disjunctive Normal Form (DNF), in order
# to get a correct `*.sif` file we will need to process the first interpretation we do of
# the tree into sif format.
#
#' Build a SIF table from a logic rule written in a string
#' 
#' @param rule_str String containing the rule to be parsed 
#' @param target Name of the node affected by the rule
#' @param last_and_num If the rule contains `and` gates, their numeration will start after the number provided here (default is 0)
#'
#' @return data.frame with the network structure derived from the rule.
#' The column `sif_str` contains the string that can be written to a file 
#' and then read with `readSIF()` in order to load a CellNOpt compatible network.
#' 
#' @examples
#' CellNOptR:::build_sif_table_from_rule("B & (C | D)", "A", last_and_num=2)
#' 
#' test_rule <- list()
#' test_rule[[1]] <- "AMP_ATP | (ATM & ATR) | HIF1 | !(EGFR | FGFR3)"
#' test_rule[[2]] <- "A & ((B | C) & !(D & E))"
#' test_rule[[3]] <- "A & B | C"
#' test_rule[[4]] <- "A & B & C"
#' test_rule[[5]] <- "A & (B | C)"
#' test_rule[[6]] <- "(A | B) & (C | D)"
#' test_rule[[7]] <- "!(C & D) | (E & F)"
#' test_rule[[8]] <- "(A | B) & (C | !D) & (E | F)"
#' parsed_rule <- list()
#' for (i in c(1:length(test_rule))){
#'   parsed_rule[[i]] <- CellNOptR:::build_sif_table_from_rule(test_rule[[i]], "T")
#' }
#' 
build_sif_table_from_rule <- function(rule_str, target, last_and_num=0) {
	required_pcks = list("dplyr","tidyr","readr")
	
	if(!all(unlist(lapply(required_pcks,function(str) {require(str,character.only = TRUE)})))){
		print("the following packages need to be installed to use readBND:")	
		print(unlist(required_pcks))
		print("Please, install the packages manually for this feature.")
		return(-1)
	}
	# remove all whitespaces
	rule_str <-  gsub(" ", "", rule_str)
	# split rule in its different atomic components
	tokens <- strsplit(rule_str, "(?=[\\|\\&\\(\\)!])", perl=TRUE)[[1]]
	# the next set of variables will be used and/or modified by the inner functions
	current_token_pos <- 0
	symbol <- ""
	root <- ""
	negation_flag <- FALSE
	sif_str <- ""
	sif_list <- list()
	or_list <- list()
	and_list <- list()
	not_list <- list()
	sif_num <- 0
	or_num <- 0
	and_num <- last_and_num
	not_num <- 0
	
	# This function is used to retrieve the next token from the rule string
	get_next_token <- function(){
		current_token_pos <<- current_token_pos + 1
		#cat("\n", current_token_pos, tokens[current_token_pos], sep=" ")
		if (current_token_pos<=length(tokens)){
			symbol <<- tokens[current_token_pos]
			# apply de Morgan's Law if necessary
			if (negation_flag){
				if (symbol=="|"){
					symbol <<- "&"
				} else if (symbol=="&"){
					symbol <<- "|"
				}
			}
		} else {
			symbol <<- ""
		}
	}
	
	# helper function to parse each expression
	get_expression <- function(){
		get_term()
		while (symbol=="|") {
			left_part <- root
			get_term()
			right_part <- root
			
			or_num <<- or_num + 1
			or_list[[or_num]] <<- c(left_part, right_part)
			root <<- paste("or", or_num, sep="")
			
			sif_num <<- sif_num + 1
			sif_list[[sif_num]] <<- c(left_part, right_part, root)
		}
	}
	
	# helper function to parse each term
	get_term <- function(){
		get_factor()
		while (symbol=="&") {
			left_part <- root
			get_factor()
			right_part <- root
			
			and_num <<- and_num + 1
			and_list[[and_num]] <<- c(left_part, right_part)
			root <<- paste("and", and_num, sep="")
			
			sif_num <<- sif_num + 1
			sif_list[[sif_num]] <<- c(left_part, right_part, root)
		}
	}
	
	# helper function
	get_factor <- function(){
		get_next_token()
		if (symbol=="!") {
			# negate next expression
			negation_flag <<- !negation_flag
			get_factor()
			negation_flag <<- !negation_flag
			# revert changes to & and | symbols if they where acquired before the reset of the negation flag
			if (symbol=="|"){
				symbol <<- "&"
			} else if (symbol=="&"){
				symbol <<- "|"
			}
		} else if (symbol=="(") {
			get_expression()
			get_next_token()
		} else if (symbol==")") {
			# we don't care about ')'
		} else{
			if (negation_flag) {
				symbol <<- paste("!", symbol, sep="")
			}
			root <<- symbol
			get_next_token()
		}
	}
	
	# helper function to rename `and` or `or` gates (i.e. reset how they are numbered)
	rename_gates <- function(sif_df, type, last_num=0){
		if (!(type %in% c("and", "or"))){
			stop("type should be \"and\" or \"or\".")
		}
		type_gates <- unique(c(sif_df$node_in, sif_df$node_out))
		type_gates <- type_gates[grepl(paste("^", type, "\\d{1,}", sep=""), type_gates, perl=TRUE)]
		if (length(type_gates)>0){
			type_gates <- cbind(type_gates, paste("and", (c(1:length(type_gates))+last_num), sep=""))
			type_gates <- data.frame(type_gates)
			colnames(type_gates) <- c("old_name", "new_name")
			type_gates <- type_gates %>% dplyr::mutate_if(is.factor, as.character)
			
			dic = type_gates$new_name
			names(dic) = type_gates$old_name
			
			sif_df <- sif_df %>% 
				dplyr::mutate(node_in = dplyr::recode(node_in, !!!dic)) %>% 
				dplyr::mutate(node_out = dplyr::recode(node_out,  !!!dic))
		}
		return(sif_df)
	}
	
	# helper function to simplify a chain of `and` or `or` gates
	simplify_gates <- function(sif_df, type){
		if (!(type %in% c("and", "or"))){
			stop("type should be \"and\" or \"or\".")
		}
		
		type_pattern <- paste("^", type, "\\d{1,}", sep="")
		while (any(grepl(type_pattern, sif_df$node_in, perl=TRUE) &
				   grepl(type_pattern, sif_df$node_out, perl=TRUE))){
			consecutive_type_gates <- sif_df %>% filter(grepl(type_pattern, sif_df$node_in, perl=TRUE), grepl(type_pattern, sif_df$node_out, perl=TRUE))
			
			dic = consecutive_type_gates$node_out
			names(dic) = consecutive_type_gates$node_in
			
			sif_df <- sif_df %>% 
				dplyr::mutate(node_in = dplyr::recode(node_in, !!!dic)) %>% 
				dplyr::mutate(node_out = dplyr::recode(node_out, !!!dic)) %>% 
				filter(node_in!=node_out)
		}
		return(sif_df)
	}
	
	# helper function to interpret the parsed expression stored in the `sif_list` variable as a data.frame
	# `sif_list` contains the parsed expression in a tree like structure and this function takes care of making
	# several simplifications until a SIF format friendly version is reached (i.e. no concatenated `and` nodes and
	# no `or` nodes).
	interpret_sif_list <- function(sif_list){
		tree_df <- data.frame(matrix(unlist(sif_list), nrow=length(sif_list), byrow=TRUE))
		colnames(tree_df) <- c("left_part", "right_part", "root")
		
		sif_df <- tree_df[,c("left_part", "root")]
		colnames(sif_df) <- c("node_in", "node_out")
		sif_df <- rbind(sif_df, setNames(tree_df[,c("right_part", "root")], names(sif_df)))
		# add target
		sif_df <- rbind(sif_df, data.frame(node_in=root, node_out=target))
		sif_df <- sif_df %>% dplyr::mutate_if(is.factor, as.character)
		
		# simplify cascade of "and" and "or" gates
		sif_df <- simplify_gates(sif_df, "and")
		#sif_df <- simplify_gates(sif_df, "or")
		
		# we need to have the expression in disjunctive normal form (DNF)
		# find if there are or gates (node_in) connected to and gates (node_out)
		or_to_and_gates <- sif_df %>% dplyr::filter(grepl("^or\\d{1,}", sif_df$node_in, perl=TRUE), grepl("^and\\d{1,}", sif_df$node_out, perl=TRUE))
		while (dim(or_to_and_gates)[1]>0){
			current_pair <- or_to_and_gates[1,]
			or_group_df <- sif_df %>% dplyr::filter(grepl(current_pair$node_in, sif_df$node_out, perl=TRUE)) %>% 
				dplyr::mutate(num_or = 1) %>% 
				dplyr::mutate(num_or = cumsum(num_or)) %>% 
				dplyr::mutate(new_node_out = paste(current_pair$node_out, node_out, num_or, sep="_")) %>% 
				dplyr::mutate(old_node_out = node_out) %>% 
				dplyr::mutate(node_out = new_node_out)
			and_group_df <- sif_df %>% dplyr::filter(grepl(current_pair$node_out, sif_df$node_out, perl=TRUE) &
											  	!grepl(current_pair$node_in, sif_df$node_in, perl=TRUE)) %>% 
				dplyr::mutate(new_node_out = paste(c(or_group_df$new_node_out), collapse=",")) %>% 
				dplyr::mutate(new_node_out = strsplit(as.character(new_node_out), ",")) %>% 
				tidyr::unnest(cols = new_node_out) %>% 
				dplyr::mutate(old_node_out = node_out) %>% 
				dplyr::mutate(node_out = new_node_out)
			gate_group_df <- sif_df %>% dplyr::filter(grepl(current_pair$node_out, sif_df$node_out, perl=TRUE) &
											   	grepl(current_pair$node_in, sif_df$node_in, perl=TRUE)) %>% 
				dplyr::mutate(new_node_out = node_in) %>% 
				dplyr::mutate(new_node_in = paste(c(or_group_df$new_node_out), collapse=",")) %>% 
				dplyr::mutate(new_node_in = strsplit(as.character(new_node_in), ",")) %>% 
				tidyr::unnest(cols = new_node_in) %>% 
				dplyr::mutate(old_node_in = node_in) %>% 
				dplyr::mutate(old_node_out = node_out) %>% 
				dplyr::mutate(node_in = new_node_in) %>% 
				dplyr::mutate(node_out = new_node_out)
			root_group_df <- sif_df %>% dplyr::filter(grepl(current_pair$node_out, sif_df$node_in, perl=TRUE)) %>% 
				dplyr::mutate(new_node_in = current_pair$node_in) %>% 
				dplyr::mutate(old_node_in = node_in) %>% 
				dplyr::mutate(node_in = new_node_in)
			new_sif_df_part <- rbind(or_group_df[,c("node_in", "node_out")],
									 and_group_df[,c("node_in", "node_out")],
									 gate_group_df[,c("node_in", "node_out")],
									 root_group_df[,c("node_in", "node_out")])
			
			# if the or gate is connected to a different and gate than the one being processed now
			# we will rename it in those instances so that we don't lose these connections when filtering
			# the part of the dataframe that we are just about to change
			if (nrow(or_to_and_gates %>% filter(node_in==current_pair$node_in))>1) {
				duplicate_sif_df_part <- sif_df %>% dplyr::filter((grepl(current_pair$node_in, sif_df$node_in, perl=TRUE) &
																!grepl(current_pair$node_out, sif_df$node_out, perl=TRUE)) |
																	(grepl(current_pair$node_in, sif_df$node_out, perl=TRUE)))
				or_num <<- or_num + 1
				new_or_gate_name <- paste("or", or_num, sep="")
				
				dic = new_or_gate_name
				names(dic) = current_pair$node_in
				
				duplicate_sif_df_part <- duplicate_sif_df_part %>% 
					dplyr::mutate(node_in = dplyr::recode(node_in, !!!dic)) %>% 
					dplyr::mutate(node_out = dplyr::recode(node_out, !!!dic))
				sif_df <- sif_df %>% filter(!(grepl(current_pair$node_in, sif_df$node_in, perl=TRUE) &
											  	!grepl(current_pair$node_out, sif_df$node_out, perl=TRUE))) %>% 
					rbind(duplicate_sif_df_part)
			}
			
			# substitute the corresponding part of sif_df with the newly calculated df
			sif_df <- sif_df %>% dplyr::filter(!((grepl(paste("^", current_pair$node_in, "$", sep=""), sif_df$node_out, perl=TRUE)) |
										  	(grepl(paste("^", current_pair$node_out, "$", sep=""), sif_df$node_out, perl=TRUE) &
										  	 	!grepl(paste("^", current_pair$node_in, "$", sep=""), sif_df$node_in, perl=TRUE)) |
										  	(grepl(paste("^", current_pair$node_out, "$", sep=""), sif_df$node_out, perl=TRUE) &
										  	 	grepl(paste("^", current_pair$node_in, "$", sep=""), sif_df$node_in, perl=TRUE)) |
										  	grepl(paste("^", current_pair$node_out, "$", sep=""), sif_df$node_in, perl=TRUE)))
			
			sif_df <- rbind(sif_df, new_sif_df_part)
			sif_df <- sif_df %>% distinct()
			
			# simplify cascade of "and" and "or" gates
			sif_df <- simplify_gates(sif_df, "and")
			sif_df <- simplify_gates(sif_df, "or")
			
			# update list of or to and gates
			or_to_and_gates <- sif_df %>% dplyr::filter(grepl("^or\\d{1,}", sif_df$node_in, perl=TRUE), grepl("^and\\d{1,}", sif_df$node_out, perl=TRUE))
		}
		
		sif_df <- rename_gates(sif_df, type="and", last_num=last_and_num)
		
		# list or gates
		or_gates <- sif_df %>% dplyr::filter(grepl("^or\\d{1,}", node_out, perl=TRUE)) %>% 
			dplyr::group_by(node_out) %>% 
			dplyr::summarise(or_members = paste(node_in, collapse=",")) %>% 
			dplyr::mutate(root = node_out) %>% 
			dplyr::select(root, or_members) %>% 
			dplyr::ungroup()
		
		
		# substitute or gates by their inputs
		while (any(grepl("^or\\d{1,}", sif_df$node_in, perl=TRUE))) {
			
			dic = or_gates$or_members
			names(dic) = or_gates$root
			
			sif_df <- sif_df %>% dplyr::mutate(node_in = dplyr::recode(node_in, !!!dic))
			sif_df <- sif_df %>% dplyr::filter(!grepl("^or\\d{1,}", node_out, perl=TRUE))
			sif_df <- sif_df %>% dplyr::mutate(node_in = strsplit(as.character(node_in), ",")) %>% tidyr::unnest(cols = node_in)
		}
		return(sif_df)
	}
	
	# helper function to write the column with the string for the SIF file
	write_sif <- function(sif_df){
		sif_df <- sif_df %>% dplyr::mutate(sign1 = !grepl("^!", node_in, perl=TRUE)) %>% 
			dplyr::mutate(sign2 = !grepl("^!", node_out, perl=TRUE)) %>% 
			dplyr::mutate(sign = ifelse(sign1 & sign2, "1", "-1")) %>% 
			dplyr::mutate(sif_str = paste(node_in, sign, node_out, sep="\t")) %>% 
			dplyr::mutate(sif_str = gsub("^!", "", sif_str, perl=TRUE))
	}
	
	# After having defined all the helper functions, we parse the input expression.
	
	# if the rule only has one input (or a negated input), we output the sif format directly
	if (length(tokens)==1){
		sif_df <- data.frame(node_in=tokens, node_out=target)
		sif_df <- write_sif(sif_df)
		return(sif_df)
	} else if (length(tokens)==2){
		if (tokens[1]=="!"){
			sif_df <- data.frame(node_in=paste("!", tokens[2], sep=""), node_out=target)
			sif_df <- write_sif(sif_df)
			return(sif_df)
		}
	}
	
	# if there is more than one element, we parse the expression
	get_expression()
	sif_df <- interpret_sif_list(sif_list)
	sif_df <- write_sif(sif_df)
	
	#return(list(sif_df, sif_list))
	return(sif_df)
}

#
#  This file is part of the CNO software
#
#  Copyright (c) 2011-2012 - EMBL - European Bioinformatics Institute
#
#  File author(s): CNO developers (cno-dev@ebi.ac.uk)
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  CNO website: http://www.cellnopt.org
#
##############################################################################
# $Id$
plotCNOlist2<-function(cnolist, simulated_cnolist=NULL, ymin=0, ymax=1){
    if (!requireNamespace("ggplot2", quietly = TRUE)) {
    stop("ggplot2 needed for this function to work. Please install it: install.packages('ggplot2')",
      call. = FALSE)
  }
    #check that CNOlist is a CNOlist


    if (is.null(simulated_cnolist)==FALSE){
        d1 = .cnolist2ggdata(cnolist)
        d2 = .cnolist2ggdata(simulated_cnolist, label="simulated")
        d = rbind(d1, d2)
    } else{
        d = .cnolist2ggdata(cnolist)
    }

    times=NULL # just to prevent warnings
    values=NULL # just to prevent warnings
    type=NULL # just to prevent warnings
    # in ggplot, you can remove colour=type to get only black lines
    # remove shape=type to get black-circle symbol
    sp <- ggplot(d, aes(x=times, y=values, group=type, shape=type, colour=type)) +
        geom_point(size=3, fill="white") +
        geom_line(size=1.) +
        xlab("") +
        ylab("Conditions")
        #scale_shape_manual(name="data type", values=c(22,21))
        #scale_color_manual(name="data type", values=c("black","red"))

    sp + facet_grid(conditions~species) + theme_bw()+
        theme(
            strip.text.x=element_text(size=8,angle=0),
            strip.text.y = element_text(size=12, face="bold"),
            strip.background = element_rect(colour="red", fill="#CCCCFF")) +
         ylim(ymin, ymax)
    #+ggtitle("fff")

    # does not seem to work well if the following code is uncommented, ggplot
    # does nto show the plot even though the code does ot enter in the if
    # statement.
    # save in PDF or whateve accepted format by ggsave.
    #if (is.null(filename)==FALSE){
        #ggsave(filename, useDingbats=FALSE)
    #}
    #cannot return anything otherwise ggplot does not show the plot...
    #return(d)
}

#convert a cnolist signal data to a data structure suitable for ggplot
.cnolist2ggdata <- function(cnolist, label="measured"){
    allvalues = c()
    alltimes  = c()
    allspecies = c()
    allconditions = c()
    allcues = c()
    times = cnolist@timepoints
    nCues = length(cnolist@cues[1,])

    for(r in 1:dim(cnolist@signals[[1]])[1]){
        for(c in 1:dim(cnolist@signals[[1]])[2]){
            yVal = c()
            for (time in 1:length(times)){
                yVal<- c(yVal, as.numeric(cnolist@signals[[time]][r,c]))
            }
            specy = colnames(cnolist@signals[[1]])[c]
            species = rep(specy, length(yVal))
            cues = cnolist@cues[r,]
            conditions=rep(r, length(yVal))

            allcues = c(allcues, cues)
            allvalues = c(allvalues, yVal)
            allspecies = c(allspecies, species)
            alltimes = c(alltimes, times)
            allconditions = c(allconditions, conditions)
        }
    }
    n1 = length(allvalues)
    d1 = data.frame(values=allvalues, species=allspecies, times=alltimes,
        conditions=allconditions, type=rep(label, n1))

    ma = matrix(allcues, ncol=nCues)
    colnames(ma) = names(cnolist@cues[1,])
    d1 = cbind(d1, ma)

    return(d1)
}

#
#  This file is part of the CNO software
#
#  Copyright (c) 2011-2012 - EBI
#
#  File author(s): CNO developers (cno-dev@ebi.ac.uk)
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  CNO website: http://www.ebi.ac.uk/saezrodriguez/cno
#
##############################################################################
# $Id$
#' @export
CNOdata <- function(filename, verbose=FALSE, url=NULL){
	
	stop("This function is depricated. The EBI wesite is not available. 
		 Consult https://saezlab.github.io/CellNOptR/5_Models%20and%20Documentation/ for models.")
	
	valid_filenames = c(
		"PKN-ToyMMB.sif",
		"PKN-ToyPB.sif", 
		"PKN-ToyMSB2009.sif",
		"PKN-ToyMMB_T2.sif",
		"MD-ToyMMB_T2.csv",
		"PKN-ExtLiverPCB.sif",
		"MD-ToyMMB.csv",
		"MD-ToyPB.csv",
		"MD-ToyMSB2009.csv",
		"MD-ExtLiverPCB.csv",
		"PKN-ToyPCB.sif",
		"MD-ToyPCB.csv")

    if ((filename %in% valid_filenames)==FALSE){
        print("Provided filename not registered. Please use one of ")
        print(valid_filenames)
        stop()
    }

    #directory = strsplit(filename, split="-", fixed=TRUE)[[1]][2]
    #print(directory)
    #directory = strsplit(directory, split=".", fixed=TRUE)[[1]][1]
    #print(directory)

    if (is.null(url)==TRUE){
        url = "http://www.ebi.ac.uk/~cokelaer/cellnopt/data/_downloads"
    }

    #filename = paste(URL, directory, filename, sep="/")
    filename = paste(url,  filename, sep="/")
    if (verbose==TRUE){
        print(filename)
    }
    #library(RCurl) -already loaded by Depends fiels of Description
    # data = getURL(filename, FOLLOW=TRUE) updated?
    data = getURL(filename)
    fh = tempfile("cellnopt_", fileext=".dat")
    if (verbose==TRUE){
        print(paste("data downloaded and copied into ", fh, sep=" "))
    }
    write(data, fh)

    return (fh)

}

#
#  This file is part of the CNO software
#
#  Copyright (c) 2011-2012 - EMBL - European Bioinformatics Institute - Massachusetts Institute of Technology
#
#  File author(s): CNO developers (cno-dev@ebi.ac.uk)
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  CNO website: http://www.ebi.ac.uk/saezrodriguez/software/cno
#
##############################################################################
# $Id$

defaultParameters<-function(data=NA, model=NA){

    paramsList<-list()
    paramsList$data<-data
    paramsList$model<-model

    # GA parameters
    paramsList$sizeFac<-1e-04
    paramsList$NAFac<-1
    paramsList$popSize<-50
    paramsList$pMutation<-0.5
    paramsList$maxTime<-3*60
    paramsList$maxGens<-500
    paramsList$stallGenMax<-100
    paramsList$selPress<-1.2
    paramsList$elitism<-5
    paramsList$relTol<-0.1
    paramsList$verbose<-FALSE

    return(paramsList)
}

#
#  This file is part of the CNO software
#
#  Copyright (c) 2011-2012 - EMBL - European Bioinformatics Institute
#
#  File author(s): CNO developers (cno-dev@ebi.ac.uk)
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  CNO website: http://www.cellnopt.org
#
##############################################################################
# $Id$
sif2graph<-function(sif){

  #if the input is a character it shoud be the name ot the sif file
  #otherwise a matrix in the sif format
  if (is.vector(sif) && (typeof(sif) == "character")){
	sif = read.table(sif)
  }

  # build the unique vertices from the column 1 and 3 of the SIF file
  vertices = unique(c(as.character(sif[,1]), as.character(sif[,3])))
  # some aliases
  v1 = sif[,1]
  v2 = sif[,3]
  edges = as.numeric(sif[,2])

  l = length(vertices) - 1
  g <- new("graphNEL", nodes=vertices, edgemode="directed")
  #weights = rep(1, l)
  weights = edges
  for (i in 1:length(v1)){
    g <- addEdge(as.character(v1[i]), as.character(v2[i]), g, weights[i])
  }
  return(g)

}

#
#  This file is part of the CNO software
#
#  Copyright (c) 2011-2012 - EMBL - European Bioinformatics Institute
#
#  File author(s): CNO developers (cno-dev@ebi.ac.uk)
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  CNO website: http://www.cellnopt.org
#
##############################################################################

cutAndPlotResultsT1 <- function(model, bString, simList=NULL, CNOlist, indexList=NULL,
 plotPDF=FALSE, tag=NULL, tPt=CNOlist@timepoints[2], plotParams=list(maxrow=10))
{

    if (!is(CNOlist,"CNOlist")){
        CNOlist = CellNOptR::CNOlist(CNOlist)
    }
    if (is.null(simList)==TRUE){
        simList = prep4sim(model)
    }
    if (is.null(indexList)==TRUE){
        indexList = indexFinder(CNOlist, model)
    }

    if ("maxrow" %in% names(plotParams) == FALSE){
        plotParams$maxrow = 10
    }


    # keep simList and indxList for back compatibility ?
    modelCut <- cutModel(model, bString)
    simListCut <- cutSimList(simList, bString)
    # t0
    Sim0 <- simulatorT0(CNOlist=CNOlist, model=modelCut, simList=simListCut, indexList=indexList)
    simRes0 <- as.matrix(Sim0[,indexList$signals,drop=FALSE])
    #simRes0 = Sim0
    # t1
    Sim <- simulatorT1(CNOlist=CNOlist, model=modelCut, simList=simListCut, indexList=indexList)
    simRes <- as.matrix(Sim[,indexList$signals,drop=FALSE])
    #simRes = Sim

    simResults <- list(t0=simRes0, t1=simRes)

    # if there is a lot of data, split up cnolist
    # make the max dimensions 10 x 10

    dim1 = dim(CNOlist@signals[[1]])[1]
    dim2 = dim(CNOlist@signals[[1]])[2]

    CNOlistSet = list()
    simResultsSet = list()

    if(dim1 > plotParams$maxrow) { #|| dim2 > 10) {

        par1 = ceiling(dim1/plotParams$maxrow)
        div1 = ceiling(dim1/par1)
        #par2 = ceiling(dim2/plotParams$maxrow)
        #div2 = ceiling(dim2/par2)

        count1 = 1
        for(a in 1:par1) {
            CNOdiv = CNOlist
            simDiv = simResults
            finalN = div1 * a
            if(finalN > dim1) {finalN = dim1}
            CNOdiv@cues = CNOdiv@cues[count1:finalN,,drop=FALSE]
            CNOdiv@stimuli = CNOdiv@stimuli[count1:finalN,,drop=FALSE]
            CNOdiv@inhibitors = CNOdiv@inhibitors[count1:finalN,,drop=FALSE]
            for(b in 1:length(CNOdiv@signals)) {
                CNOdiv@signals[[b]] = CNOdiv@signals[[b]][count1:finalN,,drop=FALSE]
            }
            for(d in 1:length(simDiv)) {
                simDiv[[d]] = simDiv[[d]][count1:finalN,,drop=FALSE]
            }
            count1 = count1 + div1
            CNOlistSet = c(CNOlistSet, list(CNOdiv))
            simResultsSet = c(simResultsSet, list(simDiv))
        }
    } else {

        CNOlistSet = list(CNOlist)
        simResultsSet = list(simResults)
    }

    outputFilenames = list()
    for(f in 1:length(CNOlistSet)) {

        mse = plotOptimResultsPan(
            simResults=simResultsSet[[f]],
            CNOlist=CNOlistSet[[f]],
            formalism="ss1",
            tPt=tPt,
            plotParams=plotParams
            )

        if(plotPDF == TRUE) {
            if(is.null(tag)) {
                filename <- paste("SimResultsT1_", f, ".pdf", sep="")
            } else {
                filename <- paste(tag,"SimResultsT1",f,".pdf",sep="_")
            }
            mse = plotOptimResultsPan(
                simResults=simResultsSet[[f]],
                CNOlist=CNOlistSet[[f]],
                pdf=TRUE,
                formalism="ss1",
                pdfFileName=filename,
                tPt=tPt,
                plotParams=plotParams
            )
            outputFilenames[[f]] = filename
        }
    }
    return(list(filenames=outputFilenames, mse=mse, simResults=simResultsSet))
}

#
#  This file is part of the CNO software
#
#  Copyright (c) 2011-2012 - EMBL - European Bioinformatics Institute
#
#  File author(s): CNO developers (cno-dev@ebi.ac.uk)
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  CNO website: http://www.cellnopt.org
#
##############################################################################
# $Id: writeDot.R 3155 2013-01-09 15:24:58Z cokelaer $
writeDot<-function(dotNodes, dotMatrix, model, filename){

#Write the dot file for the PKN:
#Use the internal variables created by writeNetwork:
	#dotNodes is a matrix with 2 columns: the first has the node names,
		#and the second the attributes (signal, stimulated, inhibited, compressed, ncno)
		#! a node can appear twice in this matrix if it belongs to more of one of the above categories
		#! a node could also not appear here if it is is none of these categories
	#dotMatrix is a matrix with 4 or 5 columns, and a row for each reaction:
		#the first column holds the name of the input node,
		#the second column holds the sign of the reaction (-1 if negative, 1 if positive)
		#the third column holds the name of the ouptut node
		#an optional 5th column holds the weights of the edges
		#the fourth column holds the time stamp (0,1,2)

#1. Create a vector of nodes and their attribute (class: stimulated, inhibited , measured,etc.)
	nodes<-unlist(unique(c(dotMatrix[,1],dotMatrix[,3])))
	nClass<-rep(NA,length(nodes))

#this creates a vector of nodes category (stimulated, compressed, etc) in the same order as
#the nodes vector.  If a node is not in any of these categories, the entry will stay NA.

	for(i in 1:length(nodes)){
	#match only returns the first match, which implies the following order of priority for the
	#colors: signals prime over inhib which primes over stim which primes over ncno which primes
	#over compressed
	#if I later want to put different colors on the nodes with more than one attribute,
	#I could still use, instead of match, which(dotNodes[,1] == nodes[1]), which would give me
	#a vector when the node has more than one attribute, I could then look into the identity
	#of the entries of this vector and define a color depending on this
		nClass[i]<-dotNodes[match(nodes[i],dotNodes[,1]),2]

        # search for this node. Is it repeated ?
        indices = grep(nodes[i], dotNodes[,1])
        if (length(indices)>1){
            # It may belong to both signal and inhibited classes, so let us
            # check:
            classes = dotNodes[indices, 2]
            if ("signal" %in% classes && "inhibited" %in% classes){
                nClass[i] = "bicolor"
	        }
        }
    }

	AndNodes<-grep(pattern="(and\\d+$)",nodes,perl=TRUE,ignore.case=FALSE)

	if(length(AndNodes) != 0){
		nClass[AndNodes]<-"dummy"
		}

#2.Find the ranks of nodes
#source nodes = those that do not get any input
#sink nodes = those that do not have any output

	findSources<-function(x){
		notSource<-any(x == 1)
		return(notSource)
		}

	notSources<-apply(model$interMat,1,findSources)
	sources<-rownames(model$interMat[!notSources,])

	findSinks<-function(x){
		notSink<-any(x == -1)
		return(notSink)
		}

	notSinks<-apply(model$interMat,1,findSinks)
	sinks<-rownames(model$interMat)[!notSinks]

#now find the distances from nodes to source/sink
	modeltoGraphNEL<-function(model){

		vertices<-model$namesSpecies
		edgeList<-vector("list", length=length(vertices))
		names(edgeList)<-vertices
	#edgeList is a list with an element edges and an element weights, both of the same size
	#where each element	in edgeL refers to one node of the graph and contains a vector made of
	#the indexes of the nodes to which the node in question has an edge

		for(sp in 1:length(vertices)){
			reacs<-which(model$interMat[sp,] == -1)

			if(length(reacs) == 0){

				targets<-sp

				}else{

					targets<-which(model$interMat[,reacs[1]] == 1)

					if(length(reacs) > 1){

						for(r in 2:length(reacs)){
							targets<-c(targets, which(model$interMat[,reacs[r]] == 1))
							}

						}
					}
	#If the node doesn't have a target, I set it a dummy interaction with itself because
	#otherwise the graphNEL object can't be created. This won't change whether there exists
	#a path to it from a signal/cue or not
			edgeList[[sp]] <- list(edges=targets, weights=rep(1,length(targets)))

			}
		graph<-new("graphNEL", nodes=vertices, edgeL=edgeList,edgemode="directed")
		return(graph)
		}

	graphModel<-modeltoGraphNEL(model)
	distMatrix<-floyd.warshall.all.pairs.sp(graphModel)
	rankNodes<-rep(Inf, length(nodes))

	for (i in 1:length(nodes)){
		classNode<-ifelse(is.na(nClass[i]),"na",nClass[i])

	if(classNode != "dummy"){
			rankNodes[i]<-min(distMatrix[sources,nodes[[i]]])

		if(any(sinks == nodes[i])){
				rankNodes[i]<-"sink"

				}else{

					if(any(sources == nodes[i])){
						rankNodes[i]<-"source"
						}
					}
			}
		}
#Correct the ranks for 'and' nodes (I'll just put them at the same level as their inputs)
	nClass[is.na(nClass)]<-"na"

	if(any(nClass == "dummy")){
		dummyNodes<-nodes[which(nClass == "dummy")]
		dummyRank<-rep(1,length(dummyNodes))

		for(i in length(dummyNodes)){
			inN<-dotMatrix[which(dotMatrix[,3] == dummyNodes[i]),1]
			ranksIn<-rankNodes[match(inN,nodes)]
			dummyRank[i]<-min(ranksIn)

		}
		rankNodes[which(nClass == "dummy")]<-dummyRank

		}

	nClass[which(nClass == "na")]<-NA

#3.Write the header and nodes part of the dot file
#Write the header
    # if this function is called manually outside of CNOWrap, then the file may
    # not be empty and therefore the final content canot be interpreted by dot
    # executable.
    f = file(filename, "w")
    close(f)

    # now we can concatenate safely
	cat('digraph G{\nsize="8.5,11";\n{rank=source;',file=filename,append=TRUE,sep="")
	cat(which(rankNodes == "source"),file=filename,append=TRUE,sep=";")
	cat(';}\n{rank=same;',file=filename,append=TRUE,sep="")
	cat(which(rankNodes == "1"),file=filename,append=TRUE,sep=";")

	if (length(rankNodes)>=2){
        for(i in 2:max(rankNodes[-which(rankNodes=="source"|rankNodes=="sink")])){
		    cat(';}\n{rank=same;',file=filename,append=TRUE,sep="")
    		cat(which(rankNodes == i),file=filename,append=TRUE,sep=";")
	    	}
    }

	cat(';}\n',file=filename,append=TRUE,sep="")
	cat('{rank=sink;',file=filename,append=TRUE,sep="")
	cat(which(rankNodes == "sink"),file=filename,append=TRUE,sep=";")
	cat(';}\n',file=filename,append=TRUE,sep="")

#write the nodes
	for(i in 1:length(nodes)){
		cat(i,file=filename,append=TRUE,sep="")

		if(is.na(nClass[i])){

			cat(' [color="black" shape="ellipse" style="solid" label="',
				file=filename,append=TRUE,sep="")
			cat(nodes[[i]],file=filename,append=TRUE,sep="")
			cat('" fontname=Helvetica fontsize=22.0 ];\n',
				file=filename,append=TRUE,sep="")

			}else{

				if(nClass[i] == "signal"){
					cat(' [color="lightblue" shape="ellipse" style="filled" label="',
						file=filename,append=TRUE,sep="")
					}
				if(nClass[i] == "bicolor"){
					cat(' [fillcolor="lightblue" color="orangered" shape="ellipse" style="filled, bold, diagonals" label="',
						file=filename,append=TRUE,sep="")
					}

				if(nClass[i] == "inhibited"){
					cat(' [color="orangered" shape="ellipse" style="filled" label="',
						file=filename,append=TRUE,sep="")
					}

				if(nClass[i] == "stimulated"){
					cat(' [color="olivedrab3" shape="ellipse" style="filled" label="',
						file=filename,append=TRUE,sep="")
					}

				if(nClass[i] == "ncno"){
					cat(' [color="lightblue" shape="ellipse" style="filled" label="',
						file=filename,append=TRUE,sep="")
					}

				if(nClass[i] == "compressed"){
					cat(' [color="black" shape="ellipse" style="dashed" label="',
						file=filename,append=TRUE,sep="")
					}

				if(nClass[i] == "dummy"){
					cat(' [color="grey90" shape="circle" style="filled" label="" fontname=Helvetica fontsize=22.0 fixedsize=true width="0.050000" height="0.050000" ];\n',file=filename,append=TRUE,sep="")
					}

				if(nClass[i] != "dummy"){
					cat(nodes[[i]],file=filename,append=TRUE,sep="")
					cat('" fontname=Helvetica fontsize=22.0 ];\n',
						file=filename,append=TRUE,sep="")
					}

				}
		}
#4.Write the reaction part of the dot file
#in dotMatrix, the column 2 determines the arrowhead (neg/pos), the 4 determines the color
#grey90 if edge not in optimal model, forestgreen if at t1, and blue if at t1 and/or t2

	if(dim(dotMatrix)[2] < 5){

		weightsE<-rep(4,dim(dotMatrix)[1])

		}else{

			weightsE<-1+(as.numeric(dotMatrix[,5])*3)

			}

#here I need to insert the bit to make the weights from 1 to 4
	for (i in 1:dim(dotMatrix)[1]){

#a. get the reaction input->output
		cat(
			paste(match(dotMatrix[i,1],nodes),"->",match(dotMatrix[i,3],nodes),sep=" "),
			file=filename,append=TRUE,sep="")

#b.get the color
		if(dotMatrix[i,4] == 0){
			cat('[ color="grey90" label="" weight="1.000000" penwidth="',
				file=filename,append=TRUE,sep="")
			}

		if(dotMatrix[i,4] == 1){
			cat('[ color="forestgreen" label="" weight="1.000000" penwidth="',
				file=filename,append=TRUE,sep="")
			}

		if(dotMatrix[i,4] == 2){
			cat('[ color="blue" label="" weight="1.000000" penwidth="',
				file=filename,append=TRUE,sep="")
			}

#c. get the penwidth from  the weights
		cat(weightsE[i],file=filename,append=TRUE,sep="")

#get the arrowhead
		if(dotMatrix[i,2] == 1){
			cat('" arrowhead="normal" style="solid"];\n',file=filename,append=TRUE,sep="")
			}

		if(dotMatrix[i,2] == -1){
			cat('" arrowhead="tee" style="solid"];\n',file=filename,append=TRUE,sep="")
			}
		}
	cat("}",file=filename,append=TRUE,sep="")
}

#
#  This file is part of the CNO software
#
#  Copyright (c) 2011-2012 - EMBL - European Bioinformatics Institute
#
#  File author(s): CNO developers (cno-dev@ebi.ac.uk)
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  CNO website: http://www.cellnopt.org
#
##############################################################################
# $Id$

# this is the code from simulatorT1 simulatorT1 was written in c

rSimulatorT1 <- function(CNOlist, model, simList, indexList) {


    if (!is(CNOlist,"CNOlist")){
         CNOlist = CellNOptR::CNOlist(CNOlist)
     }
    
	nSp <- dim(model$interMat)[1] 
	nReacs <- dim(model$interMat)[2]
	nCond <- dim(CNOlist@stimuli)[1]
	maxIpg <- dim(simList$finalCube)[2]

    if(is.null(dim(model$interMat))) {
		nSp <- length(model$interMat)
		nReacs <- 1
        maxIpg <- length(simList$finalCube)
	}

	# this holds, for each sp, how many reactions have that sp as output
    endIx <- rep(NA,nSp)

	for(i in 1:nSp) {
		endIx[i]<-length(which(simList$maxIx == i))
	}

	maxgpo <- max(endIx)

    # this value is used to test the stop condition for difference between 2
    # iterations.
    testVal <- 1E-3

	# create an initial values matrix
    initValues <- matrix(data=NA, nrow=nCond, ncol=nSp)
    colnames(initValues) <- model$namesSpecies

    # see warning on the top
    # set the initial values of the stimuli
    initValues[,indexList$stimulated] <- CNOlist@stimuli

    # see warning on the top
    # flip the inhibitors so that 0=inhibited/1=non-inhibitted
    valueInhibitors <- 1-CNOlist@inhibitors
    valueInhibitors[which(valueInhibitors == 1)] <- NA

    # see warning on the top
    # set the initial values of the inhibited species: 0 if inhibited, untouched
    # if not inhibited.
    initValues[,indexList$inhibited] <- valueInhibitors

    # initialise main loop
    newInput <- initValues
    termCheck1<- TRUE
    termCheck2 <- TRUE
    count <- 1

    minNA <- function(x) {
		if(all(is.na(x))) {
			return(NA)
        } else {
            return(min(x,na.rm=TRUE))
        }
    }

    filltempCube <- function(x) {
        cMatrix <- matrix(data=x,nrow=nReacs,ncol=nCond)
        cVector <- apply(cMatrix,1,function(x){return(x)})
    	return(cVector)
    }

    # the ixNeg and ignore matrices can be filled once for all before the loop
    if(nReacs > 1) {
        tempIxNeg <- apply(simList$ixNeg,2,filltempCube)
        tempIgnore <- apply(simList$ignoreCube,2,filltempCube)
    } else {
        tempIxNeg < -matrix(
        	simList$ixNeg,nrow=nCond,
            ncol=length(simList$ixNeg),byrow=TRUE)
        tempIgnore <- matrix(
            simList$ignoreCube,nrow=nCond,
            ncol=length(simList$ignoreCube),byrow=TRUE)
    }

    # main loop
    while(termCheck1 && termCheck2 ){

        outputPrev <- newInput
        
        # this is now a 2 columns matrix that has a column for each input
        # (column in finalCube)
        # and a set of rows for each reac
        # (where a set contains as many rows as conditions)
        # all concatenated into one long column.
        if(nReacs > 1) {
            tempStore <- apply(simList$finalCube,2,function(x){return(outputPrev[,x])})
        } else {
            tempStore <- outputPrev[,simList$finalCube]
        }

        # compute all the ands by taking, for each gate,
        # the min value across the inputs of that gate
        if(nReacs > 1){
            
            # new code with inf instead of NA. Speed up the code
            # set to Inf the values that are "dummies", so they won't influence the min
            tempStore[tempIgnore] <- Inf

            # flip the values that enter with a negative sign
            tempStore[tempIxNeg] <- 1-tempStore[tempIxNeg]

            # IMPORTANT NOTE: when multiple inputs and one or more is NA, then
            # this code should return NA (not the case in this frozen code). The
            # C simulator has the fix.
            outputCube <- apply(tempStore, 1, min)

            # outputCube is now a vector of length (nCond*nReacs) that contains the input of each reaction in
            # each condition, concatenated as such allcond4reac1, allcond4reac2, etc...
            # this is transformed into a matrix with a column for each reac and a row for each cond
            outputCube <- matrix(outputCube, nrow=nCond, ncol=nReacs)

            # go through each species, and if it has inputs, then take the max across those input reactions
            # i.e. compute the ORs

            # create the newInput matrix once for all. Faster than implicit
            # creation
            newInput = matrix(ncol=nSp, nrow=nCond, byrow=FALSE)
            for(s in 1:nSp){
                if(endIx[s] != 0) {
                    selection <- outputCube[,simList$maxIx == s]
                    if (length(selection) == nCond){
                        newInput[,s] <- selection
                    }
                    else{
                        selection[is.na(selection)] <- -Inf
                        newInput[,s] <- apply(selection, 1, max)
                    }
                }
            }
            newInput[is.infinite(newInput)] <- NA
        } else {
            # OLD code with NA. slower but we hardly enter in this loop
            # set to NA the values that are "dummies", so they won't influence the min
            tempStore[tempIgnore] <- NA
            # flip the values that enter with a negative sign
            tempStore[tempIxNeg] <- 1-tempStore[tempIxNeg]

            outputCube <- ifelse(all(is.na(tempStore)),NA,min(tempStore,na.rm=TRUE))
            newInput[,simList$maxIx] <- outputCube
        }

    	# reset the inhibitors and stimuli
    	for (stim in 1:length(indexList$stimulated)) {
            stimM <- cbind(
                CNOlist@stimuli[,stim],
                newInput[,indexList$stimulated[stim]])
            maxNA <- function(x){
                return(max(x,na.rm=TRUE))
            }
            stimV <- apply(stimM,1,maxNA)
            newInput[,indexList$stimulated[stim]] <- stimV
		}
    	
    	valueInhibitors <- 1-CNOlist@inhibitors
    	newInput[,indexList$inhibited] <- valueInhibitors*newInput[,indexList$inhibited]

    	# replace NAs with zeros to avoid having the NA penalty applying to unconnected species
    	newInput[is.na(newInput)] <- 0
    	outputPrev[is.na(outputPrev)] <- 0

    	# check the 2 termination conditions
    	termCheck1 <- !all(abs(outputPrev-newInput)<testVal)
   	 	termCheck2 <- (count < (nSp*1.2))
    	count <- count+1
    }

	# set the non-resolved bits to NA
    newInput[which(abs(outputPrev-newInput) > testVal)] <- NA

    return(newInput)
}

#
#  This file is part of the CNO software
#
#  Copyright (c) 2019 - SaezLab - Heidelberg Universit
#
#  File author(s): CNO developers
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  CNO website: http://www.cellnopt.org
#
##############################################################################

create_binary_variables_for_all_species_in_experiment_k_i <- 
  function(model, numberOfExperiment){
    
    number_of_species <- length(model$namesSpecies)
    identifier <- c()
    variables <- c()
    for(i in 1:number_of_species){
      identifier[i] <- paste("species", model$namesSpecies[i], 
                            "in experiment k =", 
                            numberOfExperiment)
      variables[i] <- paste0("x_",i,"^",numberOfExperiment)
    }
    binary_variables_list <- list(0:length(model$namesSpecies),
                                 variables, 
                                 identifier)
    return(binary_variables_list)
    
}

#
#  This file is part of the CNO software
#
#  Copyright (c) 2011-2012 - EMBL - European Bioinformatics Institute
#
#  File author(s): CNO developers (cno-dev@ebi.ac.uk)
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  CNO website: http://www.cellnopt.org
#
##############################################################################
makeCNOlist<-function(dataset,subfield, verbose=TRUE){

    #check that all the needed elements are present
    if(!is.list(dataset)){
        stop("The input to this function should be a list with elements 'dataMatrix', 'TRcol','DAcol',and 'DVcol'")
        }

    if(length(dataset) != 4){
        stop("The input to this function should be a list with elements 'dataMatrix', 'TRcol','DAcol',and 'DVcol'")
        }

    if(sum(c("dataMatrix","TRcol","DAcol","DVcol") %in% names(dataset))!=4){
        stop("The input to this function should be a list with elements 'dataMatrix', 'TRcol','DAcol',and 'DVcol'")
        }

    # first, we summarise the replicates in the dataMatrix: replicates are rows in the dataset$dataMatrix
    # that have the exact same values in all TR: and DA: columns
    duplCond<-as.matrix(dataset$dataMatrix[,c(dataset$TRcol,dataset$DAcol)])
    duplRows<-which(duplicated(duplCond) == TRUE)
    if (verbose == TRUE){
        if (length(duplRows)>0){
            print("Cleaning duplicated rows")
        }
    }

    # creates a variance matrix
    variances = dataset$dataMatrix * 0


    while(length(duplRows) != 0){
        # the all(x == ) is buggy in the case of NA hence the compareNA function.
        #dupIndex<-apply(duplCond,MARGIN=1,function(x) all(x == duplCond[duplRows[1],]))
        dupIndex<-apply(duplCond,MARGIN=1,function(x)all(compareNA(x,duplCond[duplRows[1],])))

        dupIndex<-which(dupIndex == TRUE)
        dupMatrix<-dataset$dataMatrix[dupIndex,]
        #compute the new row as the average across duplicate rows
        newRow<-colMeans(dupMatrix, na.rm=TRUE)
        # variance for these rows
        newVariance = apply(dupMatrix, MARGIN=2, FUN=var, na.rm=TRUE)

        #replace the first duplicated row by the summarised one
        dataset$dataMatrix[dupIndex[1],]<-newRow

        # same for the variance
        variances[dupIndex[1],]<-newVariance

        #remove the other summarised rows
        dataset$dataMatrix<-dataset$dataMatrix[-dupIndex[2:length(dupIndex)],]
        variances<-variances[-dupIndex[2:length(dupIndex)],]

        duplCond<-as.matrix(dataset$dataMatrix[,c(dataset$TRcol,dataset$DAcol)])
        duplRows<-which(duplicated(duplCond) == TRUE)
    }

    if (any(is.nan(as.matrix(dataset$dataMatrix)))){
        dataset$dataMatrix[is.nan(as.matrix(dataset$dataMatrix))] <- NA
    }

    #now extract the names of the cues, and the inhibitors/stimuli
    namesCues<-colnames(dataset$dataMatrix)[dataset$TRcol]

    if(subfield == TRUE){
        namesCues<-sub(pattern="(TR:)",x=namesCues, replacement="",perl=TRUE)
        tagInhib<-grep(pattern=":Inhibitor", x=namesCues)
        # tagInhibL is a logical version of the previous statement.
        # must be set now before namesCues is changed. See JIRA bug 27
        tagInhibL<-grepl(pattern=":Inhibitor", x=namesCues)

        # remove the trailing :Inhibitors and :Stimuli
        namesCues<-sub(pattern="(:\\w*$)",x=namesCues, replacement="",perl=TRUE)

        # remove trailing i
        namesCues[tagInhib]<-sub(pattern="(i$)", x=namesCues[tagInhib], replacement="", perl=TRUE)

        # if no inhibitors, grep returns integer(0), so we now need to use grepl
        # (logical version of grep)
        namesStimuli<-namesCues[tagInhibL==FALSE]
        namesInhibitors<-namesCues[tagInhibL==TRUE]
        }

    if(subfield == FALSE){
        namesCues<-sub(pattern="(TR:)",x=namesCues, replacement="",perl=TRUE)
        tagInhib<-grep(pattern="(i$)", x=namesCues, perl=TRUE,ignore.case=FALSE)
        # tagInhibL must be set now before namesCues is changed See JIRA bug 27
        tagInhibL<-grepl(pattern="(i$)", x=namesCues, perl=TRUE,ignore.case=FALSE)
        namesCues[tagInhib]<-sub(pattern="(i$)", x=namesCues[tagInhib], replacement="", perl=TRUE)
        # if no inhibitors, grep returns integer(0), so we now need to use grepl
        # (logical version of grep)
        namesStimuli<-namesCues[tagInhibL==FALSE]
        namesInhibitors<-namesCues[tagInhibL==TRUE]

    }

    if(sum("NOCYTO" %in% namesCues) != 0){
        namesCues<-namesCues[-grep(pattern="NOCYTO", namesCues)]
        namesStimuli<-namesStimuli[-grep(pattern="NOCYTO", namesStimuli)]
        }

    if(sum("NOINHIB" %in% namesCues) != 0){
        namesCues<-namesCues[-grep(pattern="NOINHIB", namesCues)]
        namesStimuli<-namesStimuli[-grep(pattern="NOINHIB", namesStimuli)]
        }

    if(sum("NO-INHIB" %in% namesCues) != 0){
        stop("Found a column with NO-INHIB tag. MIDAS files must use NOINHIB instead. Fix your MIDAS file please")
    }
    if(sum("NO-LIG" %in% namesCues) != 0){
        stop("Found a column with NO-LIG tag. MIDAS files do not accept NO-LIG. use NOINHIB or NOCYTO instead. Fix your MIDAS file please")
    }
    if(sum("NOLIG" %in% namesCues) != 0){
        stop("Found a column with NO-LIG tag. MIDAS files do not accept NO-LIG. use NOINHIB or NOCYTO instead. Fix your MIDAS file please")
    }
    if(sum("NO-CYTO" %in% namesCues) != 0){
        stop("Found a column with NO-CYTO tag. MIDAS file must use NOCYTO instead. Fix your MIDAS file please")
    }

    #now extract the names of the signals
    namesSignals<-colnames(dataset$dataMatrix)[dataset$DAcol]
    namesSignals<-sub(
        pattern="(DA:p-)",
        x=namesSignals,
        replacement="",
        perl=TRUE)
    namesSignals<-sub(
        pattern="(DA:)",
        x=namesSignals,
        replacement="",
        perl=TRUE)

    #now extract the time signals
    times<-as.factor(as.vector(as.character(as.matrix(dataset$dataMatrix[,dataset$DAcol]))))
    timeSignals<-sort(as.double(levels(times)))

    #Build the valueCues matrix (i.e. a matrix with nrows=nrows in dataMatrix and ncol=number of cues,
    #filled with 0/1 if the particular cue is present or not)

#1.I create a matrix that is a subset of the data, and only contains the TR columns
#(the cellLine TR column was removed previously)

#2.I remove the columns with NOCYTO or NOINHIB (if they exist), they don't bring any info

    if(length(grep(pattern="NOCYTO",colnames(dataset$dataMatrix)[dataset$TRcol])) != 0){
        nocyto<-grep(pattern="NOCYTO",colnames(dataset$dataMatrix)[dataset$TRcol])
        TRcol<-dataset$TRcol[-nocyto]
        cues<-dataset$dataMatrix[,TRcol]

        }else{
            # use as.matrix and then set colnames to cope for particular case of
            # only one cue
            cues<-as.matrix(dataset$dataMatrix[,dataset$TRcol])
            colnames(cues) = colnames(dataset$dataMatrix)[dataset$TRcol]
            }

    if(length(grep(pattern="NOINHIB",colnames(cues))) != 0){
        noinhib<-grep(pattern="NOINHIB",colnames(cues))
        cues<-cues[,-noinhib]
        }

#3. The cues sub-data frame now contains 1 if the cue is present and 0/NA otherwise,
#so I just need to transform the data frame into a numerical matrix
#and replace the NA in there by zeros
    cues<-as.matrix(cues,nrow=dim(cues)[1],ncol=dim(cues)[2],byrow=TRUE)
    cues[is.na(cues)]<-0

#Build the valueSignals matrices. I am going to build one big matrix
#that includes all the time points, and then I will split it into one matrix for each time point
#And then I will arrange the valueCues matrix accordingly
    valueSignals<-as.matrix(dataset$dataMatrix[,dataset$DVcol])
    valueVariance<-as.matrix(dataset$dataMatrix[,dataset$DVcol])

#This bit will create an index that contains all rows with timept1, 2, 3,...

#1.First I create a matrix that holds the time information for each row
    times<-as.matrix(dataset$dataMatrix[,dataset$DAcol])

#2. Now I check that all the columns are tha same, i.e. that each row
#will contain data on the same time point

    if (length(dataset$DAcol)>1){
        check<-rep(FALSE,(length(dataset$DAcol)-1))
        for(i in 1:length(check)){
            check[i]<-all.equal(times[,i],times[,(i+1)])
            }
        if(sum(check) != length(check))    {
           warning("Each row of your data file should contain measurements at the same time point.
               The times for the first DA column will be considered as the times for all measurements")
           }
    }
#3.Now I will only use the first column of times

#First, I create a vector timeRows that contains the indexes of the rows that contain data
#about each time point (in increasing order of time), and the vector whereTimes that contain`
#the info about how many rows I have for each time (which will allow me to extract the right
#entries from timesRows)

    times<-times[,1]
    ntimes<-length(timeSignals)
    if (ntimes <2){
        stop("Error while parsing the data. Only one time was found.")
    }
    whereTimes<-rep(0,ntimes)
    timesRows<-0  # Melody uses timesRows <- rep(0,1)
    for(i in 1:ntimes){
        timesRows<-c(timesRows,which(times == timeSignals[i]))
        whereTimes[i]<-length(which(times == timeSignals[i]))
        }

    timesRows<-timesRows[2:length(timesRows)]

#Check that we have data across all conditions for all time points except zero
    if(length(unique(whereTimes[2:length(whereTimes)])) != 1){
        warning("This program expects data across all conditions at all time points (except t=0) ")
        }
    if (verbose){
        print("Please be aware that if you only have some conditions at time zero (e.g.only inhibitor/no inhibitor), the measurements for these conditions will be copied across matching conditions at t=0")
    }
#Do the t=0 matrix, and produce a new cues matrix, that does not contain duplicates
#(1 row per condition and different matrices will be build for the different times)
    valueSignals<-list(matrix(data=0,nrow=whereTimes[2],ncol=length(dataset$DVcol)))
    valueVariance<-list(matrix(data=0,nrow=whereTimes[2],ncol=length(dataset$DVcol)))

#This vector tells me which columns of the cues matrix I should pay attention to when
#copying data across for time=0

    # bug report 31 and
    if (dim(cues)[2] >1){
        # bug report 44
        if (whereTimes[1] == 1){
            zerosCond <- 0
        }
        else{
            zerosCond<-apply(cues[timesRows[1:whereTimes[1]],],1,function(x) which(x > 0))
        }
    }
    else{
        warning("unusual case with 1 dimension in cues")
        zerosCond<-which(cues[timesRows[1:whereTimes[1]],] > 0)
    }
    zerosCond<-unique(unlist(zerosCond))
    count=1
    newcues<-matrix(data=0,nrow=whereTimes[2],ncol=dim(cues)[2])

    #fix bug report 38 to be able to have mixed times in a MIDAS file
    #for(i in timesRows[1]:timesRows[whereTimes[1]]){
    for(i in timesRows[1:whereTimes[1]]){
        # 15.12.2017: is this a bug? "i" is already going through some element of timesRow variable.
    	# is this "timesRows[i]" make sense?! or we can just use i instead
    	#present<-zerosCond[which(cues[timesRows[i],zerosCond] > 0)]
    	present<-zerosCond[which(cues[i ,zerosCond] > 0)]

        if(length(present) == 0){
            for(n in timesRows[(whereTimes[1]+1):(whereTimes[1]+whereTimes[2])]){
                    if(sum(cues[n,zerosCond]) == 0){

                        valueSignals[[1]][count,]<-as.numeric(dataset$dataMatrix[i,dataset$DVcol])
                        valueVariance[[1]][count,]<-as.numeric(variances[i,dataset$DVcol])
                        newcues[count,]<-cues[n,]
                        count=count+1
                        }
                    }
        }else{

            for(n in timesRows[(whereTimes[1]+1):(whereTimes[1]+whereTimes[2])]){
                if(length(zerosCond[which(cues[n,zerosCond] > 0)]) == length(present)){
                    if(all(zerosCond[which(cues[n,zerosCond] > 0)] == present) &&   # same cues are there
                        length(which(cues[n,zerosCond] > 0)) != 0 &&  # there is at least one non-zero cue
                       # same principle as above: use i instead of timesRows[i]
                       #all(cues[timesRows[i],zerosCond] == cues[n,zerosCond]) # cues have the same level
                       all(cues[i,zerosCond] == cues[n,zerosCond]) # cues have the same level
                       ){
                    	valueSignals[[1]][count,]<-as.numeric(dataset$dataMatrix[i,dataset$DVcol])
                        valueVariance[[1]][count,]<-as.numeric(variances[i,dataset$DVcol])
                        newcues[count,]<-cues[n,]
                        count=count+1
                    }
                }
            }
        }
    }

    #Now build the matrices for the other time points
    for(i in 2:length(timeSignals)){
        valuesTi<-matrix(data=0,nrow=whereTimes[2],ncol=length(dataset$DVcol))
        valuesVarianceTi<-matrix(data=0,nrow=whereTimes[2],ncol=length(dataset$DVcol))
        for(n in 1:dim(newcues)[1]){
            rowsMatchCues<-apply(cues,1,function(x) all(x == newcues[n,]))
            rowsmatchTime<-times == timeSignals[i]
            rowsMatch<-which((rowsMatchCues + rowsmatchTime) == 2)
            valuesTi[n,]<-as.numeric(dataset$dataMatrix[rowsMatch,dataset$DVcol])
            valuesVarianceTi[n,]<-as.numeric(variances[rowsMatch,dataset$DVcol])
            }

        valueSignals[[i]]<-valuesTi
        valueVariance[[i]]<-valuesVarianceTi

        }

#Build the valueInhibitors and valueStimuli  matrices, which are a subset of the cues one

    if(subfield == TRUE){
        valueInhibitors<-newcues[,grep(
            pattern="Inhibitor",x=colnames(cues),ignore.case=TRUE)]
        valueStimuli<-newcues[,grepl(pattern="Inhibitor",x=colnames(cues),ignore.case=TRUE)==FALSE]

        }else{

            valueInhibitors<-newcues[,grep(pattern="(i$)",x=colnames(cues),ignore.case=FALSE,perl=TRUE)]
            valueStimuli <- newcues[,grepl(pattern="(i$)",x=colnames(cues),ignore.case=FALSE,perl=TRUE)==FALSE]

            }
    if(is.null(dim(valueInhibitors))){
        valueInhibitors<-matrix(valueInhibitors,nrow=dim(newcues)[1])
        }

    if(is.null(dim(valueStimuli))){
        valueStimuli<-matrix(valueStimuli,nrow=dim(newcues)[1])
        }


    return(list(
        namesCues=namesCues,
        namesStimuli=namesStimuli,
        namesInhibitors=namesInhibitors,
        namesSignals=namesSignals,
        timeSignals=timeSignals,
        valueCues=newcues,
        valueInhibitors=valueInhibitors,
        valueStimuli=valueStimuli,
        valueSignals=valueSignals,
        valueVariances=valueVariance
    ))

}


compareNA <- function(v1,v2) {
    # This function returns TRUE wherever elements are the same, including NA's,
    # and false everywhere else.
    same <- (v1 == v2)  |  (is.na(v1) & is.na(v2))
    same[is.na(same)] <- FALSE
    return(same)
}

#
#  This file is part of the CNO software
#
#  Copyright (c) 2011-2012 - EBI
#
#  File author(s): CNO developers (cno-dev@ebi.ac.uk)
#
#  Distributed under the GPLv2 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-2.0.html
#
#  CNO website: http://www.ebi.ac.uk/saezrodriguez/software.html
#
##############################################################################
# $Id$
# This test runs T3 data set to compute the scores (must be zero), and run
# cutAndPlot plot on each time points.

# DO NOT MODIFY please (TC, June 2012)

library(CellNOptR)
pknmodel = readSIF(system.file("ToyModelT3/ToyModelT3.sif", package="CellNOptR"))
cnolist = CNOlist(system.file("ToyModelT3/ToyDataT3.csv",  package="CellNOptR"))

model = preprocessing(cnolist, pknmodel)

# computeScoreT1 with init string made of ones
verbose = FALSE
initBstring<-rep(1,length(model$reacID))
score = computeScoreT1(cnolist, model,bString=rep(1,length(model$reacID)))
bestBS = c(1, 1, 1, 0, 1, 1, 1, 1, 0, 1, 1, 1, 0, 0, 0, 0,0)
bestBS2 <- c(0,0,0,0,0,0,1) # given by running gaBinaryTN
bestBS3 <- c(0,0,1,0,0,0) # given by running gaBinaryTN


SimT1<-simulateTN(CNOlist=cnolist, model=model, bString=list(bestBS))
SimT2<-simulateTN(CNOlist=cnolist, model=model, bStrings=list(bestBS, bestBS2))
SimT3<-simulateTN(CNOlist=cnolist, model=model, bStrings=list(bestBS, bestBS2, bestBS3))

score1 = computeScoreT1(cnolist, model, bString=bestBS)
score2 = computeScoreTN(cnolist, model, bStrings=list(bestBS,bestBS2))
score3 = computeScoreTN(cnolist, model, bStrings=list(bestBS,bestBS2, bestBS3))

print(score1)
print(score2)
print(score3)
if (score1>0.01 || score2>0.024){
   stop("errore")
}
cutAndPlot(cnolist, model,bStrings=list(bestBS),plotPDF=TRUE, tag="test1")
cutAndPlot(cnolist, model,bStrings=list(bestBS,bestBS2),plotPDF=TRUE, tag="test2")
cutAndPlot(cnolist, model, bStrings=list(bestBS,bestBS2,bestBS3),plotPDF=TRUE, tag="test3")

#
#  This file is part of the CNO software
#
#  Copyright (c) 2011-2012 - EMBL - European Bioinformatics Institute
#
#  File author(s): CNO developers (cno-dev@ebi.ac.uk)
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  CNO website: http://www.cellnopt.org
#
# Modifications:
#   - the Cue values are displayed by bars in the range [0,1] (31.03.2016 AG)
#
##############################################################################
# $Id: plotCNOlist.R 3635 2013-05-28 09:21:46Z cokelaer $
plotCNOlist<-function(CNOlist){

#check that CNOlist is a CNOlist

    if (!is(CNOlist,"CNOlist")){
         CNOlist = CellNOptR::CNOlist(CNOlist)
    }

    # save current par() and recall at end to avoid permanent changes
    oldPar = par(no.readonly = TRUE)

    #set graphic parameters
    par(
        mfrow=c(nr=dim(CNOlist@signals[[1]])[1]+1,nc=dim(CNOlist@signals[[1]])[2]+1),
        cex=0.5,pch=20,mar=c(0.5,0.5,0,0),oma=c(3,2,2,2)
    )

    yMax<-max(unlist(lapply(CNOlist@signals,function(x) max(x, na.rm=TRUE))))
    if (yMax<=1){
        yMax = 1
    }
    yMin<-min(unlist(lapply(CNOlist@signals,function(x) min(x, na.rm=TRUE))))
    xVal<-CNOlist@timepoints

    # set margin sizes
    marginSize = c(0.5,0.5,0.5,0)
    if (dim(CNOlist@signals[[1]])[2]>10) marginSize[[2]] = 0.1
    if (dim(CNOlist@signals[[1]])[1]>20) marginSize[c(1,3)] = c(.1,.1)
    if (dim(CNOlist@signals[[1]])[1]>40) marginSize[c(1,3)] = c(.0,.0)

    for(c in 1:dim(CNOlist@signals[[1]])[2]){
    par(fg="blue",mar=marginSize)
        plot(
            x=xVal,
            y=rep(-5,length(xVal)),
            ylim=c(yMin, yMax),xlab=NA,ylab=NA,xaxt="n",yaxt="n")
        text(
            labels=colnames(CNOlist@signals[[1]])[c],
            x=((xVal[length(xVal)]-xVal[1])/2),
            y=(yMin+((yMax-yMin)/2)),cex=2)
        }

    plot(
        x=xVal,
        y=rep(-5,length(xVal)),
        ylim=c(yMin, yMax),xlab=NA,ylab=NA,xaxt="n",yaxt="n")
    text(
        labels="Cues",
        x=((xVal[length(xVal)]-xVal[1])/2),
        y=(yMin+((yMax-yMin)/2)),cex=2)

    par(fg="black",mar=marginSize)

    for(r in 1:dim(CNOlist@signals[[1]])[1]){

        for(c in 1:dim(CNOlist@signals[[1]])[2]){
            yVal<-lapply(CNOlist@signals,function(x) {x[r,c]})
            plot(x=xVal,y=yVal,ylim=c(yMin, yMax),xlab=NA,ylab=NA,xaxt="n",yaxt="n")
            lines(x=xVal,y=yVal,ylim=c(yMin, yMax),xlab=NA,ylab=NA,xaxt="n",yaxt="n")
            tryCatch(
                {ystd = unlist(lapply(CNOlist@variances,function(x) {x[r,c]}))
                .error.bar(unlist(xVal), unlist(yVal), ystd)},
                error=function(e){}
            )

            #add the annotation of the axis: if we're on the last row we need an x-axis
            if(r == dim(CNOlist@signals[[1]])[1]){
                axis(side=1,at=xVal)
                }

            #add the annotation of the axis: if we're on the first column we need a y-axis
            if(c == 1){
                axis(side=2,at=c(-0.5,0,0.5))
                }

            }

            # The image (cues) last columns
            if (length(CNOlist@inhibitors) != 0){
                data = t(matrix(c(CNOlist@stimuli[r,],CNOlist@inhibitors[r,]),nrow=1))
                # the axis last column
                if(r == dim(CNOlist@signals[[1]])[1]){
                    barplot(t(data),yaxt="n",ylim=c(0,1),names.arg = c(colnames(CNOlist@stimuli),paste(colnames(CNOlist@inhibitors),"-i",sep="")),las=3 )
                    axis(4)
                }else{
                    barplot(t(data),xaxt="n",yaxt="n",ylim=c(0,1))
                    axis(4)
                }
            }
            else{ # special case of no inhibitors
                data = t(matrix(CNOlist@stimuli[r,],nrow=1))
                # the axis last column
                if(r == dim(CNOlist@signals[[1]])[1]){
                    barplot(t(data),yaxt="n",ylim=c(0,1),names.arg = colnames(CNOlist@stimuli),las=3)
                    axis(4)
                }else{
                    barplot(t(data),xaxt="n",yaxt="n",ylim=c(0,1))
                    axis(4)
                }
            }
    }
    par(oldPar)

}


.error.bar <- function(x, y, upper, lower=upper, length=0.1,...){
     if(length(x) != length(y) | length(y) !=length(lower) | length(lower) != length(upper))
         stop("vectors must be same length")

     positives = upper > 0
     x = x[positives]
     y = y[positives]
     upper = upper[positives]
     lower = lower[positives]
     arrows(x,y+upper, x, y-lower, angle=90, code=3, length=length, col=rgb(0, 0, 0, 0.1),...)
     }

#
#  This file is part of the CNO software
#
#  Copyright (c) 2011-2012 - EMBL - European Bioinformatics Institute
#
#  File author(s): CNO developers (cno-dev@ebi.ac.uk)
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  CNO website: http://www.cellnopt.org
#
##############################################################################
# $Id$

cutAndPlotResultsTN <-function(CNOlist, model,bStrings, plotPDF=FALSE,
    tag=NULL, plotParams=list(maxrow=10))
{

 if (!is(CNOlist,"CNOlist")){
        CNOlist = CellNOptR::CNOlist(CNOlist)
    } 




  tPt=CNOlist@timepoints[1:length(bStrings)+1]
  simList = prep4sim(model)
  indexList = indexFinder(CNOlist, model)

    if ("maxrow" %in% names(plotParams) == FALSE){
        plotParams$maxrow = 10
    }   



    modelCut <- cutModel(model, bStrings[[1]])
    simListCut <- cutSimList(simList, bStrings[[1]])

    # t0
    Sim0 <- simulatorT0(CNOlist=CNOlist,model=modelCut,simList=simListCut,indexList=indexList)
    simResT0 <- as.matrix(Sim0[,indexList$signals, drop=FALSE])
    #simResT0 = Sim0

    # simulate
    simResults<-list()
    simResults[[1]]<-simResT0


    for(i in 1:length(bStrings)){
      cutRes = simulateTN(CNOlist, model, bStrings[1:i])
      cutRes<-cutRes[,indexList$signals]
      simResults[[i+1]]<-cutRes
    }


    mse = plotOptimResultsPan(
      simResults=simResults,
      CNOlist=CNOlist,
      formalism="ssN",
      tPt=tPt,
      plotParams=plotParams
      #timePoints=length(tPt)
    )

  if(plotPDF == TRUE) {
    if(is.null(tag)==TRUE) {
      filename <- paste("SimResultsTN", ".pdf", sep="")
    }
    else {
      filename<-paste(tag, "SimResultsTN.pdf", sep="_")
    }

    mse = plotOptimResultsPan(
      simResults=simResults,
      CNOlist=CNOlist,
      formalism="ssN",
      tPt=tPt,
      pdfFileName=filename,
      pdf=TRUE,
        plotParams=plotParams
      #TimePoints=length(tPt)
    )
  }

  outputFileNames = list()
  return(list(mse=mse, filenames=outputFileNames, simResults=simResults))


}

#
#  This file is part of the CNO software
#
#  Copyright (c) 2011-2012 - EMBL - European Bioinformatics Institute
#
#  File author(s): CNO developers (cno-dev@ebi.ac.uk)
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  CNO website: http://www.cellnopt.org
#
##############################################################################
findNONC<-function(model,indexes,verbose=FALSE){

    #use the floyd warshall algorithm implemented in the package RBGL
    #eg floyd.warshall.all.pairs.sp(coex) that returns a distance matrix
    #for that I need to create a graphNEL object
    modeltoGraphNEL<-function(model){

        vertices<-model$namesSpecies
        edgeList<-vector("list", length=length(vertices))
        names(edgeList)<-vertices
        #edgeList is a list with an element edges and an element weights, both of the same size
        #where each element    in edgeL refers to one node of the graph and contains a vector made of
        #the indexes of the nodes to which the node in question has an edge

        for(sp in 1:length(vertices)){
            reacs<-which(model$interMat[sp,] == -1)

            if(length(reacs) == 0){
                targets<-sp

                }else{
                    targets<-which(model$interMat[,reacs[1]] == 1)

                    if(length(reacs) > 1){

                        for(r in 2:length(reacs)){
                            targets<-c(targets, which(model$interMat[,reacs[r]] == 1))
                            }

                        }
                    }
    #If the node doesn't have a target, I set it a dummy interaction with itself because
    #otherwise the graphNEL object can't be created. This won't change whether there exists
    #a path to it from a signal/cue or not
            edgeList[[sp]] <- list(edges=targets, weights=rep(1,length(targets)))
            }

        graph<-new("graphNEL", nodes=vertices, edgeL=edgeList,edgemode="directed")
        return(graph)
        }

    #Now I use the function above to create a GraphNEL object
    graphModel<-modeltoGraphNEL(model)

    #now I can run the floyd warshall algo
    distMatrix<-floyd.warshall.all.pairs.sp(graphModel)

    #Find the species that are non-observable, i.e. there are no path from that species to any
    #of the signals
    #And the species that are not controllable, ie there are no paths from any of the cues
    #to those species
    ncno<-0

    for(sp in 1:length(model$namesSpecies)){
    #if all the distances from this species to signals are infinite and the species is not a signal
    #then this species is not observable
        if(
            all(is.infinite(distMatrix[sp,indexes$signals]))  &&
            !any(indexes$signals == sp)){

            ncno<-c(ncno,sp)

            }

        if(
            all(is.infinite(distMatrix[c(indexes$stimulated,indexes$inhibited),sp])) &&
            !any(c(indexes$stimulated,indexes$inhibited) == sp)){

            ncno<-c(ncno,sp)

            }
        }
    ncno<-ncno[-1]
    ncno<-unique(ncno)

    #check that none of the cues or signals are in there
    cs<-intersect(unlist(indexes),ncno)

    if(length(cs != 0)){
        ncno<-ncno[-match(cs,ncno)]
        }

    if(verbose){
        print(paste(
            "The following species are not observable and/or not controllable:",
            toString(model$namesSpecies[ncno])))
        }
    return(ncno)
}

#
#  This file is part of the CNO software
#
#  Copyright (c) 2011-2012 - EMBL - European Bioinformatics Institute
#
#  File author(s): CNO developers (cno-dev@ebi.ac.uk)
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  CNO website: http://www.cellnopt.org
#
##############################################################################
# $Id$
gaBinaryTN <-function(
    CNOlist,
    model,
    bStrings,
    sizeFac=0.0001,
    NAFac=1,
    popSize=50,
    pMutation=0.5,
    maxTime=60,
    maxGens=500,
    stallGenMax=100,
    selPress=1.2,
    elitism=5,
    relTol=0.1,
    verbose=TRUE,
    priorBitString=NULL,
    timeIndex=NULL){

    #Find the bits to optimise
    N = length(bStrings)
    bStringPrev = bStrings[[N]]
    if (is.null(timeIndex)==TRUE){
        timeIndex = N + 2   # +1 for T0 and +1 for the current time (bStrings length is N-1 indeed)
    }

    bits2optimise<-which(bStringPrev == 0)
    bLength<-length(bits2optimise)

    checkSignals(CNOlist, model)


   if (is.list(bStrings)==FALSE){
         stop("CellNOpt Error: 3d argument called bStrings must be a list of vectors. Each vector representing a bit string")
     }

    # compute it once for all (recomputed many times in computeScore otherwise)
    simResPrev<-simulateTN(CNOlist=CNOlist, model=model, bStrings=bStrings)
    simList = prep4sim(model)
    indexList = indexFinder(CNOlist, model)

    # ---- section related to T2  end ----

    Pop <- round(matrix(runif(bLength*(popSize)), nrow=(popSize),ncol=bLength))

    Pop <- addPriorKnowledge(Pop, priorBitString)

    bestbit<-Pop[1,]
    bestobj<-Inf
    stop<-FALSE
    obj<-rep(0,popSize)
    g<-0
    stallGen<-0
    res<-rbind(
        c(g,bestobj,toString(bestbit),stallGen,Inf,Inf,toString(bestbit),0),
        c(g,bestobj,toString(bestbit),stallGen,Inf,Inf,toString(bestbit),0))
    colnames(res)<-c("Generation","Best_score","Best_bitString","Stall_Generation",
        "Avg_Score_Gen","Best_score_Gen","Best_bit_Gen","Iter_time")
    PopTol<-rep(NA,bLength)
    PopTolScores<-NA

    # library(hash) -already loaded by Depends fiels of Description
    scores2Hash_env = new.env(hash = TRUE)

    #Function that produces the score for a specific bitstring
    getObj<-function(x){

        key = toString(.int2dec(x))
        if (!is.null(scores2Hash_env[[key]])){
            return(scores2Hash_env[[key]])
        } else {
            bStrings[[N+1]] = x

            Score = computeScoreTN(CNOlist, model, simList, indexList, simResPrev, bStringPrev=NULL,  bStringNext=NULL,
                bStrings=bStrings, timeIndex=timeIndex, sizeFac=sizeFac, NAFac=NAFac)
            if (length(scores2Hash_env)<1000){
            	scores2Hash_env[[key]] =  Score
            }
        }

        return(Score)
    }

    #Loop
    t0<-Sys.time()
    t<-t0


    # if bitstring has only 1 bit to optimize, enter this simple loop:
    # we should have an exhaustive optimisation as well for simple cases.
    if (bLength==1){
        # build a population made of 2 vectors: c(0) and c(1)
        Pop = matrix(c(0,1), nrow=2)
        scores<-apply(Pop,1,getObj, scoresHash=NULL)
        rankP<-order(scores,decreasing=TRUE)
        # extract the best solution.
        iBest = rankP[2]
        return(list(
            bString=Pop[iBest,],
            bScore=scores[iBest],
            results=res,
            stringsTol=PopTol,
            stringsTolScores=PopTolScores))
    }

    while(!stop){

        #compute the scores
        #l1 = length(scores2Hash)
        scores <- apply(Pop, 1, getObj)
        #l2 = length(scores2Hash)
        #if (verbose == TRUE){print(paste("new scores:", l2-l1))}


        #Fitness assignment: ranking, linear
        rankP<-order(scores,decreasing=TRUE)
        Pop<-Pop[rankP,]
        scores<-scores[rankP]
        fitness<-2-selPress+(2*(selPress-1)*(c(1:popSize)-1)/(popSize-1))

        #selection:stochastic uniform sampling
        wheel1<-cumsum(fitness/sum(fitness))
        breaks<-runif(1)*1/popSize
        breaks<-c(breaks,breaks+((1:(popSize-1)))/popSize)
        sel<-rep(1,popSize)

        for(i in 1:length(breaks)){
            sel[i]<-which(wheel1>breaks[i])[1]
            }

        #intermediate generation
        Pop2<-Pop[sel,]
        PSize2<-dim(Pop2)[1]
        PSize3<-popSize-elitism

        #Recombination: uniform: each bit has a .5 proba of being inherited from each parent
        mates<-cbind(ceiling(runif(PSize3)*PSize2),ceiling(runif(PSize3)*PSize2))

        #This holds the probability, for each bit, to be inherited from parent 1 (if TRUE) or 2 (if FALSE)
        InhBit<-matrix(runif((PSize3*bLength)),nrow=PSize3,ncol=bLength)
        InhBit<-InhBit < 0.5

        #Try one point crossover
        #xover<-ceiling(runif(PSize3)*(bLength-1))
        #indices<-matrix(1:bLength,nrow=PSize3,ncol=bLength,byrow=TRUE)
        #InhBit<-matrix(rep(FALSE,PSize3*bLength),nrow=PSize3,ncol=bLength)
        #for(i in 1:PSize3){
        #    InhBit[i,]<-indices[i,]<xover[i]
        #    }
        #

        Pop3par1<-Pop2[mates[,1],]
        Pop3par2<-Pop2[mates[,2],]
        Pop3<-Pop3par2
        Pop3[InhBit]<-Pop3par1[InhBit]

        #Mutation
        MutProba<-matrix(runif((PSize3*bLength)),nrow=PSize3,ncol=bLength)
        MutProba<-(MutProba < (pMutation/bLength))
        Pop3[MutProba]<-1-Pop3[MutProba]

        #Compute stats
        t<-c(t,Sys.time())
        g<-g+1
        thisGenBest<-scores[length(scores)]
        thisGenBestBit<-Pop[length(scores),]

        if(is.na(thisGenBest)){
            thisGenBest<-min(scores, na.rm=TRUE)
            thisGenBestBit<-Pop[which(scores == thisGenBest)[1],]
        }

        if(thisGenBest < bestobj){
            bestobj<-thisGenBest
            bestbit<-thisGenBestBit
            stallGen<-0
            }else{
                stallGen<-stallGen+1
                }

        resThisGen<-c(
            g,
            bestobj,
            toString(bestbit),
            stallGen,
            (mean(scores,na.rm=TRUE)),
            thisGenBest,
            toString(thisGenBestBit),
            as.numeric((t[length(t)]-t[length(t)-1]), units="secs"))

        names(resThisGen)<-c("Generation","Best_score","Best_bitString","Stall_Generation",
            "Avg_Score_Gen","Best_score_Gen","Best_bit_Gen","Iter_time")

        if(verbose) print(resThisGen)

        res<-rbind(res,resThisGen)

        #Check stopping criteria
        Criteria<-c((stallGen > stallGenMax),
            (as.numeric((t[length(t)]-t[1]), units="secs") > maxTime),
            (g > maxGens))
        if(any(Criteria)) stop<-TRUE

        #Check for bitstrings that are within the tolerance of the best bitstring
        tolScore<-scores[length(scores)]*relTol
        TolBs<-which(scores < scores[length(scores)]+tolScore)

        if(length(TolBs) > 0){
            PopTol<-rbind(PopTol,Pop[TolBs,])
            PopTolScores<-c(PopTolScores,scores[TolBs])
            }

        if(elitism > 0){
            Pop<-rbind(Pop3,Pop[(popSize-elitism+1):popSize,])
            }else{
                Pop<-Pop3
                }
        Pop <- addPriorKnowledge(Pop, priorBitString)
    }
    #end of the while loop

    PopTol<-as.matrix(PopTol[-1,])
    PopTolScores<-PopTolScores[-1]
    TolBs<-which(PopTolScores < scores[length(scores)]+tolScore)
    PopTol<-as.matrix(PopTol[TolBs,])
    PopTolScores<-PopTolScores[TolBs]
    PopTolT<-cbind(PopTol,PopTolScores)
    PopTolT<-unique(PopTolT,MARGIN=1)

    if(!is.null(dim(PopTolT))){
        PopTol<-PopTolT[,1:(dim(PopTolT)[2]-1)]
        PopTolScores<-PopTolT[,dim(PopTolT)[2]]
        }else{
            PopTol<-PopTolT[1:(length(PopTolT)-1)]
            PopTolScores<-PopTolT[length(PopTolT)]
            }

    res<-res[3:dim(res)[1],]
    rownames(res)<-NULL

    return(list(
        bString=bestbit,
        bScore=bestobj,
        results=res,
        stringsTol=PopTol,
        stringsTolScores=PopTolScores))
    }


addPriorKnowledge <- function(pop, priorBitString){
    if (is.null(priorBitString) == TRUE){
        return(pop)
    }
    else{
        for (i in 1:dim(pop)[1]){
            pop[i,!is.na(priorBitString)] = priorBitString[!is.na(priorBitString)]
        }
    }
   return(pop)
}


.int2dec <- function(x){
    return(sum(x*2^(rev(seq(x))-1)))
}

#
#  This file is part of the CNO software
#
#  Copyright (c) 2011-2012 - EMBL - European Bioinformatics Institute
#
#  File author(s): CNO developers (cno-dev@ebi.ac.uk)
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  CNO website: http://www.cellnopt.org
#
##############################################################################

cutNONC <- function(model, NONCindexes) {

    #####     FUNCTIONS    #####

    multipleInOut <- function(x) {

        spInReac = which(x != 0)
        inputs = length(which(x==-1))

        if((inputs > 1) && length(intersect(spInReac,NONCindexes))) {
            return(TRUE)
        } else {
            return(FALSE)
        }
    }

    emptyInOut<-function(x) {

        input <- match(-1,x,nomatch=0)
        output <- match(1,x,nomatch=0)

        if((input == 0) | (output == 0)) {
            return(TRUE)
        } else {
            return(FALSE)
        }
    }

    #####    /FUNCTIONS/    #####


    if(length(NONCindexes) == 0) {

        newModel=model

    } else {

        # if there are AND gates in the PKN, find out
        # if any NONCs are parts of AND gates
        editReac = apply(model$interMat,2,multipleInOut)
        newSpecies <- model$namesSpecies[-NONCindexes]
        newInterMat <- model$interMat[-NONCindexes,]
        newNotMat <- model$notMat[-NONCindexes,]

        # this function finds whether a given vector contains at least one input and one output or not
        # it returns true if the re is an in/out missing

        # modify to account for AND gates:
        # some NONC sp may be part of an AND gate with non-NONC sp
        # if this is the case, remove the NONC edge, not the reaction from reacID

        # rebuild reacIDs from matrix
        toEdit = as.numeric(which(editReac==TRUE))
        for(a in toEdit) {
            andInput = rownames(newInterMat)[which(newInterMat[,a] == -1)]
            andInd = which(newInterMat[,a] == -1)
            if(length(intersect(which(newNotMat[,a]==1),which(newInterMat[,a]==-1)))) {
                andNeg = intersect(which(newNotMat[,a]==1),which(newInterMat[,a]==-1))
                    for(p in 1:length(andNeg)) {
                        andInput[which(andInd==andNeg[p])] = paste("!", andInput[which(andInd==andNeg[p])], sep="")
                    }
            }

            LHS = paste(andInput,collapse="+", sep="")
            colnames(newInterMat)[a] = paste(LHS, "=", rownames(newInterMat)[which(newInterMat[,a] == 1)], sep="")

            colnames(newNotMat)[a] = colnames(newInterMat)[a]
        }

        reac2remove <- apply(newInterMat,2,emptyInOut)
        # check if reac2remove contains selfloops
        # the condition below matches before '=' and checks this pattern after
        matchIn = gsub("^!*([[:alnum:]]+)=.*","\\1", names(reac2remove))
        potentialLoops = paste(matchIn,"=",matchIn,sep="")
        # check for + or - loops
        reac2remove[which(paste("!",potentialLoops,sep="")==names(reac2remove) | potentialLoops==names(reac2remove))] = FALSE

        if(any(reac2remove)) {

            reac2remove <- which(reac2remove)
            newInterMat <- newInterMat[,-reac2remove]
            newNotMat <- newNotMat[,-reac2remove]
            newreacID <- colnames(newInterMat)

            newModel <- list(
                reacID=newreacID,
                namesSpecies=newSpecies,
                interMat=newInterMat,
                notMat=newNotMat
            )
        } else {
            newModel <- list(
                reacID=colnames(newInterMat),
                namesSpecies=newSpecies,
                interMat=newInterMat,
                notMat=newNotMat)
        }
    }

    return(newModel)

}

#
#  This file is part of the CNO software
#
#  Copyright (c) 2011-2012 - EBI
#
#  File author(s): CNO developers (cno-dev@ebi.ac.uk)
#
#  Distributed under the GPLv2 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-2.0.html
#
#  CNO website: http://www.ebi.ac.uk/saezrodriguez/software.html
#
##############################################################################
# $Id$

# This test runs optimisation using Boolean logic at different time points
# Author: S.Schrier, T.Cokelaer



manySteadyStates <-function(
  CNOlist,
  model,
  sizeFac=0.0001,
  NAFac=1,
  popSize=50,
  pMutation=0.5,
  maxTime=60,
  maxGens=500,
  stallGenMax=100,
  selPress=1.2,
  elitism=5,
  relTol=0.1,
  verbose=FALSE,
  priorBitString=NULL){


    #initialize Opt array
    Opt<-list()    
    #initialze a bString array  
    bStrings<-list()
    simRes<-list()


    T1opt<-gaBinaryT1(CNOlist=CNOlist,
                  model=model,
                  stallGenMax=stallGenMax,
                  sizeFac=sizeFac,
                  NAFac=NAFac,
                  popSize=popSize,
                  pMutation=pMutation,
                  maxTime=maxTime,
                  maxGens=maxGens,
                  selPress=selPress,
                  elitism=elitism,
                  relTol=relTol,
                  verbose=verbose,
                  priorBitString=priorBitString)

    Opt[[1]]<-T1opt
    simT1<-simulateTN(CNOlist=CNOlist, model=model, bStrings=list(T1opt$bString))
    simRes[[1]]<-simT1
    bStrings[[1]] = T1opt$bString

    if (length(CNOlist@signals)>2){
        for(i in 3:length(CNOlist@signals)){
            Opt[[i-1]]<-gaBinaryTN(CNOlist=CNOlist,
                        model=model,
                        bStrings=bStrings,
                        stallGenMax=stallGenMax,
                        maxTime=maxTime,
                        sizeFac=sizeFac,
                        NAFac=NAFac,
                        popSize=popSize,
                        pMutation=pMutation,
                        maxGens=maxGens,
                        selPress=selPress,
                        elitism=elitism,
                        relTol=relTol,
                        verbose=verbose,
                        priorBitString=priorBitString)

            bStrings[[i-1]] = Opt[[i-1]]$bString

            simRes[[i]]<-simulateTN(CNOlist,model,bStrings)
        }
    }   
    return(list(bStrings=bStrings, Opt=Opt, simRes=simRes))
}



library(CellNOptR)

# one steady state
data(CNOlistToy, package="CellNOptR")
cnolist = CNOlist(CNOlistToy)
data(ToyModel, package="CellNOptR")
results = manySteadyStates(cnolist, ToyModel)
print(results$bStrings)

# two steady state
data(CNOlistToy2, package="CellNOptR")
cnolist = CNOlist(CNOlistToy2)
data(ToyModel2, package="CellNOptR")
results = manySteadyStates(cnolist, ToyModel2)
print(results$bStrings)

# 3 steady state
ToyModel3 = readSIF(system.file("ToyModelT3/ToyModelT3.sif", package="CellNOptR"))
cnolist = CNOlist(system.file("ToyModelT3/ToyDataT3.csv", package="CellNOptR"))
results = manySteadyStates(cnolist, ToyModel3)
print(results$bStrings)

#  Copyright (c) 2019 - SaezLab - Heidelberg Universit
#
#  File author(s): CNO developers
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  CNO website: http://www.cellnopt.org
#
##############################################################################

writeFile <- function(objectiveFunction,
                      constraints,
                      bounds,
                      binaries){
  
  data = "testFile.lp"
  write("enter Problem", data)
  write("", data, append = TRUE)
  write("Minimize", data, append = TRUE)
  
  write(objectiveFunction[[2]], data, append = TRUE)
  
  write("Subject To", data, append = TRUE)
  write(constraints, data, append = TRUE)
  
  write("Bounds", data, append = TRUE)
  write(bounds, data, append = TRUE)
  
  write("Binaries",data, append = TRUE)
  write(binaries[[1]], data, append = TRUE)
  
  write("End", data, append = TRUE)
  write("optimize Problem", data, append = TRUE)
  write("write solution_cplex_primaryObjective.txt", data, append = TRUE)
  write("sol", data, append = TRUE)
  write("y", data, append = TRUE)
  write("quit", data, append = TRUE)
  
}

#
#  This file is part of the CNO software
#
#  Copyright (c) 2011-2012 - EMBL - European Bioinformatics Institute
#
#  File author(s): CNO developers (cno-dev@ebi.ac.uk)
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  CNO website: http://www.cellnopt.org
#
##############################################################################
# $Id: plotOptimResults.R 3155 2013-01-09 15:24:58Z cokelaer $
plotOptimResults<-function(
    simResults=simResults,
    expResults=expResults,
    times=times,
    namesCues=namesCues,
    namesSignals=namesSignals,
    valueCues=valueCues, formalism="new"){

    oldPar = par(no.readonly = TRUE)
    on.exit(par(oldPar))

#Set graphical parameters
    par(
        mfrow=c(nr=dim(simResults[[1]])[1]+1,nc=dim(simResults[[1]])[2]+1),
        cex=0.5,
        pch=20,
        mar=c(0.5,0.5,0,0),
        oma=c(3,2,2,2))
    yMax<-max(max(unlist(lapply(expResults,function(x) max(x, na.rm=TRUE)))),1)
    yMin<-min(min(unlist(lapply(expResults,function(x) min(x, na.rm=TRUE)))),0)
    xVal<-times

#Write the boxes on top with the labels of the signals

    for(c in 1:dim(expResults[[1]])[2]){
        par(fg="blue",mar=c(0.5,0.5,0.5,0))
        plot(
            x=xVal,
            y=rep(-5,length(xVal)),
            ylim=c(yMin, yMax),
            xlab=NA,
            ylab=NA,
            xaxt="n",
            yaxt="n")
        text(
            labels=as.character(namesSignals[c]),
            x=((xVal[length(xVal)]-xVal[1])/2),
            y=(yMin+((yMax-yMin)/2)),
            cex=2)
        }

        plot(
            x=xVal,
            y=rep(-5,length(xVal)),
            ylim=c(yMin, yMax),
            xlab=NA,
            ylab=NA,
            xaxt="n",
            yaxt="n")

        text(
            labels="Cues",
            x=((xVal[length(xVal)]-xVal[1])/2),
            y=(yMin+((yMax-yMin)/2)),
            cex=2)

    par(fg="black",mar=c(0.5,0.5,0,0))
#Go through each elements of the results matrix
#(i.e. one plot for each signal (column) for each condition (row)

    for(r in 1:dim(expResults[[1]])[1]){

        for(c in 1:dim(expResults[[1]])[2]){

#Determine the simulated and experimental values
            yVal<-lapply(expResults,function(x) {x[r,c]})
            yValS<-lapply(simResults,function(x) {x[r,c]})

#Take care of the NA's which would mess up our mean difference used for bg colouring

            if(sum(is.na(yVal)) != 0){
                yVal4Diff<-yVal[-which(is.na(yVal))]
                yValS4Diff<-yValS[-which(is.na(yVal))]
                }else{
                    yVal4Diff<-yVal
                    yValS4Diff<-yValS
                    }

            if(sum(is.na(yValS4Diff)) != 0){
                yVal4Diff<-yVal4Diff[-which(is.na(yValS4Diff))]
                yValS4Diff<-yValS4Diff[-which(is.na(yValS4Diff))]
                }else{
                    yVal4Diff<-yVal4Diff
                    yValS4Diff<-yValS4Diff
                    }

#Compute the mean difference between data and simulation, taking into account t0 but not NAs
            if (formalism == "new"){
                diff<-mean(abs(unlist(yVal4Diff)[1:length(yVal4Diff)]-unlist(yValS4Diff)[1:length(yValS4Diff)]),na.rm=TRUE)
            }
            else{
                diff<-mean(abs(unlist(yVal4Diff)[2:length(yVal4Diff)]-unlist(yValS4Diff)[2:length(yValS4Diff)]),na.rm=TRUE)
           }

#Set the bg colour based on the above
            if(is.na(diff)){
                bgcolor<-'white'
                }else{
                    if(diff > 0.9){
                        bgcolor<-'red'
                        }else{
                            if(diff > 0.8){
                                bgcolor<-'indianred1'
                                }else{
                                    if(diff > 0.7){
                                        bgcolor<-'lightpink2'
                                        }else{
                                            if(diff > 0.6){
                                                bgcolor<-'lightpink'
                                                }else{
                                                    if(diff > 0.5){
                                                        bgcolor<-'mistyrose'
                                                        }else{
                                                            if(diff > 0.4){
                                                                bgcolor<-'palegoldenrod'
                                                                }else{
                                                                    if(diff > 0.3){
                                                                        bgcolor<-'palegreen'
                                                                        }else{
                                                                            if(diff > 0.2){
                                                                                bgcolor<-'darkolivegreen3'
                                                                                }else{
                                                                                    if(diff > 0.1){
                                                                                        bgcolor<-'chartreuse3'
                                                                                        }else{
                                                                                            bgcolor<-'forestgreen'
                                                                                            }
                                                                                    }
                                                                            }
                                                                    }
                                                            }
                                                    }
                                            }
                                    }
                            }
                    }

#Plot
            plot(x=xVal,y=yVal,ylim=c(yMin, yMax),xlab=NA,ylab=NA,xaxt="n",yaxt="n")
            rect(par("usr")[1], par("usr")[3], par("usr")[2], par("usr")[4], col = bgcolor)
            points(x=xVal,y=yVal,ylim=c(yMin, yMax),xlab=NA,
                    ylab=NA,xaxt="n",yaxt="n")
            lines(x=xVal,y=yVal,ylim=c(yMin, yMax),xlab=NA,
                    ylab=NA,xaxt="n",yaxt="n")
            lines(x=xVal,y=yValS,ylim=c(yMin, yMax),xlab=NA,
                    ylab=NA,xaxt="n",yaxt="n",col="blue",lty=2)
            points(x=xVal,y=yValS,ylim=c(yMin, yMax),xlab=NA,
                    ylab=NA,xaxt="n",yaxt="n",col="blue")

#add the axis annotations: if we're on the last row, add the x axis
            if(r == dim(expResults[[1]])[1]){
                axis(side=1,at=xVal)
                }

#add the axis annotations: if we're on the first column, add the y axis
            if(c == 1){
                axis(side=2,at=c(-0.5,0,0.5))
                }


            }
    #Plot the image plots that tell preseence/absence of cues
    #
    data = t(matrix(as.numeric(valueCues[r,]),nrow=1))
    if (all(data==1)==TRUE){
        col=c("black")
    }
    else if (all(data==0)==TRUE){
        col=c("white")
    }
    else{
        col=c("white", "black")
    }
    image(data,col=col,xaxt="n",yaxt="n")

        if(r == dim(expResults[[1]])[1]){
            axis(
                side=1,
                at=seq(from=0, to=1,length.out=length(namesCues)),
                labels=as.character(namesCues),
                las=3)
            }
        }
}

#
#  This file is part of the CNO software
#
#  Copyright (c) 2011-2012 - EMBL - European Bioinformatics Institute
#
#  File author(s): CNO developers (cno-dev@ebi.ac.uk)
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  CNO website: http://www.cellnopt.org
#
##############################################################################
# $Id: writeReport.R 3155 2013-01-09 15:24:58Z cokelaer $
writeReport<-function(
	modelOriginal,
	modelOpt,
	optimResT1,
	optimResT2,
	CNOlist,
	directory,
	namesFiles=list(
		dataPlot=NA,
		evolFitT1=NA,
		evolFitT2=NA,
		simResultsT1=NA,
		simResultsT2=NA,
		scaffold=NA,
		scaffoldDot=NA,
		tscaffold=NA,
		wscaffold=NA,
        PKN=NA,
		PKNdot=NA,
		wPKN=NA,
		nPKN=NA),
	namesData=list(CNOlist=NA,model=NA),
	resE=NULL){

 if (!is(CNOlist,"CNOlist")){
        CNOlist = CellNOptR::CNOlist(CNOlist)
    }





    if (is.null(resE)==TRUE){
        resE<-residualError(CNOlist)
    }


#Create a report directory and copy css and logos in there
	dir.create(directory)
	cpfile<-dir(system.file("templates",package="CellNOptR"),full.names=TRUE)
	resultsdir<-file.path(getwd(),directory)
	file.copy(from=cpfile,to=resultsdir,overwrite=TRUE)

#copy all the graphs into the directory

	for(i in 1:length(namesFiles)){

		if(!is.na(namesFiles[[i]])){
			cpfile<-file.path(getwd(),namesFiles[[i]])
			file.copy(from=cpfile,to=resultsdir,overwrite=TRUE)
			unlink(cpfile)
			}

		}

#Produce the html page
	htmlfile=paste(resultsdir,'/CellNOptReport.html',sep="")
	cat(
		'<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN"http://www.w3.org/TR/html4/loose.dtd"> \n <html> ',
		file = htmlfile, append=TRUE)
	cat(
		'\n <link rel="stylesheet" type="text/css" href="CNOR.css" /> \n ',
		file = htmlfile, append=TRUE)
	cat(
		'<head> <title> CellNOptR Analysis Report </title> </head> \n ',
		file = htmlfile, append=TRUE)
	cat(
		'<body> \n <table class="border"> \n <tr class="border top"> \n ',
		file = htmlfile, append=TRUE)
	cat(
		'<td class="border corner"> &nbsp&nbsp&nbsp&nbsp </td>\n <td class="border top"> ',
		file = htmlfile, append=TRUE)
	cat(
		'<h1>Report from CellNOptR &nbsp &nbsp </h1> </td> \n ',
		file = htmlfile, append=TRUE)
	cat(
		'<td class="border corner2"> &nbsp&nbsp&nbsp&nbsp </td>\n </tr>\n <tr class="border"> \n <td class="border"></td>\n ',
		file = htmlfile, append=TRUE)
	cat(
		'<td class="main">\n <table>\n <tr>\n ',
		file = htmlfile, append=TRUE)
	cat(
		'<img src="Rlogo.png" width="70" height="70" ALIGN=RIGHT>&nbsp ',
		file = htmlfile, append=TRUE)
	cat(
		'<img src= "phinou.jpeg" width="110" height="110" ALIGN=RIGHT>&nbsp ',
		file = htmlfile, append=TRUE)
	cat(
		'<img src="logo_saezrodriguez_mid.gif" width="110" height="80" ALIGN=RIGHT>\n </tr>\n <tr>',
		file = htmlfile, append=TRUE)
	cat(
		paste(
			'\n  <td class="tabs"> <h3><a href="',
			namesFiles$dataPlot,
			'" title="data" target=blank >Data plot</a></h3> </td> ',
			sep=""),
		append = TRUE, file = htmlfile)
	cat(
		paste(
			'\n  <td class="tabs"> <h3><a href="',
			namesFiles$simResultsT1,
			'" title="Simulation results t1" target=blank >Simulation Results t1</a></h3> </td> ',
			sep=""),
		append = TRUE, file = htmlfile)
	cat(
		paste(
			'\n  <td class="tabs"> <h3><a href="',
			namesFiles$evolFitT1,
			'" title="Evolution of fit t1" target=blank >Evolution of fit t1</a></h3> </td> ',
			sep=""),
		append = TRUE, file = htmlfile)

	if(!is.na(namesFiles$evolFitT2)){
		cat(
			paste(
				'\n  <td class="tabs"> <h3><a href="',
				namesFiles$simResultsT2,
				'" title="Simulation results t2" target=blank >Simulation Results t2</a></h3> </td> ',
				sep=""),
			append = TRUE, file = htmlfile)
		cat(
			paste(
				'\n  <td class="tabs"> <h3><a href="',
				namesFiles$evolFitT2,
				'" title="Evolution of fit t2" target=blank >Evolution of fit t2</a></h3> </td> ',
				sep=""),
			append = TRUE, file = htmlfile)
		}

	cat('\n </tr> ', append = TRUE, file = htmlfile)
	cat(
		paste('<tr> \n <br> General information: \n <UL> \n <LI>data: ',namesData$CNOlist,sep=""),
		append = TRUE, file = htmlfile)
	cat(
		paste('\n <LI>time point(s): ',ifelse(is.na(namesFiles$evolFitT2),1,2),sep=""),
		append = TRUE, file = htmlfile)
	cat(
		paste('\n <LI>Residual error: t1: ',resE["t1"],sep=""),
		append = TRUE, file = htmlfile)
	nDataP<-sum(!is.na(CNOlist@signals[[2]]))
	scaledResE<-resE["t1"]/nDataP
	cat(
		paste(' (scaled: ',round(scaledResE,digits=4),' )',sep=""),
		append = TRUE, file = htmlfile)

	if(!is.na(resE["t2"])) {
		cat(paste('; t2: ',resE["t2"],sep=""), append = TRUE, file = htmlfile)
		nDataP<-sum(!is.na(CNOlist@signals[[3]]))
		scaledResE<-resE["t2"]/nDataP
		cat(
			paste(' (scaled: ',round(scaledResE,digits=4),' )',sep=""),
			append = TRUE, file = htmlfile)
		cat(
			paste('; t1andt2: ',resE["t1andt2"],sep=""),
			append = TRUE, file = htmlfile)
		nDataP<-sum(!is.na(CNOlist@signals[[3]]))+sum(!is.na(CNOlist@signals[[2]]))
		scaledResE<-resE["t1andt2"]/nDataP
		cat(
			paste(' (scaled: ',round(scaledResE,digits=4),' )',sep=""),
			append = TRUE, file = htmlfile)
		}

	cat(
		paste('\n <LI>previous knowledge network: ',namesData$model,sep=""),
		append = TRUE, file = htmlfile)
	cat(
		paste(
			'\n <LI>PKN: ',
			dim(modelOriginal$interMat)[1],' species and ',
			dim(modelOriginal$interMat)[2],' interactions',sep=""),
		append = TRUE, file = htmlfile)
	cat(
		paste(
			'\n <LI>Scaffold (compressed and expanded): ',
			dim(modelOpt$interMat)[1],' species and ',
			dim(modelOpt$interMat)[2],' interactions', '\n </UL>\n'),
		append = TRUE, file = htmlfile)
	cat(
		paste(
			'<br> Optimisation t1: \n<UL> \n <LI>generations: ',
			dim(optimResT1$results)[1],' (best model obtained after ',
			optimResT1$results[dim(optimResT1$results)[1],"Stall_Generation"],') \n', sep=""),
		append = TRUE, file = htmlfile)
 	cat(
 		paste(
 			'<LI>best string: ',sum(optimResT1$bString),
 			' interactions, objective function = ',
 			optimResT1$results[dim(optimResT1$results)[1],
 			"Best_score_Gen"],'\n', sep=""),
 		append = TRUE, file = htmlfile)
 	cat(
 		paste(
 			'<LI>number of strings within the tolerance limits: ',
 			dim(optimResT1$stringsTol)[1],' \n</UL>\n', sep=""),
 		append = TRUE, file = htmlfile)

	if(!is.na(namesFiles$evolFitT2)){
		cat(paste(
			'<br> Optimisation t2: \n<UL> \n <LI>generations: ',
			dim(optimResT2$results)[1],' (best model obtained after ',
			optimResT2$results[dim(optimResT2$results)[1],
			"Stall_Generation"],') \n', sep=""),append = TRUE, file = htmlfile)
 		cat(paste(
 			'<LI>best string: ',sum(optimResT2$bString),
 			' additional interactions, objective function = ',
 			optimResT2$results[dim(optimResT2$results)[1],"Best_score_Gen"],
 			'\n', sep=""),append = TRUE, file = htmlfile)
 		cat(paste(
 			'<LI>number of strings within the tolerance limits: ',
 			dim(optimResT2$stringsTol)[1],'\n</UL>\n', sep="")
 			,append = TRUE, file = htmlfile)
		}

 	cat(paste(
 		'<br> Scaffold network:\n <UL>\n <LI>cytoscape sif format: ',
 		namesFiles$scaffold,'\n',sep=""),append = TRUE, file = htmlfile)
 	cat(paste(
 		'<LI>graphviz dot format: ',namesFiles$scaffoldDot,'\n',sep=""),
 		append = TRUE, file = htmlfile)
 	cat(paste(
 		'<LI>edge attribute files: ',namesFiles$wscaffold,' and ',
 		namesFiles$tscaffold,'\n',sep=""),append = TRUE, file = htmlfile)
 	cat(paste(
 		'<LI>edge attributes respectively reflect the weight of the edge calculated as the frequency of the edge in the best solutions in the last generation of optimisation(s), and the time stamp on each edge (i.e. absent/present at t1/present at t2) \n</UL>\n',
 		sep=""),append = TRUE, file = htmlfile)
 	cat(paste(
 		'<br> Previous knowledge network:\n<UL> \n <LI>cytoscape sif format: ',
 		namesFiles$PKN,'\n',sep=""),append = TRUE, file = htmlfile)
 	cat(paste(
 		'<LI>graphviz dot format: ',namesFiles$PKNdot,'\n',sep=""),
 		append = TRUE, file = htmlfile)
 	cat(paste(
 		'<LI>edge attribute file: ',namesFiles$wPKN,' \n',sep=""),
 		append = TRUE, file = htmlfile)
 	cat(paste(
 		'<LI>node attribute file: ',namesFiles$nPKN,'\n',sep=""),
 		append = TRUE, file = htmlfile)
 	cat(paste(
 		'<LI>the edge attribute is a time stamp on each edge of the scaffold network (i.e. absent/present at t1/present at t2) mapped back to the original network\n',
 		sep=""),append = TRUE, file = htmlfile)
 	cat(paste(
 		'<LI>the node attribute reflects the information about which nodes are signals/inhibited/stimulated/non-controlable/non-observable\n </UL> \n',
 		sep=""),append = TRUE, file = htmlfile)
 	cat(paste(
 		'<br> <datestamp> ',date(),'</datestamp> ',sep=""),
 		append = TRUE, file = htmlfile)
 	cat(
 		'<hr/> \n </tr> \n </table> \n </td> \n <td class="border right"> </td> \n </tr> \n </table> \n </body> \n </html>',
 		append = TRUE, file = htmlfile)
	}

#
#  This file is part of the CNO software
#
#  Copyright (c) 2011-2012 - EMBL - European Bioinformatics Institute
#
#  File author(s): CNO developers (cno-dev@ebi.ac.uk)
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  CNO website: http://www.cellnopt.org
#
##############################################################################
# $Id$

#Function that computes the score of a specific bitstring
# todo: this is similar to wha is done in gaBinaryT1. need to do the same for T2
computeScoreT1<-function(CNOlist, model, bString, simList=NULL, indexList=NULL, 
    sizeFac=0.0001, NAFac=1, timeIndex=2){
    # simList and indexList are computed inside this function. 
    # However, for back-compatibility, we keep the arguments so that if
    # provided, we can still use them.

    # uncomment if required for debugging.
    # if (timeIndex<2){ stop("timeIndex must be >=2")}
    # if (timeIndex>length(CNOlist@timeSignals)){ 
    #      stop(paste("timeIndex must be <= ", length(CNOlist@timeSignals),sep=" "))
    # }


    if (is.null(simList)==TRUE){
        simList = prep4sim(model)
    }
    if (is.null(indexList)==TRUE){
        indexList = indexFinder(CNOlist, model)
    }

	# TC oct 2012. cutModel is a function. It performs the cut to select interMat, 
    # reacID, nameSpecies, and notMat. Here, we need only need the 3  and possibly only 2, so let us 
    # just copy and paste the code. This will save 10% of computational time
    # modelCut = cutModel(model, bString)
    bs = as.logical(bString)
    modelCut <- list()
    modelCut$interMat <- model$interMat[, bs]
    modelCut$reacID <- model$reacID[bs]
    modelCut$namesSpecies <- model$namesSpecies


    simListCut <- cutSimList(simList, bString)


    # Compute the simulated results
    nStimuli = length(indexList$stimulated)
    nInhibitors <- length(indexList$inhibited)
    nCond <- dim(CNOlist@stimuli)[1]
    nReacs <- length(modelCut$reacID)
    nSpecies <- length(model$namesSpecies) # this is correct. No need to get modelCut$namesSpecies 
    nMaxInputs <- dim(simListCut$finalCube)[2]


    # simList matrices. C code must handle the matrix indices carefully.
    # This is faster than transforming into a vector as in the previous code.
    finalCube = as.integer(simListCut$finalCube-1)
    ixNeg = as.integer(simListCut$ixNeg)
    ignoreCube = as.integer(simListCut$ignoreCube)
    maxIx = as.integer(simListCut$maxIx-1)

    # index. convertion from R to C indices convention.
    indexSignals <- as.integer(indexList$signals-1)
    indexStimuli <- as.integer(indexList$stimulated-1)
    indexInhibitors <- as.integer(indexList$inhibited-1)
    nSignals <- length(indexSignals)


    # cnolist
    valueInhibitors <- as.integer(CNOlist@inhibitors)
    valueStimuli <- as.integer(CNOlist@stimuli)

	simResults = .Call("simulatorT1", nStimuli, nInhibitors,
		nCond, nReacs, nSpecies, nSignals, nMaxInputs,
        finalCube, ixNeg, ignoreCube, maxIx,
        indexSignals, indexStimuli, indexInhibitors, valueInhibitors,
        valueStimuli, as.integer(1))

    simResultsT0 = .Call("simulatorT1", nStimuli, nInhibitors,
        nCond, nReacs, nSpecies, nSignals, nMaxInputs,
        finalCube, ixNeg, ignoreCube, maxIx,
        indexSignals, indexStimuli, indexInhibitors, 
        valueInhibitors, valueStimuli, as.integer(0))
 
    # Unlike in computeScoreTN, the C code does not cut the simResults so you need to do it
    # since it is done in getFit, no need to do it. 
    # If we cut the simRsults in the C code, we must comment these lines AND set indexlist=NULL in the 
    # get Fit call here below. However, the C code is called elwhere in simulateTN where the expected input to
    # the C code simulateTN expect the full simResults matrix !!!
    #simResultsT0 = simResultsT0[, indexList$signals]
    #simResults = simResults[, indexList$signals]
    nInTot = length(which(model$interMat == -1))

    #Compute the score
    Score <- getFit(
        simResults=simResults,
        CNOlist=CNOlist,
        model=modelCut,
        indexList=indexList,
        timePoint=timeIndex,
        sizeFac=sizeFac,
        NAFac=NAFac,
        nInTot=length(which(model$interMat == -1)),
        simResultsT0=simResultsT0)
    
    
    if (!is(CNOlist,"CNOlist")){
    	CNOlist = CellNOptR::CNOlist(CNOlist)
    }
    nDataP <- sum(!is.na(CNOlist@signals[[timeIndex]]))
    Score <- Score/nDataP
    
    
    return(Score)
}

#
#  This file is part of the CNO software
#
#  Copyright (c) 2011-2012 - EMBL - European Bioinformatics Institute
#
#  File author(s): CNO developers (cno-dev@ebi.ac.uk)
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  CNO website: http://www.cellnopt.org
#
##############################################################################
# $Id$
plotModel <- function(model, CNOlist=NULL, bString=NULL, indexIntegr=NULL,
					  signals=NULL, stimuli=NULL, inhibitors=NULL, NCNO=NULL, compressed=NULL,
					  output="STDOUT", filename=NULL,graphvizParams=list(), show=TRUE, remove_dot=TRUE,removeEmptyAnds=TRUE){
	# Quick example:
	# ---------------
	#   filename = "ToyPKNMMB.sif"
	#   g = plotModel(model, cnolist=cnolist)
	#   # g$graph contains the model transformed into a graph object
	
	# user parameters to refine the layout, color, ...
	if (is.null(graphvizParams$arrowsize)==TRUE) {
		graphvizParams$arrowsize=2
	}
	if (is.null(graphvizParams$size)==TRUE) {
		graphvizParams$size="15,15"
	}
	if (is.null(graphvizParams$fontsize)==TRUE) {
		graphvizParams$fontsize=22
	}
	if (is.null(graphvizParams$edgecolor)==TRUE) {
		graphvizParams$edgecolor="black"
	}
	if (is.null(graphvizParams$nodeLabels)==TRUE) {
		graphvizParams$nodeLabels=NULL
	}
	if (is.null(graphvizParams$nodeWidth)==TRUE) {
		graphvizParams$nodeWidth=2
	}
	if (is.null(graphvizParams$nodeHeight)==TRUE) {
		graphvizParams$nodeHeight=1
	}
	if (is.null(graphvizParams$viewEmptyEdges)==TRUE) {
		graphvizParams$viewEmptyEdges = TRUE
	}
	# mode must be before andWidth and andHeight
	if (is.null(graphvizParams$mode)==TRUE) {
		graphvizParams$mode = "sbgn"
	}
	
	if (graphvizParams$mode %in% c("sbgn", "classic") == FALSE){
		stop("mode must be in 'classic' or 'sbgn'")
	}
	
	if (is.null(graphvizParams$andWidth)==TRUE) {
		if (graphvizParams$mode == "classic"){
			graphvizParams$andWidth = 0.2
		}
		if (graphvizParams$mode == "sbgn"){
			graphvizParams$andWidth = 0.5
		}
		
	}
	if (is.null(graphvizParams$andHeight)==TRUE) {
		if (graphvizParams$mode == "classic"){
			graphvizParams$andHeight = 0.2
		}
		if (graphvizParams$mode == "sbgn"){
			graphvizParams$andHeight = 0.5
		}
	}
	
	# Some required library to build the graph and plot the results using
	# graphviz.  -already loaded by Depends fiels of Description
	# library(Rgraphviz)
	# library(RBGL)
	
	# Set the output filename
	if (is.null(filename)){
		filename="model"
		output_dot = "model.dot"
	}
	else{
		output_dot = paste(filename, ".dot", sep="")
	}
	
	# Set the image format if any
	if (output %in% c("STDOUT", "PDF", "PNG", "SVG") != TRUE){
		stop("wrong output format.Must be in PDF, PNG, SVG")
	}
	else{
		if (output=="PDF"){
			pdf(paste(filename, ".pdf", sep=""))
		}
		else if (output=="PNG"){
			png(paste(filename, ".png", sep=""))
		}
		else if (output=="SVG"){
			svg(paste(filename, ".svg", sep=""), pointsize=22, width=10, height=10)
		}
	}
	
	# If the cnolist is a NULL, nothing to do
	if (is.null(CNOlist)==TRUE){
		cnolist = NULL
	} else { # if not a CNOlist, try to convert it
		if (!is(CNOlist,"CNOlist")){
			cnolist = CellNOptR::CNOlist(CNOlist)
		} else{
			cnolist = CNOlist
		}
	}
	
	# TODO: The following piece of code should be made modular
	
	
	# Input data. If the model is a character, we guess that the user provided
	# the MODEL filename (sif format) that we can read directly.
	if (typeof(model) == "character"){
		raw = read.table(model)  # read the PKN data
		# build the unique vertices from the column 1 and 3 of the SIF file
		vertices = unique(c(as.character(raw$V1), as.character(raw$V3)))
		v1 = raw$V1 # alias to vertices in column 1
		v2 = raw$V3 # alias to vertices in column 2
		edges = raw$V2 # alias to the edges
		if (is.null(bString)){# default is only 1 so all edges are accepted
			BStimes<-rep(1,length(edges))
		}else{
			BStimes<-bString
		}
		Integr<-rep(0,length(edges))  #
	}
	# otherwise, the user probably provided the model already read by readSIF
	# in which case, and gates must be extracted from strings such as
	# "node1+node2=node3"
	# This block is the tricky part of the function. Change with care.
	else if (typeof(model)=="list" && any("namesSpecies" == names(model))){
		if (length(bString)!=length(model$reacID) & is.null(bString)==FALSE){
			stop(paste("If the bString argument is provided it must have the same",
					   "  length as model$reacID. ", "The model has ", length(model$reacID),
					   "  edges whereas the bitstring has a length of ", length(bString), sep=""))
		}
		# namesSpecies == names(model) try to check if model resemble the output of readSIF ?
		# ideally we should have a type.
		
		# build the unique vertices from the nameSpecies
		vertices = model$namesSpecies
		
		# now, we need to split the reaction to get back the different edges
		mysplit = function(x){strsplit(x, "=")}
		reacs = mysplit(model$reacID) # separate the reactions into left and right parts
		tmp <- unlist(mysplit(model$reacID))
		reacs = t(matrix(unlist(mysplit(model$reacID)), ncol=length(tmp)/2)) # reordering
		
		# Use the bString and indexIntegr input arguments to build up
		if (is.null(bString)){# default is only 1 so all edges are accepted
			optimBStimes<-rep(1,dim(reacs)[1])
		}else{
			optimBStimes<-bString
		}
		
		optIntegr<-rep(0,length(optimBStimes))
		if (is.null(indexIntegr)==FALSE){
			optIntegr[indexIntegr]<-1
		}
		
		# finally, build the v1, v2 and edges
		BStimes<-vector()
		Integr<-vector()
		
		CountReac<-1
		CountAnds<-1
		mysplit2 = function(x){strsplit(x, "+", fixed=TRUE)}
		v1<-vector()
		v2<-vector()
		edges<-vector()
		for (i in 1:dim(reacs)[1]){
			inputs<-unlist(strsplit(reacs[i,1],"+", fixed=TRUE))
			if (length(inputs)==1){
				v1[CountReac] = reacs[i,1]
				edges[CountReac] = 1
				v2[CountReac] = reacs[i,2]
				if (length(grep("!", v1))){
					v1[CountReac] = sub("!", "", v1[CountReac])
					edges[CountReac] = -1
				}
				BStimes[CountReac]<-optimBStimes[i]
				Integr[CountReac]<-optIntegr[i]
				CountReac<-CountReac+1
			}else{
				for (j in seq_along(inputs)){
					v1[CountReac]<-inputs[j]
					edges[CountReac] = 1
					v2[CountReac]<-paste("and",CountAnds,sep="")
					if (length(grep("!", v1[CountReac]))){
						v1[CountReac] = sub("!", "", v1[CountReac])
						edges[CountReac] = -1
					}
					BStimes[CountReac]<-optimBStimes[i]
					Integr[CountReac]<-optIntegr[i]
					CountReac<-CountReac+1
				}
				v1[CountReac]<-paste("and",CountAnds,sep="")
				edges[CountReac] = 1
				v2[CountReac] = reacs[i,2]
				BStimes[CountReac]<-optimBStimes[i]
				Integr[CountReac]<-optIntegr[i]
				CountReac<-CountReac+1
				vertices<-c(vertices,paste("and",CountAnds,sep=""))
				CountAnds<-CountAnds+1
			}
		}
	}
	
	if (is.null(cnolist) == FALSE){ # if a cnolist is provided, fill
		# signals/stimulis/inhitors
		stimuli <- colnames(cnolist@stimuli)
		signals <- colnames(cnolist@signals[[1]])
		inhibitors <- colnames(cnolist@inhibitors)
	}
	
	
	# check that the signal and stimuli are indeed in the list of vertices
	# otherwise they will be failures later.
	if (all(signals %in% vertices)==FALSE){
		msg = signals[signals %in% vertices == FALSE]
		print("Those signals were not found in the vertices: ")
		print(msg)
		vertices = union(vertices,signals)  # add it here to avoid failures in create_layout()
	}
	
	# build the edges. IGraph does not use names for the vertices but ids
	# that starts at zero. Let us build a data.frame to store the correspondence
	# between the ids and names.
	l = length(vertices) - 1
	
	# build the graph
	g <- new("graphNEL", nodes=vertices, edgemode="directed")
	weights = rep(1, length(v1)) # for now, the weights are to 1.
	for (i in 1:length(v1)){
		g <- addEdge(as.character(v1[i]), as.character(v2[i]), g, weights[i])
	}
	
	# The graph is now built. We can proceed with node and edge attributes for
	# the output files
	
	recipEdges="distinct" # an edge A->B does not overlap with B->A
	
	# --------------------------------------- Build the node and edges attributes list
	nodeAttrs = createNodeAttrs(g, vertices, stimuli, signals, inhibitors, NCNO,
								compressed, graphvizParams)
	
	res = createEdgeAttrs(v1, v2, edges, BStimes, Integr,
						  user_edgecolor=graphvizParams$edgecolor,
						  view_empty_edge=graphvizParams$viewEmptyEdges,removeEmptyAnds=removeEmptyAnds)
	# an alias
	edgeAttrs = res$edgeAttrs
	
	# createEdge returns the edgeAttrs and a list of edges that correspond to a
	# bistring element that is zero. In such case, the edge is useless and can
	# be removed.
	toremove = res$toremove
	for (x in toremove){
		y = unlist(strsplit(x, "~"))
		g = removeEdge(y[1], y[2], g)
	}
	# Some nodes are now connected to no other nodes. These nodes can be
	# removed. In principle, this is only "and" nodes.
	orphans = nodes(g)[(degree(g)$inDegree  + degree(g)$outDegree) ==0]
	for (x in orphans){
		if (x %in% stimuli == FALSE & x %in% inhibitors == FALSE & x %in% signals == FALSE){
			g = removeNode(x, g)
			# What was returned by createEdgeAttrs in now obsolet and need some cleanup
			f<-function(y){return(x %in% strsplit(y, "~", fixed=TRUE)[[1]])}
			indices = lapply(names(edgeAttrs$color),  f) == TRUE
			edgeAttrs$color[indices] <-NULL
			edgeAttrs$label[indices] <-NULL
			edgeAttrs$style[indices] <-NULL
			edgeAttrs$arrowhead[indices] <-NULL
		}
	}
	# we must rebuild the edges attributes
	
	# --------------------------- the ranks computation for the layout
	clusters = create_layout(g, signals, stimuli)
	
	# ------------------------------ general attributes
	# for some obscure reasons, must set style="filled" right her even though it
	# is then overwritten by nodesAttrs$style later on otherwise the output.dot
	# does not contain any style option
	
	# size does not seem to work in Rgraphviz version 1.32 wait and see for new version.
	fontsize=graphvizParams$fontsize
	attrs <- list(
		node=list(fontsize=fontsize,fontname="Helvetica",style="filled,bold"),
		edge=list(style="solid",weight="1.0",arrowsize=graphvizParams$arrowsize,minlen=3),
		graph=list(splines=TRUE,size=graphvizParams$size,bgcolor="white",ratio="fill",pad="0.5,0.5",dpi=72)
	)
	
	copyg <- g
	
	# current version of Rgraphviz (1.32 feb 2012) does not handle edgewidth
	# properly. A
	if (installed.packages()[,"Version"]["Rgraphviz"] <= "1.33.0"){
		#nodeAttrs$lty = "solid"
		print("plotModel: please upgrade to Rgraphviz >1.33.0 for best output")
		nodelty = "solid"
		savedEdgeAttrs = edgeAttrs$color
		edgeAttrs$color = NULL
	}
	else{
		nodelty=nodeAttrs$lty
	}
	
	# Set the node Rendering in Rgraphviz
	nodeRenderAttrs  <- setNodeRenderInfo(nodeAttrs, list(lwd=2, lty=nodelty,
														  cex=0.4, fontsize=fontsize, fixedsize=FALSE))
	nodeRenderInfo(g) <- nodeRenderAttrs
	
	# the arrowhead "normal" is buggy in Rgraphviz version 1.32 so switch to
	# "open" for now. However, the dot output keeps using the normal arrow.
	arrowhead2 = edgeAttrs$arrowhead
	#arrowhead2[arrowhead2=="normal"] = "open"
	
	# this statement must set recipEdges before calling edgeRenderInfo
	# otherwise feedback loops are not shown properly.
	graphRenderInfo(g) <-  list(recipEdges=recipEdges)
	
	edgeRenderAttrs  <- setEdgeRenderInfo(edgeAttrs,
										  list(arrowhead=arrowhead2, head=v2, tail=v1,
										  	 lwd=3, style="solid"))
	edgeRenderInfo(g) <- edgeRenderAttrs
	
	# Set the edge Rendering in Rgraphviz
	#   edgeRenderInfo(g) <- list(
	#        col=edgeAttrs$color,
	#        arrowhead=arrowhead2,
	#        head=v2,
	#        tail=v1,
	#        label=edgeAttrs$label,
	#        lwd=3,
	#        lty="solid"    #this fails in some cases even with version >=1.33.1
	#    )
	# hack for version of Rgraphviz 1.32. Set back the edgeAttr for the dot
	# output
	if (installed.packages()[,"Version"]["Rgraphviz"] <= "1.33.0"){
		edgeAttrs$color = savedEdgeAttrs
	}
	
	if (is.null(clusters)==TRUE){
		# finally, the layout for a R plot
		if (show==TRUE){
			x <- layoutGraph(g,layoutType="dot",recipEdges=recipEdges,attrs=attrs)
			renderGraph(x)
			nodeRenderInfo(x) <- nodeRenderAttrs
		}
		#edgeRenderInfo(x) <- edgeRenderAttrs
		edgeAttrs$style=NULL    # why ?
		toDot(copyg, output_dot, nodeAttrs=nodeAttrs,edgeAttrs=edgeAttrs,attrs=attrs, recipEdges=recipEdges)
		
		# bug introduced in Rgraphviz 1.34 that set node attributes border.lwd
		# and border.color that are not understood by dot. Best solution is to
		# change Rgraphviz but large latency so we can change the written files
		# afterwards to change the dot file itself
		clean_dot(output_dot)
	}
	else{
		# finally, the layout for a R plot
		#attrs$node = nodeRenderInfo(g)
		#attrs$edge = edgeRenderInfo(g)
		if (show==TRUE){
			x <- layoutGraph(g,layoutType="dot",subGList=clusters,recipEdges=recipEdges,attrs=attrs)
			
			
			# known bug in Rgraphviz: renderInfo should be called again after the
			# layout otherwise some attributes are lost in the renderGraph (e.g.,
			# color)
			# note that rendering is now made on x variable (not g)
			nodeRenderInfo(x) <- nodeRenderAttrs
			edgeRenderInfo(x) <- edgeRenderAttrs
			
			renderGraph(x)
		}
		
		# and save into dot file.
		toDot(graph=copyg, filename = output_dot, nodeAttrs=nodeAttrs, subGList=clusters,
			  attrs=attrs, recipEdges=recipEdges, edgeAttrs=edgeAttrs)
		# bug introduced in Rgraphviz 1.34 that set node attributes border.lwd
		# and border.color that are not understood by dot. Best solution is to
		# change Rgraphviz but large latency so we can change the written files
		# afterwards to change the dot file itself
		#
		# dot files can be read in Cytoscape but subgraphs produce an error. 
		# MAke sure to remove them before visualisation. 
		clean_dot(output_dot)
		
		
	}
	
	if (output != "STDOUT"){dev.off()}
	if (remove_dot==TRUE){file.remove(output_dot)}
	output = list(graph=g, attrs=attrs, nodeAttrs=nodeAttrs, edgeAttrs=edgeAttrs,clusters=clusters, v1=v1, v2=v2, edges=edges)
}


# if a cnolist, or at least signals/stimuli, then we can create ranking for the
# layout
create_layout <- function(g, signals, stimuli){
	
	# this algo will tell us the distance between vertices
	# distMatrix columns contains the distance from a vertex to the others
	distMatrix <- floyd.warshall.all.pairs.sp(g)
	#distMatrix <- johnson.all.pairs.sp(g)
	distMatrix[is.infinite(distMatrix) == TRUE] <- -Inf # convert -Inf to be able to grep numbers
	
	# if signals provided by the user is empty, let us find ourself the nodes
	# with input degrees set to zero:
	if (is.null(stimuli)==TRUE){
		stimuli = nodes(g)[degree(g)$inDegree==0]
	}
	
	# we will need to know the sinks, which are defined by an outDegree equal to
	# zero. Yet, if signals are already provided, we want to restrict the sinks
	# to this subsets.
	if (is.null(signals)==TRUE){
		sinks  <- nodes(g)[degree(g)$outDegree == 0]
	}
	else{
		sinks  <- signals[degree(g, signals)$outDegree == 0]
	}
	
	# compute max rank for each column
	ranks <- apply(distMatrix, 2, max)
	
	
	mrank = max(ranks, na.rm=TRUE)-1  # -1 because we already know the sinks
	if (mrank == -Inf){
		print("Issue while computing max rank. Skipping the clustering step");
		return (NULL)
	}
	
	
	
	clusters <- list()
	if (mrank >= 1){ # otherwise, nothing to do.
		# for each different rank select the names of the column to create a
		# cluster
		for (rank in 1:mrank){  # starts at 1 although ranks starts at 0.
			# However, 0 corresponds to the source that are
			# known already
			# nodes found for a particular rank may contain a sink, that should
			# be removed
			# some ranks might not contain any nodes (rank is derived from shortest distance.) -- then we skip for next. 
			nodes2keep = NULL
			nodes <- names(ranks[which(ranks==rank)])
			if(length(nodes)==0) next()
			for (n in nodes){
				if (any(n==sinks) == FALSE){ nodes2keep <- c(nodes2keep, n)}
			}
			
			if(length(nodes2keep)==0) next()
			
			# may fail sometimes
			tryCatch({
				
				thisCluster <- subGraph(nodes2keep, g)
				thisGraph <-list(graph=thisCluster,cluster=FALSE,attrs=c(rank="same"))
				clusters[[length(clusters)+1]] =  thisGraph},
				error=function(e){
					#print(rank)
					
					print("warning: clustering in the layout failed. carry on...")})
			
		}
	}
	# first the stimulis
	tryCatch(
		{
			tryCatch({
				clusterSource <- subGraph(stimuli, g);
				clusterSource<-list(graph=clusterSource,cluster=FALSE,attrs=c(rank="source"))},
				error=function(e){
					print("error during clustering in subGraph(stimuli, g)? Does the stimuli from your MIDAS are present in the model?"); print(stimuli)
					})
			tryCatch(
				{clusters[[length(clusters)+1]] = clusterSource},
				error=function(e){print("error in clusters2. should never be here")}
			)
			
		},
		error=function(e){print("warning: clustering the source/stimuli failed. carry on...")}
	)
	
	# then the signals keeping only those with outDegree==0
	tryCatch(
		{
			clusterSink <- subGraph(sinks, g)
			clusterSink <- list(graph=clusterSink, cluster=FALSE,  attrs=c(rank="sink"))
			tryCatch(
				{clusters[[length(clusters)+1]] = clusterSink},
				error=function(e){print("error in clusters3. should never be here")}
			)
		}, error=function(e){print("warning: clustering the sink failed. carry on...")}
	)
	
	
	return(clusters)
}


# Create the node attributes and save in a list to be used either by the
# plot function of the nodeRenderInfo function.
createNodeAttrs <- function(g, vertices, stimuli, signals, inhibitors, NCNO,
							compressed, graphvizParams){
	
	
	nodeLabels = graphvizParams$nodeLabels
	nodeWidth = graphvizParams$nodeWidth
	nodeHeight = graphvizParams$nodeHeight
	andHeight = graphvizParams$andHeight
	andWidth = graphvizParams$andWidth
	mode = graphvizParams$mode
	
	# ----------------------------------------------- Build the node attributes list
	fillcolor <- list()
	color <- list()
	style <- list()  # the style of what is inside the node.
	lty <- list()    #style of the contour node
	height <-list()
	label <- list()
	width <- list()
	fixedsize <- list()
	shape <- list()
	
	# default. Must be filled in case no signals/stimulis/cnolist are provided
	# Default node attributes
	for (s in vertices){
		color[s] <- "black"         # color edge
		fillcolor[s] <- "white"     # fill color
		style[s] <- "filled,bold"
		lty[s] <- "solid"
		label[s] <- s
		height[s] <- nodeHeight
		width[s] <- nodeWidth
		fixedsize[s] <- FALSE
		shape[s] <- "rectangle"
	}
	
	# user can provide a list of labels for the nodes. Usuful to show attributes
	# of a node.
	if (is.null(nodeLabels)==FALSE){
		names(nodeLabels)<-vertices
		if (length(nodeLabels)!=length(vertices)){
			stop("if nodeLabels provided, it must be of same size as vertices")
		}
		else{
			for (s in vertices){
				label[s] = nodeLabels[s]
			}
		}
	}
	
	# The stimulis node
	for (s in stimuli){
		fillcolor[s] <- "olivedrab3";
		color[s] <- "olivedrab3";
		style[s] <- "filled"
		color[s] <- "black";
	}
	# The signal nodes
	for (s in signals){ #must be before the inhibitors to allow bicolors
		fillcolor[s] <- "lightblue";
		color[s] <-"lightblue";
		color[s] <- "black";
	}
	# The inhibitor node, that may also belong to the signal category.
	for (s in inhibitors){
		if (length(grep(s, signals))>=1){
			#fillcolor[s] <- "SkyBlue2"
			shape[s]="ellipse"
			style[s] <- "filled,bold"
			color[s] <-"orangered"
		} else {
			if (length(grep(s, stimuli))>=1){
				#fillcolor[s] <- "SkyBlue2"
				shape[s]="ellipse"
				style[s] <- "filled,bold"
				color[s] <-"orangered"
			}
			else{
				fillcolor[s] <- "orangered";
				color[s] <-"orangered";
				color[s] <- "black";
			}
		}
	}
	# The compressed nodes
	for (s in compressed){
		fillcolor[s] <- "white";
		color[s] <- "black";
		style[s] <- "dashed,bold";
		lty[s]="dashed";
	}
	# The NCNO node
	for (s in NCNO){
		fillcolor[s] <- "white";
		color[s] <- "grey";
		fillcolor[s] = "grey"
	}
	# the and gate nodes
	for (s in vertices){
		if (length(grep("^and", s))>=1){
			color[s] = "black"
			if (mode=="sbgn"){
				fillcolor[s] = "white"
				label[s] = "and"
			} else{
				fillcolor[s] = "black"
				label[s] = ""
			}
			width[s]=andWidth
			height[s]=andHeight
			fixedsize[s]=FALSE
			shape[s]="circle"
			
			if (degree(g)$inDegree[s]==3){
				#fillcolor[s] = "blue"
				if (mode=="classic"){
					shape[s]="triangle"
				}
				width[s]=andWidth
				height[s]=andHeight
			}
			if (degree(g)$inDegree[s]==4){
				#fillcolor[s] = "blue"
				if (mode=="classic"){
					shape[s]="rectangle"
				}
				width[s]=andWidth
				height[s]=andHeight
			}
			
		}
	}
	nodeAttrs <- list(fillcolor=fillcolor, color=color, label=label, width=width, height=height,
					  style=style, lty=lty, fixedsize=fixedsize, shape=shape)
	return(nodeAttrs)
}


# Create the node attributes and save in a list to be used either by the
# plot function of the edgeRenderInfo function.
createEdgeAttrs <- function(v1, v2, edges, BStimes ,Integr, user_edgecolor,
							view_empty_edge=TRUE,removeEmptyAnds=TRUE){
	
	edgewidth_c = 3 # default edge width
	
	# The edge attributes
	arrowhead <- list()
	edgecolor <- list()
	edgewidth <- list()
	label <- list()
	toremove <- list()
	lty <- list() # not used yet.
	
	
	for (i in 1:length(edges)){
		edgename = paste(v1[i], "~", v2[i], sep="")
		edgewidth[edgename] = edgewidth_c    # default edgewidth
		label[edgename] = ""                 # no label on the edge by default
		lty[edgename] = "solid"
		edgecolor[edgename] = "black"        # set a default (useless but safe)
		
		if (edges[i] == 1){
			arrowhead[edgename] <- "normal"
			edgecolor[edgename] <- user_edgecolor
			if (Integr[i]==1){edgecolor[edgename] <- "purple"}
		}
		else if (edges[i] == -1){
			arrowhead[edgename] <- "tee"
			edgecolor[edgename] <- "red"
			if (Integr[i]==1){edgecolor[edgename] <- "purple"}
		}
		else{
			arrowhead[edgename] <- "normal"
			edgecolor[edgename] <- "blue"
		}
		
		# BStimes contains the bitstring. color the edges according to its value
		v = (BStimes[i]*100)%/%1
		
		# width of the edges
		if (v != 100){
			# first, let us build the color
			if (edgecolor[edgename] == 'red'){
				# if red, go from red to light pink color according to v value
				if (view_empty_edge==TRUE){
					print("showing red link even if empty")
					color = rgb(1,1-(max(20,v)/100),1-(max(20,v)/100))
				}
				else{
					color = rgb(1,1-v/100,1-v/100)
				}
			}
			else if (edgecolor[edgename] == 'black'){
				# if black, go from grey dark to grey light color according to v value
				if (view_empty_edge==TRUE){
					color = paste("grey", as.character(100.-max(20,v)), sep="")
				}
				else{
					color = paste("grey", as.character(100.-v), sep="")
				}
			}
			else{
				# otherwise, just keep the color identical and only add label
				color = edgecolor[edgename]
			}
			# now, we fill the fields with color and width
			if (v == 0){
				edgewidth[edgename]  = edgewidth_c*(10./100)
				edgecolor[edgename] = color
				if(removeEmptyAnds){# and gates that have no link are removed.
					if (length(grep("and", edgename))>=1){
						toremove <- append(toremove, edgename)
					}
				}
			}
			else{
				edgecolor[edgename] = color
				edgewidth[edgename] = edgewidth_c*(v/100)
				if (v!=0){
					label[edgename] = as.character(v)
				}else{label[edgename]="0"}
			}
		}
		else {
			#already set at the beginning of the loop: width = edgewidth_c
		}
	}
	
	edgeAttrs <- list(color=edgecolor,arrowhead=arrowhead,
					  label=label, style=lty)
	
	return(list(toremove=toremove, edgeAttrs=edgeAttrs))
}



clean_dot <- function(filename)
{
	savedata = readLines(filename)
	data = savedata
	for (i in 1:length(data)){
		data[i] = sub("border.lwd=1,", "", data[i])
		data[i] = sub("border.color=black", "", data[i])
	}
	
	# if died, need to save back the "savedata"
	# otherwise, we can overwrite model.dot
	tryCatch(write(data, file=filename), error=function(e){print("Could not scan the dot file for cleaning (border.lwd and border.color). "); })
}

# Create the list of attributes for the nodes
setNodeRenderInfo <- function(nodeAttrs, extraAttrs)
{
	attrs  <- list(
		fill=nodeAttrs$fillcolor,
		col=nodeAttrs$color,
		style=nodeAttrs$style,
		style=extraAttrs$style,
		lwd=extraAttrs$lwd,
		label=nodeAttrs$label,
		shape=nodeAttrs$shape,
		cex=extraAttrs$cex,
		fontsize=extraAttrs$fontsize,
		iwidth=nodeAttrs$width,
		iheight=nodeAttrs$height,
		fixedsize=extraAttrs$fixedsize)
	return(attrs)
}

# Create the list of attributes for the edges
setEdgeRenderInfo <- function(edgeAttrs, extraAttrs)
{
	attrs <- list(
		col=edgeAttrs$color,
		arrowhead=extraAttrs$arrowhead,
		head=extraAttrs$v2,
		tail=extraAttrs$v1,
		label=edgeAttrs$label,
		lwd=extraAttrs$lwd,
		style=extraAttrs$style    #this fails in some cases even with version >=1.33.1
	)
	return(attrs)
}

#
#  This file is part of the CNO software
#
#  Copyright (c) 2011-2012 - EMBL - European Bioinformatics Institute
#
#  File author(s): CNO developers (cno-dev@ebi.ac.uk)
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  CNO website: http://www.cellnopt.org
#
##############################################################################
# $Id$
cutSimList <- function(simList, bString)
{
    bString = as.logical(bString)
    simListCut <- simList
    finalCube <- simListCut$finalCube[bString,,drop=FALSE]
    ixNeg <-simListCut$ixNeg[bString,,drop=FALSE]
    ignoreCube <- simListCut$ignoreCube[bString,,drop=FALSE]
    maxIx <- simListCut$maxIx[bString]
    # in some cases the finalcube is a matrix but list of integer, so we
    # need to convert back to a matrix. Happens for simple models only hence
    # the warning.
    if (is.matrix(finalCube) == FALSE){
        #warning("converting back to matrix in prep4sim")
        simListCut$finalCube<-matrix(finalCube,
            dimnames=list(names(finalCube), 1))
        simListCut$ixNeg<-matrix(ixNeg, dimnames=list(names(ixNeg), 1))
        simListCut$ignoreCub<-matrix(ignoreCube,dimnames=list(names(ignoreCube), 1))
        simListCut$maxIx<-matrix(maxIx,dimnames=list(names(maxIx), 1))
    }
    else{
        simListCut$finalCube = finalCube
        simListCut$ixNeg<-ixNeg
        simListCut$ignoreCube<-ignoreCube
        simListCut$maxIx<-maxIx
    }
    return(simListCut)
}

#
#  This file is part of the CNO software
#
#  Copyright (c) 2011-2012 - EMBL - European Bioinformatics Institute
#
#  File author(s): CNO developers (cno-dev@ebi.ac.uk)
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  CNO website: http://www.cellnopt.org
#
##############################################################################

preprocessing<-function(data=NULL, model, cutNONC=TRUE, compression=TRUE,
    expansion=TRUE, ignoreList=NA, maxInputsPerGate=2,verbose=TRUE){

    # why not doing this check here ? Does not cost too much
    if (is.null(data)!=TRUE){
        checkSignals(CNOlist=data,model=model)
    }
	
	
    # a copy of the model
    cutModel <- model

    if (cutNONC==TRUE && is.null(data)!=TRUE){
        # Find the indices, in the model, of the species that are inh/stim/sign
        indices<-indexFinder(CNOlist=data, model=model,    verbose=verbose)

        # Find the indices of the non-osb/non-contr
        temp_indices <- findNONC(model=model, indexes=indices,verbose=verbose)
        # Cut the nonc off the model
        cutModel <-cutNONC(model=model, NONCindexes=temp_indices)
    }

    if (compression == TRUE && is.null(data)!=TRUE){
        # Recompute the indices
        temp_indices<-indexFinder(CNOlist=data, model=cutModel)

        # Compress the model
        cutModel<-compressModel(model=cutModel,indexes=temp_indices)
    }

    # Recompute the indices. We can do it now because the expanson gate does not
    # remove species but only add and gates.
    #if (is.null(data)!=TRUE){
    #    indices<-indexFinder(CNOlist=data,model=cutModel)
    #}
    #else{
    #    indices <- NULL
    #}

    # Expand the gates
    if (expansion == TRUE){
        cutModel <- expandGates(model=cutModel, ignoreList=ignoreList,maxInputsPerGate=maxInputsPerGate)
    }

    # since version 1.3.28 return only model, indices are recomputed in other
    # functions
    return(cutModel)
}

readSif <- function(sifFile){
    warning("readSif is a deprecated function. Use readSIF instead. Calling readSIF for you")
    return(readSIF(sifFile))
}

prep4Sim <- function(model, params){
    warning("prep4Sim is a deprecated function. Use prep4sim instead. ")
    return(prep4sim(model))
}


simulateT1 <- function(CNOlist, model, bStringT1,simList, indexList){
    warning("simulateT1 is a deprecated function. Use simulateTN instead. ")
    return(simulateTN(CNOlist, model, bStrings=list(bStringT1)))
}

simulateT2 <- function(CNOlist, model, bStringT1,simList, indexList){
    warning("simulateT1 is a deprecated function. Use simulateTN instead. ")
    return(simulateTN(CNOlist, model, bStrings=list(bStringT1)))
}


gaBinaryT2 <- function(CNOlist, model, simList, indexList, bStringT1,
    sizeFac=0.0001, NAFac=1, popSize=50, pMutation=0.5, maxTime=60,
    maxGens=500, stallGenMax=100, selPress=1.2, elitism=5, relTol=0.1,
    verbose=TRUE, priorBitString=NULL){

    warning("gaBinaryT2 is a deprecated function. Use gaBinaryTN instead.")
    return(gaBinaryTN(CNOlist=CNOlist, model=model, bStrings=list(bStringT1),
     sizeFac=sizeFac, NAFac=NAFac, popSize=popSize,  pMutation=pMutation,  maxTime=maxTime,
     maxGens=maxGens,  stallGenMax=stallGenMax,  selPress=selPress,  elitism=elitism,
     relTol=relTol,  verbose=verbose, priorBitString=priorBitString))
}

#
#  This file is part of the CNO software
#
#  Copyright (c) 2011-2013 - EMBL-EBI
#
#  File author(s): CNO developers (cno-dev@ebi.ac.uk)
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  CNO website: http://www.cellnopt.org
#
##############################################################################
# $Id$

# This function reads a SBMLQual written for storing SIF file.
# The listOfQualitativeSpecies should contain the species
# From the listOfTransitions, the edgse can be extracted
# TODO: what if there is no transittion for a given specy. Is it possible ? If
# so this is an isolated node.

readSBMLQual <- function(filename){
    warning("experimental SBML reader. use with care July 2013.")
    # library(XML) - already loaded because of package Depends field
    doc = xmlTreeParse(filename)
    r = xmlRoot(doc)

    sif = data.frame(matrix(0, ncol=3))

    species = xmlChildren(xmlChildren(r)[[1]])['listOfQualitativeSpecies']
    nameSpecies = as.vector(unlist(xmlApply(species[[1]], function(x) xmlGetAttr(x, "id"))))

    transitions = xmlChildren(xmlChildren(r)[[1]])['listOfTransitions']

    andCounter = 1
    for (i in seq_along((transitions[[1]]))){
        # somehow we can not loop over the transitions so we use
        # seq_along(length(transitions))
        transition = transitions[[1]][[i]]
        outputs = xmlChildren(transition)$listOfOutputs
        inputs = xmlChildren(transition)$listOfInputs
        signs = as.vector(unlist(sapply(xmlChildren(inputs), function(x) xmlGetAttr(x,"sign"))))
        LHS = as.vector(unlist(sapply(xmlChildren(inputs), function(x) xmlGetAttr(x,"qualitativeSpecies"))))
        RHS = as.vector(unlist(sapply(xmlChildren(outputs), function(x) xmlGetAttr(x,"qualitativeSpecies"))))
        functions = xmlChildren(transition)$listOfFunctionTerms

        # from function, figure out if we have a OR or AND gate
        logics = xmlChildren(xmlChildren(xmlChildren(xmlChildren(functions)$functionTerm)$math)$apply)
        logics = names(logics)

        # all links other than AND should be here
        if (("and" %in% logics)==FALSE){
            for (j in seq_along(LHS)){
                if (signs[[j]] == "positive"){
                    sif = rbind(c(LHS[[j]],1,RHS), sif)
                } else if (signs[[j]] == "negative") {
                    sif = rbind(c(LHS[[j]],-1,RHS), sif)
                } else {
                    stop("signs must be negative or positive")
                }
            }
        }

        # the AND case only
        if ("and" %in% logics){
            andName = paste("and", andCounter, sep="")
            for (j in seq_along(LHS)){
                if (signs[[j]] == "positive"){
                    sif = rbind(c(LHS[[j]],1,andName), sif)
                } else if (signs[[j]] == "negative") {
                    sif = rbind(c(LHS[[j]],-1,andName), sif)
                } else {
                    stop("signs must be negative or positive")
                }
            }
            sif = rbind(c(andName, 1, RHS), sif)
            andCounter = andCounter + 1
        }
    }

    # remove last row (0,0,0) set at the beginning to define the data frame.
    sif = sif[-dim(sif)[[1]],]

    fh = tempfile()
    write.table(sif,file=fh,
            row.names=FALSE,col.names=FALSE,quote=FALSE,sep="\t")

    sif = readSIF(fh)

    # hack/trick to replace self loop with a dummy node.
    #sif = .add_dummies(sif)

    return(sif)

}





# this function reads a model, find self loops.
# If any self loops are found, there are replaced by A->dummy->A
# Does handle simple direct self loops e.g. A->A but not A-|A
.add_dummies <- function(model){

    # identify self-loops
    indices = c()
    for (ireac in seq_along(model$reacID)){
        reac = model$reacID[ireac]
        lhs = strsplit(reac, "=")[[1]][1]
        rhs = strsplit(reac, "=")[[1]][2]
        if (lhs == rhs){
            indices = c(indices, ireac)
        }
    }

    # don't do anything if no self loops are found
    if (length(indices)==0){
        return(model)
    }

    # remove self loops
    warning("Self inhibited loop not handled yet in this version")
    newmodel = model
    newmodel$reacID = newmodel$reacID[-indices]
    newmodel$interMat <- newmodel$interMat[,-indices]
    newmodel$notMat <- newmodel$notMat[,-indices]

    # add them back with the dummies
    counter = 1
    for (i in seq_along(indices)){
        index = indices[i]
        dummy = paste("dummy" , counter, sep="")
        reac = model$reacID[index]
        species = strsplit(reac, "=")[[1]][1]
        reac1 = paste(species, "=", dummy, sep="")
        reac2 = paste(dummy, "=", species, sep="")
        counter = counter + 1

        # add the 2 new reactions only in reacID for now
        newmodel$reacID = c(newmodel$reacID, c(reac1, reac2))

        # add the dummy species
        newmodel$namesSpecies = c(newmodel$namesSpecies, dummy)

        ########## INTERMAT ############
        # now add first the new species row in interMat
        rowvec = rep(0, length(newmodel$reacID)-2 )
        newmodel$interMat = rbind(newmodel$interMat, rowvec)
        names = rownames(newmodel$interMat)
        names[length(names)] = dummy
        rownames(newmodel$interMat) = names

        # add species=dummy column in interMat
        colvec = rep(0, length(newmodel$namesSpecies)) # !! the dummy node is there
        colvec[newmodel$namesSpecies == species] = -1
        colvec[newmodel$namesSpecies == dummy] = 1
        colvec = colvec[1:length(colvec)]
        newmodel$interMat = cbind(newmodel$interMat, colvec)
        names = colnames(newmodel$interMat)
        names[length(names)] = reac1
        colnames(newmodel$interMat) = names

        # add dummy=species column in interMat
        colvec = rep(0, length(newmodel$namesSpecies))
        colvec[newmodel$namesSpecies == species] = 1
        colvec[newmodel$namesSpecies == dummy] = -1
        newmodel$interMat = cbind(newmodel$interMat, colvec)
        names = colnames(newmodel$interMat)
        names[length(names)] = reac2
        colnames(newmodel$interMat) = names


        ########## NOTMAT ############
        # now add the new species row in interMat
        rowvec = rep(0, length(newmodel$reacID)-2)
        newmodel$notMat = rbind(newmodel$notMat, rowvec)
        names = rownames(newmodel$notMat)
        names[length(names)] = dummy
        rownames(newmodel$notMat) = names

        # add species=dummy column in NotMat only zeros
        colvec = rep(0, length(newmodel$namesSpecies))
        newmodel$notMat = cbind(newmodel$notMat, colvec)
        names = colnames(newmodel$notMat)
        names[length(names)] = reac1
        colnames(newmodel$notMat) = names

        ## add dummy=species column in notMat only zeros
        colvec = rep(0, length(newmodel$namesSpecies))
        newmodel$notMat = cbind(newmodel$notMat, colvec)
        names = colnames(newmodel$notMat)
        names[length(names)] = reac2
        colnames(newmodel$notMat) = names


    }
    return(newmodel)
}

#
#  This file is part of the CNO software
#
#  Copyright (c) 2011-2012 - EMBL - European Bioinformatics Institute
#
#  File author(s): CNO developers (cno-dev@ebi.ac.uk)
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  CNO website: http://www.cellnopt.org
#
##############################################################################
# $Id$
simulatorT0<-function(CNOlist,model,simList,indexList){

    if (!is(CNOlist,"CNOlist")){
        CNOlist = CellNOptR::CNOlist(CNOlist)
    }

    # This simulator is dedicated to the time0 case.
    # It creates a new structure called CNOlistT0, which is a subset of a CNOlist
    # It also sets the valueInhibitors and valueStimuli to zero.
    mi = CNOlist@inhibitors
    ms = CNOlist@stimuli
    # only inhibitors and stimuli are required but need to convert this into a
    # a valid CNOlist class, so we must provide all values

    CNOlistT0 = CNOlist
    CNOlistT0@inhibitors = matrix(1, dim(mi)[1], dim(mi)[2])
    CNOlistT0@stimuli    = matrix(0, dim(ms)[1], dim(ms)[2])

    # !! hack. SimulatorT1 expect the inhibitors values to be 0 or 1.
    # Then, it flips them. So, the 0-values are flipped to 1. finally,
    # the values that are equal to 1 are set to NA...
    # Here, since this is T0, all inhibitors are set to 0. simulatorT1 will
    # therefore flip them to 1 and finally NA. So, we set all inhibitors to 1.

    # Need to be very careful if simulatorT1 changes

    # Finally run the simulator with the particular set of experiments at t0
    newInput = simulatorT1(CNOlistT0, model, simList, indexList, mode=0)
    return(newInput)
    }

#  Copyright (c) 2019 - SaezLab - Heidelberg Universit
#
#  File author(s): CNO developers
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  CNO website: http://www.cellnopt.org
#
##############################################################################

writeCplexCommandFile <- function(commandFileName, 
                                  inputFileName, 
                                  outputFileName,
                                  mipGap=mipGap,
                                  relGap=relGap,
                                  timelimit=timelimit, 
                                  cplexPath=cplexPath, 
                                  numSolutions = numSolutions, 
                                  limitPop = limitPop, 
                                  poolIntensity = poolIntensity, 
                                  poolReplace = poolReplace){
  
  data = commandFileName
  write(paste0("read ", inputFileName), data)
  write(paste0("set mip tolerances mipgap ",mipGap), data, append = TRUE)
  write(paste0("set mip pool relgap ",relGap), data, append = TRUE)
  write(paste0("set mip pool replace ", poolReplace), data, append = TRUE)
  write(paste0("set mip pool intensity ",poolIntensity), data, append = TRUE)
  write(paste0("set timelimit ",timelimit=timelimit), data, append = TRUE)
  write(paste0("set mip limits populate ", limitPop), data, append = TRUE)
  write(paste0("set mip pool capacity ", numSolutions), data, append = TRUE)
  write(paste0("populate"), data, append = TRUE)
  write(paste0("write ", outputFileName, " sol all"), data, append = TRUE)
  write("quit", data, append = TRUE)
  
}

writeMIDAS <- function(CNOlist, filename, timeIndices=NULL, overwrite=FALSE)
{

    if (!is(CNOlist,"CNOlist")){
         cnolist = CellNOptR::CNOlist(CNOlist)
    }
    else{
        cnolist = CNOlist
    }

    # remove exiisting file if required
    if (overwrite==TRUE){
        if (file.exists(filename)==TRUE){
            file.remove(filename)
        }
    } else {
        if (file.exists(filename)==TRUE){
            stop(paste("File ", filename, "already exists.",  sep=""))
        }
    }

    timeSignals = cnolist@timepoints
    namesCues = colnames(cnolist@cues)
    namesStimuli = colnames(cnolist@stimuli)
    namesInhibitors = colnames(cnolist@inhibitors)
    namesSignals = colnames(cnolist@signals[[1]])

    #nCues = length(namesCues)
    nStimuli = length(namesStimuli)
    nInhibitors = length(namesInhibitors)
    nSignals = length(namesSignals)
    input_nTimes = length(timeSignals)
    output_nTimes = length(timeSignals)
    # if timeIndices provided, the inputs nTimes and output nTimes are different
    if (is.null(timeIndices)==TRUE){
        timeIndices = 1:input_nTimes
    } else{
        output_nTimes = length(timeIndices)
    }

    # First column is cellline
    nCols = 1 + nStimuli + nInhibitors + nSignals * 2 
    nConds = dim(cnolist@signals[[1]])[1]
    nRows = nConds * output_nTimes

    # build the header
    colNames = c("TR:mock:CellLine")
    for (x in namesStimuli){
        colNames = cbind(colNames, paste("TR:", x, sep=""))
    }
    for (x in namesInhibitors){
        colNames = cbind(colNames, paste("TR:", x, "i", sep=""))
    }
    for (x in namesSignals){
        colNames = cbind(colNames, paste("DA:", x, sep=""))
    }
    for (x in namesSignals){
        colNames = cbind(colNames, paste("DV:", x, sep=""))
    }

    data = matrix(NA, nRows, nCols)

    # cellline is made of ones for now
    data[,1] = 1

    # fill the experiments first


    for (time in seq_along(timeIndices)){

            i1 = 1 + (time - 1) * nConds
            i2 = i1 + nConds - 1

            j1 = 2; j2 = j1 + nStimuli - 1

            data[i1:i2, j1:j2] = cnolist@stimuli

            if (nInhibitors>0){
                j1 = j2 + 1; j2 = j1 + nInhibitors - 1
                data[i1:i2, j1:j2] = cnolist@inhibitors
            } 

            j1 = j2 + 1; j2 = j1 + nSignals - 1
            data[i1:i2, j1:j2] = rep(timeSignals[timeIndices[time]], nSignals)

            j1 = j2 + 1; j2 = j1 + nSignals - 1
            data[i1:i2,j1:j2] = cnolist@signals[[timeIndices[time]]] 
    }


    # and save it into a file
    write.table(data, file=filename,
        row.names=FALSE, col.names=colNames,quote=FALSE,sep=",")

}

#
#  This file is part of the CNO software
#
#  Copyright (c) 2011-2012 - EMBL - European Bioinformatics Institute
#
#  File author(s): CNO developers (cno-dev@ebi.ac.uk)
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  CNO website: http://www.cellnopt.org
#
##############################################################################
# $Id: plotOptimResultsPDF.R 3155 2013-01-09 15:24:58Z cokelaer $
plotOptimResultsPDF<-function(
    simResults=simResults,
    expResults=expResults,
    times=times,
    namesCues=namesCues,
    namesSignals=namesSignals,
    valueCues=valueCues,
    filename, formalism="new"){

    if(sum(dim(simResults[[1]])) < 20){

        pdf(file=filename,width=14,height=7)

        }else{

            pdf(file=filename,width=21,height=10)

            }

    plotOptimResults(
        simResults=simResults,
        expResults=expResults,
        times=times,
        namesCues=namesCues,
        namesSignals=namesSignals,
        valueCues=valueCues,
        formalism=formalism)

    dev.off()
    }

#
#  This file is part of the CNO software
#
#  Copyright (c) 2011-2012 - EMBL - European Bioinformatics Institute
#
#  File author(s): CNO developers (cno-dev@ebi.ac.uk)
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  CNO website: http://www.cellnopt.org
#
##############################################################################
# $Id: writeNetwork.R 3155 2013-01-09 15:24:58Z cokelaer $
##the global function is still called writeNetwork and still needs the same arguments
#(modelComprExpanded, optimResT1, optimResT2, modelOriginal, CNOlist), but it is divided into:
#    -a function writeSNetworkW that does the actual writing to files,
#    -a function that gets the info for the sif and dot files
writeNetwork<-function(modelOriginal, modelComprExpanded, optimResT1,
    optimResT2=NA, CNOlist, tag=NULL, verbose=FALSE)
{

    nwInfo<-getNetworkInfo(
        modelOriginal=modelOriginal,
        modelComprExpanded=modelComprExpanded,
        optimResT1=optimResT1,
        optimResT2=optimResT2,
        CNOlist=CNOlist,verbose=verbose)

    writeNetworkW(
        dN=nwInfo$dN,
        dM=nwInfo$dM,
        modelOriginal=modelOriginal,
        sifFile=nwInfo$sifFile,
        EAweights=nwInfo$EAweights,
        nodesAttr=nwInfo$nodesAttr,
        tag=tag)

    }


#########################################################################
##############these are the functions used above
#########################################################################
#this is the bit that writes
writeNetworkW<-function(dN,dM,modelOriginal,sifFile,EAweights,nodesAttr, tag=NULL){

 create_filename<-function(x, tag=NULL){
        if (is.null(tag)){
            return(x)
        }
        else{
            return(paste(c(tag, "_", x), collapse=""))
        }
    }


    writeDot(
        dotNodes=dN,
        dotMatrix=dM,
        model=modelOriginal,
        filename=create_filename("PKN.dot", tag=tag))

    write.table(
        sifFile[,1:3],
        file=create_filename("PKN.sif", tag=tag),
        row.names=FALSE,col.names=FALSE,quote=FALSE,sep="\t")

    write.table(
        EAweights,
        file=create_filename("TimesPKN.EA", tag=tag),
        row.names=FALSE,col.names="Times",quote=FALSE,sep="\t")

    write.table(
        nodesAttr,
        file=create_filename("nodesPKN.NA", tag=tag),
        row.names=FALSE,col.names="NodesType",quote=FALSE,sep="\t")

    }
############
#this is the bit that produces the info that is needed above
getNetworkInfo<-function(
    modelOriginal,
    modelComprExpanded,
    optimResT1,
    optimResT2,
    CNOlist,
    verbose){

    #These are used to create the string that holds the information about whether an edge is present
#or not (BStimes)
    bString1<-optimResT1$bString

    if(is.na(optimResT2[1])){
        bString2<-optimResT1$bString[which(optimResT1$bString == 0)]
        }else{
            bString2<-optimResT2$bString
            }

#BStimes is a string containing 0,1 and 2 depending on whether the interaction is
#absent, present at t1 or present at t2
    BStimes<-bString1
    BStimes[which(BStimes == 0)]<-bString2*2
#!!! BStimes relates to strings in the compressed and expanded model
    reacs<-modelComprExpanded$reacID
    reacsOriginal<-modelOriginal$reacID

#check if there are ANDs that were split, and if yes, set the score of the original AND
#to the max score of its resulting branches (i.e.the max of the branches, if one branch was present
#at t1 the AND gets 1, same if one was present at t2, if one of the branches was present at T1
#and another one at t2, then the AND gets a score of 3)

    if(names(modelComprExpanded$SplitANDs) != "initialReac"){

        for(i in 1:length(modelComprExpanded$SplitANDs)){

            map<-match(modelComprExpanded$SplitANDs[[i]],reacs)
            mapScore<-BStimes[map]
        #this forces the score to be 3 if a branch was present at t1 and another at t2
            if(any(mapScore == 1) && any(mapScore == 3)) mapScore[1]<-3
            mapScore<-max(mapScore)
            splitAnd<-names(modelComprExpanded$SplitANDs)[i]
            splitAnd<-which(reacs == splitAnd)
            BStimes[splitAnd]<-mapScore

            }

        }

#check if there were ands that were created, and if yes, assign to each branch of the original OR
#the max of the score of the and or of that particular branch

    if(length(modelComprExpanded$newANDs) != 0){

        for(i in 1:length(modelComprExpanded$newANDs)){
            branches<-modelComprExpanded$newANDs[i]
            branchesPos<-match(unlist(branches),reacs)
            BStimes[unlist(branches)]<-pmax(
                BStimes[match(unlist(branches),reacs)],
                BStimes[match(names(modelComprExpanded$newANDs)[i],reacs)])
            }

        }

#The new BStimes vector now contain weights that have been adapted to propagate weights coming
#from new ANDs or split ANDs
#Now create an adjacency list for the compressed/expanded model: col1=in node, col2=out node, col3=weight

findOutput<-function(x){
        sp<-which(x == 1)
        sp<-modelComprExpanded$namesSpecies[sp]
        }

findInput<-function(x){
        sp<-which(x == -1)
        sp<-modelComprExpanded$namesSpecies[sp]
        }

n<-1
adj<-matrix(NA,nrow=(length(reacs)*3),ncol=3)

for(i in 1:length(reacs)){

    input<-findInput(modelComprExpanded$interMat[,i])
    output<-findOutput(modelComprExpanded$interMat[,i])
    weight<-BStimes[i]

    for(j in 1:length(input)){
        adj[n,1]<-input[j]
        adj[n,2]<-output
        adj[n,3]<-weight
        n<-n+1
        }

    }

adj<-adj[-which(is.na(adj[,1])),]

#remove duplicates that arise from new ANDs and split ANDs
adj<-cbind(
    adj[order(adj[,3],decreasing=TRUE),1],
    adj[order(adj[,3],decreasing=TRUE),2],
    adj[order(adj[,3],decreasing=TRUE),3])
adj<-unique(adj)

for(i in 1:dim(unique(adj[,1:2]))[1]){

    findMatch<-apply(adj,1,function(x) all(x[1:2] == adj[i,1:2]))

    if(sum(findMatch) > 1){
        findMatch[which(findMatch == TRUE)[1]]<-FALSE
        adj<-adj[-which(findMatch==TRUE),]
        }

    }

#Now do the same for the original model: col1=in node, col2=out node, col3=reaction #

findOutput<-function(x){
        sp<-which(x == 1)
        sp<-modelOriginal$namesSpecies[sp]
        }

findInput<-function(x){
        sp<-which(x == -1)
        sp<-modelOriginal$namesSpecies[sp]
        }

n<-1
adjOrig<-matrix(NA,nrow=(length(reacsOriginal)*3),ncol=3)

for(i in 1:length(reacsOriginal)){

    input<-findInput(modelOriginal$interMat[,i])
    output<-findOutput(modelOriginal$interMat[,i])

    for(j in 1:length(input)){
        adjOrig[n,1]<-input[j]
        adjOrig[n,2]<-output
        adjOrig[n,3]<-i
        n<-n+1
        }

    }

adjOrig<-adjOrig[-which(is.na(adjOrig[,1])),]

#Go through the adjacency list of the compressed model and find which path is the shortest
#between those species
#this will hold the max weight of anything that has been mapped to this row of adjOrig so far
OrigMap<-rep(0,dim(adjOrig)[1])

for(i in 1:dim(adj)[1]){
#If the row of the compressed adjacency list matches a row of the original adjacency list,
#we just copy the weights
    rowAdj<-adj[i,1:2]
    matchR<-apply(adjOrig[,1:2],1,function(x) all(x == rowAdj))

    if(sum(matchR) > 0){

        OrigMap[which(matchR)]<-max(OrigMap[which(matchR)],adj[i,3])

        }else{

#Otherwise we need to find the shortest path between the nodes in that row of adj, within the rows
#of adjOrig, and map the weights of the row of adj to each of the row of adjOrig that are involved
#in that path
#This only works for paths of length 2 at most: improve this!!!!!
            ins<-which(adjOrig[,1] == adj[i,1])
            outN<-adjOrig[ins,2]
            target<-FALSE
            n<-1

            while(n <= length(outN) && !target){
                outs<-which(adjOrig[,1] == outN[n])
                outN2<-adjOrig[outs,2]

                if(any(outN2 == adj[i,2])){

                    rOut<-which(outN2 == adj[i,2])
                    rOut<-outs[rOut]
                    rIn<-ins[n]
                    n<-n+1
                    target<-TRUE

                    }else{

                        n<-n+1
                        #this sets those variables to NA if the path couldn't be mapped
                        rIn<-NA
                        rOut<-NA

                        }

                }
            #if the path could be mapped, we record the mapping
            if(!is.na(rIn) && !is.na(rOut)){

                OrigMap[rIn]<-max(OrigMap[rIn],adj[i,3])
                OrigMap[rOut]<-max(OrigMap[rOut],adj[i,3])
            #if the path couldn't be mapped (path too long), we print a warning, nad OrigMap
            #stays 0
                }else{

                    if(verbose == TRUE){
                        print("Please be aware that when mapping the scaffold network back to the PKN, compressed paths of length > 2 are ignored.")
                        }

                    }

            }
    }
    BStimesOrig<-cbind(adjOrig[,3],OrigMap)

    for(i in 1:dim(BStimesOrig)[1]){
        BStimesOrig[i,2]<-max(BStimesOrig[i,2],BStimesOrig[which(BStimesOrig[,2] == BStimesOrig[i,1]),2])
        }

    BStimesOrig<-unique(BStimesOrig)
    BStimesOrig<-BStimesOrig[,2]

#These mini functions are used to find the outputs and inputs of a reaction

    findOutput<-function(x){
        sp<-which(x == 1)
        sp<-modelOriginal$namesSpecies[sp]
        }

    reacOutput<-apply(modelOriginal$interMat,2,findOutput)

    findInput<-function(x){
        sp<-which(x == -1)
        sp<-modelOriginal$namesSpecies[sp]
        }

    reacInput<-apply(modelOriginal$interMat,2,findInput)

#This mini function is used to create a reaction label as used in a cystoscape edge attribute file

    createReac<-function(x){
        r<-paste(x[1]," (",x[2],") ",x[3],sep="")
        return(r)
        }

#if the class of reacInput is not a list, then there are no AND gates

    if(!is(reacInput,"list")){

        isNeg<-function(x){
            isNegI<-any(x == 1)
            return(isNegI)
            }

        inpSign<-apply(modelOriginal$notMat,2,isNeg)
        inpSign<-!inpSign
        inpSign[inpSign]<-1
        inpSign[!inpSign]<--1
        sifFile<-cbind(reacInput,inpSign,reacOutput)
        EApresent<-apply(sifFile,1,createReac)
        EAweights<-cbind(EApresent, BStimesOrig)

        }else{

#in this case there are AND gates and so we need to create dummy "and#" nodes
            sifFile<-matrix(0,nrow=200,ncol=4)
            nR<-1
            nANDs<-1

            for(i in 1:length(reacOutput)){

                if(length(reacInput[[i]]) == 1){

                    sifFile[nR,1]<-reacInput[[i]]
                    sifFile[nR,3]<-reacOutput[i]
                    sifFile[nR,2]<-ifelse(any(modelOriginal$notMat[,i] == 1),-1,1)
                    sifFile[nR,4]<-BStimesOrig[i]
                    nR<-nR+1

                    }else{

                        for(inp in 1:length(reacInput[[i]])){

                            sifFile[nR,1]<-reacInput[[i]][inp]
                            sifFile[nR,3]<-paste("and",nANDs,sep="")
                            sifFile[nR,2]<-ifelse(modelOriginal$notMat[inp,i] == 1,-1,1)
                            sifFile[nR,4]<-BStimesOrig[i]
                            nR<-nR+1

                            }

                        sifFile[nR,1]<-paste("and",nANDs,sep="")
                        sifFile[nR,3]<-reacOutput[i]
                        sifFile[nR,2]<-1
                        sifFile[nR,4]<-BStimesOrig[i]
                        nANDs<-nANDs+1
                        nR<-nR+1

                        }
                }

            sifFile<-sifFile[1:(nR-1),]
            EApresent<-apply(sifFile[,1:3],1,createReac)
            EAweights<-cbind(EApresent,sifFile[,4])

            }
    #this bit creates a matrix of reaction that will be used for the dot file
    dM<-cbind(sifFile,BStimesOrig)

    #this mini function makes edge attributes in the format e1 (sign) e2 = attr

    makeEA<-function(x){
        ea<-paste(x[1],"=",x[2])
        return(ea)
        }

    EAweights<-apply(EAweights,1,makeEA)

#write the nodes attributes
    nodesCompr<-modelComprExpanded$speciesCompressed
    indexes<-indexFinder(CNOlist,modelOriginal)
    nodesSig<-modelOriginal$namesSpecies[indexes$signals]
    nodesInh<-modelOriginal$namesSpecies[indexes$inhibited]
    nodesStim<-modelOriginal$namesSpecies[indexes$stimulated]
    nodesNCNO<-findNONC(modelOriginal,indexes)
    nodesAttrNames<-nodesSig
    nodesAttr<-rep("signal",length(nodesSig))

    if(length(nodesInh) != 0){
        nodesAttrNames<-c(nodesAttrNames,nodesInh)
        nodesAttr<-c(nodesAttr,rep("inhibited",length(nodesInh)))
        }

    if(length(nodesStim) != 0){
        nodesAttrNames<-c(nodesAttrNames,nodesStim)
        nodesAttr<-c(nodesAttr,rep("stimulated",length(nodesStim)))
        }

    if(length(nodesNCNO) != 0){
        nodesAttrNames<-c(nodesAttrNames,nodesNCNO)
        nodesAttr<-c(nodesAttr,rep("ncno",length(nodesNCNO)))
        }

    if(length(nodesCompr) != 0){
        nodesAttrNames<-c(nodesAttrNames,nodesCompr)
        nodesAttr<-c(nodesAttr,rep("compressed",length(nodesCompr)))
        }

    nodesAttr<-cbind(nodesAttrNames,nodesAttr)

#this bit creates a matrix of nodes attributes that will be used for the dot file
    dN<-nodesAttr

#this is the node attributes in the format as will be used by cytoscape
    nodesAttr<-apply(nodesAttr,1,makeEA)

#nodes that are neither inh/stim/sig nor NONC will not be in thi snode attribute so I'll add them
    other<-setdiff(modelOriginal$namesSpecies,dN[,1])
    other<-paste(other," = NA",sep="")
    nodesAttr<-c(nodesAttr, other)

#this is the edge attribute matrix that contains, for each edge, whether it is
#absent from the model (0), present at t1(1) or present at t2(2)
    return(list(
        dN=dN,
        dM=dM,
        sifFile=sifFile,
        nodesAttr=nodesAttr,
        EAweights=EAweights))
    }

#
#  This file is part of the CNO software
#
#  Copyright (c) 2011-2012 - EMBL - European Bioinformatics Institute
#
#  File author(s): CNO developers (cno-dev@ebi.ac.uk)
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  CNO website: http://www.cellnopt.org
#
##############################################################################
# $Id$
model2sif<-function(model,optimRes=NA,writeSif=FALSE, filename="Model"){
	
  if (is.na(optimRes[1])){
    BStimes<-rep(1,length(model$reacID))
  }else{
    BStimes<-optimRes$bString
  }	

	findOutput<-function(x){
		sp<-which(x == 1)
		sp<-model$namesSpecies[sp]
		}
		
	reacOutput<-apply(model$interMat,2,findOutput)
	
	findInput<-function(x){
		sp<-which(x == -1)
		sp<-model$namesSpecies[sp]
		}
		
	reacInput<-apply(model$interMat,2,findInput)
		
#if the class of reacInput is not a list, then there are no AND gates
	if(!is(reacInput,"list")){
	
		isNeg<-function(x){
			isNegI<-any(x == 1)
			return(isNegI)
			}
			
		inpSign<-apply(model$notMat,2,isNeg)
		inpSign<-!inpSign
		inpSign[inpSign]<-1
		inpSign[!inpSign]<- -1
    
		sifFile<-cbind(reacInput,inpSign,reacOutput)
    sifFile<-sifFile[BStimes==1,]
		colnames(sifFile)=NULL
    rownames(sifFile)=NULL
    
		}else{
		
#in this case there are AND gates and so we need to create dummy "and#" nodes
			sifFile<-matrix(ncol=3)
			nANDs<-1
			for(i in 1:length(reacOutput)){
			  if (BStimes[i]==1){

				  if(length(reacInput[[i]]) == 1){
            tmp<-matrix(0,nrow=1,ncol=3)
					  tmp[1,1]<-reacInput[[i]]
					  tmp[1,3]<-reacOutput[[i]]
					  tmp[1,2]<-ifelse(
						  any(model$notMat[,i] == 1),-1,1)
              sifFile<-rbind(sifFile,tmp)
          
					}else{
					
						for(inp in 1:length(reacInput[[i]])){
              tmp<-matrix(0,nrow=1,ncol=3)
							tmp[1,1]<-reacInput[[i]][inp]
							tmp[1,3]<-paste("and",nANDs,sep="")
							tmp[1,2]<-ifelse(
								model$notMat[which(reacInput[[i]][inp]==rownames(model$notMat)),i] == 1,
								-1,1)
              sifFile<-rbind(sifFile,tmp)
							}
						tmp<-matrix(0,nrow=1,ncol=3)	
						tmp[1,1]<-paste("and",nANDs,sep="")
						tmp[1,3]<-reacOutput[[i]]
						tmp[1,2]<-1
            sifFile<-rbind(sifFile,tmp)
						
            nANDs<-nANDs+1
            }
			    }
				}
				
      sifFile<-sifFile[2:dim(sifFile)[1],]
			
			}

#this is the edge attribute matrix that contains, for each edge, whether it is
#absent from the model (0), present at t1(1) or present at t2(2)
  if (writeSif==TRUE){
	filename<-paste(filename, ".sif", sep="")
    writeSIF(sifFile, filename=filename)
  }

  return(sifFile)
}

#  Copyright (c) 2019 - SaezLab - Heidelberg Universit
#
#  File author(s): CNO developers
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  CNO website: http://www.cellnopt.org
#
##############################################################################

write_bounds <- function(model,
                         midasTreatmentPart,
                         y_vector,
                         binary_variables){
  
  # writing of the boundaries
  y_bounds_vector = writeBoundsForEdges_y_i(y_vector)
  linkerVector = link_midas_colnames_to_namesSpecies(model, midasTreatmentPart)
  bounds = c(paste0("0<= xb",1:(length(binary_variables[[1]])), " <= 1"))
  treatmentMatrix <- midasTreatmentPart
  for(i in 1:dim(midasTreatmentPart)[1]){
    for(j in 1:dim(midasTreatmentPart)[2]){
      if(midasTreatmentPart[i,j] == 1 && 
         !(str_sub(colnames(midasTreatmentPart)[j], -1) == "i")){
        bounds[length(y_vector) + 
                 (i-1)*length(model$namesSpecies)+
                 linkerVector[j]] =
          paste0("xb", length(y_vector) + 
                   (i-1)*length(model$namesSpecies)+linkerVector[j], " = 1")
      } else if(midasTreatmentPart[i,j] == 0 && 
                !(str_sub(colnames(midasTreatmentPart)[j], -1) == "i")){
        bounds[length(y_vector) + 
                 (i-1)*length(model$namesSpecies)+
                 linkerVector[j]] = 
          paste0("xb", length(y_vector) + 
                   (i-1)*length(model$namesSpecies)+linkerVector[j], " = 0")
        
      }
      else if(midasTreatmentPart[i,j] == 1 && 
              str_sub(colnames(midasTreatmentPart)[j], -1) == "i"){
        bounds[length(y_vector) + 
                 (i-1)*length(model$namesSpecies)+
                 linkerVector[j]] = 
          paste0("xb", length(y_vector) + 
                   (i-1)*length(model$namesSpecies)+linkerVector[j], " = 0")
      } else{
        ## nothing. stays at it was.
      }
    }
  }
  
  return(bounds)
}

#  Copyright (c) 2019 - SaezLab - Heidelberg Universit
#
#  File author(s): CNO developers
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  CNO website: http://www.cellnopt.org
#
##############################################################################

createILPBitstringAll<- function(cplexSolutionFileName,
                                 y_vector,
                                 binary_variables){
  
  #
  bitStringAll <- list()
  
  #
  readOutBinariesAll<- function(cplexSolutionFileName, binary_variables){
    
    cplexSolutionData <- xmlParse(cplexSolutionFileName)
    cplexSolution <- xmlToList(cplexSolutionData, simplify = TRUE)
    binariesAll <- list()
    for(i in 1:length(cplexSolution)){
      
      if(all(names(cplexSolution[[i]])=="variable")){
        
        bitstring <- c()
        binaries <- c()
        for(j in 1:length(cplexSolution[[i]])){
          
          binaries[which(binary_variables[[1]]==cplexSolution[[i]][[j]][1])] = 
            cplexSolution[[i]][[j]][3]
          
        }
        
        binariesAll[[length(binariesAll)+1]] <- binaries
        
      }
      
    }
    
    return(binariesAll)
    
  }
  
  #
  binaries <- readOutBinariesAll(cplexSolutionFileName, binary_variables)
  for(i in 1:length(binaries)){
    bitString <- c()
    bitString <- binaries[[i]][1:length(y_vector)]
    bitStringAll[[length(bitStringAll)+1]] <- bitString
  }
  
  #
  bitStringILPAll <- bitStringAll
  for(i in 1:length(bitStringILPAll)){
    
    bitStringILPAll[[i]] <- round(as.numeric(bitStringILPAll[[i]]))
    
  }
  bitStringILPAll <- unique(bitStringILPAll)
  idx <- c()
  for(i in 1:length(bitStringILPAll)){
    
    if(sum(as.numeric(bitStringILPAll[[i]]))==0){
      
      idx <- c(idx, i)
      
    }
    
  }
  if(length(idx) > 0){
    
    bitStringILPAll <- bitStringILPAll[-idx]
    
  }
  
  return(bitStringILPAll)
  
}

#
#  This file is part of the CNO software
#
#  Copyright (c) 2011-2012 - EMBL - European Bioinformatics Institute
#
#  File author(s): CNO developers (cno-dev@ebi.ac.uk)
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  CNO website: http://www.cellnopt.org
#
##############################################################################
# $Id: CNORwrap.R 3155 2013-01-09 15:24:58Z cokelaer $
#This function is a wrapper around the whole CNOR analysis, it performs the following steps:
#1.Plot the CNOlist
#2.Checks data to model compatibility
#3.Cut the nonc off the model
#4.Compress the model
#5.Expand the gates
#6.Compute the residual error
#7.Prepare for simulation
#8.Optimisation t1
#9.Plot simulated and experimental results
#10.Plot the evolution of fit
#11.Optimise t2 (not implemented in this version)
#12.Write the scaffold and PKN
#13.Write the report

#' @export
CNORwrap<-function(paramsList=NA, data=NA, model=NA, name, namesData=NA, time=1,
compression=TRUE, expansion=TRUE, cutNONC=TRUE)
{

    # aliases
    Name = name
    NamesData = namesData
    Time = time

    # if paramsList empty, we will fill it with
    # default values and the Data and model provided
    if(is.na(paramsList[1])==TRUE){
        #print("paramList not provided")
        # Data must be provided
        # is.na raise warning, so let us use is.list
        if (is.na(data[1])==TRUE){
            stop("if paramsList not provided, Data must be provided")
        }
        # model must be provided
        if (is.na(model[1])==TRUE){
            stop("if paramsList not provided, model must be provided")
        }
        paramsList = defaultParameters(data, model)
    

    # if paramsList is provided, and model and Data are NA, then we must find
    # Data and model in paramsList
    }else if (is.na(paramsList[1])==FALSE){
        #print("paramsList  provided")

        if (is.na(data[1])==TRUE && is.na(paramsList$data[1])==TRUE){
            stop("Data must be provided either in paramsList or Data argument")
        }
        if (is.na(model[1])==TRUE && is.na(paramsList$model[1])==TRUE){
            stop("model must be provided either in paramsList or model argument")
        }

        # Data must be provided
        if (is.na(data[1])==FALSE){
            #print("Data provided")
            if (is.na(paramsList$data[1])==FALSE){
                warning("overwritting paramsList$data with provided Data")
            }
            paramsList$data = data
        }
        # model must be provided
        if (is.na(model[1])==FALSE){
            #print("model provided")
            if (is.na(paramsList$model[1])==FALSE){
                warning("overwritting paramsList$Model with provided Model")
            }
            paramsList$model = model
        }

    }

    if (is.list(NamesData)==FALSE){
        if (is.na(NamesData)==TRUE){
            NamesData <- list(
                CNOlist=paste(Name, "Data",sep=""),
               model=paste(Name,"Model", sep=""))
        }
        else{
            stop("NamesData must be a list or kept to the default values (NA)")
        }
    }


    #1.Plot the CNOlist
    plotCNOlist(paramsList$data)
    plotCNOlistPDF(
        CNOlist=paramsList$data,
        filename=paste(Name,"DataPlot.pdf",sep="")
        )

    #2. Checks data to model compatibility
    checkSignals(CNOlist=paramsList$data,model=paramsList$model)

    #3.Cut the nonc off the model
    #4.Compress the model
    #5.Expand the gates
    newmodel = preprocessing(paramsList$data, paramsList$model,
        compression=compression, expansion=expansion, cutNONC=cutNONC)
    NCNOcutCompExp <- newmodel

    #6.Compute the residual error

    #7.Prepare for simulation

    #8.Optimisation t1
    initBstring<-rep(1,length(NCNOcutCompExp$reacID))
    T1opt<-gaBinaryT1(CNOlist=paramsList$data,
        model=NCNOcutCompExp,
        initBstring=initBstring,
        sizeFac=paramsList$sizeFac,
        NAFac=paramsList$NAFac,
        popSize=paramsList$popSize,
        pMutation=paramsList$pMutation,
        maxTime=paramsList$maxTime,
        maxGens=paramsList$maxGens,
        stallGenMax=paramsList$stallGenMax,
        selPress=paramsList$selPress,
        elitism=paramsList$elitism,
        relTol=paramsList$relTol,
        verbose=paramsList$verbose)

    #9.Plot simulated and experimental results
    cutAndPlot(
        model=NCNOcutCompExp,
        bStrings=list(T1opt$bString),
        CNOlist=paramsList$data,
        plotPDF=TRUE)


    #10.Plot the evolution of fit
    pdf(paste(Name,"evolFitT1.pdf",sep=""))
    plotFit(optRes=T1opt)
    dev.off()
    plotFit(optRes=T1opt)

    #11.Optimise t2
    if(Time==2){

        T2opt<-gaBinaryTN(
            CNOlist=paramsList$data,
            model=NCNOcutCompExp,
            bStrings=list(T1opt$bString),
            sizeFac=paramsList$sizeFac,
            NAFac=paramsList$NAFac,
            popSize=paramsList$popSize,
            pMutation=paramsList$pMutation,
            maxTime=paramsList$maxTime,
            maxGens=paramsList$maxGens,
            stallGenMax=paramsList$stallGenMax,
            selPress=paramsList$selPress,
            elitism=paramsList$elitism,
            relTol=paramsList$relTol,
            verbose=paramsList$verbose)

        cutAndPlot(model=NCNOcutCompExp,CNOlist=paramsList$data,
            bStrings=list(T1opt$bString, bStringT2=T2opt$bString), plotPDF=TRUE)

        pdf(paste(Name,"evolFitT2.pdf",sep=""))
        plotFit(optRes=T2opt)
        dev.off()
        plotFit(optRes=T2opt)

        }else{
        T2opt<-NA
    }
    #13.Write the scaffold and PKN
    #and
    #14.Write the report
    writeScaffold(
        modelComprExpanded=NCNOcutCompExp,
        optimResT1=T1opt,
        optimResT2=T2opt,
        modelOriginal=paramsList$model,
        CNOlist=paramsList$data)
    writeNetwork(
        modelOriginal=paramsList$model,
        modelComprExpanded=NCNOcutCompExp,
        optimResT1=T1opt,
        optimResT2=T2opt,
        CNOlist=paramsList$data)

    if(Time==2){

        namesfiles<-list(
            dataPlot=paste(Name,"DataPlot.pdf",sep=""),
            evolFitT1=paste(Name,"evolFitT1.pdf",sep=""),
            evolFitT2=paste(Name,"evolFitT2.pdf",sep=""),
            simResultsT2="SimResultsTN.pdf",
            simResultsT1="SimResultsT1_1.pdf",
            scaffold="Scaffold.sif",
            scaffoldDot="Scaffold.dot",
            tscaffold="TimesScaffold.EA",
            wscaffold="weightsScaffold.EA",
            PKN="PKN.sif",
            PKNdot="PKN.dot",
            wPKN="TimesPKN.EA",
            nPKN="nodesPKN.NA")

    }
    else{
        namesfiles<-list(
            dataPlot=paste(Name,"DataPlot.pdf",sep=""),
            evolFitT1=paste(Name,"evolFitT1.pdf",sep=""),
            evolFitT2=NA,
            simResultsT2=NA,
            simResultsT1="SimResultsT1_1.pdf",
            scaffold="Scaffold.sif",
            scaffoldDot="Scaffold.dot",
            tscaffold="TimesScaffold.EA",
            wscaffold="weightsScaffold.EA",
            PKN="PKN.sif",
            PKNdot="PKN.dot",
            wPKN="TimesPKN.EA",
            nPKN="nodesPKN.NA")
    }

    writeReport(
        modelOriginal=paramsList$model,
        modelOpt=NCNOcutCompExp,
        optimResT1=T1opt,
        optimResT2=T2opt,
        CNOlist=paramsList$data,
        directory=Name,
        namesFiles=namesfiles,
        namesData=NamesData)

}

#
#  This file is part of the CNO software
#
#  Copyright (c) 2011-2012 - EMBL - European Bioinformatics Institute
#
#  File author(s): CNO developers (cno-dev@ebi.ac.uk)
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  CNO website: http://www.cellnopt.org
#
##############################################################################
# $Id$

cutAndPlot <- function(CNOlist, model, bStrings, plotPDF=FALSE, tag=NULL,
 plotParams=list(maxrow=10))
{


     if (!is(CNOlist,"CNOlist")){
        CNOlist = CellNOptR::CNOlist(CNOlist)
    } 



    # bitStrings must be a list of bitString (T1, T2, ...TN)
    tPt = length(bStrings)+1

    simList <- prep4sim(model)
    indexList <- indexFinder(CNOlist=CNOlist,model=model)

    

    # if tPt nothing to plot
    if (tPt==1){
        stop("noting to do with time data at time 0")
    }

    # if tPt=2 (default), call cutAndPlotResultsT1
    if (tPt == 2){
       #print("Entering cutAndPlotResultsT1")

       outputs = cutAndPlotResultsT1(model=model, bString=bStrings[[1]], simList=simList, 
            CNOlist=CNOlist, indexList=indexList, plotPDF=plotPDF, tag=tag,
            plotParams=plotParams)
    }

    if (tPt>=3){
       #print("Entering cutAndPlotResultsTN")
       outputs = cutAndPlotResultsTN(
         CNOlist=CNOlist,
         model=model,
         bStrings=bStrings,
         plotPDF=plotPDF,
         tag=tag) 
    }
    # if tPt=2 (default), call cutAndPlotResultsT2
    return(outputs)
}

#  Copyright (c) 2019 - SaezLab - Heidelberg Universit
#
#  File author(s): CNO developers
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  CNO website: http://www.cellnopt.org
#
##############################################################################

createAndRunILP <- function(model = model, 
                            midas = midas, 
                            cnolist = cnolist, 
                            accountForModelSize = accountForModelSize, 
                            sizeFac = sizeFac, 
                            source_path = source_path, 
                            mipGap = mipGap, 
                            relGap = relGap, 
                            timelimit = timelimit,
                            cplexPath = cplexPath, 
                            method = method, 
                            numSolutions = numSolutions, 
                            limitPop = limitPop, 
                            poolIntensity = poolIntensity, 
                            poolReplace = poolReplace){
  
  # Creating auxilliary variables
  time_start_writing <- Sys.time()
  md <- midas
  TimeOfEvaluation <- max(md$dataMatrix[,md$DAcol[1]])
  
  # Creating the vector of edge identifiers
  create_y_vector_from_preprocessed_model <- function(model){
    
    y_vector <- model$reacID
    return(y_vector)
    
  }
  
  ## Creating set of variables to inhibitory effects of each interactions
  create_inhibitor_set_for_y_i <- function(model, y_i){
    
    inhibitor_set <- c()
    inhibitor_set <- 
      names(which(model$notMat[,which(model$reacID == y_i)] =="1"))
    return(inhibitor_set)
    
  }
  
  # Creating set of variables to the reactants of each interactions
  create_reactant_set_for_y_i <- function (model, y_i, inhibitor_set){
    
    reactant_set <- c()
    reactant_set <- 
      names(which(model$interMat[,which(model$reacID == y_i)] == "-1"))
    remove <- inhibitor_set
    reactant_set <- reactant_set[!reactant_set %in% remove]
    return(reactant_set)
    
  }
  
  ## Creating set of variables to products of each interactions
  create_product_set_for_y_i <- function(model, y_i){
    
    product_set <- c()
    product_set <- 
      names(which(model$interMat[,which(model$reacID == y_i)] == "1"))
    return(product_set)
    
  }
  
  ## Combining all interaction set of variables
  create_reaction_for_y_i <- function(model, y_i){
    
    inhibitor_set <- create_inhibitor_set_for_y_i(model, y_i) 
    reactant_set <- create_reactant_set_for_y_i(model, y_i, inhibitor_set)
    product_set <- create_product_set_for_y_i(model, y_i)
    reaction_set <- list(reactant_set, inhibitor_set, product_set)
    return(reaction_set)
    
  }
  
  ## Creating all the sets
  create_all_sets <- function(model){
    
    y_vector <- create_y_vector_from_preprocessed_model(model)
    all_reaction_sets <- 
      data.frame(reac_name = c("reactant_set", "inhibitor_set", "product_set"))
    for(i in 1:length(y_vector)){
      y_i <- y_vector[i]
      reaction_for_y_i <- create_reaction_for_y_i(model, y_i)
      all_reaction_sets$y_1 <- reaction_for_y_i
      colnames(all_reaction_sets)[i+1] <- y_vector[i]
    }
    all_reaction_sets[[1]] <- NULL
    return(all_reaction_sets)
    
  }
  
  ## Extracting only the measurements from the MIDAS file
  extractMeasurementPartFromMidas <- function(midas, TimeOfEvaluation){
    
    md <- midas$dataMatrix
    if (length(midas$DAcol)>1) {
      md_out <- md[which(md[midas$DAcol[1]] ==TimeOfEvaluation),midas$DVcol]
    } else { 
      md_out<-matrix(NA,length(which(md[midas$DAcol[1]] ==TimeOfEvaluation)),1);
      md_out[,1]<-md[which(md[midas$DAcol[1]] ==TimeOfEvaluation),midas$DVcol];
      colnames(md_out) <- colnames(md)[midas$DAcol]
    }
    return(md_out)
    
  }
  
  # Extracting only the treatment part from the MIDAS file
  extractTreatmentPartFromMidas <- function(midas, TimeOfEvaluation){
    
    md <- midas$dataMatrix
    md_out <- md[which(md[midas$DAcol[1]] ==TimeOfEvaluation),midas$TRcol]
    return(md_out)
    
  }
  
  # Means of measurements
  meansOfMeasurements <- function(midasExperimentPart_at_t0, 
                                  midasExperimentPart, midasTreatmentPart){
    
    ValueMatrix <- midasExperimentPart_at_t0
    if (!is.null(dim(ValueMatrix))) {
      if(all(midasTreatmentPart[1,]==0)){
        ValueMatrix <- rbind(ValueMatrix, midasExperimentPart[1,])
      }
      out <- colMeans(ValueMatrix, na.rm = TRUE)
    } else {
      out <- ValueMatrix
    }
    return(out)
    
  }
  
  # Creating and running the ILP problem
  reaction_sets <- create_all_sets(model)
  midasExperimentPart <- extractMeasurementPartFromMidas(md, TimeOfEvaluation)
  midasExperimentPart_at_t0 <- extractMeasurementPartFromMidas(md,0)
  midasTreatmentPart <- extractTreatmentPartFromMidas(md, TimeOfEvaluation)
  meansOfMeasurements_at_t0 <- meansOfMeasurements(midasExperimentPart_at_t0, 
                                                  midasExperimentPart, 
                                                  midasTreatmentPart)
  numberOfExperiments <- if(!is.null(dim(midasExperimentPart)[1])){
    dim(midasExperimentPart)[1]}else{1}
  y_vector <- create_y_vector_from_preprocessed_model(model)
  binary_variables <- create_binaries(model, md, numberOfExperiments, y_vector)
  objectiveFunction <- writeObjectiveFunction(model, midasExperimentPart, 
                                             y_vector = y_vector, 
                                             accountForModelSize, sizeFac, 
                                             meansOfMeasurements_at_t0, 
                                             method = method)
  constraints <- write_constraints(model, midasExperimentPart, 
                                  midasTreatmentPart, reaction_sets, 
                                  y_vector, md)
  bounds <- write_bounds(model, midasTreatmentPart,y_vector, binary_variables)
  writeFile(objectiveFunction, constraints, bounds, binary_variables)
  time_cplex_start <- Sys.time()
  invokeCPLEX(inputFileName = "testFile.lp", 
              outputFileName = "cplexSolution.txt", mipGap=mipGap, 
              relGap = relGap, timelimit=timelimit, cplexPath=cplexPath,
              numSolutions=numSolutions, limitPop = limitPop, 
              poolIntensity = poolIntensity, poolReplace = poolReplace)
  time_cplex_end <- Sys.time()
  bitstringILPAll <- createILPBitstringAll("cplexSolution.txt", y_vector, 
                                          binary_variables)
  
  time_end_writing <- Sys.time()
  total_time <- difftime(time_end_writing, time_start_writing, units = "secs")
  
  time_cplex_only <- difftime(time_cplex_end, time_cplex_start, units = "secs")
  bScore <- c()
  for(ii in 1:length(bitstringILPAll)){
    bScore <- computeScoreT1(CNOlist = cnolist, model = model, 
                            bString = bitstringILPAll[[ii]])
  }
  
  return(list(bitstringILP = bitstringILPAll, bScore = bScore, 
              time_cplex_only = time_cplex_only, total_time = total_time))
  
}

#
#  This file is part of the CNO software
#
#  Copyright (c) 2019 - SaezLab - Heidelberg Universit
#
#  File author(s): CNO developers
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  CNO website: http://www.cellnopt.org
#
##############################################################################

create_binaries <- function(model, 
                            midas, 
                            numberOfExperiments, 
                            y_vector){
  
  ## creating auxilliary variables
  numbers <- c()
  variables <- c()
  identifiers <- c()
  
  ## Creating binary variables for the edges
  y_binaries <- create_binary_variables_for_y_vector(y_vector)
  numbers <- y_binaries[[1]]
  variables <- y_binaries[[2]]
  identifiers <- y_binaries[[3]]
  
  ## creating binary variables for the species
  for(i in 1:numberOfExperiments){
    binList <- 
      create_binary_variables_for_all_species_in_experiment_k_i(model, i)
    numbers= append(numbers, binList[[1]])
    variables <- append(variables, binList[[2]])
    identifiers <- append(identifiers, binList[[3]])
  }
  for(i in 1:numberOfExperiments){
    binList <- 
      create_binary_variables_for_all_reactions_z_i_in_experiment_k_i(model, 
                                                                      i)
    numbers= append(numbers, binList[[1]])
    variables <- append(variables, binList[[2]])
    identifiers <- append(identifiers, binList[[3]])
  }
  
  ## combining and returning all the binary variables
  binaries <- list(c(paste0("xb",1:length(variables))), variables, identifiers)
  return(binaries)
  
}

#
#  This file is part of the CNO software
#
#  Copyright (c) 2011-2012 - EMBL - European Bioinformatics Institute
#
#  File author(s): CNO developers (cno-dev@ebi.ac.uk)
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  CNO website: http://www.cellnopt.org
#
##############################################################################
expandGates<-function(model, ignoreList=NA,maxInputsPerGate=2){

    Model = model
    # check that Model is a Model list
    if(!is.list(Model)) stop("This function expects as input a Model as output by readSIF")
    if(length(Model) == 4) {
        if(all(names(Model) != c("reacID", "namesSpecies","interMat","notMat"))){
            stop("This function expects as input a Model as output by readSIF")
            }
        }
    if(length(Model) == 5) {
        if(all(names(Model) != c("reacID", "namesSpecies","interMat","notMat","speciesCompressed"))) {
            stop("This function expects as input a Model as output by readSIF")
        }
    }

    SplitANDs <- list(initialReac=c("split1","split2"))
    splitR <- 1

    # split all the ANDs
    # remove any ANDs 2/3 and save >3 to add later
    # +3 won't get added here again but are part of prior knowledge if contained within PKN
    andToAdd = c()
    remove.and = c()
    reacs2Ignore = c()
    initialReacN <- length(Model$reacID)

  #TODO: move into readSIF ?
  if (initialReacN == 1){
        Model$interMat <- as.matrix(Model$interMat)
   }

    # which reactions have ignoreList as output?
    if(!is.na(ignoreList[1])) {
        for(s in 1:initialReacN) {
            if(any(Model$interMat[ignoreList,s] == 1)) {
                reacs2Ignore = c(reacs2Ignore, s)
            }
        }
    }

    for(r in 1:initialReacN) {
        inNodes <- which(Model$interMat[,r] == -1)
        if(length(inNodes) > 1) {
            if(length(inNodes) > 3) {
                andToAdd = c(andToAdd, r)
            }
            remove.and = c(remove.and, r)

            if(!any(reacs2Ignore == r)) {
                outNode <- which(Model$interMat[,r] == 1)
                newReacs <- matrix(data=0,nrow=dim(Model$interMat)[1],ncol=length(inNodes))
                newReacsNots <- matrix(data=0,nrow=dim(Model$interMat)[1],ncol=length(inNodes))
                newReacs[outNode,] <- 1
                newReacsIDs <- rep("a",length(inNodes))
                for(i in 1:length(inNodes)) {
                    newReacs[inNodes[i],i] <- -1
                    newReacsNots[inNodes[i],i] <- Model$notMat[inNodes[i],r]
                    newReacsIDs[i] <- paste(Model$namesSpecies[inNodes[i]],"=",Model$namesSpecies[outNode],sep="")
                    if(Model$notMat[inNodes[i],r] == 1) newReacsIDs[i] <- paste("!",newReacsIDs[i],sep="")
                }
                colnames(newReacs) <- newReacsIDs
                colnames(newReacsNots) <- newReacsIDs
                SplitANDs[[splitR]] <- newReacsIDs
                names(SplitANDs)[splitR] <- Model$reacID[r]
                splitR <- splitR+1
                Model$notMat <- cbind(Model$notMat,newReacsNots)
                Model$interMat <- cbind(Model$interMat,newReacs)
                Model$reacID <- c(Model$reacID,newReacsIDs)
            }
        }
    }

    # ***** ADDED *****
    # remove 'AND' gates that will be made anyway
    # save anything > 3 to add at end

    if(length(andToAdd)) {
        toAdd = list()
        toAdd$notMat <- Model$notMat[,andToAdd]
        toAdd$interMat <- Model$interMat[,andToAdd]
        toAdd$reacID <- Model$reacID[andToAdd]
    } else {
        toAdd <- NA
    }

    if(length(remove.and)) {
        Model$notMat <- Model$notMat[,-remove.and]
        Model$interMat <- Model$interMat[,-remove.and]
        Model$reacID <- Model$reacID[-remove.and]
    }

    # ***** ADDED *****

    # the list SplitANDs now contains a named element for each AND reac that has been split,
    # and each element contains a vector with the names of the reactions that result from the split
    # create combinations of ORs
    # the newANDs list will contain an element for each new '&' gate, named by the name of this new and
    # reac, and containing a vector of the names of the reactions from which it was created

    newANDs <- list(finalReac=c("or1","or2"))
    ANDsadded <- 1
    total.list = 1:length(Model$namesSpecies)


    # functions to get lhs and rhs of reactions
    getlhs <- function(x) {
        spec1 = strsplit(x, "=")[[1]][1]
    }
    getrhs <- function(x) {
        spec2 = strsplit(x, "=")[[1]][2]
    }

    # scan all species and build and gates if required
    for(sp in total.list) {
        inReacsIndex <- which(Model$interMat[sp,] == 1)
        if(length(inReacsIndex) > 1) {
            inReacs <- Model$interMat[,inReacsIndex]
            findInput <- function(x) {
                inNode<-which(x == -1)
            }
            inSp <- apply(inReacs,2,findInput)

            # let
            # find the input species first and store in a vector
            inSpecies = apply(as.matrix(colnames(inReacs)), 1, getlhs)
            outname = Model$namesSpecies[sp]

            # just for sanity check, all outputs must be the same
            outnames = apply(as.matrix(colnames(inReacs)), 1, getrhs)
            if (length(unique(outnames))!=1 | outname!=outnames[1]){
                stop("error in expandGates. should not happen here.please report")
            }

            # an alias
            myrownames = rownames(Model$interMat)

            # first the 2 inputs cases

            combinations = combn(seq(1,length(inSpecies)), 2)

            for (this in seq(1, dim(combinations)[2])){
                i = combinations[1,this]
                j = combinations[2,this]
                # names[i] and names[j] contains possibly the !
                # sign,let us get the real species names
                realname1 = ifelse(substr(inSpecies[i], 1,1)=="!",substr(inSpecies[i],2,10000), inSpecies[i])
                realname2 = ifelse(substr(inSpecies[j], 1,1) =="!",substr(inSpecies[j],2,10000), inSpecies[j])

                realnames = c(realname1,realname2)
                if (any(combn(realnames,2)[1,] == combn(realnames,2)[2,])){  # exclude reaction if any name are indentical
                    next()
                }

                # create the new reaction Id to be used as a column name
                newcolname = paste(paste(inSpecies[i], inSpecies[j],sep="+"),outname, sep="=")
                if (newcolname %in% colnames(Model$interMat)){
                    next() # skip if exist already
                }
                Model$reacID <- c(Model$reacID,newcolname)

                # fill the interMat (-1 if in lhs, 1 if in rhs)
                values = as.matrix(rep(0, length(Model$namesSpecies)))
                colnames(values)<-newcolname
                values[which(myrownames == realname1)]<- -1
                values[which(myrownames == realname2)]<- -1
                values[which(myrownames == outname)]<- 1
                Model$interMat= cbind(Model$interMat, values)

                # now, the notMat, 0 by default
                values = as.matrix(rep(0, length(Model$namesSpecies)))
                colnames(values)<-newcolname
                if (substr(inSpecies[i],1,1) == "!"){ #look first specy
                    values[which(myrownames == realname1)]<- 1
                }
                if (substr(inSpecies[j],1,1) == "!"){# and second one
                    values[which(myrownames == realname2)]<- 1
                }
                Model$notMat= cbind(Model$notMat, values)

                # finally, fill the newAnd list
                newreac1 = paste(inSpecies[i], outname, sep="=")
                newreac2 = paste(inSpecies[j], outname, sep="=")
                newANDs[[length(newANDs)+1]] <- c(newreac1, newreac2)
                names(newANDs)[[length(newANDs)]] <- newcolname
            }

            # Same code as above but to create the 3 inputs combinations
            if (length(inSpecies)>=3 & maxInputsPerGate>=3){
                combinations = combn(seq(1,length(inSpecies)), 3)
                indices = seq(1,dim(combinations)[2])
            }
            else{
                indices = seq(length=0)
            }

            for (this in indices){
                i = combinations[1,this]
                j = combinations[2,this]
                k = combinations[3,this]
                realname1 = ifelse(substr(inSpecies[i], 1,1)=="!",substr(inSpecies[i],2,10000), inSpecies[i])
                realname2 = ifelse(substr(inSpecies[j], 1,1) =="!",substr(inSpecies[j],2,10000), inSpecies[j])
                realname3 = ifelse(substr(inSpecies[k], 1,1) =="!",substr(inSpecies[k],2,10000), inSpecies[k])

                realnames = c(realname1,realname2, realname3)
                if (any(combn(realnames,2)[1,] == combn(realnames,2)[2,])){  # exclude reaction if any name are indentical
                    next()
                }
                newcolname <- paste(paste(inSpecies[i], inSpecies[j],inSpecies[k],
                               sep="+"), outname, sep="=")
                if (newcolname %in% colnames(Model$interMat)){
                    next() # skip if exist already
                }
                Model$reacID <- c(Model$reacID,newcolname)

                # intermat first
                values = as.matrix(rep(0, length(Model$namesSpecies)))
                colnames(values)<-newcolname
                for (name in inSpecies){
                    realname = ifelse(substr(name, 1,1)=="!",substr(name,2,10000), name)
                    values[which(myrownames == realname)]<- -1
                }
                values[which(myrownames == outname)]<- 1
                Model$interMat= cbind(Model$interMat, values)

                # now, the notMat
                values = as.matrix(rep(0, length(Model$namesSpecies)))
                colnames(values)<-newcolname
                for (name in inSpecies){
                    if (substr(name,1,1) == "!"){
                        realname = ifelse(substr(name, 1,1) =="!", substr(name,2,10000), name)
                        values[which(myrownames == realname)]<- 1
                    }
                }
                Model$notMat= cbind(Model$notMat, values)

                # finally the newAnd
                newreac1 = paste(inSpecies[i], outname, sep="=")
                newreac2 = paste(inSpecies[j], outname, sep="=")
                newreac3 = paste(inSpecies[k], outname, sep="=")
                newANDs[[length(newANDs)+1]] <- c(newreac1, newreac2, newreac3)
                names(newANDs)[[length(newANDs)]] <- newcolname
            }

            # Same code as above but to create the 4 inputs combinations
            if (length(inSpecies)>=4 & maxInputsPerGate>=4){
                combinations = combn(seq(1,length(inSpecies)), 4)
                indices = seq(1,dim(combinations)[2])
            }
            else{
                indices = seq(length=0)
            }

            for (this in indices){
                i = combinations[1,this]
                j = combinations[2,this]
                k = combinations[3,this]
                l = combinations[4,this]
                realname1 = ifelse(substr(inSpecies[i], 1,1)=="!",substr(inSpecies[i],2,10000), inSpecies[i])
                realname2 = ifelse(substr(inSpecies[j], 1,1) =="!",substr(inSpecies[j],2,10000), inSpecies[j])
                realname3 = ifelse(substr(inSpecies[k], 1,1) =="!",substr(inSpecies[k],2,10000), inSpecies[k])
                realname4 = ifelse(substr(inSpecies[l], 1,1) =="!",substr(inSpecies[l],2,10000), inSpecies[l])
                realnames = c(realname1,realname2, realname3, realname4)
                if (any(combn(realnames,2)[1,] == combn(realnames,2)[2,])){  # exclude reaction if any name are indentical
                    next()
                }
                newcolname <- paste(paste(inSpecies[i], inSpecies[j],inSpecies[k],inSpecies[l],
                               sep="+"), outname, sep="=")
                if (newcolname %in% colnames(Model$interMat)){
                    next() # skip if exist already
                }
                Model$reacID <- c(Model$reacID,newcolname)

                # intermat first
                values = as.matrix(rep(0, length(Model$namesSpecies)))
                colnames(values)<-newcolname
                for (name in inSpecies){
                    realname = ifelse(substr(name, 1,1)=="!",substr(name,2,10000), name)
                    values[which(myrownames == realname)]<- -1
                }
                values[which(myrownames == outname)]<- 1
                Model$interMat= cbind(Model$interMat, values)

                # now, the notMat
                values = as.matrix(rep(0, length(Model$namesSpecies)))
                colnames(values)<-newcolname
                for (name in inSpecies){
                    if (substr(name,1,1) == "!"){
                        realname = ifelse(substr(name, 1,1) =="!", substr(name,2,10000), name)
                        values[which(myrownames == realname)]<- 1
                    }
                }
                Model$notMat= cbind(Model$notMat, values)

                # finally the newAnd
                newreac1 = paste(inSpecies[i], outname, sep="=")
                newreac2 = paste(inSpecies[j], outname, sep="=")
                newreac3 = paste(inSpecies[k], outname, sep="=")
                newreac4 = paste(inSpecies[l], outname, sep="=")
                newANDs[[length(newANDs)+1]] <- c(newreac1, newreac2, newreac3, newreac4)
                names(newANDs)[[length(newANDs)]] <- newcolname

            } # end if length(inSp) == 2


 if (maxInputsPerGate >= 5) {
        for (mip in 5:maxInputsPerGate) {
          if (length(inSpecies) >= mip & maxInputsPerGate >=
              mip) {
            combinations = combn(seq(1, length(inSpecies)),
              mip)
            indices = seq(1, dim(combinations)[2])
          }
          else {
            indices = seq(length = 0)
          }
          for (this in indices) {
            combs <- c()
            realnames <- c()
            for (i in 1:mip) {
              combs[i] = combinations[i, this]
              realnames[i] = ifelse(substr(inSpecies[combs[i]], 1, 1) ==
                         "!", substr(inSpecies[i], 2, 10000), inSpecies[combs[i]])
            }
            if (any(combn(realnames, 2)[1, ] == combn(realnames,
                                         2)[2, ])) {
              (next)()
            }
            newcolname <- paste(paste(inSpecies[combs], collapse = "+"), outname,
                                sep = "=")
            if (newcolname %in% colnames(Model$interMat)) {
              (next)()
            }
            Model$reacID <- c(Model$reacID, newcolname)
            values = as.matrix(rep(0, length(Model$namesSpecies)))
            colnames(values) <- newcolname
            for (name in inSpecies) {
              realname = ifelse(substr(name, 1, 1) == "!",
                substr(name, 2, 10000), name)
              values[which(myrownames == realname)] <- -1
            }
            values[which(myrownames == outname)] <- 1
            Model$interMat = cbind(Model$interMat, values)
            values = as.matrix(rep(0, length(Model$namesSpecies)))
            colnames(values) <- newcolname
            for (name in inSpecies) {
              if (substr(name, 1, 1) == "!") {
                realname = ifelse(substr(name, 1, 1) == "!",
                  substr(name, 2, 10000), name)
                values[which(myrownames == realname)] <- 1
              }
            }
            Model$notMat = cbind(Model$notMat, values)
            newreac1 = paste(inSpecies[i], outname, sep = "=")
            newreac2 = paste(inSpecies[j], outname, sep = "=")
            newreac3 = paste(inSpecies[k], outname, sep = "=")
            newreac4 = paste(inSpecies[l], outname, sep = "=")
            newreacs <- c()
            for (i in 1:mip) {
              newreacs[i] <- paste(inSpecies[combs[i]], outname, sep = "=")
            }
            newANDs[[length(newANDs) + 1]] <- newreacs # c(newreac1, newreac2, newreac3, newreac4)
            names(newANDs)[[length(newANDs)]] <- newcolname
          }
        }
      }

        }
    }
#    dup.index2 = c(1:length(Model$reacID))
#    dup.index2 = dup.index2[duplicated(Model$reacID)]
#    Model$reacID = Model$reacID[-dup.index2]
#    Model$interMat = Model$interMat[,-dup.index2]
#    Model$notMat = Model$notMat[,-dup.index2]

    if(!is.na(toAdd)) {
        Model$notMat = cbind(Model$notMat, toAdd$notMat)
        Model$interMat = cbind(Model$interMat, toAdd$interMat)
        Model$reacID = c(Model$reacID, toAdd$reacID)
    }

    modelExp <- Model
    modelExp$SplitANDs <- SplitANDs
    modelExp$newANDs <- newANDs
    return(modelExp)
}

#
#  This file is part of the CNO software
#
#  Copyright (c) 2011-2012 - EMBL - European Bioinformatics Institute
#
#  File author(s): CNO developers (cno-dev@ebi.ac.uk)
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  CNO website: http://www.cellnopt.org
#
##############################################################################
indexFinder<-function(CNOlist, model, verbose=FALSE){

    if (!is(CNOlist,"CNOlist")){
        CNOlist = CellNOptR::CNOlist(CNOlist)
    }


    #check that Model is a model list

    if(!is.list(model)) stop("This function expects as input a model as output by readSIF")

    if( !all( c("reacID", "namesSpecies","interMat","notMat") %in% names(model))){
        stop("This function expects as input a model as output by readSIF")
    }




    #Find the indexes of the signals
    signals<-match(colnames(CNOlist@signals[[1]]),model$namesSpecies)

    #Find the indexes of the stimulated species
    stimulated<-match(colnames(CNOlist@stimuli),model$namesSpecies)

    #Find the indexes of the inhibited species
    inhibited<-match(colnames(CNOlist@inhibitors),model$namesSpecies)

    #Print summaries
    if(verbose){
        print(paste(
            "The following species are measured:",
            toString(model$namesSpecies[signals])))
        print(paste(
            "The following species are stimulated:",
            toString(model$namesSpecies[stimulated])))
        print(paste(
            "The following species are inhibited:",
            toString(model$namesSpecies[inhibited])))
        }

    #Return a list of indexes
    return(list(signals=signals,stimulated=stimulated,inhibited=inhibited))

}

#
#  This file is part of the CNO software
#
#  Copyright (c) 2011-2012 - EMBL - European Bioinformatics Institute
#
#  File author(s): CNO developers (cno-dev@ebi.ac.uk)
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  CNO website: http://www.cellnopt.org
#
##############################################################################
# $Id: residualError.R 3155 2013-01-09 15:24:58Z cokelaer $
residualError<-function(CNOlist){


    if (!is(CNOlist,"CNOlist")){
         CNOlist = CellNOptR::CNOlist(CNOlist)
     }

    nTimes = length(CNOlist@timepoints) - 1  # we do not take into account t0


	resErr<-rep(NA, nTimes + 2) # for back compatibility, we will store t1andt2
                                # (+1) but we also store the total (+1)

    # build up the name of the columns: t1, t2, ...tN, t1andt2, total
    namesresErr = NULL
    for (i in 1:nTimes){
        namesresErr = cbind(namesresErr, paste("t", i, sep=""))
    }
    namesresErr = cbind(namesresErr, "t1andt2")
    namesresErr = cbind(namesresErr, "total")
    names(resErr) <- namesresErr

    # compute errors at each time
    for (i in 1:nTimes){
        # we skip time 0. t1 starts at index 2 hence the i+1
        Diff <- round(CNOlist@signals[[i+1]])-CNOlist@signals[[i+1]]
        resErr[i]<-sum(Diff^2, na.rm=TRUE)
    }

    # compute the total and t1andt2 columns
    resErr["total"]<-sum(resErr, na.rm=TRUE) # should be before t1ant2 being filled
	if(length(CNOlist@signals) >= 3){
        resErr[nTimes+1]<-resErr["t1"]+resErr["t2"]
    }

	return(resErr)
	}

#
#  This file is part of the CNO software
#
#  Copyright (c) 2011-2012 - EMBL - European Bioinformatics Institute
#
#  File author(s): CNO developers (cno-dev@ebi.ac.uk)
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  CNO website: http://www.cellnopt.org
#
##############################################################################
getFit<-function(
    simResults,
    CNOlist,
    model,
    indexList=NULL,
    timePoint=c("t1","t2"),
    sizeFac=0.0001,
    NAFac=1,
    nInTot,
    simResultsT0=NULL
    ){

    if (!is(CNOlist,"CNOlist")){
         CNOlist = CellNOptR::CNOlist(CNOlist)
     }
	
	
	allStatesSimResults = simResults
	
    if (is.null(indexList)==FALSE){
        simResults<-simResults[,indexList$signals]
        if (is.null(simResultsT0)==FALSE){
            simResultsT0<-simResultsT0[,indexList$signals]
        }
    }


    # for back compatibility, timePoint ca be "t1" or "t2" but developers should
    # use an integer.
    if(timePoint == "t1"){
        tPt<-2
    }
    else{
        if(timePoint == "t2"){
            tPt<-3
        }
        else{
            tPt<-timePoint
        }
    }


    # if t0 is provided and we are interested in t1
    # then  score is based on t1 but also t0
    if (tPt == 2 && is.null(simResultsT0)==FALSE){
        Diff0 <- simResultsT0 - CNOlist@signals[[1]]
        Diff <- simResults - CNOlist@signals[[tPt]]
        r0 <- Diff0^2
        r <- Diff^2
        r <- rbind(r0, r) # we can concatenate because it's matricial computation.
        deviationPen<-sum(r[!is.na(r)])/2
    }# otherwise, no need to take to into account
    else{
        Diff<-simResults-CNOlist@signals[[tPt]]
        r<-Diff^2
        deviationPen<-sum(r[!is.na(r)])
    }


    NAPen<-NAFac*length(which(is.na(allStatesSimResults)))
    
    nDataPts<-dim(CNOlist@signals[[tPt]])[1]*dim(CNOlist@signals[[tPt]])[2]

    nInputs<-length(which(model$interMat == -1))

    # nInTot: number of inputs of expanded model
    # nInputs: number of inputs of cut model
    sizePen<-(nDataPts*sizeFac*nInputs)/nInTot
    score<-deviationPen+NAPen+sizePen
    return(score)

    }

# Export a Boolean network <network> to an sbml-qual file <fileName>.
# This file can then be read in using other software that supports SBMLqual standards.
# The function also takes a bit string as input. 
#It cuts the model according to the values in bitstrings and write the new model object to SBMLqual.



toSBMLStandard <- function(network, file, bitString = c(rep(1,length(network$reacID))))
{
    if (!requireNamespace("stringi", quietly = TRUE)) {
    stop("stringi needed for this function to work. Please install it.",
      call. = FALSE)
  }
  if (!requireNamespace("stringr", quietly = TRUE)) {
    stop("stringr needed for this function to work. Please install it.",
      call. = FALSE)
  }
    network = cutModel(network, bitString)
    
    
    # generate a network identifier from the file name
    id <- sub(".sbml", "", basename(file), fixed=TRUE)
    id <- gsub("[^a-zA-Z0-9_]+","_",id)
    
    # open a string connection
    output <- NULL
    f <- textConnection("output",  encoding="UTF-8", open="w", local=TRUE)
    
    # write document header
    cat(file=f, "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n")
    cat(file=f, "<sbml xmlns=\"http://www.sbml.org/sbml/level3/version1/core\" level=\"3\" version=\"1\" xmlns:qual=\"http://www.sbml.org/sbml/level3/version1/qual/version1\" qual:required=\"true\">\n")
    cat(file=f, "\t<model id=\"", id , "\">\n", sep="")
    
    # write default compartment
    cat(file=f, "\t\t<listOfCompartments>\n")
    cat(file=f, "\t\t\t<compartment id=\"default\" constant=\"true\"/>\n")
    cat(file=f, "\t\t</listOfCompartments>\n")
    
    # write genes
    geneList <- NULL
    cat(file=f, "\t\t<qual:listOfQualitativeSpecies>\n")
    for (gene in network$namesSpecies)
    {
        Input_interaction <- which(network$interMat[gene,] == 1)
        Output_Interaction <- which(network$interMat[gene,] == -1)
        #if gene is has no input interaction and no output interaction, do not write it out to SBMLqual 
        if (length(Input_interaction) == 0 && length(Output_Interaction) == 0)next
        cat(file=f, "\t\t\t<qual:qualitativeSpecies qual:id =\"", gene, "\" qual:compartment=\"main\"",
            " qual:constant=\"false\"/>\n", sep = "")
        geneList <- c(geneList, gene)
        
    }
    cat(file=f, "\t\t</qual:listOfQualitativeSpecies>\n")
    
    # write transition functions
    cat(file=f, "\t\t<qual:listOfTransitions>\n")
    
    t_count = 1 
    
    for (gene in geneList)
    {
        inputs = c()
        signs = c()
        interactions <- which(network$interMat[gene,] == 1)
        t_name <- ""
        t_name = paste0("t", t_count, sep='')
        
        cat(file=f, "\t\t\t<qual:transition qual:id=\"", t_name , "\">\n",sep = "")
        cat(file=f,"\t\t\t\t <qual:listOfInputs> \n")
        
        for (i in interactions)
        {
            
            intName <- colnames(network$interMat)[i]
            if (grepl("+", intName,fixed = TRUE) == TRUE) 
            {
                tmp = unlist(strsplit(intName,split = "="))[1] #take the inputs on the left hand side, removing the output gene
                LHS = unlist(strsplit(tmp,split = "+",fixed = TRUE)) #list of inputs
                for (input in LHS){
                    sign <- "positive"
                    if(substr(input,1,1) == "!"){
                        input = stringi::stri_sub(input,2)
                        sign <- "negative"
                        
                    }
                    full_name <- ""
                    full_name <- paste0("theta_", t_name, "_", input, sep='')
                    signs <- c(signs, sign)
                    cat(file=f,"\t\t\t\t\t <qual:input  qual:id=\"", full_name , 
                        "\" qual:qualitativeSpecies=\"",input,"\" qual:transitionEffect=\"none\" qual:sign=\"", sign, "\" qual:thresholdLevel=\"1\"/>\n",sep = "")
                }
                andGate = paste(unlist(LHS), collapse='&')
                andGate = stringr::str_replace_all(andGate,'!','')
                if (length(inputs)!=0)
                    andGate = paste0('|', andGate)
                inputs = c(inputs, andGate)
                
            } 
            else 
            {
                sign <- "positive"
                LHS <- unlist(strsplit(intName,split = "="))[1] #single OR input
                if (substr(LHS,1,1) == "!"){
                    sign <- "negative"
                    LHS <- stringi::stri_sub(LHS,2)
                }
                
                full_name <- ""
                full_name <- paste0("theta_", t_name, "_",LHS,sep='')
                signs <- c(signs, sign)
                cat(file=f,"\t\t\t\t\t <qual:input  qual:id=\"", full_name , 
                    "\" qual:qualitativeSpecies=\"", LHS, "\" qual:transitionEffect=\"none\" qual:sign=\"", sign, "\" qual:thresholdLevel=\"1\"/>\n",sep = "")
                
                if (length(inputs)!=0)
                    orGate = paste0('|',LHS, collapse = '')
                else orGate = LHS
                inputs = c(inputs, orGate)
            }
        
            
        }
        cat(file=f,"\t\t\t\t </qual:listOfInputs>\n")
        cat(file=f,"\t\t\t\t <qual:listOfOutputs>\n")
        cat(file=f,"\t\t\t\t\t <qual:output qual:qualitativeSpecies=\"", gene ,"\" qual:transitionEffect=\"assignmentLevel\"/>\n",sep = "")
        cat(file=f,"\t\t\t\t </qual:listOfOutputs>\n")
        cat(file=f, "\t\t\t\t<qual:listOfFunctionTerms>\n")
        cat(file=f, "\t\t\t\t\t<qual:defaultTerm qual:resultLevel=\"0\"/>\n")
        cat(file=f, "\t\t\t\t\t<qual:functionTerm qual:resultLevel=\"1\">\n")
        cat(file=f, "\t\t\t\t\t\t<math xmlns=\"http://www.w3.org/1998/Math/MathML\">\n")
        cat(file=f, "\t\t\t\t\t\t\t<apply>\n")
        transition = paste(unlist(inputs), collapse='')
       
        #only OR transitions
        if (grepl("&", transition,fixed = TRUE) == FALSE && transition!='')
        {
            transition = unlist(strsplit(transition,split = '|', fixed = TRUE))
            if (length(transition) > 1)
                cat(file=f, "\t\t\t\t\t\t\t\t<", 'or', "/>\n",sep = "")
            for (i in 1:length(transition)){
                cat(file=f, "\t\t\t\t\t\t\t\t<apply>\n")
                cat(file=f, "\t\t\t\t\t\t\t\t\t<eq/>\n")
                cat(file=f, "\t\t\t\t\t\t\t\t\t<ci>" , transition[i] , "</ci>\n")
                if (signs[i] == 'positive')
                    integer = 1
                else integer = 0
                cat(file=f, "\t\t\t\t\t\t\t\t\t<cn type='integer'>" , integer , "</cn>\n")
                cat(file=f, "\t\t\t\t\t\t\t\t</apply>\n")
            }
            
                    
                
        }
        #only AND transitions
        else if (grepl("&", transition,fixed = TRUE) == TRUE && transition!='' && grepl("|", transition,fixed = TRUE) == FALSE){
            transition = unlist(strsplit(transition,split = '&', fixed = TRUE))
            cat(file=f, "\t\t\t\t\t\t\t\t<", 'and', "/>\n",sep = "")
            for (i in 1:length(transition)){
                cat(file=f, "\t\t\t\t\t\t\t\t<apply>\n")
                cat(file=f, "\t\t\t\t\t\t\t\t\t<eq/>\n")
                cat(file=f, "\t\t\t\t\t\t\t\t\t<ci>" , transition[i] , "</ci>\n")
                if (signs[i] == 'positive')
                    integer = 1
                else integer = 0
                cat(file=f, "\t\t\t\t\t\t\t\t\t<cn type='integer'>" , integer , "</cn>\n")
                cat(file=f, "\t\t\t\t\t\t\t\t</apply>\n")
            
            }
        
        }
        else if (transition==''){
            #if gene has no input transition, do nothing 
        }
        #both And and OR gates 
        else 
        {
            count_sign = 1
            transition = unlist(strsplit(transition,split = '|', fixed = TRUE))
            cat(file=f, "\t\t\t\t\t\t\t\t<", 'or', "/>\n",sep = "")
            for (i in 1:length(transition)){
                cat(file=f, "\t\t\t\t\t\t\t\t<apply>\n")
                if (grepl("&", transition[i], fixed = TRUE) == TRUE){
                    cat(file=f, "\t\t\t\t\t\t\t\t<", 'and', "/>\n",sep = "")
                    andTransition = unlist(strsplit(transition[i],split = '&', fixed = TRUE))
                    for (k in andTransition){
                        cat(file=f, "\t\t\t\t\t\t\t\t<apply>\n")
                        cat(file=f, "\t\t\t\t\t\t\t\t\t<eq/>\n")
                        cat(file=f, "\t\t\t\t\t\t\t\t\t<ci>" , k , "</ci>\n")
                        if (signs[count_sign] == 'positive')
                            integer = 1
                        else integer = 0
                        cat(file=f, "\t\t\t\t\t\t\t\t\t<cn type='integer'>" , integer , "</cn>\n")
                        cat(file=f, "\t\t\t\t\t\t\t\t</apply>\n")
                        count_sign = count_sign + 1 
                    }
                }
                else {
                    cat(file=f, "\t\t\t\t\t\t\t\t\t<eq/>\n")
                    cat(file=f, "\t\t\t\t\t\t\t\t\t<ci>" , transition[i] , "</ci>\n")
                    if (signs[i] == 'positive')
                        integer = 1
                    else integer = 0
                    cat(file=f, "\t\t\t\t\t\t\t\t\t<cn type='integer'>" , integer , "</cn>\n")
                    count_sign = count_sign + 1 
                }
                    
                cat(file=f, "\t\t\t\t\t\t\t\t</apply>\n")
            }
            
        }
        cat(file=f, "\t\t\t\t\t\t\t</apply>\n")
        cat(file=f, "\t\t\t\t\t\t</math>\n")
        cat(file=f, "\t\t\t\t\t</qual:functionTerm>\n")
        cat(file=f, "\t\t\t\t</qual:listOfFunctionTerms>\n")
        cat(file=f, "\t\t\t</qual:transition>\n")
        t_count = t_count + 1
    }
    
    # finish document
    cat(file=f, "\t\t</qual:listOfTransitions>\n")
    cat(file=f, "\t</model>\n")
    cat(file=f, "</sbml>\n")
    close(f)
    
    # open file and write the complete XML string
    f <- file(file, encoding="UTF-8", open="w")
    cat(file=f,output,sep="\n")
    close(f)  
    
}

#  Copyright (c) 2019 - SaezLab - Heidelberg Universit
#
#  File author(s): CNO developers
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  CNO website: http://www.cellnopt.org
#
##############################################################################

invokeCPLEX <- function(inputFileName, 
                        outputFileName, 
                        mipGap=mipGap, 
                        relGap = relGap, 
                        timelimit=timelimit, 
                        cplexPath = cplexPath, 
                        numSolutions = numSolutions, 
                        limitPop = limitPop, 
                        poolIntensity = poolIntensity, 
                        poolReplace = poolReplace){
  
  ## Writing and executing the cplex commands
  ## setwd(gsub(pattern = "cplex", replacement = "", x = cplexPath))
  writeCplexCommandFile(commandFileName = "cplexCommand.txt", inputFileName, 
                        outputFileName, mipGap=mipGap, relGap = relGap, 
                        timelimit=timelimit, cplexPath = cplexPath, 
                        numSolutions = numSolutions, limitPop = limitPop, 
                        poolIntensity = poolIntensity, 
                        poolReplace = poolReplace)
  if(file.exists(outputFileName)){
    file.remove(outputFileName)
  }
  # system(paste0(cplexPath, " -f cplexCommand.txt"))
  
  if (Sys.info()[1]=="Windows") {
    file.copy(from = cplexPath,to = getwd())
    system(paste0("cplex.exe -f cplexCommand.txt"))
    file.remove("cplex.exe")
  } else {
    system(paste0(cplexPath, " -f cplexCommand.txt"))
  }
  
}

#
#  This file is part of the CNO software
#
#  Copyright (c) 2011-2012 - EBI
#
#  File author(s): CNO developers (cno-dev@ebi.ac.uk)
#
#  Distributed under the GPLv2 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-2.0.html
#
#  CNO website: http://www.ebi.ac.uk/saezrodriguez/cno
#
##############################################################################
# $Id$

# This is a test of the ToyModel and gaBinaryT1
# 

library(CellNOptR)
data(CNOlistToy2, package="CellNOptR")
data(ToyModel2, package="CellNOptR")
cnolist = CNOlistToy2
pknmodel = ToyModel2

model = preprocessing(cnolist, pknmodel, verbose=FALSE)
T1opt<-gaBinaryT1(CNOlist=cnolist,model=model,verbose=FALSE)
print(T1opt$bString)

truebs = c(1, 1, 1, 1, 1, 0, 1, 1, 0, 0, 1, 1, 1, 1, 0, 0, 0, 0)


T2opt<-gaBinaryTN(CNOlist=cnolist,model=model,bStrings=list(truebs),verbose=FALSE)
truebs2 = c(0, 0, 0, 1, 0, 0, 0)

print(T2opt$bString)

#
#  This file is part of the CNO software
#
#  Copyright (c) 2011-2012 - EMBL - European Bioinformatics Institute
#
#  File author(s): CNO developers (cno-dev@ebi.ac.uk)
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  CNO website: http://www.cellnopt.org
#
##############################################################################
# $Id: plotFit.R 3522 2013-04-23 11:01:03Z cokelaer $
plotFit<-function(optRes, filename=NULL){

   oldPar = par(no.readonly = TRUE)
   on.exit(par(oldPar))

   if (is.null(filename)!=TRUE){
        pdf(filename)
    }

    par(mfrow=c(2,1),mar=c(0.5,4,4,0))
    plot(
        x=optRes$results[,"Generation"],
        y=optRes$results[,"Avg_Score_Gen"],
        xlab=NA,
        xaxt="n",
        ylab="Average score of generation",
        type="l")
    par(mar=c(4,4,0,0))
    plot(
        x=optRes$results[,"Generation"],
        y=optRes$results[,"Best_score_Gen"],
        xlab="Generations",
        ylab="Best Score",
        type="l")

    if (is.null(filename)!=TRUE){
        dev.off()
    }

   # par(oldPar)

}

#
#  This file is part of the CNO software
#
#  Copyright (c) 2011-2012 - EMBL - European Bioinformatics Institute
#
#  File author(s): CNO developers (cno-dev@ebi.ac.uk)
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  CNO website: http://www.ebi.ac.uk/saezrodriguez/cno
#
##############################################################################
# $Id$

crossInhibitedData <- function(object){
          times = object@timepoints 

          # identify names found in inhibitors and signals list
          inhibitors = colnames(object@inhibitors)[colnames(object@inhibitors) %in% colnames(object@signals[[1]])]

          # only those ones must be crossed
          for (inhibitor in inhibitors){
              for (time in seq_along(times)){
                  mask = object@inhibitors[,inhibitor] == 1
                  object@signals[[time]][mask, inhibitor] = NA
              }
          }
    return(object)
}

#
#  This file is part of the CNO software
#
#  Copyright (c) 2011-2012 - EMBL - European Bioinformatics Institute
#
#  File author(s): CNO developers (cno-dev@ebi.ac.uk)
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  CNO website: http://www.cellnopt.org
#
##############################################################################

compressModel<-function(model, indexes){

    Model = model

    #this will hold the indices of the species that we have to look at (i.e. not signal nor cue)
    species<-seq(1:length(Model$namesSpecies))
    #species<-species[-c(indexes$signals,indexes$stimulated,indexes$inhibited)]
    # TODO: is.na proposed by Aidan fix.
    if(is.na(indexes$inhibited[1])) {
        species <- species[-c(indexes$signals,indexes$stimulated)]
    } else {
        species <- species[-c(indexes$signals,indexes$stimulated,indexes$inhibited)]
    }

    speciesCompressed<-0

    for(sp in species){
        # Find all the reacs that come out of species sp
        outReac <- which(Model$interMat[sp,] == -1)

        # Find all the reacs that come into species sp
        inReac <- which(Model$interMat[sp,] == 1)

        # If the species doesn't have any output, we can't compress it, so we do sthg only if
        #the species has one output minimum
        if(length(outReac) != 0){

            #If the species has only one input
            if(length(inReac) == 1 && length(which(Model$interMat[,inReac] == -1)) == 1) {
                #TODO aidan fix why was the second condition added?

                # find the input element
                input <- which(Model$interMat[,inReac] == -1)

                # find the output elements
                if(length(outReac) > 1) outputs <- apply(Model$interMat[,outReac],2,function(x) which(x == 1))
                if(length(outReac) == 1) outputs <- which(Model$interMat[,outReac] == 1)

                # check that the input is not an & gate, and that none of the downstream
                # elements is the output element
                if(length(input) == 1 && !any(outputs == input[1])
                    && !any(abs(Model$interMat[,inReac] + Model$interMat[,outReac[1]]) > 1)) {
                    # TODO: fix properly: the case where the input is also part of an 'AND' gate in the output. outcome: don't compress
                    # need to create new reacs, that go from input to outputs, and to remove the old reacs

                    # Create the first one, that goes from input to the first output
                    reac2Remove <- c(inReac, outReac)
                    newReacs = matrix(NA, nrow=dim(Model$interMat)[1], ncol=length(outReac))
                    rownames(newReacs) = Model$namesSpecies
                    newNots = matrix(NA, nrow=dim(Model$notMat)[1], ncol=length(outReac))
                    rownames(newNots) = Model$namesSpecies
                    newReacIDs = rep(NA, length(outReac))

                    for(r in 1:length(outReac)) {
                        newReacs[,r] <- Model$interMat[,inReac] + Model$interMat[,outReac[r]]
                        # TODO: check this?
                        newNots[,r] <- Model$notMat[,outReac[r]]
                        if(Model$notMat[sp,outReac[r]] == 1 && Model$notMat[which(Model$interMat[,inReac]==-1),inReac]==1)
                        newNots[input,r] <- 0
                        if(Model$notMat[sp,outReac[r]] == 0 && Model$notMat[which(Model$interMat[,inReac]==-1),inReac]==0)
                        newNots[input,r] <- 0
                        if(Model$notMat[sp,outReac[r]] == 1 && Model$notMat[which(Model$interMat[,inReac]==-1),inReac]==0)
                        newNots[input,r] <- 1
                        if(Model$notMat[sp,outReac[r]] == 0 && Model$notMat[which(Model$interMat[,inReac]==-1),inReac]==1)
                        newNots[input,r] <- 1
                        if(length(grep("+",names(outReac[r])))) {
                            andInput = rownames(newReacs)[which(newReacs[,r] == -1)]
                            andInd = which(newReacs[,r] == -1)
                            if(length(intersect(which(newNots[,r]==1),which(newReacs[,r]==-1)))) {
                                andNeg = intersect(which(newNots[,r]==1),which(newReacs[,r]==-1))
                                for(p in 1:length(andNeg)) {
                                    andInput[which(andInd==andNeg[p])] = paste("!", andInput[which(andInd==andNeg[p])], sep="")
                                }
                            }

                            LHS = paste(andInput,collapse="+", sep="")
                            newReacIDs[r] = paste(LHS, "=", rownames(newReacs)[which(newReacs[,r] == 1)], sep="")

                        } else {
                            newReacIDs[r] <- paste(rownames(newReacs)[which(newReacs[,r] == -1)], "=", rownames(newReacs)[which(newReacs[,r] == 1)],sep="")
                            if(any(newNots[,r] == 1)) newReacIDs[r] <- paste("!", newReacIDs[4], sep="")
                        }

                    }

                    colnames(newReacs) <- newReacIDs
                    colnames(newNots) <- newReacIDs

                    # now remove the reacs to remove and append the ones that have just been created
                    Model$interMat <- Model$interMat[,-reac2Remove]
                    Model$notMat <- Model$notMat[,-reac2Remove]
                    Model$reacID <- Model$reacID[-reac2Remove]
                    Model$interMat <- cbind(Model$interMat, newReacs)
                    Model$notMat <- cbind(Model$notMat,newNots)
                    Model$reacID <- c(Model$reacID,newReacIDs)

                    if(length(outReac) == 1) {
                        colnames(Model$interMat)[dim(Model$interMat)[2]] <- newReacIDs
                        colnames(Model$notMat)[dim(Model$notMat)[2]] <- newReacIDs
                    }

                    # store which species has been compressed. This will be used to remove rows of the matrices
                    # after all is compressed (I don't do it know because then the indexes of the species that we
                    # have decided to look at in the beginning won't be valid anymore)

                    speciesCompressed <- c(speciesCompressed,sp)
                } #end of if(length(input) == 1 && !any(outputs == input[1]) && !any....
            } else {

                # If the species has more than one input, but only one output
                if(length(outReac) == 1 && length(which(Model$interMat[,outReac] == -1)) == 1) {

                    # find the output element
                    output <- which(Model$interMat[,outReac] == 1)
                    # find the input element(s)
                    inputs = list()

                    if(length(inReac) > 1) {
                        for(a in inReac) {
                            inputs = c(inputs, list(which(Model$interMat[,a] == -1)))
                        }
                    }
                    if(length(inReac) == 1) inputs <- list(which(Model$interMat[,inReac] == -1))

                    # check that none of the downstream elements is the output element
                    inputsUnlist = unlist(inputs)
                    if(!any(inputsUnlist == output[1])) {
                        # create the first reac, that goes from the first input to the output
                        reac2Remove <- c(inReac, outReac)
                        newReacs = matrix(NA, nrow=dim(Model$interMat)[1], ncol=length(inReac))
                        rownames(newReacs) = Model$namesSpecies
                        newNots = matrix(NA, nrow=dim(Model$notMat)[1], ncol=length(inReac))
                        rownames(newNots) = Model$namesSpecies
                        newReacIDs = rep(NA, length(inReac))
                        if (length(inReac)>0){
                        for(r in 1:length(inReac)) {
                            newReacs[,r] <- Model$interMat[,inReac[r]] + Model$interMat[,outReac]
                            newNots[,r] <- Model$notMat[,inReac[r]]
                            # if out is negative & in is single:
                            if(length(inputs[[r]])==1 && Model$notMat[sp,outReac]==1) {
                                newNots[inputs[[r]],r] <- ifelse((newNots[inputs[[r]],r] == 1), 0, 1)
                            }
                            # if out is negative & in is 'AND' gate
                            if(length(inputs[[r]]) > 1 && Model$notMat[sp,outReac]==1) {
                                for(b in 1:length(inputs[[r]])) {
                                    newNots[inputs[[r]][b]] <- ifelse((newNots[inputs[[r]][b]] == 1), 0, 1)
                                }
                            }

                            if(length(grep("+",names(inReac[r])))) {
                                andInput = rownames(newReacs)[which(newReacs[,r] == -1)]
                                andInd = which(newReacs[,r] == -1)
                                if(length(intersect(which(newNots[,r]==1),which(newReacs[,r]==-1)))) {
                                    andNeg = intersect(which(newNots[,r]==1),which(newReacs[,r]==-1))
                                    for(p in 1:length(andNeg)) {
                                        andInput[which(andInd==andNeg[p])] = paste("!", andInput[which(andInd==andNeg[p])], sep="")
                                    }
                                }

                            LHS = paste(andInput,collapse="+",sep="")
                            newReacIDs[r] = paste(LHS, "=", rownames(newReacs)[which(newReacs[,r] == 1)], sep="")

                            } else {
                                newReacIDs[r] <- paste(rownames(newReacs)[which(newReacs[,r] == -1)], "=", rownames(newReacs)[which(newReacs[,r] == 1)],sep="")
                                if(any(newNots[,r] == 1)) newReacIDs[r] <- paste("!", newReacIDs[4], sep="")
                            }

                        }}

                        colnames(newReacs) <- newReacIDs
                        colnames(newNots) <- newReacIDs

                        Model$interMat <- Model$interMat[,-reac2Remove]
                        Model$notMat <- Model$notMat[,-reac2Remove]
                        Model$reacID <- Model$reacID[-reac2Remove]
                        Model$interMat <- cbind(Model$interMat, newReacs)
                        Model$notMat <- cbind(Model$notMat, newNots)
                        Model$reacID <- c(Model$reacID, newReacIDs)
                        if(length(inReac) == 1) {
                            colnames(Model$interMat)[dim(Model$interMat)[2]] <- newReacIDs
                            colnames(Model$notMat)[dim(Model$notMat)[2]] <- newReacIDs
                        }

                        speciesCompressed <- c(speciesCompressed,sp)
                    }
                }
            }

        # If the species has more than one input and more than one output then
        # nothing is done.
        }
    } # end of the for loop over species



    ModelCompressed<-Model
    ModelCompressed$speciesCompressed<-NA

    # If species have been compressed
    if(length(speciesCompressed) > 1){

        # Remove the dummy 0 in speciesCompressed
        speciesCompressed<-speciesCompressed[2:length(speciesCompressed)]
        ModelCompressed$speciesCompressed<-Model$namesSpecies[speciesCompressed]
        ModelCompressed$namesSpecies<-Model$namesSpecies[-speciesCompressed]
        ModelCompressed$notMat<-Model$notMat[-speciesCompressed,]
        ModelCompressed$interMat<-Model$interMat[-speciesCompressed,]
    }

    # Check that no duplicates reactions have been created
    uniqueR<-unique(ModelCompressed$reacID)

    if(length(uniqueR) != length(ModelCompressed$reacID)){
        ModelCompressed$reacID<-ModelCompressed$reacID[match(uniqueR,ModelCompressed$reacID)]
        ModelCompressed$interMat<-ModelCompressed$interMat[,match(uniqueR,colnames(ModelCompressed$interMat))]
        ModelCompressed$notMat<-ModelCompressed$notMat[,match(uniqueR,colnames(ModelCompressed$notMat))]
        }

    return(ModelCompressed)

}

#
#  This file is part of the CNO software
#
#  Copyright (c) 2011-2012 - EMBL - European Bioinformatics Institute
#
#  File author(s): CNO developers (cno-dev@ebi.ac.uk)
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  CNO website: http://www.cellnopt.org
#
##############################################################################
# $Id$

plotOptimResultsPan <- function(simResults, yInterpol=NULL, xCoords=NULL,
CNOlist=CNOlist, formalism=c("ss1","ss2","ssN","dt","ode"), pdf=FALSE,
pdfFileName="", tPt=NULL, plotParams=list(margin=0.1, width=15, height=12,
cmap_scale=1, cex=1.6, ymin=NULL, Fac=1, rotation=0)) {


    if (!is(CNOlist,"CNOlist")){
         CNOlist = CellNOptR::CNOlist(CNOlist)
     }

    formalism <- match.arg(formalism)
    # index valueSignals according to tPt
    valueSignalsI = sapply(c(0,tPt), function(x) which(CNOlist@timepoints==x))

    if ("margin" %in% names(plotParams) == FALSE){
        plotParams$margin = 0.1
    }
    if ("cex" %in% names(plotParams) == FALSE){
        plotParams$cex = 1.6
    }
    if ("width" %in% names(plotParams) == FALSE){
        plotParams$width = 15
    }
    if ("height" %in% names(plotParams) == FALSE){
        plotParams$height = 12
    }
    if ("cmap_scale" %in% names(plotParams) == FALSE){
        plotParams$cmap_scale = 1
    }
    if ("lwd" %in% names(plotParams) == FALSE){
        plotParams$lwd = 3
    }
    if ("pch" %in% names(plotParams) == FALSE){
        plotParams$pch = 16
    }
    if ("ymin" %in% names(plotParams) == FALSE){
        plotParams$ymin = NULL
    }
    if ("Fac" %in% names(plotParams) == FALSE){
        plotParams$Fac = 1
    }
    if ("rotation" %in% names(plotParams) == FALSE){
        plotParams$rotation = 0
    }

# aliases
    margin = plotParams$margin
    cex = plotParams$cex

    #####    functions    #####

    list2Array = function(x, dim) {
        xUnlist = unlist(x);
        xArray=array(xUnlist,dim=dim)
    }

    #####    /functions/    #####


    opar = par(no.readonly = TRUE)
    on.exit(par(opar))

    if(pdf==TRUE) {
        pdf(file=pdfFileName, width=plotParams$width, height=plotParams$height)
    }

    Ncols = dim(CNOlist@signals[[1]])[2]
    Nrows = dim(CNOlist@signals[[1]])[1]

    split.screen(c(1, Ncols+3))
    for(a in 1:(Ncols+2)) {
        split.screen(c(Nrows+1,1),a) # why +2 ?
    }


    # TODO - do i need all these with split.screen?
    par(
        pch=2,
        # margin bottom of pcolor mesh: oma(X,Y,W,Z) where X is distance bottom
        # to the colormesh
        oma=c(3,2,2,2),
        mgp=c(3,0.9,0),
        family="Times"
    )

    Ncolors = 1000

    heatCols = heat.colors(Ncolors)

    # maximum across all data points
    #yMax <- max(unlist(lapply(CNOlist$valueSignals, function(x) max(x,na.rm=TRUE))))
    yMax=1
    # minimum across all data points

    if (is.null(plotParams$ymin)==FALSE){
        yMin = plotParams$ymin
    } else {
        yMin <- min(unlist(lapply(CNOlist@signals, function(x) min(x,na.rm=TRUE))))
    }


    # time labels
    xVal <- CNOlist@timepoints[valueSignalsI]
    if(formalism=="dt") {
        xValS = xCoords
        norm = length(CNOlist@timepoints)
    } else if (formalism == "ss1") {
        xValS = c(0,tPt[1])
        norm = 2
    } else if (formalism == "ss2") {
        xValS = c(0,tPt[1:2])
        norm = 3
    } else if (formalism == "ssN") {
      xValS = c(0,tPt)
      norm = length(tPt)+1 # should be number of time points.
    } else if (formalism == "ode") {
        xValS = CNOlist@timepoints
        xVal <- CNOlist@timepoints
        valueSignalsI = seq(1, length(xValS))
        norm = length(CNOlist@timepoints)
    }else {
        xValS = xVal
    }
    # latest time point
    xValMax = max(xVal)

    # make simResults array if not already

    if(!is.array(simResults)) {
        simResults = list2Array(simResults, dim=c(dim(simResults[[1]]),length(simResults)))
    }

    # make valueSignals an array
    valueSignalsArr = list2Array(CNOlist@signals,
        dim=c(dim(CNOlist@signals[[1]]),length(CNOlist@signals)))

    # calculate the MSE
    allDiff = matrix(NA, nrow=dim(simResults)[1], ncol=dim(simResults)[2])

    if(formalism != "dt") {
        # ss1, ss2, ssN
        for(a in 1:dim(simResults)[1]) {
            for(b in 1:dim(simResults)[2]) {
                c1 = simResults[a,b,]
                c2 = valueSignalsArr[a,b,valueSignalsI]
                NAcount = max(sum(is.na(c1)), sum(is.na(c2)))
                allDiff[a,b] = sum((c1 - c2)^2, na.rm=TRUE)/(norm - NAcount)
            }
        }
    } else {
        # dt
        for(a in 1:dim(simResults)[1]) {
            for(b in 1:dim(simResults)[2]) {
                c1 =  simResults[a,b,]
                c2 = yInterpol[a,b,]
                NAcount = max(sum(is.na(c1)), sum(is.na(c2)))
                allDiff[a,b] = sum((c1 - c2)^2, na.rm=TRUE)/(norm - NAcount)
            }
        }
    }
    # max difference between sim and exper
    #diffMax = max(unlist(!is.na(allDiff)))

    # set the count for the split screen window
    count1 = dim(CNOlist@signals[[1]])[2]+4

    # plot headers
    for(c in 1:dim(CNOlist@signals[[1]])[2]) {
        screen(count1)
        par(fg="blue",mar=c(margin, margin, margin, 0))
        plot(x=xVal, y=rep(-5,length(xVal)), ylim=c(yMin, yMax),
        xlab=NA,ylab=NA,xaxt="n",yaxt="n")
		labels=colnames(CNOlist@signals[[1]])[c]
        text(
            labels=labels,
            x=((xVal[length(xVal)]-xVal[1])/2),
            y=(yMin+((yMax-yMin)/2)),
            cex=cex
        )
        count1 = count1 + dim(CNOlist@signals[[1]])[1]+1
    }

    # stim + inhib
    screen(count1)
    par(fg="blue",mar=c(margin, margin, margin, 0))
    plot(
        x = xVal,
        y = rep(-5,length(xVal)),
        ylim = c(yMin, yMax),
        xlab = NA,ylab=NA,xaxt="n",yaxt="n"
    )
    text(
        labels = "Stim",
        x = ((xVal[length(xVal)]-xVal[1])/2),
        y = (yMin+((yMax-yMin)/2)),cex=cex
    )

    count1 = count1 + dim(CNOlist@signals[[1]])[1]+1
       screen(count1)
    #par(fg="blue",mar=c(0.5,0.5,0.7,0))
    par(fg="blue",mar=c(margin, margin, margin, 0))
       plot(
        x = xVal, y=rep(-5,length(xVal)),
        ylim = c(yMin, yMax),
           xlab = NA,ylab=NA,xaxt="n",yaxt="n")
    text(
        labels="Inh",
         x=((xVal[length(xVal)]-xVal[1])/2),
           y=(yMin+((yMax-yMin)/2)),cex=cex
       )


    # new count for plotting results
    countRow = dim(CNOlist@signals[[1]])[2]+4


    for(c in 1:dim(CNOlist@signals[[1]])[2]) {
        countRow=countRow+1
        for(r in 1:dim(CNOlist@signals[[1]])[1]) {

            screen(countRow)
            par(fg="black",mar=c(margin, margin,0,0))
            yVal <- lapply(CNOlist@signals[valueSignalsI], function(x) {x[r,c]})

            yValS <- simResults[r,c,]
            if(!is.na(allDiff[r,c])) {
                #diff = (1 - (allDiff[r,c] / diffMax)) * 1000
                diff = (1 - (allDiff[r,c] /plotParams$Fac)**plotParams$cmap_scale) * (Ncolors-1)+1
            } else {
                diff = -1
            }
            if (diff <1 & diff!=-1){diff=1}

            if(diff>Ncolors) {
                diff=Ncolors
            }
            # this is a NA or NaN so what color ?
            if(diff==-1) {
                bgcolor = "gray"
            } else{
                bgcolor = heatCols[diff]
            }
            plot(x=xVal,y=yVal,ylim=c(yMin,yMax),xlab=NA,ylab=NA,xaxt="n",yaxt="n",)
            rect(par("usr")[1], par("usr")[3], par("usr")[2], par("usr")[4], col=bgcolor)


            tryCatch(
                {ystd = unlist(lapply(CNOlist@variances[valueSignalsI],function(x) {x[r,c]}))
                .error.bar(unlist(xVal), unlist(yVal), ystd)
                }, error=function(e){}
            )




            # the measurements
            lines(xVal, yVal, ylim=c(yMin, yMax), xlab=NA, ylab=NA, xaxt="n", yaxt="n", col="black", lty=1, lwd=1)
            points(x=xVal,y=yVal,ylim=c(yMin, yMax),xlab=NA,ylab=NA,xaxt="n",yaxt="n",pch=2)

            # the simulated data
            lines(xValS, yValS, xlim=c(0,xValMax), ylim=c(yMin, yMax),
                xlab=NA, ylab=NA, xaxt="n", yaxt="n", col="blue", lty=2, lwd=plotParams$lwd)
            points(xValS, yValS, xlim=c(0,xValMax), ylim=c(yMin, yMax),
                xlab=NA, ylab=NA, xaxt="n", yaxt="n", col="blue", pch=plotParams$pch)

            # add the axis annotations: if we're on the last row, add the x axis
            if(r == dim(CNOlist@signals[[1]])[1]) {
                axis(side=1,at=CNOlist@timepoints)
            }

            # add the axis annotations: if we're on the first column, add the y axis
            if(c == 1)    {
                axis(side=2,at=c(0,0.5,1), labels=c("","0.5",""), las=1)
            }

            countRow=countRow+1
        }
    }

    sStim = countRow+1

    for(c1 in 1:dim(CNOlist@signals[[1]])[1]) {
    	screen(sStim)
    	#par(mar=c(0.5,0.5,0.5,0.5))
    	par(mar=c(margin, margin,0,0))


    	data = matrix(CNOlist@stimuli[c1,],nrow=1)
    	if(c1 == dim(CNOlist@signals[[1]])[1]){
    		barplot(data,yaxt="n",ylim=c(0,1),names.arg = colnames(CNOlist@stimuli),las=2)
    		#axis(1)
    	}else{
    		barplot(data,xaxt="n",yaxt="n",ylim=c(0,1))
    		#axis(1)
    	}


    	sStim = sStim+1
    }

    sInhib = sStim+1
    for(c1 in 1:dim(CNOlist@signals[[1]])[1]) {
        screen(sInhib)

    	#par(mar=c(0.5,0.5,0.5,0.5))
    	par(mar=c(margin, margin,0,0))

    	if (length(CNOlist@inhibitors) != 0){
    		data = matrix(c(CNOlist@inhibitors[c1,]),nrow=1)
    		if(c1 == dim(CNOlist@signals[[1]])[1]){
    			barplot(data,yaxt="n",ylim=c(0,1),names.arg = c(paste(colnames(CNOlist@inhibitors),"-i",sep="")),las=3 )
    			#axis(1)
    		}else{
    			barplot(data,xaxt="n",yaxt="n",ylim=c(0,1))
    			#axis(1)
    		}
    	}
    	else{ # special case of no inhibitors
    		image(
    			t(matrix(0,nrow=1)),
    			col=c("grey"),xaxt="n",yaxt="n"
    		)
    	}


        sInhib = sInhib+1
    }

    screen(dim(CNOlist@signals[[1]])[2]+3)
    splitProp = 1/(dim(CNOlist@signals[[1]])[1]+1)
    sSplit = matrix(c(0,1,(1-splitProp),1,0,1,0,(1-splitProp)),ncol=4, byrow=TRUE)
    split.screen(sSplit)
    screen(sInhib)
#       par(fg="blue",mar=c(0.5,0.5,0.7,0))
    par(fg="blue",mar=c(margin, margin, margin, 0))
    plot(
        x = xVal,
        y = rep(-5,length(xVal)),
        ylim = c(yMin, yMax),
        xlab = NA,ylab=NA,xaxt="n",yaxt="n"
    )
       text(
        labels = "Error",
        x = ((xVal[length(xVal)]-xVal[1])/2),
        y = (yMin+((yMax-yMin)/2)),cex=cex
        )

    screen(sInhib+1)

    Ncolors = 100
    colbar = heat.colors(Ncolors)

    # scale the colorbar in the same way as the colors used in the boxes
    # according to diff.
    colbar = colbar[as.integer((seq(0, Ncolors,length.out=100)/Ncolors)**plotParams$cmap_scale*Ncolors)]

    labels = c(1,0.5,0)
    len <- length(colbar)
    rhs <- 0.5
    rhs2 <- rhs + rhs/10
    at <- c(0, 0.5, 1)

    par(mai=c(0,0.1,0,0))
    #plot.new()

    tryCatch({
        plot.new()
        yyy <- seq(0,1,length=len+1)
        rect(0, yyy[1:len], rep(rhs, len), yyy[-1], col = colbar, border = colbar)
        rect(0, 0, rhs, 1, border="black")
        segments(rhs, at, rhs2, at)
        text(x=rhs2, y=at, labels=labels, pos=4, offset=0.2)

        },
        error=function(e){print("could not add colorbar. Skipped")}
    )


    close.screen(all.screens=TRUE)
#    par(oldPar)
    if(pdf==TRUE) {
        dev.off()
    }

    return(allDiff)
}


.error.bar <- function(x, y, upper, lower=upper, length=0.1,...){
     if(length(x) != length(y) | length(y) !=length(lower) | length(lower) !=
length(upper))
         stop("vectors must be same length")

     positives = upper > 0
     x = x[positives]
     y = y[positives]
     upper = upper[positives]
     lower = lower[positives]
     arrows(x,y+upper, x, y-lower, angle=90, code=3, length=length, ...)
     }

#
#  This file is part of the CNO software
#
#  Copyright (c) 2019 - SaezLab - Heidelberg Universit
#
#  File author(s): CNO developers
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  CNO website: http://www.cellnopt.org
#
##############################################################################

link_midas_colnames_to_namesSpecies <- function(model, reducedMidas){
  
  linker = c()
  if (!is.null(colnames(reducedMidas))) { 
    for(i in 1:length(colnames(reducedMidas))){
      if(str_sub(colnames(reducedMidas)[i],-1)=="i"){
        linker[i] <- which(model$namesSpecies==
                            substr(strsplit(
                              colnames(reducedMidas)[i],":")[[1]][2],
                              1,nchar(strsplit(colnames(reducedMidas)[i],
                                               ":")[[1]][2])-1))
      } else{  ##for everything else
        linker[i] <- which(model$namesSpecies==
                            strsplit(colnames(reducedMidas)[i],":")[[1]][2])
      }
    }
  } else {
    linker <- 1:length(model$namesSpecies)
  }
  return(linker)
  
}

#
#  This file is part of the CNO software
#
#  Copyright (c) 2011-2012 - EMBL - European Bioinformatics Institute
#
#  File author(s): CNO developers (cno-dev@ebi.ac.uk)
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  CNO website: http://www.cellnopt.org
#
##############################################################################
# $Id$

cutModel <- function(model, bString){
    bs = as.logical(bString)
    newmodel <- list()
    newmodel$interMat <- model$interMat[, bs,drop=FALSE]
    newmodel$notMat <- model$notMat[, bsdrop=FALSE]
    newmodel$reacID <- model$reacID[bs]
    newmodel$namesSpecies <- model$namesSpecies

    # could also add the times used in times > T1 if times
    # newmodel$times <- bStringTimes[which(bStringTimes != 0)]

    return(newmodel)
}

# Export a Boolean network <network> to an sbml-qual file <fileName>.
# This file can then be read in again using CellNoptR



writeLogic <- function(gene, inputs, t_name, t_count, logic, f, signs){
    cat(file=f,"\t\t\t\t </qual:listOfInputs>\n")
    cat(file=f,"\t\t\t\t <qual:listOfOutputs>\n")
    cat(file=f,"\t\t\t\t\t <qual:output qual:qualitativeSpecies=\"", gene ,"\" qual:transitionEffect=\"assignmentLevel\"/>\n",sep = "")
    cat(file=f,"\t\t\t\t </qual:listOfOutputs>\n")
    cat(file=f, "\t\t\t\t<qual:listOfFunctionTerms>\n")
    cat(file=f, "\t\t\t\t\t<qual:defaultTerm qual:resultLevel=\"0\"/>\n")
    cat(file=f, "\t\t\t\t\t<qual:functionTerm qual:resultLevel=\"1\">\n")
    cat(file=f, "\t\t\t\t\t\t<math xmlns=\"http://www.w3.org/1998/Math/MathML\">\n")
    cat(file=f, "\t\t\t\t\t\t\t<apply>\n")
    count = 1
    if (length(inputs)!=0){
        if (length(inputs)>1)
            cat(file=f, "\t\t\t\t\t\t\t\t<", logic, "/>\n",sep = "")
        for (i in inputs){
            full_name <- ""
            full_name <- paste0("theta_", t_name, "_", i ,sep='')
            cat(file=f, "\t\t\t\t\t\t\t\t<apply>\n")
            if (signs[count] == 'positive')
                cat(file=f, "\t\t\t\t\t\t\t\t\t<geq/>\n")
            else 
                cat(file=f, "\t\t\t\t\t\t\t\t\t<lt/>\n")
            cat(file=f, "\t\t\t\t\t\t\t\t\t<ci>" , i , "</ci>\n")
            cat(file=f, "\t\t\t\t\t\t\t\t\t<ci>" , full_name , "</ci>\n")
            cat(file=f, "\t\t\t\t\t\t\t\t</apply>\n")
            count = count + 1
        }
    }
    cat(file=f, "\t\t\t\t\t\t\t</apply>\n")
    cat(file=f, "\t\t\t\t\t\t</math>\n")
    cat(file=f, "\t\t\t\t\t</qual:functionTerm>\n")
    cat(file=f, "\t\t\t\t</qual:listOfFunctionTerms>\n")
    cat(file=f, "\t\t\t</qual:transition>\n")
    if (logic == "and") 
        t_count = t_count + 1
    return(t_count)
    
    
}

toSBML <- function(network, file, bitString = c(rep(1,length(network$reacID))),version=c("standard","cellnopt"))
{

	version = match.arg(version)
    if(version=="standard"){
    	toSBMLStandard(network=network, file=file, bitString = bitString)
    	return()
    } 


    if (!requireNamespace("stringi", quietly = TRUE)) {
    stop("stringi needed for this function to work. Please install it.",
      call. = FALSE)
  }
  

    network = cutModel(network, bitString)
   
    #stop()
    # generate a network identifier from the file name
    id <- sub(".sbml", "", basename(file), fixed=TRUE)
    id <- gsub("[^a-zA-Z0-9_]+","_",id)
    
    # open a string connection
    output <- NULL
    f <- textConnection("output",  encoding="UTF-8", open="w", local=TRUE)
    
    # write document header
    cat(file=f, "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n")
    cat(file=f, "<sbml xmlns=\"http://www.sbml.org/sbml/level3/version1/core\" level=\"3\" version=\"1\" xmlns:qual=\"http://www.sbml.org/sbml/level3/version1/qual/version1\" qual:required=\"true\">\n")
    cat(file=f, "\t<model id=\"", id , "\">\n", sep="")
    
    # write default compartment
    cat(file=f, "\t\t<listOfCompartments>\n")
    cat(file=f, "\t\t\t<compartment id=\"default\" constant=\"true\"/>\n")
    cat(file=f, "\t\t</listOfCompartments>\n")
    
    # write genes
    geneList <- NULL
    cat(file=f, "\t\t<qual:listOfQualitativeSpecies>\n")
    for (gene in network$namesSpecies)
    {
        Input_interaction <- which(network$interMat[gene,] == 1)
        Output_Interaction <- which(network$interMat[gene,] == -1)
        if (length(Input_interaction) == 0 && length(Output_Interaction) == 0)next
        cat(file=f, "\t\t\t<qual:qualitativeSpecies qual:id =\"", gene, "\" qual:compartment=\"main\"",
            " qual:constant=\"false\"/>\n", sep = "")
        geneList <- c(geneList, gene)
        
    }
    cat(file=f, "\t\t</qual:listOfQualitativeSpecies>\n")
    
    # write transition functions
    cat(file=f, "\t\t<qual:listOfTransitions>\n")
    
    t_count = 1 
    
    for (gene in geneList)
    {
        #keep track of all interaction for a gene
        all_interactions <- which(network$interMat[gene,] == 1)
        or_interaction = c()
        and_interaction = c()
        #write all OR interactions 
        t_name <- ""
        t_name = paste0("t", t_count, sep='')
        cat(file=f, "\t\t\t<qual:transition qual:id=\"", t_name , "\">\n",sep = "")
        cat(file=f,"\t\t\t\t <qual:listOfInputs> \n")
        orSigns <- c()
        
        for (i in all_interactions){
            int_name <- colnames(network$interMat)[i]
            if (grepl("+", int_name,fixed = TRUE) == TRUE) {
                and_interaction = c(and_interaction,int_name)
                next
            }
            else{
                
                sign <- "positive"
                LHS <- unlist(strsplit(int_name,split = "="))[1] #input
                if (substr(LHS,1,1) == "!"){
                    sign <- "negative"
                    LHS <- stringi::stri_sub(LHS,2)
                }
                or_interaction = c(or_interaction, LHS)
                orSigns <- c(orSigns, sign)
                RHS <- unlist(strsplit(int_name,split = "="))[2] #gene -- output
                full_name <- ""
                full_name <- paste0("theta_", t_name, "_",LHS,sep='')
                cat(file=f,"\t\t\t\t\t <qual:input  qual:id=\"", full_name , 
                    "\" qual:qualitativeSpecies=\"", LHS, "\" qual:transitionEffect=\"none\" qual:sign=\"", sign, "\" qual:thresholdLevel=\"1\"/>\n",sep = "")
            }
        }
        #write the logical headers for the OR transitions
        
        writeLogic(gene,or_interaction,t_name, t_count, "or", f, orSigns)
        
        #if there are AND interactions, create new transition
        if (length(and_interaction)==0)
            t_count = t_count + 1
        for (i in and_interaction){
            t_count = t_count + 1
            and_list = c()
            andSigns = c()
            t_name <- ""
            t_name = paste0("t", t_count, sep='')
            cat(file=f, "\t\t\t<qual:transition qual:id=\"", t_name ,  "\">\n",sep = "")
            cat(file=f,"\t\t\t\t <qual:listOfInputs> \n")
            tmp = unlist(strsplit(i,split = "="))[1] #input
            LHS = unlist(strsplit(tmp,split = "+",fixed = TRUE))
            for (input in LHS){
                sign <- "positive"
                if(substr(input,1,1) == "!"){
                    input = stringi::stri_sub(input,2)
                    sign <- "negative"
                }
                and_list = c(and_list,input)
                andSigns = c(andSigns, sign)
                full_name <- ""
                full_name <- paste0("theta_", t_name, "_", input, sep='')
                cat(file=f,"\t\t\t\t\t <qual:input  qual:id=\"", full_name , 
                    "\" qual:qualitativeSpecies=\"",input,"\" qual:transitionEffect=\"none\" qual:sign=\"", sign, "\" qual:thresholdLevel=\"1\"/>\n",sep = "")
            }
            t_count = writeLogic(gene,and_list,t_name, t_count, "and", f, andSigns)
            
        }
    }
    
    # finish document
    cat(file=f, "\t\t</qual:listOfTransitions>\n")
    cat(file=f, "\t</model>\n")
    cat(file=f, "</sbml>\n")
    close(f)
    
    # open file and write the complete XML string
    f <- file(file, encoding="UTF-8", open="w")
    cat(file=f,output,sep="\n")
    close(f)  
    
}

#  Copyright (c) 2019 - SaezLab - Heidelberg Universit
#
#  File author(s): CNO developers
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  CNO website: http://www.cellnopt.org
#
##############################################################################

create_binary_variables_for_all_reactions_z_i_in_experiment_k_i <- 
  function(model, numberOfExperiment){ 
    number_of_reactions <- length(model$reacID)
    identifier <- c()
    variables <- c()
    for(i in 1:number_of_reactions){
      identifier[i] <- paste("reaction", model$reacID[i], "in experiment k =", 
                            numberOfExperiment)
      variables[i] <- paste0("z_",i,"^",numberOfExperiment)
    }
    binary_variables_list <- list(0:length(model$reacID),variables, identifier)
    return(binary_variables_list)
}

#
#  This file is part of the CNO software
#
#  Copyright (c) 2019 - SaezLab - Heidelberg Universit
#
#  File author(s): CNO developers
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  CNO website: http://www.cellnopt.org
#
##############################################################################

writeObjectiveFunction <- function(model, 
                                   midasExperimentPart, 
                                   y_vector=y_vector,
                                   accountForModelSize=TRUE, 
                                   sizeFac = 0.000001, 
                                   meansOfMeasurements_at_t0, 
                                   method = "quadratic" ){
  
  # creating auxilliary objects
  reducedMidas <- midasExperimentPart
  whichObjective <- method
  objective1 <- c()
  
  # writing the objective function
  linker_vector <- link_midas_colnames_to_namesSpecies(model, reducedMidas)
  weight_of_unstimulated_experiments <- if(!is.null(dim(reducedMidas)[1])){dim(
    reducedMidas)[1]+1}else{1}
  constant <- 0
  factors <- "obj:\t"
  if(whichObjective == "linear"){
    for(i in 1:dim(reducedMidas)[1]){
      
      for(j in 1:dim(reducedMidas)[2]){
        if(!is.na(reducedMidas[i,j])){ 
          ##term1:
          if(i ==1){
            term1 <- weight_of_unstimulated_experiments * 
              meansOfMeasurements_at_t0[j]
          }else{
            term1 <- reducedMidas[i,j]
          }
          ##term2:
          if(i ==1){
            term2 <- weight_of_unstimulated_experiments * 
              (1-2*meansOfMeasurements_at_t0[j])
          }else{
            term2 <- 1-2*reducedMidas[i,j]
          }
          
          binary_index <- length(model$reacID) + 
            (i-1)*length(model$namesSpecies) + 
            linker_vector[j]  
          term3 <- paste0("xb", binary_index)
          constant <- constant + term1
          
          if(term2<0){
            factors  <- paste(factors,term2,term3)
          }else{
            factors <- paste(factors, "+", term2, term3)
          }
          
        }
        
        
      }
      if(accountForModelSize){
        for(m in 1:length(y_vector)){
          factors <- paste0(factors, " + ", sizeFac, " xb", m )
        }
      }
      
    }
    # here, the model size is accounted for:
    for(m in 1:length(y_vector)){
      factors <- paste0(factors, " + ", sizeFac, " xb", m )
    }
    
    out <- list(constant, factors) #see comment in the functon definition above
    
    
  } else if(whichObjective =="quadratic")
  {
    for(i in 1:dim(reducedMidas)[1]){
      
      for(j in 1:dim(reducedMidas)[2]){
        
        if(!is.na(reducedMidas[i, j])){
          
          binary_index <- length(model$reacID) + 
            (i-1)*length(model$namesSpecies) + linker_vector[j]
          #real term1:
          term1 <- paste0("xb", binary_index)
          #term2:
          if(reducedMidas[i, j]>=0){
            term2 <- paste0(2*abs(reducedMidas[i,j]), " xb", binary_index)
          } else {
            term2 <- paste0(0, " xb", binary_index)
          }
          #real term3:
          term3 <- reducedMidas[i,j] *reducedMidas[i,j]
          
          factors  <- paste(factors,"+", term1,"-", term2)#,"+", term3)
          
        }
        
        
      }
      
      if(accountForModelSize){
        for(m in 1:length(y_vector)){
          factors <- paste0(factors, " + ", sizeFac, " xb", m )
        }
      }
      
      out <- list(constant, factors)
    }
  }
  
  return(out)
  
}

#
#  This file is part of the CNO software
#
#  Copyright (c) 2011-2012 - EMBL - European Bioinformatics Institute
#
#  File author(s): CNO developers (cno-dev@ebi.ac.uk)
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  CNO website: http://www.ebi.ac.uk/saezrodriguez/cno
#
##############################################################################
# $Id$
prep4sim<-function(model){
  
  #Compute the max number of inputs observed in the model for a single reaction
  maxInput<-colSums(model$interMat)
  maxInput<-abs(min(maxInput))+1
  
  #Make the empty matrices
  finalCube<-matrix(NA, nrow=length(model$reacID),ncol=maxInput)
  ixNeg<-matrix(FALSE, nrow=length(model$reacID),ncol=maxInput)
  ignoreCube<-matrix(TRUE, nrow=length(model$reacID),ncol=maxInput)
  maxIx<-rep(NA,length(model$reacID))
  
  #Fill the matrices finalCube, ignoreCube and ixNeg, and maxIx
  
  for(r in 1:length(model$reacID)){
    
    input<-which(model$interMat[,r] == -1)
      
    if (length(input)>0){
      finalCube[r,1:length(input)]<-input
      
      if(length(input) < maxInput) finalCube[r,(length(input)+1):maxInput]<-1
      
      neg<-model$notMat[input,r]
      ixNeg[r,1:length(input)]<-(neg == 1)
      ignoreCube[r,1:length(input)]<-FALSE
      maxIx[r]<-which(model$interMat[,r] == 1)
    }
    
  }
  
  rownames(finalCube)<-model$reacID
  rownames(ixNeg)<-model$reacID
  rownames(ignoreCube)<-model$reacID
  names(maxIx)<-model$reacID
  
  modelname<-deparse(substitute(model))
  
  # check for self-loops here
  # rescan reactions for self-loops
  for(r in 1:length(model$reacID)){
    # pick out species
    lhs = strsplit(model$reacID[r], "=")[[1]][1]
    rhs = strsplit(model$reacID[r], "=")[[1]][2]
    lhs = unlist(strsplit(lhs, "\\+"))
    # notmat is okay so no need to record if negative or not
    lhs = gsub("!","",lhs)
    if(any(lhs==rhs)) { # this is a self loop with/without AND gate
        input = sapply(lhs, function(x) which(model$namesSpecies == x))
        
        finalCube[r,1:length(input)]<-input
        if(length(input) < maxInput) finalCube[r,(length(input)+1):maxInput]<-1
        
        neg<-model$notMat[input,r]
        ixNeg[r,1:length(input)]<-(neg == 1)
        ignoreCube[r,1:length(input)]<-FALSE
        maxIx[r]<-which(model$interMat[,r] == 1)
        
      }
    }
  
  Fields4Sim<-list(
    finalCube=finalCube,
    ixNeg=ixNeg,
    ignoreCube=ignoreCube,
    maxIx=maxIx,
    modelname=modelname, maxInput=maxInput)
  
  return(Fields4Sim)
}

#' Data used for the DREAM3 challenge
#'
#' This data object contains the model used in the package vignette, already 
#' loaded and formatted as a Model object. This is to be used with the data 
#' in "CNOListDREAM"
#'
#' J. Saez-Rodriguez, L. G. Alexopoulos, J. Epperlein, R. Samaga, D. A.
#' Lauffenburger, S. Klamt and P. K. Sorger. Discrete logic modeling as a means to
#' link protein signaling networks with functional analysis of mammalian signal
#' transduction, Molecular Systems Biology, 5:331, 2009.
#' 
#' Prill RJ, Marbach D, Saez-Rodriguez J, Sorger PK, Alexopoulos LG, Xue X,
#' Clarke ND, Altan-Bonnet G, and Stolovitzky G. Towards a rigorous assessment of
#' systems biology models: the DREAM3 challenges. PLoS One, 5(2):e9202, 2010.
#' @format DreamModel is a list with fields "reacID" (character vector), "namesSpecies" (character vector), "interMat" (numerical matrix), "notMat"(numerical matrix).
#' @usage DreamModel
#' @aliases LiverDREAM, DreamModel
"DreamModel"

#
#  This file is part of the CNO software
#
#  Copyright (c) 2011-2012 - EMBL - European Bioinformatics Institute
#
#  File author(s): CNO developers (cno-dev@ebi.ac.uk)
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  CNO website: http://www.cellnopt.org
#
##############################################################################
# $Id$


writeSIF <- function(model, filename, overwrite=FALSE){

    # convert internal model structure to a SIF matrix
    sif = model2sif(model)

    # and save it into a file
    if (file.exists(filename)==FALSE){
        write.table(sif[,1:3],file=filename,
            row.names=FALSE,col.names=FALSE,quote=FALSE,sep="\t")
    }
    else{
       if (overwrite==TRUE){
            write.table(sif[,1:3],file=filename,
                row.names=FALSE,col.names=FALSE,quote=FALSE,sep="\t")
        }
        else{
           stop(paste("File ", filename, "already exists.",  sep=""))
        }
    }


}

#' ## Crossvalidation study
#' crossvalidate
#' 
#' k-fold crossvalidation for Boolean model
#' 
#' Does a k-fold cross-validation for Boolean CellNOpt models. In k-iterations a 
#' fraction of the data is eliminated from the CNOlist. The model is trained 
#' on the 
#' remaining data and then the model predicts the held-out data. Then the 
#' prediction
#' accuracy is reported for each iteration. 
#' 
#' @param CNOlist Cnolist which contains all the experiments  
#' @param model a model prepared for the training  
#' @param nfolds number of folds - default is 10. Although nfolds can be as 
#' large as the sample size (leave-one-out CV),
#'  it is not recommended for large datasets.  
#' @param foldid an optional vector of values between `1` and `nfold` 
#' identifying what fold each observation is in. If supplied, `nfold` can be 
#' missing.  
#' @param type define the way to do the crossvalidation. The default is 
#' `type="datapoint"`, which assigns the data randomly into folds. 
#' The option `type="experiment"` uses whole experiments for crossvalidation 
#' (all data corresponding to a cue combination). The `type=observable` uses the
#' subset of nodes across all experiments for crossvalidation.  
#' @param ... further arguments are passed to gaBinaryT1  
#' @seealso \link{\code{gaBinaryT1}}  
#' @import foreach

crossvalidateBoolean <- function(CNOlist,model,nfolds=10,foldid=NULL, 
								 type=c('datapoint','experiment','observable'),
								 timeIndex = 2,parallel=FALSE, ...){
	
	if (!is(CNOlist,"CNOlist")){
		CNOlist = CellNOptR::CNOlist(CNOlist)
	}
	
	crossvalidate.call = match.call(expand.dots = TRUE)
	
	if(!(type%in%c('datapoint','experiment','observable'))){
		
		error("Wrong argument for type. It should either be 'datapoint', 
          'experiment' or 'observable'")
		
	} else {
		
		if(type=='datapoint'){
			N = prod(dim(CNOlist@signals[[timeIndex]]))
		}else if(type=='experiment'){
			N = nrow(CNOlist@signals[[timeIndex]])
		}else if(type=="observable"){
			N = ncol(CNOlist@signals[[timeIndex]])
		}
		
	}
	
	if (is.null(foldid)){
		
		foldid = sample(rep(seq(nfolds), length = N))
		
	} else {
		
		nfolds = max(foldid)
		
	}
	
	outlist = as.list(seq(nfolds))
	
	if (parallel) {
		if (!requireNamespace("doParallel", quietly = TRUE)) {
			stop("Package \"doParallel\" needed for this function to work. Please install it.",
				 call. = FALSE)
		}
		
		`%dopar%` <- foreach::`%dopar%`
		outlist = foreach::foreach(i = seq(nfolds), .packages = c("CellNOptR")) %dopar%
			{
				whichI = foldid == i
				CNOlist.sub = CNOlist
				CNOlist.cv = CNOlist
				if(type=='datapoint'){
					CNOlist.sub@signals[[timeIndex]][whichI] = NA
					CNOlist.cv@signals[[timeIndex]][!whichI] = NA
				}else if(type=='experiment'){
					CNOlist.sub@signals[[timeIndex]][whichI,] = NA
					CNOlist.cv@signals[[timeIndex]][!whichI,] = NA
				}else if(type=="observable"){
					CNOlist.sub@signals[[timeIndex]][,whichI] = NA
					CNOlist.cv@signals[[timeIndex]][,!whichI] = NA
				}
				
				outlist = list()
				outlist$fit = gaBinaryT1(CNOlist = CNOlist.sub, 
										 model=model, timeIndex = timeIndex, verbose = FALSE)
				
				outlist$cvScore = computeScoreT1(CNOlist = CNOlist.cv,model = model,
												 bString = outlist$fit$bString,
												 timeIndex = timeIndex)
				outlist
			}
		
	}
	else {
		cat("fold \t fitScore \t cvScore \t nTolModels \t bString\n")
		
		for (i in seq(nfolds)) {
			whichI = foldid == i
			CNOlist.sub = CNOlist
			CNOlist.cv = CNOlist
			if(type=='datapoint'){
				CNOlist.sub@signals[[timeIndex]][whichI] = NA
				CNOlist.cv@signals[[timeIndex]][!whichI] = NA
			}else if(type=='experiment'){
				CNOlist.sub@signals[[timeIndex]][whichI,] = NA
				CNOlist.cv@signals[[timeIndex]][!whichI,] = NA
			}else if(type=="observable"){
				CNOlist.sub@signals[[timeIndex]][,whichI] = NA
				CNOlist.cv@signals[[timeIndex]][,!whichI] = NA
			}
			
			outlist[[i]] = list()
			outlist[[i]]$fit = gaBinaryT1(CNOlist = CNOlist.sub, 
										  model=model, timeIndex = timeIndex, 
										  verbose = FALSE, ...)
			
			outlist[[i]]$cvScore = computeScoreT1(CNOlist = CNOlist.cv,model = model,
												  bString = outlist[[i]]$fit$bString,
												  timeIndex = timeIndex)
			cat(i,"\t",outlist[[i]]$fit$bScore,"\t",outlist[[i]]$cvScore,"\t",
				length(outlist[[i]]$fit$stringsTolScores),"\t\t",
				paste(outlist[[i]]$fit$bString,collapse=","),"\n")
		}
	}
	
	cvScores = data.frame(folds=seq(nfolds), cvScore= 
						  	unlist(lapply(outlist,function(x)x$cvScore)))
	fitScores = data.frame(folds=seq(nfolds), bScore= 
						   	unlist(lapply(outlist,
						   				  function(x)x$fit[c("bScore")])))
	bStrings = data.frame(folds=seq(nfolds), 
						  do.call("rbind",lapply(outlist,
						  					   function(x)matrix(x$fit$bString,
						  					   				  nrow=1))))
	colnames(bStrings) = c('folds', model$reacID)
	
	
	out = list(cvScores=cvScores,
			   fitScores=fitScores,
			   bStrings=bStrings,
			   crossvalidate.call=crossvalidate.call,
			   foldid=foldid)
	return(out)
}

#
#  This file is part of the CNO software
#
#  Copyright (c) 2011-2012 - EMBL - European Bioinformatics Institute
#
#  File author(s): CNO developers (cno-dev@ebi.ac.uk)
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  CNO website: http://www.cellnopt.org
#
##############################################################################
# $Id$
simulatorT2 <-function(
    simResultsT1,
    CNOlist,
    model,
    simList,
    indexList,
    timeIndex=3){

    warning("simulatorT2 is deprecated. Use simulatorTN instead, which has the
same prototype and is using a C simulator. You can still use this function but
it is not giong to be maintained after version 1.4.0. " )

    if (!is(CNOlist,"CNOlist")){
         CNOlist = CellNOptR::CNOlist(CNOlist)
    }

    nSp<-dim(model$interMat)[1]
    nReacs<-dim(model$interMat)[2]
    nCond<-dim(CNOlist@stimuli)[1]
    maxIpg<-dim(simList$finalCube)[2]

    if(is.null(dim(model$interMat))){
        nSp<-length(model$interMat)
        nReacs<-1
        maxIpg<-length(simList$finalCube)
    }

    #This holds, for each sp, how many reactions have that sp as output
    endIx<-rep(NA,nSp)
    for(i in 1:nSp){
        endIx[i]<-length(which(simList$maxIx == i))
        }
    maxgpo<-max(endIx)

#This value is used to test the stop condition for difference between 2 iterations
    testVal<-1E-3

#Create an initial values matrix
    initValues<-simResultsT1

#Initialise main loop
    newInput<-initValues
    termCheck1<-TRUE
    termCheck2<-TRUE
    count<-1

#First iteration
    outputPrev<-newInput
    tempStore<-apply(simList$finalCube,2,function(x){return(outputPrev[,x])})

        filltempCube<-function(x){
            cMatrix<-matrix(data=x,nrow=dim(simList$ixNeg)[1],ncol=nCond)
            cVector<-apply(cMatrix,1,function(x){return(x)})
            return(cVector)
            }

    tempIxNeg<-apply(simList$ixNeg,2,filltempCube)
    tempIgnore<-apply(simList$ignoreCube,2,filltempCube)
    tempStore[tempIgnore]<-NA
    tempStore[tempIxNeg]<-1-tempStore[tempIxNeg]

    minNA<-function(x){
        if(all(is.na(x))){
            return(NA)
            }else{
                return(min(x,na.rm=TRUE))
                }
        }

    outputCube<-apply(tempStore,1,minNA)
    outputCube<-matrix(outputCube, nrow=nCond,ncol=nReacs)

        #rewrite anything that comes to a node that also receives a t2 branch,ie set it to the same
        #as our t2 reac for those reacs, so that they won't influence the OR
    reacsT2<-which(model$times == (timeIndex-1))

    if(length(reacsT2) > 0){

        for(i in reacsT2){
            outNode<-which(model$interMat[,i] > 0)
            reacs2Overwrite<-which(model$interMat[outNode,] > 0)

            if(length(reacs2Overwrite) != 0) {
                for(n in 1:length(reacs2Overwrite)){
                    outputCube[,reacs2Overwrite[n]]<-outputCube[,i]
                    }
                }

            }

        }

    for(s in 1:nSp){

        if(endIx[s] != 0){
            compOR<-function(x){
                if(all(is.na(x[which(simList$maxIx == s)]))){
                    res<-NA
                    }else{
                        res<-max(x[which(simList$maxIx == s)],na.rm=TRUE)
                        }
                return(res)
                }
            newInput[,s]<-apply(outputCube,1,compOR)
            }

        }

    for(stim in 1:length(indexList$stimulated)){

        stimM<-cbind(CNOlist@stimuli[,stim],newInput[,indexList$stimulated[stim]])

        maxNA<-function(x){
            return(max(x,na.rm=TRUE))
            }

        stimV<-apply(stimM,1,maxNA)
        newInput[,indexList$stimulated[stim]]<-stimV

        }

    valueInhibitors<-1-CNOlist@inhibitors
    newInput[,indexList$inhibited]<-valueInhibitors*newInput[,indexList$inhibited]
    newInput[is.na(newInput)]<-0
    outputPrev[is.na(outputPrev)]<-0
    termCheck1<-!all(abs(outputPrev-newInput)<testVal)
    termCheck2<-(count < (nSp*1.2))
    count<-count+1
    firstIter<-newInput

#Main loop

    while(termCheck1 && termCheck2){

        outputPrev<-newInput

#This is now a 2 columns matrix that has a column for each input (column in finalCube)
#and a set of rows for each reac (where a set contains as many rows as conditions)
#all concatenated into one long column
        tempStore<-apply(simList$finalCube,2,function(x){return(outputPrev[,x])})
        tempIxNeg<-apply(simList$ixNeg,2,filltempCube)
        tempIgnore<-apply(simList$ignoreCube,2,filltempCube)

#Set to NA the values that are "dummies", so they won't influence the min
        tempStore[tempIgnore]<-NA

#Flip the values that enter with a negative sign
        tempStore[tempIxNeg]<-1-tempStore[tempIxNeg]

#Compute all the ands by taking, for each gate, the min value across the inputs of that gate
        outputCube<-apply(tempStore,1,minNA)

#OutputCube is now a vector of length (nCond*nReacs) that contains the input of each reaction in
#each condition, concatenated as such allcond4reac1,allcond4reac2,etc...
#This is transformed into a matrix with a column for each reac and a row for each cond
        outputCube<-matrix(outputCube, nrow=nCond,ncol=nReacs)

#Go through each species, and if it has inputs, then take the max across those input reactions
#i.e. compute the ORs
        for(s in 1:nSp){
            if(endIx[s] != 0){
                compOR<-function(x){
                    if(all(is.na(x[which(simList$maxIx == s)]))){
                        res<-NA
                        }else{
                            res<-max(x[which(simList$maxIx == s)],na.rm=TRUE)
                            }
                    return(res)
                    }
                newInput[,s]<-apply(outputCube,1,compOR)
                }
            }

#Reset the inhibitors and stimuli
        for(stim in 1:length(indexList$stimulated)){
            stimM<-cbind(CNOlist@stimuli[,stim],newInput[,indexList$stimulated[stim]])
            maxNA<-function(x){
                return(max(x,na.rm=TRUE))
                }
            stimV<-apply(stimM,1,maxNA)
            newInput[,indexList$stimulated[stim]]<-stimV
            }
        valueInhibitors<-1-CNOlist@inhibitors
        newInput[,indexList$inhibited]<-valueInhibitors*newInput[,indexList$inhibited]

#Set all the nodes that are targets of a t2 reaction to the state that they had at the first iteration
        if(length(reacsT2) != 0){
            t2reacs<-model$interMat[,reacsT2]
            for(r in 1:length(reacsT2)){
                if(length(reacsT2) == 1) {
                    target<-which(t2reacs > 0)
                    }else{
                        target<-which(t2reacs[,r] > 0)
                        }
                newInput[,target]=firstIter[,target]
                }
            }

#Replace NAs with zeros to avoid having the NA    penalty applying to unconnected species
        newInput[is.na(newInput)]<-0
        outputPrev[is.na(outputPrev)]<-0

#Check the 2 termination conditions
        termCheck1<-!all(abs(outputPrev-newInput)<testVal)
        termCheck2<-(count < (nSp*1.2))
        count<-count+1
        }

#Set the non-resolved bits to NA
    newInput[which(abs(outputPrev-newInput) > testVal)]<-NA
    return(newInput)

    }

#
#  This file is part of the CNO software
#
#  Copyright (c) 2011-2012 - EMBL - European Bioinformatics Institute
#
#  File author(s): CNO developers (cno-dev@ebi.ac.uk)
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  CNO website: http://www.ebi.ac.uk/saezrodriguez/cno
#
##############################################################################
# $Id$

## given a MIDAS file, returns a CNOlist object
## You may want to make this (and the existing readMIDAS)
## into S4 generics in order to avoid a name conflict.





## Class definition
setClass("CNOlist",
    representation(
        cues="matrix",
        inhibitors="matrix",
        stimuli="matrix",
        signals="list",
        variances="list",
        timepoints="vector"),
    ## validity method
    validity=function(object) {
        msg <- NULL
        nrow <- nrow(getCues(object))
        signalNrows <- unique(sapply(signals(object), nrow))
        if (nrow != nrow(inhibitors(object)) ||
            nrow != nrow(stimuli(object)) ||
            length(signalNrows) != 1 ||
            nrow != signalNrows)
        {
            msg <- "'nrow' differs between elements"
        }
        if (is.null(msg)) TRUE else msg
    })


## constructor
CNOlist <-function(data, verbose=FALSE){

    res = NULL
    # input can a filename or the old (still used) CNOlist returned by
    # makeCNOlist function. 
    if (is.character(data)== TRUE){
        res = internal_CNOlist_from_file(data, verbose)
    }

    if (is.list(data)==TRUE){
        if ("namesCues" %in% names(data) == TRUE){
            res = internal_CNOlist_from_makeCNOlist(data) 
        }else{
            stop("Not a valid list. Does not seem to be returned by CellNOptR::makeCNOlist")
        }
    }

    if (is(CNOlist,"CNOlist")){
        # nothing to do already a cnolist
        res = list(
                cues=data@cues,
                inhibitors=data@inhibitors,
                stimuli=data@stimuli,
                signals=data@signals,
                variances=data@variances,
                timepoints=data@timepoints)
    }
 
    if (is.null(res)==TRUE){
        stop("Input data must be a filename or the output of CellNOptR::makeCNOlist function")
    }

    new("CNOlist", cues=res$cues, inhibitors=res$inhibitors,
        stimuli=res$stimuli, signals=res$signals, variances=res$variances, timepoints=res$timepoints)
}

setGeneric("getCues", function(object){standardGeneric("getCues")})
setGeneric("getInhibitors", function(object){standardGeneric("getInhibitors")})
setGeneric("getStimuli", function(object){standardGeneric("getStimuli")})
setGeneric("getSignals", function(object){standardGeneric("getSignals")})
setGeneric("getVariances", function(object){standardGeneric("getVariances")})
setGeneric("getTimepoints", function(object){standardGeneric("getTimepoints")})


setMethod("getCues", "CNOlist", function(object){return(object@cues)})
setMethod("getInhibitors", "CNOlist", function(object){return(object@inhibitors)})
setMethod("getStimuli", "CNOlist", function(object){return(object@stimuli)})
setMethod("getSignals", "CNOlist", function(object){return(object@signals)})
setMethod("getVariances", "CNOlist", function(object){return(object@variances)})
setMethod("getTimepoints", "CNOlist", function(object){
    # timepoints may have been modify on the fly so let us recomput it
    object@timepoints = names(object@signals)
    return(object@timepoints)

})



# timepoints will be updated if signals is changed so we should not provide any
# setTimepoint method


setGeneric("setSignals<-",function(object,value){standardGeneric("setSignals<-")})
setReplaceMethod("setSignals","CNOlist", 
    function(object,value){
        object@signals<-value
        return(object)
    }
)

## internal accessors only ??
#cues <- function(cnoList, ...) cnoList@cues
inhibitors <- function(cnoList, ...) cnoList@inhibitors
stimuli <- function(cnoList, ...) cnoList@stimuli
signals <- function(cnoList, ...) cnoList@signals


## show method
setMethod(show, "CNOlist", function(object) {
    cat("class:", class(object), "\n")
    cat("cues:", colnames(getCues(object)), "\n")
    cat("inhibitors:", colnames(getInhibitors(object)), "\n")
    cat("stimuli:", colnames(getStimuli(object)), "\n")
    cat("timepoints:", names(signals(object)), "\n")
    cat("signals:", colnames(signals(object)[[1]]), "\n")
    cat("variances:", colnames(signals(object)[[1]]), "\n")
    cat("--\nTo see the values of any data contained in this instance, just use the
appropriate getter method (e.g., getCues(cnolist), getSignals(cnolist), ...\n\n")
})

#setMethod("plot", signature(x="CNOlist", y="missing"), function(x, y, ...){
#    plotCNOlist(x)
#})

setMethod("plot", "CNOlist", function(x, y, ... ){
    plotCNOlist(x)
})
setMethod("plot", signature(x="CNOlist", y="CNOlist"), function(x, y, ... ){
    plotCNOlist2(x,y)
})

setMethod("length", "CNOlist", function(x) length(x@signals))



setGeneric(name="readErrors",
           def=function(object,filename){standardGeneric("readErrors")})
setMethod("readErrors", "CNOlist", 
          definition=function(object, filename){
          # TODO check compatibility cnolist object and errors
          # read Errors from a file and save into the cnolist@variance field
          errors = CNOlist(filename)
          for (i in seq_along(object@signals)){
              object@variances[[i]] = errors@signals[[i]]
          }
          # looks like we cannot change on the fly but have to return the object
          return(object)
})

setGeneric(name="writeErrors",
           def=function(object,filename, overwrite=FALSE){standardGeneric("writeErrors")})
setMethod("writeErrors", "CNOlist", 
          definition=function(object, filename,overwrite=FALSE){
          # let us make a copy
          # TODO check compatibility cnolist object and errors
          # copy variance from object Errors from a file and save into the cnolist@variance field
          errors = object
          for (i in seq_along(object@signals)){
              errors@signals[[i]] = object@variances[[i]]
          }
          writeMIDAS(errors, filename,overwrite=overwrite)
})

# if cellnopt is loaded, then isGeneric("randomize") is true and then the setGeneric
# is not evaluated, leading to an error " Error in setMethod(... no existing definition.."
# Solution: restart the R session and run check without loading CellNOptR
#if (isGeneric("randomize")==FALSE){
    setGeneric(
        name="randomize",
        def=function(object,sd=0.1, minValue=0,maxValue=1,mode="gaussian"){standardGeneric("randomize")}
    )
#}
#lockBinding("randomize", .GlobalEnv)


setMethod("randomize", "CNOlist", 
    definition=function(object, sd=0.1, minValue=0, maxValue=1,mode="uniform"){
        res = randomizeCNOlist(object, sd=sd, mode=mode)
        return(res)
    }
)

# a method that convert back the CNOlist to a makeCNOlist useful for back
# compatibility with other packages (e.g. ODE)
setGeneric(
    name="compatCNOlist",
    def=function(object){standardGeneric("compatCNOlist")})
setMethod("compatCNOlist", "CNOlist",
    definition=function(object){
        return(internal_compatCNOlist(object))})


internal_compatCNOlist<-function(cnolist){
	if (is(cnolist,"CNOlist")){

        # conversion
        res = list(
             namesCues=colnames(cnolist@cues),
             namesStimuli=colnames(cnolist@stimuli),
             namesInhibitors=colnames(cnolist@inhibitors),
             namesSignals=colnames(cnolist@signals[[1]]),
             timeSignals=getTimepoints(cnolist),
             valueCues=cnolist@cues,
             valueInhibitors=cnolist@inhibitors,
             valueStimuli=cnolist@stimuli,
             valueVariances=cnolist@variances,
             valueSignals=cnolist@signals)

    } else{
        res = cnolist
    }

    return(res)
}



# used by the constructor not for export.
# open a MIDAS file (given a  filename) and create the instance of CNOlist
internal_CNOlist_from_file <- function(MIDASfile, verbose=FALSE)
{
    x <- readMIDAS(MIDASfile, verbose=verbose)

    # in the old makeCNOlist, there is a need for subfield. In cnolist, we
    # automatically figure it out here below:
    names = colnames(x$dataMatrix[x$TRcol])
    
    # Are all TR coded using subfield ?
    all_subfield_s = sapply(names, function(x){x = grepl(":Stimuli", x)})
    all_subfield_i = sapply(names, function(x){x = grepl(":Inhibitors", x)})
    all_subfield = all(all_subfield_s | all_subfield_i)

    # any subfield used ? 
    any_subfield_s = any(sapply(names, function(x){x = grepl(":Stimuli", x)}))
    any_subfield_i = any(sapply(names, function(x){x = grepl(":Inhibitors", x)}))

    

    any_subfield = any_subfield_s | any_subfield_i

    if (all_subfield == FALSE && any_subfield == TRUE){
        stop("If you use subfield in the MIDAS header, they should be used for all treatments adding :Stimuli or :Inhibitors after the species name (e.g., TR:<specy name>:Stimuli)")
    }

    if (all_subfield == FALSE){
        subfield = FALSE
    } else{
        subfield = TRUE
    }
    cnolist <- makeCNOlist(x, subfield=subfield, verbose=verbose)


    res <- internal_CNOlist_from_makeCNOlist(cnolist)
    return(res)
}


# used by the constructor not for export.
# open a MIDAS file (given a  filename) and create the instance of CNOlist
internal_CNOlist_from_makeCNOlist <- function(cnolist)
{

    myCues <- cnolist$valueCues
    colnames(myCues) <- cnolist$namesCues

    myInhibitors <- cnolist$valueInhibitors
    colnames(myInhibitors) <- cnolist$namesInhibitors

    myStimuli <- cnolist$valueStimuli
    colnames(myStimuli) <- cnolist$namesStimuli

    mySignals <- cnolist$valueSignals
    names(mySignals) <- cnolist$timeSignals
    mySignals <- lapply(mySignals, "colnames<-", cnolist$namesSignals)

    if ("valueVariances" %in% names(cnolist)){
        myVars <- cnolist$valueVariances
        myVars <- lapply(myVars, "colnames<-", cnolist$namesSignals)
    } else{
        myVars = mySignals
        for (time in 1:length(mySignals)){
            myVars[[time]] = myVars[[time]] * NA
        }
    }

    myTimePoints <- as.numeric(cnolist$timeSignals)

    #CNOlist(myCues, myInhibitors, myStimuli, mySignals)
    return( list(cues=myCues, inhibitors=myInhibitors, stimuli=myStimuli,
        signals=mySignals, variances=myVars, timepoints=myTimePoints))
}

#
#  This file is part of the CNO software
#
#  Copyright (c) 2011-2012 - EMBL - European Bioinformatics Institute
#
#  File author(s): CNO developers (cno-dev@ebi.ac.uk)
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  CNO website: http://www.cellnopt.org
#
##############################################################################
# $Id$
graph2sif<-function(graph,writeSif=FALSE, filename="Graph"){
  
  ed<-edges(graph)
  no<-nodes(graph)
  sifFile<-matrix(ncol=3)
  
  for (i in 1:length(no)){
    node1<-no[i]
    if (length(ed[[i]])>0){
      for (j in 1:length(ed[[i]])){
        node2<-ed[[i]][j]
        sig<-sign(as.numeric(edgeData(graph,no[i],ed[[i]][j], "weight")))
        sifFile<-rbind(sifFile,c(node1,sig,node2))
      }
    }
  }
  
  sifFile<-sifFile[-1,]
  
  if (writeSif==TRUE){
    filename<-paste(filename, ".sif", sep="")
    write.table(sifFile, file=filename, row.names=FALSE,col.names=FALSE,quote=FALSE,sep="\t")
  }

  return(sifFile)
}

#' Data used for the DREAM3 challenge
#'
#' This data object contains the DREAM data used in the package vignette, 
#' already loaded and formatted as a CNOlist object. This is to be used with 
#' the model "DreamModel". This is a data collected on HepG2 cells cultivated
#' with or without stimulation of tgfa, ilk, mek12, pi3k and p38, in
#' combination with inhibition of igf1 and/or il1a.  Seven phosphoproteins
#' are measured using Luminex xMAP assays: akt, erk12, ikb, jnk12, p38, 
#' hsp27 and mek12.
#'
#' J. Saez-Rodriguez, L. G. Alexopoulos, J. Epperlein, R. Samaga, D. A.
#' Lauffenburger, S. Klamt and P. K. Sorger. Discrete logic modeling as a means to
#' link protein signaling networks with functional analysis of mammalian signal
#' transduction, Molecular Systems Biology, 5:331, 2009.
#' 
#' Prill RJ, Marbach D, Saez-Rodriguez J, Sorger PK, Alexopoulos LG, Xue X,
#' Clarke ND, Altan-Bonnet G, and Stolovitzky G. Towards a rigorous assessment of
#' systems biology models: the DREAM3 challenges. PLoS One, 5(2):e9202, 2010.
#' 
#' @usage data(CNOlistDREAM)
"CNOlistDREAM"

#
#  This file is part of the CNO software
#
#  Copyright (c) 2011-2012 - EMBL - European Bioinformatics Institute
#
#  File author(s): CNO developers (cno-dev@ebi.ac.uk)
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  CNO website: http://www.ebi.ac.uk/saezrodriguez/cno
#
##############################################################################
# $Id: plotCNOlistLarge.R 3894 2013-08-08 12:19:23Z cokelaer $
#This function is a variant of plotCNOlist that works for bigger datasets and allows one
#to split the plots into n=nsplit plots (across the conditions dimension)

plotCNOlistLarge<-function(CNOlist,nsplit=4, newDevice=FALSE){

    if (!is(CNOlist,"CNOlist")){
         CNOlist = CellNOptR::CNOlist(CNOlist)
     }

    opar = par(no.readonly = TRUE)
    on.exit(par(opar))


    L = dim(CNOlist@cues)[1] # number of signals/Cues
    if (nsplit<=0 | nsplit>=L){
        stop("nsplit must be strictly positive and smaller than the number of signals")
    }


    splits<-dim(CNOlist@cues)[1]/nsplit
    splits<-floor(splits)
    CNOlistOriginal<-CNOlist

    for(i in 1:nsplit){
        if (newDevice==TRUE) { # this flag allows to plot the results in nsplit different
                               # devices, which is useful for scripting, otherwise only
                               # the last plot is shown
            dev.new()
        }
        CNOlist<-CNOlistOriginal

        if(i == nsplit){
            indices<-(indices[length(indices)]+1):dim(CNOlistOriginal@cues)[1]
            }

        indices<-((1:splits)+((i-1)*splits))
        CNOlist@cues<-CNOlist@cues[indices,]
        CNOlist@stimuli<-CNOlist@stimuli[indices,]
        CNOlist@inhibitors<-CNOlist@inhibitors[indices,]
        CNOlist@signals[[1]]<-CNOlist@signals[[1]][indices,]

        for(n in 2:length(CNOlist@signals)){
            CNOlist@signals[[n]]<-CNOlist@signals[[n]][indices,]
            }

        par(
            mfrow=c(nr=dim(CNOlist@signals[[1]])[1]+1,nc=dim(CNOlist@signals[[1]])[2]+2),
            cex=0.5,
            pch=20,
            mar=c(0.5,0.5,0,0),
            oma=c(3,2,2,2))
        yMax<-max(unlist(lapply(CNOlist@signals,function(x) max(x, na.rm=TRUE))))
        yMin<-min(unlist(lapply(CNOlist@signals,function(x) min(x, na.rm=TRUE))))
        xVal<-CNOlist@timepoints

        for(c in 1:dim(CNOlist@signals[[1]])[2]){
            par(fg="blue",mar=c(0.5,0.5,0.7,0))


            tryCatch(
                plot(x=xVal, y=rep(-5,length(xVal)), ylim=c(yMin, yMax),xlab=NA,ylab=NA,xaxt="n",yaxt="n"),
                error=function(e) stop(c("CNOError: an error occurred while
plotting. You may want to try again with a larger number of figures nsplit
by providing the nsplit option (suggestion: nsplit=",floor(L/25), ")")))

            text(
                labels=colnames(CNOlist@signals[[1]])[c],
                x=((xVal[length(xVal)]-xVal[1])/2),
                y=(yMin+((yMax-yMin)/2)),
                cex=1)
            }

        plot(
            x=xVal,
            y=rep(-5,length(xVal)),
            ylim=c(yMin, yMax),
            xlab=NA,ylab=NA,xaxt="n",yaxt="n")
        text(
            labels="Stim",
            x=((xVal[length(xVal)]-xVal[1])/2),
            y=(yMin+((yMax-yMin)/2)),cex=1)
        plot(
            x=xVal, y=rep(-5,length(xVal)),
            ylim=c(yMin, yMax),
            xlab=NA,ylab=NA,xaxt="n",yaxt="n")
        text(
            labels="Inh",
            x=((xVal[length(xVal)]-xVal[1])/2),
            y=(yMin+((yMax-yMin)/2)),cex=1)
        par(fg="black",mar=c(0.5,0.5,0,0))

        for(r in 1:dim(CNOlist@signals[[1]])[1]){

            for(c in 1:dim(CNOlist@signals[[1]])[2]){
                yVal<-lapply(CNOlist@signals,function(x) {x[r,c]})

                plot(x=xVal,y=yVal,ylim=c(yMin, yMax),xlab=NA,ylab=NA,xaxt="n",yaxt="n")
                lines(x=xVal,y=yVal,ylim=c(yMin, yMax),xlab=NA,ylab=NA,xaxt="n",yaxt="n")

                #add the axis annotations: if we're on the last row, add the x axis
                if(r == dim(CNOlist@signals[[1]])[1]){
                    axis(side=1,at=xVal)
                    }

                #add the axis annotations: if we're on the first column, add the y axis
                if(c == 1){
                    axis(side=2,at=c(-0.5,0,0.5))
                    }


                }

            image(
                t(matrix(CNOlist@stimuli[r,],nrow=1)),
                col=c("white","black"),xaxt="n",yaxt="n")

            if(r == dim(CNOlist@signals[[1]])[1]){
                axis(
                    side=1,
                    at=seq(from=0, to=1,length.out=length(CNOlist@stimuli)),
                    labels=CNOlist@stimuli,las=3,cex.axis=1)
                }

            image(
                t(matrix(CNOlist@inhibitors[r,],nrow=1)),
                col=c("white","black"),xaxt="n",yaxt="n")

            if(r == dim(CNOlist@signals[[1]])[1]){
                axis(
                    side=1,
                    at=seq(from=0, to=1,length.out=length(CNOlist@inhibitors)),
                    labels=paste(CNOlist@inhibitors,"-i",sep=""),
                    las=3,cex.axis=1)
            }

            }
        }
    }

#
#  This file is part of the CNO software
#
#  Copyright (c) 2011-2012 - EMBL - European Bioinformatics Institute
#
#  File author(s): CNO developers (cno-dev@ebi.ac.uk)
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  CNO website: http://www.cellnopt.org
#
##############################################################################
checkSignals<-function(CNOlist, model ){

    if (!is(CNOlist,"CNOlist")){
        CNOlist = CellNOptR::CNOlist(CNOlist)
    }

    # check that model is a Model list
    if(!is.list(model)){
        stop("This function expects as input a Model as output by readSIF")
    }
    if( !all( c("reacID", "namesSpecies","interMat","notMat") %in% names(model)))  {
        stop("This function expects as input a Model as output by readSIF")
    }

    # check that all of the signals in colnames(CNOlist@signals) match to one species in model$namesSpecies
    signalsMatch<-match(colnames(CNOlist@signals[[1]]),model$namesSpecies,nomatch=0)
    if(any(signalsMatch == 0)){
        stop(paste(
            "The following signals from your CNOlist do not match any of the species in your model, and should be removed:",
            toString(colnames(CNOlist@signals[[1]])[which(signalsMatch == 0)])
            ))
    }

    # same for stimuli
    signalsMatch<-match(colnames(CNOlist@stimuli),model$namesSpecies,nomatch=0)
    if(any(signalsMatch == 0)){
        stop(paste(
            "The following stimuli from your CNOlist do not match any of the species in your model, and should be removed:",
            toString(colnames(CNOlist@stimuli)[which(signalsMatch == 0)])
            ))
    }

    # same for inhibitors
    signalsMatch<-match(colnames(CNOlist@inhibitors),model$namesSpecies,nomatch=0)
    if(any(signalsMatch == 0)){
        stop(paste(
            "The following inhibitors from your CNOlist do not match any of the species in your model, and should be removed:",
            toString(colnames(CNOlist@inhibitors)[which(signalsMatch == 0)])
            ))
    }
    
    # check if the stimuli nodes have no incoming edge: will not treated
    #  in the simulation functions
  	stimuli <- colnames(CNOlist@stimuli)
    
   if(any(model$interMat[stimuli,] == 1)){
   		isWrongReaction = apply(model$interMat[stimuli,,drop=FALSE] == 1,2,any)
   		msg = names(isWrongReaction)[isWrongReaction]
    	stop(paste0("Stimuli node must not have incoming edge. Please check the model for reactions: ", paste(msg,collapse = ", ")))
    }
    
    
    

}

#
#  This file is part of the CNO software
#
#  Copyright (c) 2019 - SaezLab - Heidelberg Universit
#
#  File author(s): CNO developers
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  CNO website: http://www.cellnopt.org
#
##############################################################################

ilpBinaryTN <- function(cnolist, 
                        model, 
                        sizeFac = 0.0001, 
                        mipGap = 0, 
                        relGap = 0, 
                        timelimit = 3600, 
                        cplexPath, 
                        method = "quadratic",
                        numSolutions = 100, 
                        limitPop = 500, 
                        poolIntensity = 0, 
                        poolReplace = 2,
                        timeIndices = c(1, 2)){
  
  ## Initializing auxilliary objects for the analysis
  resILPAll <- list()
  exclusionList <- NULL
  cnolistReal <- cnolist
  tmpPath <- getwd()
  
  if(!file.exists(cplexPath)){
  	stop("User should provide a valid CPLEX path for using this function.")
  }
  
  ## Checking cnolist class
  if (is(CNOlist,"CNOlist")){
    writeMIDAS(CNOlist = cnolist, filename = "tempMD.csv", 
               timeIndices = timeIndices, overwrite = TRUE)
    md = readMIDAS(MIDASfile = "tempMD.csv")
    file.remove("tempMD.csv")
    cnolist <- compatCNOlist(object = cnolist)
  }
  
  ## Form of the objective function
  if(!(method%in%c("quadratic", "linear"))){
    print("'method' variable should be quadratic or linear. 
          Setting method = 'qadratic'")
    method = "quadratic"
  }
  
  ## Penalty factors as decimals
  options(scipen = 10)
  
  ## Creating CNOlist object to use from the midas md table and checking if 
  ## signals in the cnolist and pkn match
  cnolist = makeCNOlist(md,FALSE)
  
  startILP = Sys.time()
  print("Creating LP file and running ILP...")
  resILP = createAndRunILP(model, md, cnolistReal, accountForModelSize = TRUE, 
                           sizeFac = sizeFac, mipGap=mipGap, relGap=relGap, 
                           timelimit=timelimit, cplexPath = cplexPath, 
                           method = method, numSolutions = numSolutions, 
                           limitPop = limitPop, poolIntensity = poolIntensity, 
                           poolReplace = poolReplace)
  endILP = Sys.time()
  
  cleanupILP()
  
  return(resILP)
  
}

#
#  This file is part of the CNO software
#
#  Copyright (c) 2011-2012 - EMBL - European Bioinformatics Institute
#
#  File author(s): CNO developers (cno-dev@ebi.ac.uk)
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  CNO website: http://www.cellnopt.org
#
##############################################################################
# $Id$

#' @export
CNORbool<-function(CNOlist, model, paramsList=defaultParameters(),
    compression=TRUE, expansion=TRUE, cutNONC=TRUE, verbose=FALSE,
    timeIndices=NULL)
{


	if (!is(CNOlist,"CNOlist")){
        cnolist = CellNOptR::CNOlist(CNOlist)
    }else{
        cnolist = CNOlist
    }
    if (is.character(model)==TRUE){
        model = readSIF(model)
    }

    # overwrite verbosity
    paramsList$verbose = verbose

    # 1. Checks data to model compatibility
    checkSignals(cnolist, model)
    # 2.preprocessing
    model = preprocessing(cnolist, model, compression=compression,
        expansion=expansion, cutNONC=cutNONC, verbose=verbose)

    #8.Optimisation t1
    bStrings = list()
    initBstring<-rep(1,length(model$reacID))
    print("Entering gaBinaryT1")


    if (is.null(timeIndices)==TRUE){
        T0index = 2
    } else{
        T0index = timeIndices[[1]]
    }

    allRes = c()
    T1opt<-gaBinaryT1(CNOlist=cnolist,
        model=model,
        initBstring=initBstring,
        sizeFac=paramsList$sizeFac,
        NAFac=paramsList$NAFac,
        popSize=paramsList$popSize,
        pMutation=paramsList$pMutation,
        maxTime=paramsList$maxTime,
        maxGens=paramsList$maxGens,
        stallGenMax=paramsList$stallGenMax,
        selPress=paramsList$selPress,
        elitism=paramsList$elitism,
        relTol=paramsList$relTol,
        verbose=paramsList$verbose, 
        timeIndex=T0index)
    bStrings[[1]] = T1opt$bString
    allRes = c(allRes, T1opt)


    if (length(cnolist@signals)==2){
        return(list(model=model, bStrings=bStrings, results=allRes))
    }

    #.Optimise tN where N>1
    Times = 1
    T2opt<-NA # default value

    if (is.null(timeIndices)==TRUE){
        for (i in 3:length(cnolist@signals)){
            Times = Times + 1
            print(paste("Entering gaBinaryTN (time=", Times, ")", sep=" "))

            TNopt<-gaBinaryTN(
                CNOlist=cnolist,
                model=model,
                bStrings=bStrings,
                sizeFac=paramsList$sizeFac,
                NAFac=paramsList$NAFac,
                popSize=paramsList$popSize,
                pMutation=paramsList$pMutation,
                maxTime=paramsList$maxTime,
                maxGens=paramsList$maxGens,
                stallGenMax=paramsList$stallGenMax,
                selPress=paramsList$selPress,
                elitism=paramsList$elitism,
                relTol=paramsList$relTol,
                verbose=paramsList$verbose)

            allRes = c(allRes, TNopt)
            bStrings[[Times]] = TNopt$bString
            print(TNopt$bString)

            if (Times==2){
                T2opt = TNopt
            }
        }
    }else{
        for (i in 2:length(timeIndices)){
            Times = Times + 1
            print(paste("Entering gaBinaryTN (time=", Times, ")", sep=" "))

            TNopt<-gaBinaryTN(
                CNOlist=cnolist,
                model=model,
                bStrings=bStrings,
                sizeFac=paramsList$sizeFac,
                NAFac=paramsList$NAFac,
                popSize=paramsList$popSize,
                pMutation=paramsList$pMutation,
                maxTime=paramsList$maxTime,
                maxGens=paramsList$maxGens,
                stallGenMax=paramsList$stallGenMax,
                selPress=paramsList$selPress,
                elitism=paramsList$elitism,
                relTol=paramsList$relTol,
                verbose=paramsList$verbose, timeIndex=timeIndices[[i]])

            allRes = c(allRes, TNopt)
            bStrings[[Times]] = TNopt$bString
            print(TNopt$bString)

            if (Times==2){
                T2opt = TNopt
            }

        }
    }

    return(list(model=model, bStrings=bStrings, results=allRes))
}

#
#  This file is part of the CNO software
#
#  Copyright (c) 2011-2012 - EMBL - European Bioinformatics Institute
#
#  File author(s): CNO developers (cno-dev@ebi.ac.uk)
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  CNO website: http://www.cellnopt.org
#
##############################################################################
# $Id$

# Function that computes the score of a specific bitstring
# Although it is very similar to computeScoreT1, there are enough differences to
# have a different function.
computeScoreTN<-function(CNOlist, model, simList=NULL, indexList=NULL,
    simResPrev=NULL, bStringPrev=NULL, bStringNext=NULL, timeIndex=NULL,
    sizeFac=0.0001, NAFac=1, bStrings=NULL){

    # by default same behaviour as computeScoreT2
    # timeIndex=3 stands for T2 by default.
    #timeIndex = timeIndex # i.e., "tN"
    if (is.null(timeIndex)==TRUE){
        timeIndex = length(bStrings) + 1
    }

#    print(timeIndex)

    if (is.null(simList)==TRUE){
        simList = prep4sim(model)
    }
    if (is.null(indexList)==TRUE){
        indexList = indexFinder(CNOlist, model)
    }

    # if no previous results provided, we must recompute simulated results at
    # each time. This is slower but provides an easy user interface. Just
    # cnolist, model and list of bitStrings is required.
    if (is.null(simResPrev) == TRUE){
        res = buildBitString(bStrings)
        bitString = res$bs
        bStringTimes = res$bsTimes

        modelCut = cutModel(model, bitString)
        modelCut$times <- bStringTimes[which(bStringTimes != 0)]
        simListCut <- cutSimList(simList, bitString)

        if (is.null(bStrings)==FALSE){
            simResults = simulateTN(CNOlist, model, bStrings)
        }
        else{
            stop("CellNOpt erro:: you must provide either bStrings or (simResPrev, bStringPrev, bStringNext) arguments)")
        }

    }
    else{


        if (is.null(bStrings)==TRUE){
            # if no bStrings, we have prev and next bistring that must be
            # provided and it should correspond to T2 analysis
            # TODO: sanity checl that timeIndex is correct
            bStrings = list(bStringPrev, bStringNext)
        } else {
            # need to check that length of bStrings is in agreement with
            # timeIndex.
        }

        res = buildBitString(bStrings)
        bitString = res$bs
        bStringTimes = res$bsTimes

        modelCut = cutModel(model, bitString)
        modelCut$times <- bStringTimes[which(bStringTimes != 0)]
        simListCut <- cutSimList(simList, bitString)
        # Compute the simulated results
        simResults <- simulatorTN(
            simResultsPrev=simResPrev,
            CNOlist=CNOlist,
            model=modelCut,
            simList=simListCut,
            indexList=indexList,
            timeIndex=timeIndex)
    }

    # already done in the C simulator.
    simResults = simResults[, indexList$signals]


    ## todo: use the C simulator ? 

    #Compute the score
    Score <- getFit(
        simResults=simResults,
        CNOlist=CNOlist,
        model=modelCut,
        #indexList=NULL, # because we are using the C simulator thast cope with indexList directly
        timePoint=timeIndex,
        sizeFac=sizeFac,
        NAFac=NAFac,
        nInTot=length(which(model$interMat == -1)),
		simResultsT0=NA)

    if (!is(CNOlist,"CNOlist")){
          CNOlist = CellNOptR::CNOlist(CNOlist)
    }
  nDataP <- sum(!is.na(CNOlist@signals[[2]]))
  Score <- Score/nDataP

  return(Score)
}

#
#  This file is part of the CNO software
#
#  Copyright (c) 2011-2012 - EBI
#
#  File author(s): CNO developers (cno-dev@ebi.ac.uk)
#
#  Distributed under the GPLv2 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-2.0.html
#
#  CNO website: http://www.ebi.ac.uk/saezrodriguez/software.html
#
##############################################################################
# $Id$
# test if the priorBitString is used correctly in gaBinaryT1DO NOT MODIFY please (TC, June 2012)

library(CellNOptR)


# Load T1 and T2 data. Only T1 is used here below.
data(ToyModel, package="CellNOptR")
data(CNOlistToy, package="CellNOptR")
cnolist = CNOlistToy
pknmodel = ToyModel

# preprocessing
model = preprocessing(cnolist, pknmodel)

# computeScoreT1 with init string made of ones
initBstring<-rep(1,length(model$reacID))



priorBitString = rep(NA, length(model$reacID))
priorBitString[1] = 0
priorBitString[2] = 0
priorBitString[3] = 0
priorBitString[4] = 0

# Second, you call the gaBinaryT1 function by providing the priorBitString
# argument:
ToyT1opt<-gaBinaryT1(CNOlist=CNOlistToy, model=model,
    initBstring=initBstring, maxGens=10, popSize=5,
    verbose=FALSE, priorBitString=priorBitString)


for (x in ToyT1opt$results[,7]){
     x = strsplit(x ,",")[[1]]
     if (as.numeric(x[[1]]) != 0 ){ stop("first element must be 0")}
     if (as.numeric(x[[2]]) != 0 ){ stop("second element must be 0")}
     if (as.numeric(x[[3]]) != 0 ){ stop("tirdh element must be 0")}
     if (as.numeric(x[[4]]) != 0 ){ stop("fourth element must be 0")}
}

#
#  This file is part of the CNO software
#
#  Copyright (c) 2011-2012 - EMBL - European Bioinformatics Institute
#
#  File author(s): CNO developers (cno-dev@ebi.ac.uk)
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  CNO website: http://www.cellnopt.org
#
##############################################################################
# $Id$
simulateTN<-function(CNOlist, model, bStrings){
#simResT1,bStringT2,bStringTimes,simList,indexList,timeIndex){

    if (is.list(bStrings)==FALSE){
        stop("CellNOpt Error: 3d argument called bStrings must be a list of vectors. Each vector representing a bit string")
    }

    if (length(bStrings) == 1){
        simPrev = internal_simulateT1(CNOlist, model, bStrings[[1]])
    }
    else if (length(bStrings) > 1){
        # T1 first
        simPrev = internal_simulateT1(CNOlist, model, bStrings[[1]])
        # Then, loop over T2 to TN
        for (i in 2:length(bStrings)){
            res = buildBitString(bStrings[1:i])
            simPrev = internal_simulateTN(CNOlist, model, simPrev, res$bs, res$bsTimes, i+1)
        }
    }

    return(simPrev)
}

internal_simulateTN <- function(CNOlist, model, simPrev, bStringPrev, bStringTimes, timeIndex){

  simList = prep4sim(model)
  indexList = indexFinder(CNOlist, model)
  modelCut<-cutModel(model, bStringPrev)


  simListCut<-simList
  simListCut$finalCube<-simListCut$finalCube[as.logical(bStringPrev),]
  simListCut$ixNeg<-simListCut$ixNeg[as.logical(bStringPrev),]
  simListCut$ignoreCube<-simListCut$ignoreCube[as.logical(bStringPrev),]
  simListCut$maxIx<-simListCut$maxIx[as.logical(bStringPrev)]

  if(is.null(dim(simListCut$finalCube))){
    simListCut$finalCube<-matrix(simListCut$finalCube,ncol=1)
    simListCut$ixNeg<-matrix(simListCut$ixNeg,ncol=1)
    simListCut$ignoreCube<-matrix(simListCut$ignoreCube,ncol=1)
  }
  modelCut$times <- bStringTimes[which(bStringTimes != 0)]


  # simulate
  SimT2 <- simulatorTN(
    simResultsPrev=simPrev,
    CNOlist=CNOlist,
    model=modelCut,
    simList=simListCut,
    indexList=indexList,
    timeIndex=timeIndex)

  return(SimT2)
}




internal_simulateT1<-function(CNOlist, model, bStringT1, simList=NULL, indexList=NULL){
    # simList and indexList are computed inside this function. 
    # However, for back-compatibility, we keep the arguments so that if
    # provided, we can still use them.

    # cut the model
    modelCut <- cutModel(model, bStringT1)
    if (is.null(simList)==TRUE){
        simList = prep4sim(model)
    }
    if (is.null(indexList)==TRUE){
        indexList = indexFinder(CNOlist, model)
    }

    # cut the model
    newSimList = cutSimList(simList, bStringT1)

    # compute the results
    simRes <- simulatorT1(CNOlist=CNOlist, model=modelCut, simList=newSimList, 
        indexList=indexList)

    return(simRes)
}

library(CellNOptR)

# test readBND ------------------------------------------------------------------

required_pcks = list("dplyr","tidyr","readr")

if(!all(unlist(lapply(required_pcks,requireNamespace)))){
	print("the following packages need to be installed to use readBND:")	
	print(unlist(required_pcks))
	print("Please, install the packages manually for this feature.")

}else{
	#download.file("https://maboss.curie.fr/pub/example.bnd",destfile = "./tests/example.bnd")
	
	model = readBND("./data/example.bnd")
	#model = readBND("https://maboss.curie.fr/pub/example1.bnd")
	
	# basic checks for being a CellNoptR model:
	stopifnot(is.list(model))
	stopifnot(length(model)==4)
	stopifnot(all(names(model) %in% c("reacID","namesSpecies","interMat","notMat")))
	stopifnot(length(model$reacID)==ncol(model$interMat))
	stopifnot(length(model$reacID)==ncol(model$notMat))
	stopifnot(length(model$namesSpecies)==nrow(model$interMat))
	stopifnot(length(model$namesSpecies)==nrow(model$notMat))
	
	#plotModel(model)
	
	
	
}

#
#  This file is part of the CNO software
#
#  Copyright (c) 2011-2013 - EBI
#
#  File author(s): CNO developers (cno-dev@ebi.ac.uk)
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  CNO website: http://www.cellnopt.org
#
##############################################################################
# $Id$


# Remove the signals, sstimulis, inhibitors or cues in cnolist that are not found in model
cutCNOlist <- function(cnolist, model=NULL, cutTimeIndices=list(), verbose=FALSE){

  if (is.null(model)==TRUE && length(cutTimeIndices)==0){
    stop("Neither model nor time indices were provided. You must provide a
         model (to remove species in your cnolist that are not in the model) and/or a list
         of time indices to remove data at different time  points.")
  }
  if (!is(cnolist,"CNOlist")){
  	cnolist = CellNOptR::CNOlist(cnolist)
  }
	
  if (is.null(model)==FALSE){
    cutCNOlist = .cutCNOlistModel(cnolist, model, verbose)
    if (length(cutTimeIndices)>0){
      cutCNOlist = .cutCNOlistTimeIndices(cutCNOlist, cutTimeIndices)
    }
  } else{
    if (length(cutTimeIndices)>0){
      cutCNOlist = .cutCNOlistTimeIndices(cnolist, cutTimeIndices)
    }
  }

  return(cutCNOlist)
  }


.cutCNOlistModel <- function(cnolist, model, verbose=FALSE){

  cnolist_stimuli = colnames(cnolist@stimuli)
  cnolist_signals = colnames(cnolist@signals[[1]])
  cnolist_cues = colnames(cnolist@cues)
  cnolist_inhibitors = colnames(cnolist@inhibitors)
  species = model$namesSpecies
  times = cnolist@timepoints

  # not efficient to copy but midas files are small and this function should be
  # called only once.
  cutCNOlist = cnolist

  # Search for stimuli to remove
  stimuli2remove = c()
  for(stimuli in cnolist_stimuli){
    if(stimuli %in% species == FALSE){
      stimuli2remove = c(stimuli2remove, stimuli)
    }
  }

  # Search for signals to remove
  signals2remove = c()
  for (signal in cnolist_signals){
    # if cnolist specy not in the model, we will need to remove it
    if (signal %in% species == FALSE){
      signals2remove = c(signals2remove, signal)
    }
  }

  # Search for cues to remove
  cues2remove = c()
  for (cue in cnolist_cues){
    if (cue %in% species == FALSE){
      cues2remove = c(cues2remove, cue)
    }
  }

  # Search for inhibitors to remove
  inhibitors2remove = c()
  for (inhibitor in cnolist_inhibitors){
    if (inhibitor %in% species == FALSE){
      inhibitors2remove = c(inhibitors2remove, inhibitor)
    }
  }

  if (verbose == TRUE){
    cat("The following signals, cues and inhibitors were not found in the model\n")
    cat("They will be removed from the cnolist\n")
    cat("signals:", signals2remove, "\n")
    cat("cues:",cues2remove, "\n")
    cat("inhibitors:", inhibitors2remove, "\n")
  }

  # let us work with the indices instead of the names
  indeces_stimuli = c()
  for(stimuli in stimuli2remove){
    indeces_stimuli = c(indeces_stimuli,
                        grep(paste("^", stimuli, "$", sep = ""), cnolist_stimuli))
  }

  indices_signals = c()
  for (signal in signals2remove){
    # note that in the grep function with use the ^ and $ sign to ensure
    # that a pattern signal matches exactly a signal (e.g., STAT does not
    # match STAT1)
    indices_signals = c(indices_signals,
                        grep(paste("^", signal,  "$", sep=""), cnolist_signals))
  }

  indices_cues = c()
  for (cue in cues2remove){
    indices_cues = c(indices_cues,
                     grep(paste("^", cue, "$", sep=""), cnolist_cues))
  }

  indices_inhibitors = c()
  for (inhibitor in inhibitors2remove){
    indices_inhibitors = c(indices_inhibitors,
                           grep(paste("^", inhibitor, "$", sep=""), cnolist_inhibitors))
  }

  # now perform cut on stimuli
  if(length(indeces_stimuli)>0){
    #cutCNOlist@stimuli = as.matrix(cnolist@stimuli[, -indeces_stimuli])
  	cutCNOlist@stimuli = cnolist@stimuli[, -indeces_stimuli,drop=FALSE]   # rewrote to keep the names of the matrixes
  }

  # now perform the cut on the cues
  if (length(indices_cues)>0){
    #cutCNOlist@cues = as.matrix(cnolist@cues[,-indices_cues])
  	cutCNOlist@cues = cnolist@cues[,-indices_cues,drop=FALSE]
  }

  # and the signal matrices
  if (length(indices_signals)>0){
    for (time in 1:length(cnolist@signals)){
      #cutCNOlist@signals[[time]] = as.matrix(cnolist@signals[[time]][,-indices_signals])
      #cutCNOlist@variances[[time]] = as.matrix(cnolist@variances[[time]][,-indices_signals])
      cutCNOlist@signals[[time]] = cnolist@signals[[time]][,-indices_signals,drop=FALSE]
      cutCNOlist@variances[[time]] = cnolist@variances[[time]][,-indices_signals,drop=FALSE]
    }
  }

  # and the inhibitor matrices
  if (length(indices_inhibitors)>0){
    #for (time in 1:length(cnolist@inhibitors)){
    #cutCNOlist@inhibitors[[time]] = as.matrix(cnolist@inhibitors[[time]][,-indices_inhibitors])
    #}
    #cutCNOlist@inhibitors = as.matrix(cnolist@inhibitors[,-indices_inhibitors])
    cutCNOlist@inhibitors = cnolist@inhibitors[,-indices_inhibitors,drop=FALSE]
  }

  return(cutCNOlist)
}



.cutCNOlistTimeIndices <- function(cnolist, cutTimeIndices){

  cutCNOlist = cnolist
  indices = unlist(cutTimeIndices)
  if (max(indices)>length(cnolist@signals)){
    warning("one or more indices provided are larger than number of time
            points. These indices are going to be ignored.")
  }
  cutCNOlist@signals = cnolist@signals[-indices]
  cutCNOlist@timepoints = cnolist@timepoints[-indices]

  return(cutCNOlist)
  }

---
title: "Training of boolean logic models of signalling networks using prior
knowledge networks and perturbation data with CellNOptR"
author: "Camille Terfve, Thomas Cokelaer, Aidan MacNamara, Enio Gjerga, Attila Gabor, Panuwat Trairatphisan, Julio Saez-Rodriguez"
date: "`r Sys.Date()`"
output:
  html_vignette:
    toc: true
    toc_depth: 2
vignette: >
  %\VignetteIndexEntry{Training of boolean logic models of signalling networks using prior knowledge networks and perturbation data with CellNOptR}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---


## Installation


Before starting, make sure you have installed the latest version of R (4.1).
For more information and download of R, please refer to (http://www.r-project.org/).   
For more information about how to install R packages, please refer to
(http://cran.r-project.org/doc/manuals/R-admin.html#Installing-packages).

Before starting this tutorial you also need to install the package
*CellNOptR*. You can either install *CellNOptR* from Bioconductor by typing:
```{r installPackage, eval=FALSE}
if (!requireNamespace("BiocManager", quietly=TRUE))
    install.packages("BiocManager")
BiocManager::install("CellNOptR")
```

or from our Github:
```{r installPackage2, eval=FALSE}
if (!requireNamespace("remotes", quietly=TRUE))
	install.packages("remotes")
remotes::install_github("saezlab/CellNOptR")
```


A series of books about R can be found on the R project website
((http://www.r-project.org/)), and many tutorials are available on the
internet. If you are a complete beginner, all you need to know is that by typing
"?nameOfFunction" you get the help page about the function that you are
interested in.

## Introduction

The package *CellNOptR* integrates prior knowledge about protein signalling
networks and perturbation data to infer functional characteristics of a
signalling network. 
CellNOptR performs optimisation using a boolean formalism only@julio2009.
However, it includes some data importing and normalising capabilities (as in 
*DataRail* toolbox @datarail in the MatLab pipeline). Moreover it
is used by other packages that implement more complex formalisms. Such packages
are available on BioConductor as well: CNORdt, CNORode (ordinary
equation-based), CNORProb (probabilisty based logic).
More information about the methods and application of the Matlab pipeline can be
found in reference @CNObc and (http://www.cellnopt.org). 


This tutorial shows how to use *CellNOptR* to analyse 1 or 2 time points
data sets on a toy example (1 time point), a realistic example and another toy
example (with 2 time points).  The whole analysis can also be performed in one
step using a wrapper function as described in section 6.

The first step of an analysis with *CellNOptR* is to load the library, and
create a directory where you can perform your analysis, then set it as your
working directory.

```{r Ropts, include=FALSE}
options(width=70)
```

```{r loadLib, eval=TRUE, message=FALSE, warning=FALSE}
library(CellNOptR)
```

```{r newDir, eval=FALSE}
dir.create("CNOR_analysis")
```

## Quick Start

Assuming that you have a prior knowledge network stored in SIF format and a
MIDAS file, the optimisation of your problem can be done in a couple of steps:

```{r quickstart, eval=FALSE}
# ---------------------- load the library and get a SIF and MIDAS file
library(CellNOptR)

# ---------------------- examples are provided in CellNOptR
data("ToyModel", package="CellNOptR")
data("CNOlistToy", package="CellNOptR")
pknmodel = ToyModel
cnolist = CNOlist(CNOlistToy)

# ---------------------- alternatively you can read your own files:
# pknmodel = readSIF("ToyModel.sif")
# cnolist = CNOlist("ToyDataMMB.csv")

# ---------------------- preprocess the network
model <- preprocessing(cnolist, pknmodel)

# ---------------------- perform the analysis
res <- gaBinaryT1(CNOlist = cnolist,model =  model, verbose=FALSE)

# ---------------------- plot the results
cutAndPlot(cnolist, model, list(res$bString))
```



See the following sections for details.



## Loading the data and prior knowledge network.

Let us first create a directory where to store the file that will be created:

```{r directory, eval=TRUE, include=FALSE}
cpfile <- dir(system.file("ToyModel", package="CellNOptR"), full=TRUE)
file.copy(from=cpfile, to=getwd(), overwrite=TRUE)
```

The example that we use is the toy model example from CellNOpt, which is a data
set and associated network that have been created in silico. This data and
network can be found in the inst/ToyModel directory of this package.  The data
is read using the function *readMIDAS*, which as the name states expects a MIDAS
formatted CSV file (see the documentation of DataRail and @datarail for
more information about that file format). Then, you will need to convert the
data into a *CNOlist*, which is the data structure used in
*CellNOptR*. Please note that this data is already
normalised for boolean modelling. If it had not been the case we would have had
to normalise the data first to scale it between 0 and 1, which can be done using
the  *normaliseCNOlist* function of CellNOptR (see the help of this
function for more information about the normalisation procedure). This
normalisation procedure is the one used in @julio2009 as implemented in
*DataRail*. 


Before version 1.3.30, you would type the following combinaison of commands to create a
CNOlist from a MIDAS file:

```{r getData, eval=TRUE}
dataToy <- readMIDAS("ToyDataMMB.csv", verbose=FALSE)
CNOlistToy <- makeCNOlist(dataToy, subfield=FALSE, verbose=FALSE)
```

Alternatively, this data is provided within *CellNOptR* so you can also
load it as follows

```{r getData2, eval=TRUE}
data(CNOlistToy,package="CellNOptR", verbose=FALSE)
```

However, since version 1.3.30, you can use the *CNOlist* class to load the
data in a single command line

```{r cnolistClass, eval=TRUE, include=FALSE}
CNOlistToy = CNOlist("ToyDataMMB.csv")
```

Note for the users familiar with the previous commands (*readMIDAS* and
and *makeCNOlist*) that you can easily convert the old
data structure into an instance of CNOlist class as follows:

```{r cnolistClass2, eval=FALSE}
data(CNOlistToy,package="CellNOptR")
CNOlistToy = CNOlist(CNOlistToy)
```

A CNOlist is the central data object of this package; it contains
measurements of elements of a prior knowledge network under different
combinations of perturbations of other nodes in the network.  A CNOlist
comprises the following attributes: signals, cues, stimuli,
inhibitors  and timepoints. The attributes cues (and its derivatives stimuli and
inhibitors) are boolean
matrices that contain for each condition (row) a 1 when the corresponding cue
(column) is present, and a zero otherwise.  


You can have a look at your data and the CNOlist format by typing:

```{r showCNO, eval=TRUE}
CNOlistToy
```

You can also visualise your data using the method *plot* (or a function
called *plotCNOlist*) which
will produce a plot on your screen with a subplot for each signal and each
condition, and an image plot for each condition that contains the information
about which cues are present in each condition. This plot can also be produced
and stored in your working directory as a single PDF file using the function
*plotCNOlistPDF*.


```{r plotCNO, fig.width=7, fig.height=7, fig.cap="Figure 1: CNOlist data shown by plotting function (either *plot* or *plotCNOlist*)"}
plot(CNOlistToy)
```


```{r ploCNOPDF, eval=FALSE, include=TRUE}
plotCNOlistPDF(CNOlist=CNOlistToy,filename="ToyModelGraph.pdf")
```

We then load the prior knowledge network (PKN), contained in a Cytoscape SIF format file, using the function *readSIF* (alternatively, this example model can be loaded as a R data object already formatted, similarly to what is done above for the CNOlist).  Cytoscape @Cytoscape is a software for network visualisation and analysis. You can build a network within Cytoscape and simply save it as the default SIF file format, which can then be imported in *CellNOptR*.  If you choose to do this, then you should make sure that if you have 'and' gates in your network they are present as dummy nodes named 'and' followed by a number from 1 to the number of 'and' nodes that you have.

Alternatively, you can create your network file as a text file formatted as a SIF file.  Briefly, the expected file format is a tab delimited text file with a line for each directed interaction and the following three elements per line: name of source node, 1 or -1 if the source node is activating or inhibiting the target node, name of target node.  The names of the species in the model must match some of the nodes in the model (and this is case sensitive).  'And' hyperedges are expected to be represented in the SIF file as dummy nodes named 'and' followed by a number.  For example if you have an interaction of the type 'a \& b=c', your SIF file should contain the following three rows: 'a    1    and1', 'b 1 and1', 'and1 1 c'.  Please be aware that when building the scaffold network for optimisation, the software will create all possible 'and' combinations (with maximum 3 inputs) of edges coming into each node, so in the general case it is not necessary to put 2 or 3 input 'and' hyperedges in the prior knowledge network since the software will create them if the corresponding single edges are present.


```{r getModel, eval=TRUE}
pknmodel<-readSIF("ToyPKNMMB.sif")
```

```{r getModel2, eval=TRUE}
data(ToyModel,package="CellNOptR")
```

Having loaded both the data set and corresponding model, we run a check to make sure that our data and model were correctly loaded and that our data matches our model (i.e. that species that were inhibited/stimulated/measured in our data set are present in our model).

%If this is the case, then we can look for the indices of the species that are present in our dataset in our model, using *indexFinder*. This step will have to be repeated every time an operation is done on the model that removes or reorders species.

```{r indices, eval=TRUE}
checkSignals(CNOlistToy,pknmodel)
```

The SIF model that you have just loaded is visible on figure 2 as
displayed by *plotModel* (requires Rgraphviz to be installed).

```{r plotModel, eval=TRUE, fig.cap="Figure 2: Prior knowledge network (original SIF file visualised by *plotModel*) for the Toy Model example."}
plotModel(pknmodel, CNOlistToy)
```


## Preprocessing the model

Prior to optimisation, the model has to be pre-processed in 3 steps: removal of non-observable/non-controllable species, compression, and expansion.  Each one of these steps is described in more details below.

Since version 0.99.24, a *preprocessing* function is available and makes the
following 3 steps in 1 command line. However, the description of the 3 steps remains
available here below. If you do not bother about details, you can jump directly to section \@ref(sec:preproc).

### Finding and cutting the non observable and non controllable species

Non observable nodes are those that do not have a path to any measured species in the PKN, whereas non controllable nodes are those that do not receive any information from a species that is perturbed in the data.  As we won't be able to conclude anything about these species, we will find them and remove them from the model. Please note that in this particular case there are no nodes to cut, but we still include these steps here because they are necessary in a general case.

```{r NONC, eval=TRUE}
indicesToy<-indexFinder(CNOlistToy,pknmodel,verbose=TRUE)
ToyNCNOindices<-findNONC(pknmodel,indicesToy,verbose=TRUE)
ToyNCNOcut<-cutNONC(pknmodel,ToyNCNOindices)
indicesToyNCNOcut<-indexFinder(CNOlistToy,ToyNCNOcut)
```

### Compressing the model

Compressing the model consists of collapsing paths in which a series of non measured or perturbed nodes input into a measured or perturbed node.  This step is performed because such paths do not bring any additional information compared to their compressed version, and unnecessarily complicate the model.  Typically this includes linear cascades for examples, but this excludes any node that would be:

-  involved in complex logics (more than one input and also more than one output)
-  involved in self loops


Compression is performed using the function *compressModel*.

```{r compress, eval=TRUE}
ToyNCNOcutComp<-compressModel(ToyNCNOcut,indicesToyNCNOcut)
indicesToyNCNOcutComp<-indexFinder(CNOlistToy,ToyNCNOcutComp)
```

### Expanding the gates

The last preprocessing step consists in expanding the gates present in the PKN,
i.e. creating new logic combinations of gates from the ones present in the prior
knowledge network.  This is performed in 2 steps: i) any AND node present in the
PKN is split into its constituent branches, and ii) every time a nodes gets more
than one input, then all 'AND' combinations of the inputs are produced, although
only exploring combinaisons of AND gates with a maximum of 2, 3 or 4 input
nodes (for instance for a 5 inputs case, only $C_2^5$, $C_3^5$, or $C_4^5$
combinations are created). This step is performed because although connections
between nodes might be known or inferred from functional relationships, the
particular logic with which these interactions work or are combined to influence
a target node are generally not known.

This step, performed by the function *expandGates*, will create additional fields *SplitANDs* and *newANDs* in the model that inform you about new edges that have been crated from splitting 'AND' hyperedges, and about new hyperedges that have been created from combinations of edges.

```{r expand, eval=TRUE}
model <- expandGates(ToyNCNOcutComp, maxInputsPerGate=3)
```
Note that here we set the option *maxInputsPerGate* to 3 whereas the
default value is 2.

### Preprocessing function{#sec:preproc}

In *CellNOptR* (from version 1.2), a function called
*preprocessing* gathers the previous three preprocessing steps in a single command
line:

```{r expand2, eval=TRUE}
model <- preprocessing(CNOlistToy, pknmodel, expansion=TRUE,
    compression=TRUE, cutNONC=TRUE, verbose=FALSE)
```
In the previous commands, although the default behaviour of the preprocessing function is to perform the expansion, compression and removing of
non-observable and non-controlable nodes, we set these options to TRUE so as to
emphasize the usage of the function.


## Training of the model

By "optimising the model", we mean exploring the space of possible combinations of expanded gates in the PKN in order to find the combination that reproduces most closely the data.  Comparison between model and data is obtained by  simulating the steady state behaviour of the model under all conditions present in the data, and comparing these binary values to the normalised data points. The match between data and model is quantified using an objective function with parameters *sizeFac* and *NAFac*.  This function is the sum of a term that computes the fit of the simulated data to the experimental data, a term that penalises increased model size (weighted by the parameter *sizeFac*), and a term that penalises NAs in the output of the simulation (i.e. nodes that are in a non resolved state, typically negative feedbacks; weighted by the parameter *NAFac*). Typically this has the following structure: \(\frac{1}{n} \sum_{t,l,k} (M_{t,l,k}-D_{t,l,k})^2 + \alpha \frac{1}{s} \sum_{edges} e_{edges} + \beta n_{NA} \), where n is the number of data points, M the model output for time t, readout l, condition k, D is the corresponding measurement, \(\alpha\) is the size factor, \(e\) is the number of inputs for the egde considered (where \(edges\) are all edges present in the optimised model), s is the number of hyperdeges in the model, \(\beta\) is the NA factor, and \(n_{NA}\) is the number of undetermined values returned by the model.

The optimisation itself is done using a genetic algorithm that tries to optimise a string of 0s and 1s denoting the presence or absence of each gate in the model, where the fitness of each individual string is obtained based on the value of the objective function (score). This genetic algorithm uses the following methods: random initialisation of the population (although an initial string is given to the algorithm in the parameter *initBstring*) of size set by *popSize*, linear ranking based on the scores for fitness assignment (with a default selective pressure of 1.2, set by the parameter *selPress*), stochastic uniform sampling for selection (with an *elitism* parameter that allows the best x strings to be carried on 'as is' to the next generation), uniform crossover probability, and a mutation probability over the sequence set to 0.5 as default (set by *pMutation*).  The search can be stopped using three conditions: a maximum time in seconds (*maxTime*), a maximum number of generations (*maxGens*), and a maximum number of stall generations (i.e. generations where the best string is identical, *stallGenMax*).

The genetic algorithm function returns a list object that collects a number of informations such as the best string and corresponding score at each iteration, the average fit at each generation, etc.  The function also returns strings that were obtained across the whole optimisation process and that obtained scores that were closed to the best string, where 'close' is defined by a relative tolerance on the score which is set by the parameter *relTol*.  This is an important piece of information because when the data cannot constrain the model tightly then many strings are obtained with a fit that is close to the optimal one, and interpretation of edges present in the optimal model is therefore more subtle.

We start off by computing the residual error, which is the minimum error that is
unavoidable with a boolean network and comes from the discrete nature of such a
model (please remember that although the data is normalised in this pipeline, it
is not discretised, and therefore we compare 0/1 values to continuous values
between 0 and 1). This value is important because however good is our
optimisation, the value of the goodness of fit term cannot go under this
residual error. 

```{r resError, eval=TRUE}
resECNOlistToy <- residualError(CNOlistToy)
```


Then, we create an initial bit string for the optimisation,
which in this case is just a string of 1s, but could be a meaningful string if
you have prior expectation about the topology of the model.

```{r initbs, eval=TRUE}
initBstring <- rep(1,length(model$reacID))
```

We can now start the optimisation, in this case with default values for all non essential parameters of the genetic algorithm. If you set the argument verbose to TRUE, this function will print the following information, at each generation: generation number, best score and best string at this generation, stall generation number, average score of this generation and iteration time. You can also find these informations in the object that is returned by this function, as well as the best string the b*bString* field, and strings within the relative tolerance limits in *StringsTol*.


```{r optim, eval=TRUE}
ToyT1opt <- gaBinaryT1(CNOlist=CNOlistToy, model=model,
    initBstring=initBstring, verbose=FALSE)
```

We will now produce plots of our analysis.  First, we plot the results of simulating the data with our best model alongside the actual data set in a plot similar to that obtained above with *plotCNOlist*, except that the simulated data is overlaid in dashed blue lines, and the background of the plot reflects the absolute difference between simulated and experimental data (greener=closer to 0; redder=closer to 1; white=NA, either for data or simulation).  Second, we will plot the evolution of the average score and best score during the evolution of the population of models, as a function of generations.  This is useful to detect problems in the optimisation.


```{r resSim,fig.width=7, fig.height=7, fig.cap="Figure 3: Results of the cutAndPlot function on the Toy Model example."}
cutAndPlot(model=model, bStrings=list(ToyT1opt$bString),
    CNOlist=CNOlistToy,plotPDF=TRUE)
```



```{r plotFit,fig.width=7, fig.height=7, eval=TRUE, fig.cap="Figure 4: Results of the cutAndPlot function on the Toy Model example"}
plotFit(optRes=ToyT1opt)
```



Setting the plotPDF argument to "TRUE" means that a PDF figure will be produced (advised, this will then be linked to your report).  To produce a PDF of the evolution of fit as well (advised), type:


```{r simFitPDF, include=TRUE, eval=FALSE}
cutAndPlot(
    model=model,
    bStrings=list(ToyT1opt$bString),
    CNOlist=CNOlistToy,
    plotPDF=TRUE)
pdf("evolFitToyT1.pdf")
plotFit(optRes=ToyT1opt)
dev.off()
```

Note that for now, you can not set the output filename. It is going to be named
\emph{SimResultsT1\_1.pdf} (if several plots are generated, you get
\emph{SimResultsT1\_2.pdf}
and so on). In the future, users should be able to provide the name. You can
still call *cutAndPlot* without the PDF option and use the R pdf function like in
the example above when calling *plotFit* function.


## Plotting the optimised model

Once you have optimised the processed model the bitstring found correspond to
the best model and you may want to look at the result:

```{r plotModelToy1, eval=T,fig.cap="Figure 4a: Processed model. "}
plotModel(model, CNOlistToy, bString=ToyT1opt$bString)
```


You may also want to look at the correspondence on the original model. To do so,
you will need to map back your best model on top of the PKN. This is done in two
steps:

```{r plotModelToy2, eval=TRUE, fig=F, include=TRUE, fig.cap="Figure 4b: The edges are on (black or red) or off (grey or pink) according to the best set of parameters found during the optimisation (the best bit string). To obtain the right hand side model, the mapBack function has been used."}

bs = mapBack(model, pknmodel, ToyT1opt$bString)
plotModel(pknmodel, CNOlistToy, bs, compressed=model$speciesCompressed)
```


## Writing your results

The next function, *writeScaffold*, allows you to write in a Cytoscape SIF file  the scaffold network that was used for optimisation as well as two corresponding edge attribute files: one that tells you when the edge was called present in the optimised networks (ie 0=absent, 1=present), and one that tells you the weight of each edge as the fraction of models within the relative tolerance distance of the best model's score that actually included the edge. The function also writes the scaffold to a graphviz ((http://www.graphviz.org/Credits.php)) dot file, where the presence/absence is represented by the color of the edge (grey if absent, blue if present) and the weight is represented by the penwidth of the edges.

We then also write the prior knowledge network, using the function *writeNetwork*, which again produces a SIF file and corresponding attributes, and a dot file.  The SIF file has a corresponding edge attribute file that contains the present/absent information mapped back to the PKN, and the node attribute file contains information about the status of the node (compressed, non-observable/non-controllable, signal, inhibited, stimulated).  The dot file encodes the edge information as above and the node information in the color of the node: signals are in blue, inhibited nodes are in red, stimulated nodes are in green, and compressed/cut nodes are in white with dashed contour.   Examples of such files can be found on the following two figures, respectively.

![Figure 5: Scaffold network generated by the function writeScaffold for the Toy Model example.](Fig2.png){width=50%}

![Figure 6: PKN generated by the function writeNetwork for the Toy Model example.](Fig3.png){width=50%}

You can then write a report that contains relevant information about your
analysis and links to the various plots that you created.  This will be in the
form of an html file called CellNOptReport.html, which will be stored along with
all of your plots in a directory that you create and name (the name of that
folder is a parameter in the function *writeReport*).  Please not that the
function *writeReport* will create the folder and move all of the files
given by the list *namesFiles* into this directory. The files given by the
arguments *dataPlot*, *evolFitT1*, *optimResT1* will then be hyperlinked to your html report.



```{r  eval=FALSE}
writeScaffold(
    modelComprExpanded=model,
    optimResT1=ToyT1opt,
    optimResT2=NA,
    modelOriginal=pknmodel,
    CNOlist=CNOlistToy)

writeNetwork(
    modelOriginal=pknmodel,
    modelComprExpanded=model,
    optimResT1=ToyT1opt,
    optimResT2=NA,
    CNOlist=CNOlistToy)

namesFilesToy<-list(
    dataPlot="ToyModelGraph.pdf",
    evolFitT1="evolFitToyT1.pdf",
    evolFitT2=NA,
    simResultsT1="SimResultsT1_1.pdf",
    simResultsT2=NA,
    scaffold="Scaffold.sif",
    scaffoldDot="Scaffold.dot",
    tscaffold="TimesScaffold.EA",
    wscaffold="weightsScaffold.EA",
    PKN="PKN.sif",
    PKNdot="PKN.dot",
    wPKN="TimesPKN.EA",
    nPKN="nodesPKN.NA")

writeReport(
    modelOriginal=pknmodel,
    modelOpt=model,
    optimResT1=ToyT1opt,
    optimResT2=NA,
    CNOlist=CNOlistToy,
    directory="testToy",
    namesFiles=namesFilesToy,
    namesData=list(CNOlist="Toy",model="ToyModel"))
```

```{r  eval=TRUE, echo=FALSE}
unlink("testToy",recursive=TRUE)
```

## The one step version

If you do not want to bother with all of these steps, there is a function that allows you to do the whole analysis in one step.  In order to do this, you must first load the model and data, as above (assuming that you have already loaded the library and have copied the relevant files in your working directory).

```{r  eval=TRUE}
dataToy <- readMIDAS("ToyDataMMB.csv")
CNOlistToy <- makeCNOlist(dataToy,subfield=FALSE)
pknmodel <- readSIF("ToyPKNMMB.sif")
```

Then you have two possibilities, either you keep all optimisation parameters to their default values and just type

```{r wrap1, eval=FALSE, include=TRUE}
res <- CNORwrap(
    paramsList=NA,
    name="Toy",
    namesData=list(CNOlist="ToyData",model="ToyModel"),
    data=CNOlistToy,
    model=pknmodel)
```

or you want to have some control over the parameters of the optimisation, in which case you create a parameters list with fields *Data* (the CNOlist containing your data),  *model* (your model object),  *sizeFac* (default to 1e-04), *NAFac* (default to 1), *popSize* (default to 50), *pMutation* (default to 0.5), *maxTime* (default to 60), *maxGens* (default to 500), *stallGenMax* (default to 100), *selPress* (default to 1.2), *elitism* (default to 5), *relTol* (default to 0.1), *verbose* (default to FALSE ). You can then call the wrapper function with this set of parameters.

```{r eraseToyDir, eval=TRUE, include=FALSE}
unlink("Toy",recursive=TRUE)
```

```{r wrap2, eval=FALSE}
pList<-defaultParameters(CNOlistToy, pknmodel)
#pList$data = CNOlistToy
#pList$model = ToyModel
res <- CNORwrap(
    paramsList=pList,
    name="Toy1Step",
    namesData=list(CNOlist="ToyData",model="ToyModel"))
```

Both of these versions will generate the graphs produced above on your graphics window, and as PDF's that will be hyperlinked to your html report, and you will be able to find all of these hyperlinked with your html report, all in a directory called by the *Name* parameter, in your working directory.

```{r eraseData, eval=TRUE, echo=FALSE}
unlink("ToyDataMMB.csv")
unlink("ToyPKNMMB.sif")
unlink("Toy1Step",recursive=TRUE)
unlink("ToyModelMMB2.sif")
```

## A real example

This package also contains a slightly larger and more realistic data set, which is a part of the network analysed in @julio2009 and comprises 40 species and 58 interactions in the PKN. This network was also used for the signaling challenge in DREAM 4 (see (http://www.the-dream-project.org/)).  The associated data was collected in hepatocellular carcinoma cell line HepG2 (see @alexopoulos_networks_2010).   The same analysis as above can be performed on this data set.  In order to load this data, you need to copy the files in the "DREAMModel" directory to you working directory (see below) then use *readMIDAS*, *readSIF* and *makeCNOlist*, or load the pre-formatted model and CNOlist objects, as above.

```{r getDREAM, eval=TRUE}
#Option 1: copy the SIF and MIDAS files (followed by readMIDAS, makeCNOlist and readSIF)
cpfile<-dir(system.file("DREAMModel",package="CellNOptR"),full=TRUE)
file.copy(from=cpfile,to=getwd(),overwrite=TRUE)
#Option 2: load the CNOlist and model objects
data(CNOlistDREAM,package="CellNOptR")
data(DreamModel,package="CellNOptR")
```

Having loaded both data and model, you can now visualise your data using
*plotCNOlist* and *plotCNOlistPDF*.  You can now start the
preprocessing of the model, i.e. find the non-observable/non-controllable
species, compress the model, and expand the gates (*preprocessing*).

Having obtained a pre-processed model, we run the training function (*gaBinaryT1*). Finally, we plot the results of our training (*cutAndPlot*, see figure \@ref(fig:DREAMres)) as well as the information about the evolution of fit during optimisation (*plotFit*).


```{r DREAMAnalysis, eval=FALSE}
model = preprocessing(CNOlistDREAM, DreamModel, verbose=FALSE)
res = gaBinaryT1(CNOlistDREAM, model, verbose=FALSE, maxTime=10)
cutAndPlot(CNOlistDREAM, model, bStrings=list(res$bString), plotPDF=TRUE,
    plotParams=list(maxrow=25, margin=0.1, width=20, height=20))


```

![Figure 7: Plot of simulated and experimental data for the DREAM data and model presented above.](Fig4.png){width=60%}


Our analysis is then complete and we can write our results (scaffold network (*writeScaffold*), prior knowledge network (*writeNetwork*) and html report (*writeReport*)). An example of the html report generated for this analysis is shown on figure 8.  

![Figure 8: Report generated by CellNOptR for the DREAM data set and model training process.](Fig5.png){width=60%}

The scaffold network produced by *writeScaffold* is shown on figure 9.

![Figure 9: Scaffold network for the DREAM model, as produced by writeScaffold.](Fig6.png){width=60%}


## A toy example with two time points

In this section we will demonstrate the use of CellNOptR used when the data comprises two time points where we assume that different mechanisms of regulation are at play, which act on separate time scales.  The data set that we use is the toy model example from CellNOpt, adapted for an optimisation based on two time points (i.e. all signals are kept at the same value as t1 except for cJun and Jnk which go down to zero). The model that we will load is the toymodel from CellNOpt where we added a negative feedback between cJun and Jnk (!cJun=Jnk).  We are going to try and capture this feedback, assuming that this is a slow mechanism that shuts the cJun - Jnk branch down at the second, slower time scale.

In this case the data comprises two time points that we assume are pseudo steady states corresponding to different time scales. The optimisation will therefore be performed in 2 steps, one that finds the reactions present at time 1, and one that finds the reactions that were not present at time 1 and might have entered into play at time 2, explaining the evolution of the signals from time 1 to time 2. First, we load the data and model, and perform the optimisation just as we did for the Toy model above.

```{r t2load, eval=TRUE}
data(CNOlistToy2,package="CellNOptR")
data(ToyModel2,package="CellNOptR")
pknmodel = ToyModel2
cnolist = CNOlist(CNOlistToy2)
```

```{r t2plotCNOlist, fig.width=7, eval=TRUE}
plot(cnolist)
```


```{r t2Opt1, eval=TRUE}
model = preprocessing(cnolist, pknmodel, verbose=FALSE)
T1opt <- gaBinaryT1(cnolist, model, stallGenMax=10, maxTime=60, verbose=FALSE)
```

```{r t2OptT1plot, eval=TRUE, fig.width=7}
sim <- cutAndPlot(model=model, bStrings=list(T1opt$bString),
    CNOlist=cnolist, plotPDF=TRUE)
```


```{r  eval=TRUE, fig.width=6 , fig.height=6}
plotFit(optRes=T1opt)
```

We can now optimise the second time point.

```{r t2OptT2, eval=TRUE}
T2opt<-gaBinaryTN(cnolist, model,  bStrings=list(T1opt$bString),
    stallGenMax=10, maxTime=60, verbose=FALSE)
```

Finally, we produce all the plots and write the report.


```{r resSimT2, eval=FALSE}
res <- cutAndPlot(
    model=model,
    bStrings=list(T1opt$bString, T2opt$bString),
    CNOlist=cnolist,
    plotPDF=TRUE, plotParams=list(cex=0.8, cmap_scale=0.5, margin=0.2))
```

![Figure 10: Output of the *cutAndPlot* function on the 2-time data set.](Fig7.png){width=60%}


## A toy example with the ILP implementation

Besides Genetic Algorithm, another approach for training the PKN to data is by using an Integer Linear Programming (ILP) optimization technique. This optimization problem has been implemented based on the formulation from @mitsos_2009. 

In this section we will demonstrate the use of CellNOptR with the ILP implementation with the example provided in the CellNOptR package. For running this feature, users will have to first obtain an *IBM-CPLEX* license (see (https://www.ibm.com/de-de/products/ilog-cplex-optimization-studio)). Onve the *CPLEX* solver is downloaded and installed, then for running CNO-ILP users will have to specify a valid path (*cplexPath*) pointing to the *CPLEX* executable file.

```{r eval=FALSE}
# ---------------------- load the library and get a SIF and MIDAS file
library(CellNOptR)
library(stringr)
#
# ---------------------- examples are provided in CellNOptR
data("ToyModel", package="CellNOptR")
data("CNOlistToy", package="CellNOptR")
pknmodel = ToyModel
cnolist = CNOlist(CNOlistToy)
#
# ---------------------- alternatively you can read your own files:
# pknmodel = readSIF("ToyModel.sif")
# cnolist = CNOlist("ToyDataMMB.csv")
#
# ---------------------- preprocess the network
model = preprocessing(cnolist, pknmodel)
#
# ---------------------- perform the analysis
cplexPath = "path/to/cplex"
resILP = ilpBinaryT1(cnolist = cnolist, model = model, 
                     numSolutions = 3, relGap = 0.05,
                     cplexPath = cplexPath) # asking to retrieve 3 
                                            # equivalent solutions with 
                                            # tolerance gap relGap=0.05
#
# ---------------------- plot the results (optimized models + fits)
cutAndPlot(CNOlist = cnolist, model = model, bStrings = list(resILP$bitstringILP[[1]]))
plotModel(model, cnolist, bString=resILP$bitstringILP[[1]])

cutAndPlot(CNOlist = cnolist, model = model, bStrings = list(resILP$bitstringILP[[2]]))
plotModel(model, cnolist, bString=resILP$bitstringILP[[2]])

cutAndPlot(CNOlist = cnolist, model = model, bStrings = list(resILP$bitstringILP[[3]]))
plotModel(model, cnolist, bString=resILP$bitstringILP[[3]])
```

Figures 11, 12 and 13 show the 3 optimal solutions retrieved by the ILP analysis.

![Figure 11. Results of the *ilpBinaryT1* function on the Toy Model example - solution 1](model_1.png){width=50%}


![Figure 12. Results of the *ilpBinaryT1* function on the Toy Model example - solution 2](model_2.png){width=50%}


![Figure 13. Results of the *ilpBinaryT1* function on the Toy Model example - solution 3](model_3.png){width=50%}


As indicated in Figure 14, the three solutions fit the data with the same quality and the main difference is in the size of network solutions. This example shows how ILP can be used to retrieve alternative sparser solutions which have the same quality of fit and to indicate the most robust interactions or the ones which are consistently present across the solutions.

![Figure 14. Alternative solutions have the same fit to data. Plots obtained throught the *cutAndPlot* function](ilp_fit.png){width=100%}


Through the gap parameters (relGap) users can control the quality of solutions to retrieve compared to the optimal ones. As a rule of thumb the looser the gap parameters the higher the number of solutions which can be retrieved.

## k-fold Crossvalidation

Cross-validation has been implemented in order to assess the predictive performance of our models and inspect how well these models can predict data outside of its training set. Three re-sampling strategies have been implemented: resampling of 1)data-ponts, 2)experimental conditions and 3)observable nodes. Because of the multiple and possibly extensive analysis, the cross-validation functionality allow for a parallelization of the procedure.

Below we show an example about how we can use the cross-validation functionality by applying different re-sampling strategies in a parallel and serial computing environments.

```{r  eval=FALSE}
# ---------------------- load the library and get a SIF and MIDAS file
library(CellNOptR)
#
# ---------------------- examples are provided in CellNOptR
data(PKN_ToyPB, package="CellNOptR")
data(CNOlist_ToyPB, package="CellNOptR")
#
# ---------------------- preprocess the network
model = preprocessing(data = cnodata,model = pknmodel,compression = T,expansion = T)
plotModel(model,cnodata)
#
#------------------------ original CNOlist contains many timepoints, we use only a subset
plot(cnodata)
selectedTime = c(0,10)
cnodata_prep = cutCNOlist(cnodata ,model = model,
                          cutTimeIndices = which(!cnodata@timepoints %in% selectedTime))
plot(cnodata_prep)
#
# ---------------------- perform the analysis
opt = gaBinaryT1(CNOlist = cnodata_prep,model = model)
#
# ---------------------- 10-fold cross-validation procedure using T1 data
library(doParallel)
doParallel::registerDoParallel(cores=3)
system.time({R1=crossvalidateBoolean(CNOlist = cnodata_prep, model = model, 
                                     type="datapoint", nfolds=10, parallel = TRUE)})
system.time({R2=crossvalidateBoolean(CNOlist = cnodata_prep, model=model, 
                                     type="experiment", nfolds=10, parallel = TRUE)})
system.time({R3=crossvalidateBoolean(CNOlist = cnodata_prep, model=model, 
                                     type="observable", nfolds=10, parallel = TRUE)})
system.time({R4=crossvalidateBoolean(CNOlist = cnodata_prep, model=model, 
                                     type="datapoint", nfolds=10, parallel = FALSE)})
system.time({R5=crossvalidateBoolean(CNOlist = cnodata_prep, model=model, 
                                     type="experiment", nfolds=10, parallel = FALSE)})
system.time({R6=crossvalidateBoolean(CNOlist = cnodata_prep, model=model, 
                                     type="observable", nfolds=10, parallel = FALSE)})
```

## What else
More tutorials can be found in www.cellnopt.org and (http://www.cellnopt.org/)
in particulat on (http://www.cellnopt.org/doc/cnodocs/).

\begin{thebibliography}{1}

\bibitem{alexopoulos_networks_2010}
Alexopoulos, L.G., Saez-Rodriguez, J., Cosgrove, B.D., Lauffenburger, D.A.,
  Sorger, P.K.: Networks inferred from biochemical data reveal profound
  differences in toll-like receptor and inflammatory signaling between normal
  and transformed hepatocytes.
\newblock Molecular \& Cellular Proteomics: {MCP} **9**(9), 1849--1865
  (2010).

  \bibitem{CNObc}
M.K.~Morris, I.~Melas, J.~Saez-Rodriguez.
\newblock Construction of cell type-specific logic models of signalling networks using CellNetOptimizer.
\newblock {\em Methods in Molecular Biology: Computational Toxicology}, Ed. B. Reisfeld and A. Mayeno, Humana Press.

\bibitem{datarail}
J.~Saez-Rodriguez, A.~Goldsipe, J.~Muhlich, L.G.~Alexopoulos, B.~Millard, D.A.~Lauffenburger and P.K.~Sorger.
\newblock Flexible informatics for linking experimental data to mathematical models via *DataRail*.
\newblock {\em Bioinformatics}, 24:6, 2008.

\bibitem{julio2009}
J.~Saez-Rodriguez, L.~Alexopoulos, J.~Epperlein, R.~Samaga, D.~Lauffenburger, S.~Klamt and P.K.~Sorger.
\newblock Discrete logic modelling as a means to link protein signalling networks with functional analysis of mammalian signal transduction.
\newblock {\em Molecular Systems Biology}, 5:331, 2009.

\bibitem{Cytoscape}
P.~Shannon, A.~Markiel, O.~Ozier, N.S..~Baliga, J.T.~Wang, D.~Ramage,  N.~Amin, B.~Schiwkowski and T.~Ideker.
\newblock Cytoscape: a software for integrated models of biomolecular interaction networks.
\newblock {\em Genome Research}, 13(11):2498-504, 2003.

\bibitem{mitsos_2009}
Mitsos, A., Melas, I.N., Siminelakis, P., Chairakaki, A.D.,
  Saez-Rodriguez, J.: Identifying Drug Effects via Pathway Alterations using an Integer Linear Programming Optimization Formulation on Phosphoproteomic Data.
\newblock PLOS \& Computational Biology **5**(12): e1000591
  (2010).


\end{thebibliography}

\name{cutSimList}
\alias{cutSimList}
\title{
Cut a simList structure according to a bitstring
}
\description{
This function is for developers only. 
}
\usage{
cutSimList(simList, bString)
}

\arguments{
  \item{simList}{the simList object to cut.}
  \item{bString}{the bitString to be used to cut the simList object.}
}


\value{
  \item{cutSimList}{the new simList object}
}

\author{
 T. Cokelaer
}

\name{CellNOptR-package}
\alias{CellNOptR-package}
\alias{CellNOptR}
\docType{package}
\title{
R version of CellNOptR, boolean features
}
\description{
This package does optimisation of boolean logic networks of signalling pathways based on a previous knowledge network and a set of data collected upon perturbation of some of the nodes in the network.
}
\details{
\tabular{ll}{
Package: \tab CellNOptR\cr
Type: \tab Package\cr
Version: \tab 1.25.1\cr
Date: \tab 2018-01-10\cr
License: \tab GPLv3\cr
LazyLoad: \tab yes\cr
}
}
\author{
T. Cokelaer, A. MacNamara, F. Eduati, S. Schrier, C. Terfve

Maintainer: A. Gabor<gabor.attila87@gmail.com>, until 2018-01-18: T. Cokelaer <cokelaer@ebi.ac.uk>
}
\references{
J. Saez-Rodriguez, L. G. Alexopoulos, J. Epperlein, R. Samaga, D. A. Lauffenburger, S. Klamt and P. K. Sorger. Discrete logic modeling as a means to link protein signaling networks with functional analysis of
mammalian signal transduction, Molecular Systems Biology, 5:331, 2009.
}

\keyword{ package }
\examples{

# quick 1 time point optimisation of a Prior Knowledge Network to MIDAS data. 
data(CNOlistToy,package="CellNOptR")
data(ToyModel,package="CellNOptR")

pknmodel = ToyModel
cnolist = CNOlist(CNOlistToy)
model = preprocessing(cnolist, pknmodel)
results = gaBinaryT1(cnolist, model, verbose=FALSE)
plotFit(results)
cutAndPlot(cnolist, model, list(results$bString))

# Same as above and HTML report
CNORwrap(name="Toy", 
    namesData=list(CNOlist="ToyData",model="ToyModel"),
    data=CNOlistToy, model=pknmodel)
}

\name{model2sif}
\alias{model2sif}

\title{
  Convert a model object in sif format
}

\description{
  This function receives as input a model object and converts it to the cytoscape sif format. It can be used to convert either the whole model (before or after pre-processing) or the optimized one (if the corresponding bitString in provided). The resulting table can also be saved in a sif file.
}

\usage{
model2sif(model,optimRes=NA,writeSif=FALSE, filename="Model")

}


\arguments{
  \item{model}{
the model as generated using \link{readSIF}
}
  \item{optimRes}{
the output of the optimisation (as obtained using \link{gaBinaryT1}), default set to NA the whole model in converted 
}
  \item{writeSif}{
it writeSif=FALSE (default) the sif file is not saved. If writeSif=TRUE it is saved.
}
  \item{filename}{
the name of the sif file saved if writeSif=TRUE. Default is Model.sif.
}

}


\details{
All links in the model are converted in sif format that is sourceNode-tab-sign-tab-targetNode. If there are ANDs, they are converted using dummy nodes called and# (e.g. A+B=C will be A-tab-1-tab-and1; B-tab-1-tab-and1; and1-tab-1-tab-C).
}

\value{
\item{sifFile}{a table with all the links in the model in the format sourceNode-tab-sign-tab-targetNode}
}

\author{
    F.Eduati
}

\seealso{
\link{graph2sif}, \link{sif2graph}, \link{readSIF}, 
}

\examples{
    data(ToyModel,package="CellNOptR")
    model2sif(model=ToyModel,writeSif = FALSE)
}

\name{defaultParameters}
\alias{defaultParameters}
\title{
Create a list of default parameters
}
\description{
This function provides a list of default parameters including the Genetic
Algorithm parameters.}
\usage{
defaultParameters(data=NA, model=NA)
}

\arguments{
  \item{data}{
   a CNOlist structure, as created by \code{makeCNOlist}
  }
  \item{model}{
    a model structure, as created by \code{readSIF}, normally pre-processed but 
    that is not a requirement of this function
  }
}


\details{
  The list contains the Genetic Algorithm parameter, a verbose option and can be
  used to store the Data and Model.
}

\value{
  \item{params}{a list with the fields: \code{data}, \code{model},
\code{verbose} and all default parameters of \link{gaBinaryT1} }
}
\author{
 T. Cokelaer
}
\examples{
data(ToyModel, package="CellNOptR")
data(CNOlistToy, package="CellNOptR")
params = defaultParameters(CNOlistToy, ToyModel)
}

\name{create_binaries}
\alias{create_binaries}
\alias{create_binaries}

\title{
Defining the set of binary variables for the ILP implementation of CellNOptR.
}
\description{
This function takes as input the model and data in a CNOlist object
in order to define the set of binary variablese. 
}
\usage{
create_binaries(model, 
                            midas, 
                            numberOfExperiments, 
                            y_vector)
}
\arguments{
  \item{model}{the model
  }
  \item{midas}{the midas table
  }
  \item{numberOfExperiments}{the number of experimental conditions
  }
  \item{y_vector}{the variables for each interaction in the PKN
  }


}

\author{
E Gjerga, H Koch
}
\references{
Alexander Mitsos, Ioannis N. Melas, Paraskeuas Siminelakis, Aikaterini D. Chairakaki, Julio Saez-Rodriguez, and Leonidas G. Alexopoulos. Identifying Drug Effects via Pathway Alterations using an Integer Linear Programming Optimization Formulation on Phosphoproteomic Data. PLoS Comput Biol. 2009 Dec; 5(12): e1000591.
}

\name{writeSIF}
\alias{writeSIF}
\alias{writeSif}

\title{
Convert a model into a SIF format and save the result in a file.
}
\description{
This function takes as input a model (as created by e.g., read from a SIF data set with
readSIF function) and save it into a file. 
}
\usage{
writeSIF(model, filename,  overwrite = FALSE)
}
\arguments{
  \item{model}{the model
  }
  \item{filename}{the filename
  }
  \item{overwrite}{by default, do not overwrite a file.
  }


}

\author{
T Cokelaer
}
\examples{
cpfile<-dir(system.file("ToyModel",package="CellNOptR"),full=TRUE)
file.copy(from=cpfile,to=getwd(),overwrite=TRUE)
ToyModel<-readSIF(sifFile="ToyPKNMMB.sif")
writeSIF(ToyModel, "ToyPKNMMB_copy.sif")
}

\name{getFit}
\alias{getFit}
\title{
Compute the score of a model
}
\description{
This function computes the value of the objective function for a model and an associated data set, as a sum of a term that computes the fit of model to data, a term that penalises the NA values produced by the model, and a term that penalises increasing size of the model.
}
\usage{
getFit(simResults, CNOlist, model, indexList=NULL, timePoint=c("t1", "t2"),
sizeFac=1e-04, NAFac=1, nInTot, simResultsT0=NULL)
}
\arguments{
  \item{simResults}{
matrix of simulated results (the full one as output by the simulator)
}
  \item{CNOlist}{
a CNOlist to compare the simulated results with
}
  \item{model}{
a model that has already been cut to contain only the reactions in the optimal bitstring
}
  \item{indexList}{
list of indexes as produced by indexFinder. User should not use this parameter
that is kept for back compatibility with previous version of the simulator
written in R. The new simulator (C code) does not neccesitate the usage of the
indexList parameter in this function anymore.
}

  \item{timePoint}{
"t1" or "t2" tells which time point we are looking at. If timePoint=t1 then we will compare the simResults to the results stored in CNOlist$valueSignals[[2]].  If timePoint=t1 then we will compare the simResults to the results stored in CNOlist$valueSignals[[2]]
}
  \item{sizeFac}{
weights the penalty for the size of the model, default=0.0001
}
  \item{NAFac}{
weights the penalty for the number of NAs
}
  \item{nInTot}{
the number of inputs in the model prior to cutting, used to normalised the size penalty
}
  \item{simResultsT0}{
   Results of the time 0 simulator (internal usage of gaBinaryT1)
}

}
\details{
BE AWARE: contrary to what is done in the Matlab version of CellNOpt, here the simulation results are computed beforehand and the model that is input into this function is a model that has already been cut i.e. that only contains the reactions present in the optimised model (i.e.should be the same model as the one that you input into the simulator). Also, the simResults matrix is the full one as output by the simulator, i.e. it contains results for all species in the model, not only the signals
}
\value{
This function returns a single number, the value of the objective function.
}
\references{
J. Saez-Rodriguez, L. G. Alexopoulos, J. Epperlein, R. Samaga, D. A. Lauffenburger, S. Klamt and P. K. Sorger. Discrete logic modeling as a means to link protein signaling networks with functional analysis of
mammalian signal transduction, Molecular Systems Biology, 5:331, 2009.
}
\author{
C. Terfve
}
\seealso{
\link{gaBinaryT1}, \link{simulatorT1}
}
\examples{
#Here we will evaluate the fit of the full initial model, 
#without pre-processing or any optimisation

data(CNOlistToy,package="CellNOptR")
data(ToyModel,package="CellNOptR")

indicesToy<-indexFinder(CNOlistToy,ToyModel,verbose=FALSE)
ToyFields4Sim<-prep4sim(ToyModel)
simResults<-simulatorT1(
	CNOlist=CNOlistToy,
	model=ToyModel,
	simList=ToyFields4Sim,
	indexList=indicesToy)
simResults = simResults[, indicesToy$signals]
Score<-getFit(
	simResults=simResults,
	CNOlist=CNOlistToy,
	model=ToyModel,
	timePoint="t1", 
    nInTot=length(which(ToyModel$interMat == -1))
    )
}

\name{mapBack}
\alias{mapBack}

\title{
  Map an optimised model back onto the PKN model.
}

\description{
  Map an optimised model back onto the PKN model.
}

\usage{
mapBack(model, PKN, bString)

}


\arguments{
  \item{model}{
the optimised model
}
  \item{PKN}{
the Prior Knowledge network
}
  \item{bString}{
the optimised bitString
}
}



\value{
\item{bStringPKN}{the corresponding bitstring corresponding to the original PKN.
}
}

\author{
    F.Eduati
}

\seealso{
\link{graph2sif}, \link{sif2graph}, \link{readSIF}, 
}

\name{randomizeCNOlist}
\alias{randomizeCNOlist}
\title{
add noise to the data contained in a CNOlist.
}
\description{
This function takes in a CNOlist and does the normalisation of the data between
0 and 1, according to two different procedures (see details).
}
\usage{
randomizeCNOlist(cnolist, sd=0.1, minValue=0, maxValue=1, mode="gaussian")
}


\arguments{
  \item{cnolist}{
a CNOlist
}
  \item{sd}{standard deviation to be used when adding gaussian noise. Not used
if mode is uniform.
}
  \item{minValue}{When adding Gaussian noise, the result may be below the
minValue(default 0). If so, the value is set to minValue.
}
  \item{maxValue}{When adding Gaussian noise, the result may be above the
maxValue(default 1). If so, the value is set to maxValue.
}
  \item{mode}{The mode can be either 'gaussian', 'shuffle' or uniform'. In gaussian mode, a
gaussian noise is added to the data. The mean parameter is the
data and the standard deviation is defined by the sd parameter. In uniform mode,
the data is simply replaced by values taken from a uniform distribution between
0 and 1. In 'shuffle' mode all rows and columns are shuffled.
}

}

\value{
a noisy CNOlist
}
\author{
T. Cokelaer
}
\examples{
data(CNOlistToyMMB, package="CellNOptR")
cnolist = CNOlistToyMMB
cnolist2 = randomizeCNOlist(cnolist, mode="uniform")

# a method called randomize is available in the CNOlist class so you could type:
cnolist2 = randomize(cnolist, mode="uniform")
}

\name{cutAndPlot}
\alias{cutAndPlot}
\title{
Interface to cutAndPlotResults functions.
}
\description{
This function takes a model and cnolist as well as a list of optimised bitstring at different time points. It calls the appropriate cutAndPlotResultsTX function.
}
\usage{
cutAndPlot(CNOlist, model,  bStrings, plotPDF=FALSE, tag=NULL,
plotParams = list(maxrow = 10))
 
}
\arguments{
  \item{CNOlist}{
a CNOlist, corresponding to the optimisation one
}
  \item{model}{
a model (the full one that was used for optimisation)
}
  \item{bStrings}{
a bitstring for T1  as output by gaBinaryT1 (i.e. a vector of 1s and 0s)
}
  \item{plotPDF}{
TRUE or FALSE; tells whether you want a pdf to be produced or not
}
  \item{tag}{
NULL or string; tells whether you want to prefix filenames with a tag (replaces
the default behaviour).
}
\item{plotParams}{
a list of option related to the PDF and plotting outputs. (1) maxrow
is the maximum number of row used to plot the results. See
\link{plotOptimResultsPan} for other fields.
}

}
\value{
This function returns nothing. It plots a graph in your graphic window and sqve it in a file if asked
}
\author{
T. Cokelaer
}
\seealso{
\link{cutAndPlotResultsT1}
}
\examples{
#load data

data(CNOlistToy,package="CellNOptR")
data(ToyModel,package="CellNOptR")

#pre-process model
model = preprocessing(CNOlistToy, ToyModel)

cutAndPlot(CNOlistToy, model,
    bStrings=list(rep(1,length(model$reacID))),	plotPDF=FALSE)
}

\name{crossvalidateBoolean}
\alias{crossvalidateBoolean}
\title{
k-fold crossvalidation for Boolean model.
}
\description{
Cross-validation analysis for the boolean case.
}
\usage{
crossvalidateBoolean(CNOlist,model,nfolds=10,foldid=NULL, 
                                type=c('datapoint','experiment','observable'),timeIndex = 2,parallel=FALSE,...)
}
\arguments{
  \item{CNOlist}{
a CNOlist on which the score is based (based on valueSignals[[2]], i.e. data at time 1).
}
  \item{model}{
a model structure, as created by \code{readSIF}, normally pre-processed but that is not a requirement of this function.
}
  \item{nfolds}{
number of folds - default is 10. Although nfolds can be as large as the sample size (leave-one-out CV), it is not recommended for large datasets.
}
  \item{foldid}{
an optional vector of values between `1` and `nfold` identifying what fold each observation is in. If supplied, `nfold` can be missing.
}
  \item{type}{
define the way to do the crossvalidation. The default is type="datapoint"`, which assigns the data randomly into folds. The option `type="experiment"` uses whole experiments for crossvalidation (all data corresponding to a cue combination). The `type=observable` uses the subset of nodes across all experiments for crossvalidation.  
}
  \item{timeIndex}{
the index of the time point to optimize. Must be greater or equal to 2 (1 corresponds to time=0). Must be less than the number of time points. Default is 2.
}
  \item{parallel}{
verbose parameter, indicating wheter to parallelize the cross-validation procedure or not (default set to FALSE).
}
 \item{...}{
 further arguments are passed to gaBinaryT1  
}

}
\details{
Does a k-fold cross-validation for Boolean CellNOpt models. In k-iterations a  fraction of the data is eliminated from the CNOlist. The model is trained on the  remaining data and then the model predicts the held-out data. Then the prediction accuracy is reported for each iteration.
}
\value{
This function returns a list with elements:
\item{cvScores}{cross-validation scores}
\item{fitScores}{fitting scores}
\item{bStrings}{the optimal bit-string list for each run}
\item{crossvalidate.call}{echo of the function which was called}
\item{foldid}{the fold id's}
}

\author{
A. Gabor, E. Gjerga
}
\examples{


data("ToyModel", package="CellNOptR")
data("CNOlistToy", package="CellNOptR")
pknmodel = ToyModel
cnodata = CNOlist(CNOlistToy)

# original and preprocessed network 
plotModel(pknmodel,cnodata)
model = preprocessing(data = cnodata,
					model = pknmodel,
					compression = TRUE,
					expansion = TRUE)
plotModel(model,cnodata)

# original CNOlist contains many timepoints, we use only a subset
plot(cnodata)
selectedTime = c(0,10)
cnodata_prep = cutCNOlist(cnodata,
                          model = model,
                          cutTimeIndices = which(!getTimepoints(cnodata) \%in\% selectedTime))

plot(cnodata_prep)

# optimise and show results
opt = gaBinaryT1(CNOlist = cnodata_prep,model = model,verbose = FALSE)

# 10-fold crossvalidation using T1 data
# We use only T1 data for crossvalidation, because data in the T0 matrix is not independent.
# All rows of data in T0 describes the basal condition.

# Crossvalidation produce some text in the command window:  
\dontrun{
library(doParallel)
registerDoParallel(cores=3)
R=crossvalidateBoolean(CNOlist = cnodata_prep,
						model = model,
						type = "datapoint",
						nfolds = 10,
						parallel = TRUE)

}


}

\name{preprocessing}
\alias{preprocessing}
\title{
Performs the pre-processing steps
}
\description{
This function performs any of the following preprocessing steps:
    \enumerate{
        \item{removes Non-Controllable and Non-Observables nodes}
        \item{compress the model}
        \item{and-gates expansion}
    }
}
\usage{
preprocessing(data, model, cutNONC=TRUE, compression=TRUE,
    expansion=TRUE, ignoreList=NA, maxInputsPerGate=2, verbose=TRUE)
}

\arguments{
  \item{data}{
the CNOlist that contains the data that you will use
}
  \item{model}{
the model object as returned by \link{readSIF}
}
  \item{cutNONC}{
Removes the NONC nodes using \link{cutNONC} and \link{findNONC} (Default is TRUE).
}
  \item{compression}{
Compress the model using \link{compressModel} (Default is TRUE).
}
  \item{expansion}{
Add and gates using \link{expandGates} (Default is TRUE).
}
  \item{ignoreList}{
   an index vector of states to ignore incoming edges in \link{expandGates}.
}
  \item{maxInputsPerGate}{
used by the \link{expandGates} function to set maximum inputs per and gates.
}
  \item{verbose}{
verbose option (Default is TRUE).
}
}


\details{
  The function can apply any or none of the pre-processing steps. It returns the
new model and the indices returned by indexFinder.

}

\value{
  the new model
}

\author{
 T. Cokelaer
}


\seealso{
\link{readSIF}, \link{readMIDAS}, \link{cutNONC}, \link{findNONC},
\link{compressModel}, \link{expandGates}.

}
\examples{
data(ToyModel,package="CellNOptR")
data(CNOlistToy,package="CellNOptR")
model = preprocessing(CNOlistToy, ToyModel, cutNONC=FALSE)
}

\name{plotCNOlistLarge}
\alias{plotCNOlistLarge}
\title{
Plot the data in a  CNOlist, for lists with many conditions.
}
\description{
This function plots the data in a CNOlist as a matrix of plots with a  row for each condition and a column for each signal, and an extra plot for each row that specifies which cues are present.
}
\usage{
plotCNOlistLarge(CNOlist,nsplit=4, newDevice=FALSE)
}
\arguments{
  \item{CNOlist}{
    a CNOlist
  }
  \item{nsplit}{
    the number of splits in the condition dimension (one new plot window will be produced for each split, i.e. if you have 80 conditions and specify 4 splits you will get 4 plots with 20 conditions each).
  }

  \item{newDevice}{
    nsplit plots are created within the same device. In principle, most of the R
    Graphical USer Interface will allow the user to navigate between the different
    plots. However, if scripting only the last plot will be seen. If you want to
    create new device for each different plot, then set this option to TRUE.
  }

}
\details{
This function can plot normalised values or the un-normalised ones, it just needs a CNOlist. This function makes plots of CNOlists that are more readable when many conditions are present in the data.  In addition to plotting the conditions divided into multiple plots, this function also plots the cues divided in two columns, one for inhibitors and one for stimuli.
}
\value{
This function just produces plots on your graphics window.
}
\author{
C. Terfve
}
\seealso{
\link{plotCNOlist}, \link{plotCNOlistPDF}, \link{plotCNOlistLargePDF}
}
\examples{
data(CNOlistDREAM,package="CellNOptR")
plotCNOlistLarge(CNOlistDREAM, nsplit=2)
}

\name{writeNetwork}
\alias{writeNetwork}
\title{
Write a previous knowledge network model to a sif file (with attribute files), as well as a dot file
}
\description{
This function writes the original previous knowledge network (the model that you loaded in the beginning of your analysis)  in a sif file, with a nodes attribute file that specifies if each node was stimulated/inhibited/signal/compressed/non-controllable-non-observable and an edge attribute file that specifies if the edge was absent in the optimal model (0) present in the optimal model at t1 (1) or present in the optimal model at t2 (2).

This function also writes a Graphviz dot file that contains the same information (see \code{writeDot} for more information about the dot file conventions).
}
\usage{
writeNetwork(modelOriginal, modelComprExpanded, optimResT1, optimResT2, CNOlist,
tag = NULL,verbose=FALSE)
}
\arguments{
  \item{modelOriginal}{
The PKN model
}
  \item{modelComprExpanded}{
The scaffold model (i.e. compressed and expanded)
}
  \item{optimResT1}{
The results of the optimisation process at t1
}
  \item{optimResT2}{
The results of the optimisation process at t2 (set this to NA if you have performed a one time point optimisation).
}
  \item{CNOlist}{
The CNOlist on which the optimisation is based
}
  \item{tag}{
NULL or string; tells whether you want to prefix filenames with a tag (replaces
the default behaviour).
}
  \item{verbose}{
If verbose=TRUE, the function prints a message every time an edge in the scaffold network couldn't be mapped back to the PKN
}
}
\details{
The weights of the edges are computed as the mean across models within the relative tolerance limits, as output in the results from the optimisation \code{$stringsTol}. Strings that are in \code{$stringsTol} are the ones that are within the relative tolerance limits around the best solution in the population across all generations of the optimisation. 

!If there is no time 2, then the argument optimResT2 should be = NA

This function maps back the edges weights from the optimised (expanded and compressed) model to the original model. The mapping back only works if the path has length 2 at most (i.e. you have node1-comp1-comp2-node2, where comp refer to nodes that have been compressed).
}
\value{
This function does not have any output, it just writes a sif file, an edge attribute file, and a node attribute file
}
\author{
C. Terfve
}
\note{
The mapback of this function is still an open question, even in the Matlab version.  Future developments will include more robust versions of the mapping back algorithm, probably as a separate mapback function.
}
\seealso{
writeScaffold, writeDot
}
\examples{

#load data

data(CNOlistToy,package="CellNOptR")
data(ToyModel,package="CellNOptR")

#pre-process model

indicesToy<-indexFinder(CNOlistToy,ToyModel,verbose=TRUE)
ToyNCNOindices<-findNONC(ToyModel,indicesToy,verbose=TRUE)
ToyNCNOcut<-cutNONC(ToyModel,ToyNCNOindices)
indicesToyNCNOcut<-indexFinder(CNOlistToy,ToyNCNOcut)
ToyNCNOcutComp<-compressModel(ToyNCNOcut,indicesToyNCNOcut)
indicesToyNCNOcutComp<-indexFinder(CNOlistToy,ToyNCNOcutComp)
ToyNCNOcutCompExp<-expandGates(ToyNCNOcutComp)

#optimise

ToyFields4Sim<-prep4sim(ToyNCNOcutCompExp)
initBstring<-rep(1,length(ToyNCNOcutCompExp$reacID))
ToyT1opt<-gaBinaryT1(
	CNOlist=CNOlistToy,
	model=ToyNCNOcutCompExp,
	initBstring=initBstring,
	verbose=TRUE,
	maxGens=2,
	popSize=5)

#write network

writeNetwork(
	modelOriginal=ToyModel,
	modelComprExpanded=ToyNCNOcutCompExp,
	optimResT1=ToyT1opt,
	optimResT2=NA,
	CNOlist=CNOlistToy)
}

\name{plotCNOlistLargePDF}
\alias{plotCNOlistLargePDF}
\title{
Plots a CNOlist into a pdf file, for lists with many conditions.
}
\description{
This function is a wrapper for plotCNOlistLarge, that plots the output directly in a pdf file.
}
\usage{
plotCNOlistLargePDF(CNOlist, filename, nsplit, width=14, height=7)
}
\arguments{
  \item{CNOlist}{
a CNOlist
}
  \item{filename}{
a name for your pdf file, eg. "plot.pdf"
}
  \item{nsplit}{
the number os splits along the condition dimension (see plotCNOlistLarge)
}
  \item{width}{
set the width of the PDF document.
}
  \item{height}{
set the height of the PDF document.
}
}


\details{
This function makes plots of CNOlists that are more readable when many conditions are present in the data.  In addition to plotting the conditions divided into multiple plots, this function also plots the cues divided in two columns, one for inhibitors and one for stimuli.
}
\value{
This function doesn't return anything, it just produces a pdf file with your plots, in your current working directory.
}
\author{
C. Terfve
}
\seealso{
\link{plotCNOlistLarge}, \link{plotCNOlist}, \link{plotCNOlistPDF}
}
\examples{
data(CNOlistDREAM,package="CellNOptR")
plotCNOlistLargePDF(CNOlistDREAM, filename="dreamData.pdf",nsplit=2)
}

\name{simulatorT2}
\alias{simulatorT2}
\title{
Deprecated since 1.3.28. Use \link{simulateTN} function instead.
Simulation of a boolean model for time 2
}
\description{
This function simulates a boolean model at time 2 where time 2 is assume to be a pseudo-steady states at a time scale slower than the pseudo-steady state evaluated at time 1
}
\usage{
simulatorT2(simResultsT1, CNOlist, model, simList, indexList, timeIndex=3)
}
\arguments{
  \item{simResultsT1}{
a matrix that is the output of simulatorT1 (i.e. one row per condition and one column per species IN THE MODEL)
}
  \item{CNOlist}{
a CNOlist
}
  \item{model}{
a Model that only contains the reactions to be evaluated, and the additional field \code{model$times} that should have been created inside the gabinaryTN optimisation engine.}
  \item{simList}{
a simList as created by prep4sim, that has also already been cut to contain only the reactions to be evaluated
}
  \item{indexList}{
an indexList as created by indexFinder
}
\item{timeIndex}{
argument requiter only for steady states T3 and above to specify the Time
indices.
}
}
\details{
This is the simulator for time T2, it is very similar to simulatorT1 but here we assume that we start from the simulated results at t1 (i.e. we start from a pseudo-steady state) then it does a first iteration, and whatever branch is set to be active at t2 has an effect that cannot be changed after the first iteration, i.e. the output node of a t2 iteration is fixed. We assume here that the model has already been cut, and that the cutting is based on keeping  all the edges that are set to either 1 or 2, and there is an additional field \code{$times} in the model that keeps the info of the t1/t2 (it is a vector of 1s and 2s of length=number of reaches present). Structurally the function is almost identical to simulatorT1 but it does a first iteration where all the gates that lead to a node that also receives a T2 gates are set to the same value as the t2 gate in the ANDs calculation.In the main loop, the nodes that are targets of t2 interactions are constantly reset to their value at the first iteration, at the end of each iteration (similarly to what is done with stimulated and inhibited species)

The \code{model$times} field is a vector of 1s and 2s that tells the simulator which interactions are expected to be active at t1 and which are at t2.
}
\value{
This function outputs a single matrix of format similar to valueSignals in the CNOlist but that contains an output for each species in the model.  This matrix is the simulated equivalent of valueSignals at time 2 if you consider only the columns given by indexSignals
}
\author{
C. Terfve
}
\seealso{
\link{simulateTN}, \link{cutAndPlotResultsT1}, \link{simulatorT1}
}
\examples{

# This computes the output of the full model, which is normally not done on a
# stand alone basis, but if you have a model and would like to visualise its
# output compared to your data, then this is what you should do

data(CNOlistToy2,package="CellNOptR")
data(ToyModel2,package="CellNOptR")

#Sim<-simulatorTN(
#   CNOlist=CNOlistToy2,
#   model=ToyModel2)

#Sim2<-simulatorTN(
#   simResultsT1=Sim, 
#  CNOlist=CNOlistToy2, 
#   model=ToyModel2, 
#   simList=ToyFields4Sim, 
#   indexList=indicesToy, timeIndex=3)


}
\keyword{internal}

\name{residualError}
\alias{residualError}
\title{
Compute the residual error for a dataset
}
\description{
This function takes in a CNOlist and computes the residual error, which is the minimum error between the scaled continuous data and a binary boolean approximation of this data.
}
\usage{
residualError(CNOlist)
}
\arguments{
  \item{CNOlist}{
a CNOlist
}
}
\details{
Be aware that it is expected that \code{$valueSignals[[1]]} holds t0 (all
signals=0) and \code{$valueSignals[[2]]} holds t1, \code{$valueSignals[[3]]}
holds t2 and so on. The output is a list of residual errors at each time greater
than 2. In addition, the total error is stored. For back compatibility, an
additional field called t1andt2 is stored (NA is only t1 provided). 

}
\value{
a vector with named entries t1, t2, ...tn, t1andt2 and total. If only t1 is
provided, t1andt2 is set to NA. The field t1andt2 may be removed in the future.
Use the field called total instead.
}
\author{
C. Terfve, T. Cokelaer
}
\seealso{
\link{makeCNOlist}, \link{normaliseCNOlist}, \link{getFit}
}
\examples{
data(CNOlistToy,package="CellNOptR")
resECNOlistToy<-residualError(CNOlistToy)
}

\name{write_bounds}
\alias{write_bounds}

\title{
Writing the set of boundaries for each integer variable for the ILP implementation of CellNOptR.
}
\description{
This function takes as input the integer variables assigned to each of the elements in the network and data in a CNOlist object
in order to produce the set boundaries for each integer variable. 
}
\usage{
write_bounds(model,
             midasTreatmentPart,
             y_vector,
             binary_variables)
}
\arguments{
  \item{model}{the model
  }
  \item{midasTreatmentPart}{the treatment part of CNOlist where data is stored
  }
  \item{y_vector}{the variables for each interaction in the PKN
  }
  \item{binary_variables}{the set of binary variables
  }


}

\author{
E Gjerga, H Koch
}
\references{
Alexander Mitsos, Ioannis N. Melas, Paraskeuas Siminelakis, Aikaterini D. Chairakaki, Julio Saez-Rodriguez, and Leonidas G. Alexopoulos. Identifying Drug Effects via Pathway Alterations using an Integer Linear Programming Optimization Formulation on Phosphoproteomic Data. PLoS Comput Biol. 2009 Dec; 5(12): e1000591.
}

\name{write_constraints}
\alias{write_constraints}
\alias{write_constraints}

\title{
Writing the set of constraints for the ILP implementation of CellNOptR.
}
\description{
This function takes as input the integer variables assigned to each of the elements in the network and data in a CNOlist object
in order to produce the set of constraints needed to be optimized by the ILP solver. 
}
\usage{
write_constraints(model,
                              midasExperimentPart,
                              midasTreatmentPart,
                              reaction_sets,
                              y_vector, 
                              midas, 
                              binary_variables)
}
\arguments{
  \item{model}{the model
  }
  \item{midasExperimentPart}{the experimental part of CNOlist where data is stored
  }
  \item{midasTreatmentPart}{the treatment part of CNOlist where data is stored
  }
  \item{reaction_sets}{the set of reactions
  }
  \item{y_vector}{the variables for each interaction in the PKN
  }
  \item{midas}{the midas table
  }
  \item{binary_variables}{the set of binary variables
  }


}

\author{
E Gjerga, H Koch
}
\references{
Alexander Mitsos, Ioannis N. Melas, Paraskeuas Siminelakis, Aikaterini D. Chairakaki, Julio Saez-Rodriguez, and Leonidas G. Alexopoulos. Identifying Drug Effects via Pathway Alterations using an Integer Linear Programming Optimization Formulation on Phosphoproteomic Data. PLoS Comput Biol. 2009 Dec; 5(12): e1000591.
}

\name{CNOlist-class}
\docType{class}
\alias{CNOlist-class}
%\alias{CNOlist,ANY}
\alias{CNOlist}


\alias{compatCNOlist,CNOlist-method}
\alias{getCues,CNOlist-method}
\alias{getInhibitors,CNOlist-method}
\alias{getSignals,CNOlist-method}
\alias{getStimuli,CNOlist-method}
\alias{getTimepoints,CNOlist-method}
\alias{getVariances,CNOlist-method}
\alias{plot,CNOlist,CNOlist-method}
%\alias{plot,CNOlist,missing-method}
%\alias{plot,CNOlist,CNOlist,CNOlist-method}
\alias{length,CNOlist-method}
\alias{randomize,CNOlist-method}
\alias{setSignals<-,CNOlist-method}
\alias{readErrors,CNOlist-method}
\alias{writeErrors,CNOlist-method}

%\alias{plot,CNOlist-method}
%\alias{plot,CNOlist,ANY,CNOlist-method}
%\alias{plot,CNOlist,CNOlist,CNOlist-method}
%\alias{plot,CNOlist,ANY}
%\alias{plot,CNOlist,CNOlist}







\title{Class \code{"CNOlist"}}
\description{
This function takes as input the filename of a MIDAS file (or the list returned
by \link{makeCNOlist}) and returns an instance of CNOlist class. It provides an 
object oriented approach to manipulate CNOlist. This function calls
\link{readMIDAS} and \link{makeCNOlist}.
}
\section{Objects from the Class}{
Objects can be created by calls of the form \code{new("CNOlist", ...)}.
%%  ~~ describe objects here ~~ 
}
\section{Slots}{
  \describe{
    \item{\code{cues}:}{Object of class \code{"matrix"} }
    \item{\code{inhibitors}:}{Object of class \code{"matrix"}  }
    \item{\code{stimuli}:}{Object of class \code{"matrix"} }
    \item{\code{signals}:}{Object of class \code{"list"}  }
    \item{\code{variances}:}{Object of class \code{"list"}  }
    \item{\code{timepoints}:}{Object of class \code{"vector"} timepoints
    contained in the signals matrix. }

  See \link{CNOlist-methods} for details
}
}


\section{Methods}{
  Available methods are plot, compatCNOlist, randomize, length.  See
\link{CNOlist-methods} for details.
}


\author{
T. Cokelaer
}
\seealso{
\code{\link{randomizeCNOlist}}, \code{\link{plotCNOlist}}, \code{\link{plotCNOlist2}}
}
\examples{
showClass("CNOlist")

files<-dir(system.file("ToyModel",package="CellNOptR"),full=TRUE)
cnolist  = CNOlist(files[[1]])
# getters:
getCues(cnolist)
getInhibitors(cnolist)
getSignals(cnolist)
getVariances(cnolist)
getTimepoints(cnolist)
getStimuli(cnolist)
# In version 1.3.30 and above, use the plot method instead of former plotCNOlist function.
plot(cnolist)
new_cnolist = randomize(cnolist)
length(cnolist)
}
\keyword{classes}

\name{writeFile}
\alias{writeFile}

\title{
Writing the ILP problem.
}
\description{
This function takes as input the objective function. constraints, bounds and the solver path in order to generate a file containing the ILP problem. 
}
\usage{
writeFile(objectiveFunction,
                      constraints,
                      bounds,
                      binaries)
}
\arguments{
  \item{objectiveFunction}{the objective function of the ILP problem
  }
  \item{constraints}{the set of constraints of the ILP problem
  }
  \item{bounds}{the set of bounds for each integer variable
  }
  \item{binaries}{the set of binary variables
  }

}

\author{
E Gjerga, H Koch
}

\name{readMIDAS}
\alias{readMIDAS}
\alias{readMidas}
\title{Reads in a CSV MIDAS file}
\description{
This function takes in a single argument, the name of a CSV MIDAS file
containing the data, and returns a list that contains all the elements to build
a CNOlist. The output of this function should be used as input for
\code{\link{makeCNOlist}}.
}
\usage{
readMIDAS(MIDASfile, verbose=TRUE)
}

\arguments{
    \item{MIDASfile}{
        a CSV MIDAS file (see details)
    }
    \item{verbose}{
        logical (default to TRUE).
    }
}

\details{
This function does not return a CNOlist, but the output of this function can be 
used directly into makeCNOlist to create one. The MIDAS file format is described
in Saez-Rodriguez et al. (2008).

If you have all of the readouts measured at the same series of time points, you
can specify a unique DA: column which must have the format "DA:ALL". 

}
\value{
this function returns a list with fields:
\item{dataMatrix}{matrix containing the data in the MIDAS file}
\item{TRcol}{indexes of the columns that contain the treatments (excluding cell line)}
\item{DAcol}{indexes of the columns that contain the data time points}
\item{DVcol}{indexes of the columns that contain the actual values (measurements)}
}
\references{
J. Saez-Rodriguez, A. Goldsipe, J. Muhlich, L. Alexopoulos, B. Millard, D. A. Lauffenburger, P. K. Sorger 
\emph{Flexible Informatics for Linking Experimental Data to Mathematical Models via DataRail.} Bioinformatics, 24:6, 840-847 (2008). 
}
\author{
C.Terfve
}
\seealso{
\link{makeCNOlist}
}
\examples{
   
   cpfile<-dir(system.file("ToyModel",package="CellNOptR"),full=TRUE)
   file.copy(from=cpfile,to=getwd(),overwrite=TRUE)
   dataToy<-readMIDAS(MIDASfile='ToyDataMMB.csv')
   CNOlistToy<-makeCNOlist(dataset=dataToy,subfield=FALSE)
}

\name{buildBitString}
\alias{buildBitString}
\title{
Build the final bit string vector and times vector based on a list of optimised bit strings at different
time points.
}
\description{
This function takes as input a list of vectors (can be only one). Each vector
represents an optimised bit string at a different time point (as returned by 
the gaBinary functions). The first optimised bit string have the same length as
the model to be optimised. The length of the following vectors corresponds to
the number of zeros found in the previous bitstring. For instance,
the following list of bit strings \code{bStrings = list(c(1,1,1,0,0,0),
c(1,0,0))} is correct whereas \code{bStrings = list(c(1,1,1,0,0,0), c(1,0))} is
incorrect.


This function is used internally by \link{computeScoreTN} and
\link{simulateTN}. It should not be used by a user in principle.
However, it may be useful for post processing.

New in version 1.3.28.
}
\usage{
buildBitString(bStrings)
}
\arguments{
    \item{bStrings}{
        a list of bit strings as returned by the optimisation at different time points.
    }
}


\value{
This function returns 2 components. The first one as explained in the
description is a vector of same length as the first input vector in the
bStrings argument. The second component is a vector that keeps track 
of the time point at which each bit was optimised (see example).
}
\author{
T. Cokelaer
}
\examples{

# Considering the optimised bitstrings at T1, T2 and T3 to be c(1,1,0,1,0,0),
# c(0,1,0) and c(0,1), then we can build the overall bitStrings as follows:

res = buildBitString(list(c(1,1,0,1,0,0), c(0,1,0), c(0,1)))

# The results bit string is accessed through the bs field:
res$bs
#[1] 1 1 0 1 1 1

# and times at which each bit is activated with the bsTimes field:
res$bsTimes

# res$bsTimes = c(1,1,0,1,2,3)

}
\keyword{internal}

\name{plotCNOlist}
\alias{plotCNOlist}
\title{
Plot the data in a  CNOlist
}
\description{
This function plots the data in a CNOlist as a matrix of plots with a  row for
each condition and a column for each signal, and an extra plot for each row that
specifies which cues are present..
}
\usage{
plotCNOlist(CNOlist)
}
\arguments{
  \item{CNOlist}{
a CNOlist
}
}
\details{
This function can plot the normalised values or the un-normalised ones, it just needs a CNOlist.
}
\value{
This function just produces a plot on your graphics window
}
\author{
C. Terfve
}
\seealso{
plotCNOlistPDF, plotCNOlistLarge, plotCNOlistLargePDF
}
\examples{
data(CNOlistToy,package="CellNOptR")
plotCNOlist(CNOlistToy)
}

\name{readSIF}
\alias{readSif}
\alias{readSIF}
\title{
Read a SIF file and create a model object
}
\description{
This function reads in a cytoscape SIF file and creates a model object that can be used in the CellNOptR procedure.
}
\usage{
readSIF(sifFile)
}
\arguments{
  \item{sifFile}{
The name of a SIF file (Cytoscape format). See details for the accepted format.
}
}
\details{
The input argument is the name of a SIF file that contains a prior knowledge
network. The SIF format (See http://wiki.cytoscape.org/Cytoscape_User_Manual/Network_Formats/) 
can be of the form:

\code{ nodeA typeA node2}

or

\code{ node2 typeB node3 node4}


with space or tabulations in between types and nodes. Spaces and tabulations
have different meaning in the original cytoscape format. However, we do not
differentiate them. Therefore names of the nodes cannot have white space inside
them.

The accepted values for the types are -1 (inhibitor) or 1 (normal relation).

If there are ANDs they should be introduced as dummy nodes called "and#" (don't
forget the number after "and" otherwise this won't be recognised).  Please be
aware that "and" nodes are not expected to be negated, i.e. there are not
supposed to be !and1=xyz because that amounts to inverting the sign of all
inputs of and1, which is more simply done at the inputs level.

The SIF format can also include unconnected node that is a row with a single
name:

\code{nodeA}

Although there would be no error, these type of rows are ignored. 

}
\value{
a model list with fields:
    \item{interMat}{contains a matrix with column for each reaction and a row for each species, with a -1 where the species is the source node and a +1 where the species is a target node, and 0 otherwise}
    \item{notMat}{has the same format as interMat but just contains a 1 if the source node enters the reac with a negative effect, and 0 otherwise}
    \item{namesSpecies}{vector that contains the names of the species in the same order as the rows of the interMat and notMat matrices}
    \item{reacID}{vector that holds character strings specifying the reaction in full letters, in the same order as the columns of interMat and notMat}
}

\references{
Shannon P, Markiel A, Ozier O, Baliga NS, Wang JT, Ramage D, Amin N, Schwikowski
B, Ideker T. Cytoscape: a software environment for integrated models of
biomolecular interaction networks. Genome Research 2003 Nov; 13(11):2498-504.
}
\author{
C. Terfve
}
\examples{

cpfile<-dir(system.file("ToyModel",package="CellNOptR"),full=TRUE)
file.copy(from=cpfile,to=getwd(),overwrite=TRUE)
ToyModel<-readSIF(sifFile="ToyPKNMMB.sif")
}

\name{writeReport}
\alias{writeReport}
\title{
Write a report of a CellNOptR analysis
}
\description{
This function writes a short report of a CellNOptR analysis in an html page, that is linked to the various graphs produced
}
\usage{
writeReport(modelOriginal, modelOpt, optimResT1, optimResT2, CNOlist, directory,
    namesFiles = list(dataPlot = NA, evolFitT1=NA, evolFitT2=NA, simResultsT1=NA,
    simResultsT2=NA, scaffold=NA, scaffoldDot=NA, tscaffold=NA, wscaffold=NA,
    PKN=NA, PKNdot=NA, wPKN=NA,nPKN=NA), namesData = list(CNOlist=NA, model=NA),
    resE=NULL)
}


\arguments{
  \item{modelOriginal}{
the original previous knowledge network (i.e. model that you loaded) in a model list format
}
  \item{modelOpt}{
the model that was actually used for optimisation (i.e. the scaffold network, after compression and expansion) in a model list format
}
  \item{optimResT1}{
the results of the optimisation at t1, as output by gabinaryT1
}
  \item{optimResT2}{
the results of the optimisation at t2, as output by gabinaryTN. Set this to NA if you have performed a one time point optimisation.
}
  \item{CNOlist}{
a CNOlist
}
  \item{directory}{
the name of a new directory that will be created, where your results will be moved
}
  \item{namesFiles}{
 a list of the names of the files that should have been created. Depending on
whether a t2 optimisation was performed or not, all or some of the following
fields are expected: dataPlot, evolFitT1, evolFitT2, simResultsT1, simResultsT2,
scaffoldDot, scaffold, tscaffold, wscaffold, PKN, PKNdot, wPKN, nPKN.
}
  \item{namesData}{
a list with fields \code{$CNOlist} and \code{$model} that contain strings that
are meaningful identifiers of your data and previous knowledge network (for your
own record).
}
  \item{resE}{
a vector with named entries t1, t2 t1andt2, as produced by the function
\code{ResidualError}, that contains the residual error associated with the
discretisation of the data. Since version 1.3.29, there is no need to provide
this argument. Kept for back compatibility.
}
}
\details{
Future versions of this function might directly write and compile a tex file.
}
\value{
This function produces a directory and moves all the files of namesFiles to it,
then it creates an html report that contains infos about the optimisation
process.
}
\author{
C. Terfve
}
\seealso{
\link{writeNetwork}, \link{writeScaffold}
}
\examples{

#load data

data(CNOlistToy,package="CellNOptR")
data(ToyModel,package="CellNOptR")

#pre-process model (partial)

indicesToy<-indexFinder(CNOlistToy,ToyModel,verbose=TRUE)
ToyNCNOcutComp<-compressModel(ToyModel,indicesToy)
indicesToyNCNOcutComp<-indexFinder(CNOlistToy,ToyNCNOcutComp)
ToyNCNOcutCompExp<-expandGates(ToyNCNOcutComp)

#optimise

ToyFields4Sim<-prep4sim(ToyNCNOcutCompExp)
initBstring<-rep(1,length(ToyNCNOcutCompExp$reacID))
ToyT1opt<-gaBinaryT1(
	CNOlist=CNOlistToy,
	model=ToyNCNOcutCompExp,
	initBstring=initBstring, 
	maxGens=2,
	popSize=5,
	verbose=TRUE)

#write report

namesFilesToy<-list(
	dataPlot=NA,
	evolFitT1=NA,
	evolFitT2=NA,
	simResultsT1=NA,
	simResultsT2=NA,
	scaffold=NA,
	scaffoldDot=NA,
	tscaffold=NA,
	wscaffold=NA,
	PKN=NA,
	PKNdot=NA,
	wPKN=NA,
	nPKN=NA)

writeReport(
	modelOriginal=ToyModel,
	modelOpt=ToyNCNOcutCompExp,
	optimResT1=ToyT1opt,
	optimResT2=NA,
	CNOlist=CNOlistToy,
	directory="testToy",
	namesFiles=namesFilesToy,
	namesData=list(CNOlist="Toy",model="ToyModel"),
	resE=NA)
}

\name{CNOlistDREAM}
\docType{data}
\alias{CNOlistDREAM}
\title{Data used for the DREAM3 challenge}
\description{This data object contains the DREAM data used in the package vignette, already loaded and formatted as a CNOlist object. This is to be used with the model "DreamModel". This is a data collected on HepG2 cells cultivated with or without stimulation of tgfa, ilk, mek12, pi3k and p38, in combination with inhibition of igf1 and/or il1a.  Seven phosphoproteins are measured using Luminex xMAP assays: akt, erk12, ikb, jnk12, p38, hsp27 and mek12.}
\usage{data(CNOlistDREAM)}
\format{CNOlistDREAM is a list with the fields "namesCues" (character vector), "namesStimuli" (character vector), "namesInhibitors" (character vector), "namesSignals" (character vector), "timeSignals" (numerical vector), "valueCues" (numerical matrix), "valueInhibitors" (numerical matrix), "valueStimuli"(numerical matrix), "valueSignals"(numerical matrix).}
\source{This data and model is extracted from the Matlab version of CellNOpt1.0 (\url{http://www.ebi.ac.uk/saezrodriguez/software.html#CellNetOptimizer}).}
\references{
\enumerate{
\item{J. Saez-Rodriguez, L. G. Alexopoulos, J. Epperlein, R. Samaga, D. A.
Lauffenburger, S. Klamt and P. K. Sorger. Discrete logic modeling as a means to
link protein signaling networks with functional analysis of mammalian signal
transduction, Molecular Systems Biology, 5:331, 2009.}
\item{ Prill RJ, Marbach D, Saez-Rodriguez J, Sorger PK, Alexopoulos LG, Xue X,
Clarke ND, Altan-Bonnet G, and Stolovitzky G. Towards a rigorous assessment of
systems biology models: the DREAM3 challenges. PLoS One, 5(2):e9202, 2010. }
}
}
\keyword{datasets}

\name{cutNONC}
\alias{cutNONC}
\title{
Cuts the non-observable/non-controllable species from the model
}
\description{
This function cuts the non-observable and/or non-controllable species from the model, and returns a cut model.
}
\usage{
cutNONC(model, NONCindexes)
}
\arguments{
  \item{model}{
a model structure, as produced by readSIF
}
  \item{NONCindexes}{
a vector of indices of species to remove in that model, as produced for example by findNONC
}
}
\details{
This function takes in a model and a vector of indices of species to remove in that model and it removes those species and any reaction involving them (be aware, if you have x&y=z and x is to be removed, then the function produces y=z, because it works by removing entire rows of the model matrices and then removes the columns that do not have either an input or an output). This function could actually be used to cut any species, not only NONC species.
}
\value{
a model
}
\author{
C.Terfve
}
\seealso{
\link{findNONC}, \link{readSIF}
}
\note{No need to call this function directly since version 0.99.24. Use
\link{preprocessing} instead.}

\examples{
data(CNOlistToy,package="CellNOptR")
data(ToyModel,package="CellNOptR")
indicesToy<-indexFinder(CNOlistToy,ToyModel,verbose=FALSE)
ToyNCNOindices<-findNONC(ToyModel,indicesToy,verbose=FALSE)
ToyNCNOcut<-cutNONC(ToyModel,ToyNCNOindices)
}

\name{simulatorT1}
\alias{simulatorT1}
\title{
Simulation of a boolean model
}
\description{
This is the simulator, inspired from BoolSimEngMKM in the Matlab CellNOpt, to be used on one time point simulations
}
\usage{
simulatorT1(CNOlist, model, simList, indexList, mode=1)
}
\arguments{
  \item{CNOlist}{
a CNOlist
}
  \item{model}{
a model that only contains the reactions to be evaluated
}
  \item{simList}{
a simList as created by prep4sim, that has also already been cut to contain only the reactions to be evaluated
}
  \item{indexList}{
an indexList as created by indexFinder
}
  \item{mode}{switch to use the cSimualor for time 0 or 1}

}
\details{
Differences from the BoolSimEngMKM simulator include: 
the valueInhibitors has not been previously flipped;
the function outputs the values across all conditions for all species in the model, instead of only for the signal species. This is because then the  output of this function can be used as initial values for the version of the simulator that works on time point 2 (not implemented in this version).

If you would like to compute the output of a model that contains some of the gates in the model but not all, we suggest that you use the function \code{SimulateT1} and specify in the \code{bStringT1} argument which gates you want to be included.  Indeed, SimulateT1 is a wrapper around simulatorT1 that takes care of cutting the model for you before simulating it.
}
\value{
This function outputs a single matrix of format similar to valueSignals in the CNOlist but that contains an output for each species in the model.  This matrix is the simulated equivalent of valueSignals at time 1, if you consider only the columns given by indexSignals.
}
\references{
\enumerate{
\item{J. Saez-Rodriguez, L. G. Alexopoulos, J. Epperlein, R. Samaga, D. A.
Lauffenburger, S. Klamt and P. K. Sorger. Discrete logic modeling as a means to
link protein signaling networks with functional analysis of mammalian signal
transduction, Molecular Systems Biology, 5:331, 2009.}
\item{M. K. Morris, J. Saez-Rodriguez, D. Clarke, P. K. Sorger, D. A.
Lauffenburger. Training Signaling Pathway Maps to Biochemical Data with
Constrained Fuzzy Logic: Quantitative Analysis of Liver Cell Responses to
Inflammatory Stimuli, PLoS Comp. Biol., 7(3): e1001099, 2011.}
}
}
\author{
C. Terfve
}
\seealso{
\link{simulateTN}, \link{cutAndPlotResultsT1}
}
\examples{
#This computes the output of the full model, which is normally not done on a stand alone basis, but if you have a model and would like to visualise its output compared to your data, then this is what you should do

data(CNOlistToy,package="CellNOptR")
data(ToyModel,package="CellNOptR")

indicesToy<-indexFinder(CNOlistToy,ToyModel,verbose=TRUE)
ToyFields4Sim<-prep4sim(ToyModel)

Sim<-simulatorT1(
	CNOlist=CNOlistToy,
	model=ToyModel,
	simList=ToyFields4Sim,
	indexList=indicesToy)
}

\name{writeDot}
\alias{writeDot}
\title{
Write a model, and attached features, to a dot file 
}
\description{
This function writes a model to a Graphviz dot file with encoded features such as edge weight and nodes status (see details).
}
\usage{
writeDot(dotNodes,dotMatrix,model,filename)
}
\arguments{
  \item{dotNodes}{
internal variables created by writeNetwork or writeScaffold; dotNodes is a matrix with 2 columns: the first has the node names,and the second the attributes (signal, stimulated, inhibited, compressed, nano). A node can appear twice in this matrix if it belongs to more of one of the above categories; a node could also not appear here if it is is none of these categories
}
  \item{dotMatrix}{
internal variables created by writeNetwork or writeScaffold; dotMatrix is a matrix with 4 or 5 columns, and a row for each reaction:the first column holds the name of the input node, the second column holds the sign of the reaction (-1 if negative, 1 if positive), the third column holds the name of the output node, the fourth column holds the time stamp (0,1,2), an optional 5th column holds the weights of the edges
}
  \item{model}{
A model to be plotted, if used inside writeNetwork then this should be the previous knowledge network (modelOriginal), if inside writeScaffold then this should be the scaffold (modelComprExpanded)
}
  \item{filename}{
a name for the file
}
}
\details{
This function is not to be used on its own, it should be used internally to writeNetwork or writeScaffold. For the colouring of the nodes, nodes that are both stimulated and inhibited or any other combination, only one colour per category is used, and the following order of priority for the colours is used: signals prime over inhibited nodes which primes over stimulated nodes which primes over non-controllable/non-observable nodes, which primes over compressed.  Nodes that are neither of those have a black contour, stimulated nodes are green, inhibited are red, measure are blue, compressed and non-controllable/non-observable nodes are black and dashed.  Edges are coloured according to time stamp in the optimal model (green=t1, blue=t1 and/or t2, grey=neither); on the scaffold, the strokes of the edges reflects the weights in the models within reltol (i.e. for each edge, the weight is the frequency with which it appeared among the models within the relative tolerance boundaries around the best solution).
}
\value{
This function does not have any output, it just writes a dot file in your working directory. 
}
\references{
Emden R. Gansner ,  Stephen C. North. An Open Graph Visualization System and Its Applications to Software Engineering. Software - Practice and Experience (1999)
}
\author{
C. Terfve
}
\seealso{
\link{writeNetwork}, \link{writeScaffold}
}
\examples{

#load data

data(CNOlistToy,package="CellNOptR")
data(ToyModel,package="CellNOptR")

#pre-process model

indicesToy<-indexFinder(CNOlistToy,ToyModel,verbose=TRUE)
ToyNCNOindices<-findNONC(ToyModel,indicesToy,verbose=TRUE)
ToyNCNOcut<-cutNONC(ToyModel,ToyNCNOindices)
indicesToyNCNOcut<-indexFinder(CNOlistToy,ToyNCNOcut)
ToyNCNOcutComp<-compressModel(ToyNCNOcut,indicesToyNCNOcut)
indicesToyNCNOcutComp<-indexFinder(CNOlistToy,ToyNCNOcutComp)
ToyNCNOcutCompExp<-expandGates(ToyNCNOcutComp)

#optimise

ToyFields4Sim<-prep4sim(ToyNCNOcutCompExp)
initBstring<-rep(1,length(ToyNCNOcutCompExp$reacID))
ToyT1opt<-gaBinaryT1(
	CNOlist=CNOlistToy,
	model=ToyNCNOcutCompExp,
	initBstring=initBstring, 
	maxGens=2,
	popSize=5, 
	verbose=TRUE)

#write network

writeNetwork(
	modelOriginal=ToyModel,
	modelComprExpanded=ToyNCNOcutCompExp,
	optimResT1=ToyT1opt,
	optimResT2=NA,
	CNOlist=CNOlistToy)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/PKN_ToyPB.R
\docType{data}
\name{pknmodel}
\alias{pknmodel}
\title{pknmodel}
\format{
An object of class \code{list} of length 4.
}
\usage{
data(PKN_ToyPB)
}
\description{
Small in-silico case study used for demonstration. Use with CNOlist_ToyPB data.
}
\keyword{datasets}

\name{prep4sim}
\alias{prep4sim}
\alias{prep4Sim}
\title{
Prepare a model for simulation
}
\description{
Adds to the model some fields that are used by the simulation engine
}
\usage{
prep4sim(model)
}
\arguments{
  \item{model}{
a model list, as output by readSIF, normally pre-processed but that is not a requirement of this function
}
}
\details{
This adds fields that are necessary for the simulation engine in a version that is extensible for constrained Fuzzy logic extension of the methods applied here (in development).
}
\value{
this function returns a list with fields:
\item{finalCube}{stores, for each reac(row) the location of its inputs (col)}
\item{ixNeg}{stores, for each reac(row) and each input (col) whether it is a negative input}
\item{ignoreCube}{logical matrix of the same size as the 2 above, that tells whether the particular cell is filled or not}
\item{maxIx}{row vector that stores, for each reac, the location of its output}
\item{modelname}{stores the name of the model from which these fields were derived}
\item{maxInput}{stores the max number of inputs observed in the model for a single reaction}
}
\author{
C. Terfve, T. Cokelaer
}
\seealso{
\link{simulatorT1}
}
\note{For internal usage since version 1.3.28}
\examples{
data(ToyModel,package="CellNOptR")
ToyFields4Sim<-prep4sim(ToyModel)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/readRules.R
\name{build_sif_table_from_rule}
\alias{build_sif_table_from_rule}
\title{Build a SIF table from a logic rule written in a string}
\usage{
build_sif_table_from_rule(rule_str, target, last_and_num = 0)
}
\arguments{
\item{rule_str}{String containing the rule to be parsed}

\item{target}{Name of the node affected by the rule}

\item{last_and_num}{If the rule contains `and` gates, their numeration will start after the number provided here (default is 0)}
}
\value{
data.frame with the network structure derived from the rule.
The column `sif_str` contains the string that can be written to a file 
and then read with `readSIF()` in order to load a CellNOpt compatible network.
}
\description{
Build a SIF table from a logic rule written in a string
}
\examples{
CellNOptR:::build_sif_table_from_rule("B & (C | D)", "A", last_and_num=2)

test_rule <- list()
test_rule[[1]] <- "AMP_ATP | (ATM & ATR) | HIF1 | !(EGFR | FGFR3)"
test_rule[[2]] <- "A & ((B | C) & !(D & E))"
test_rule[[3]] <- "A & B | C"
test_rule[[4]] <- "A & B & C"
test_rule[[5]] <- "A & (B | C)"
test_rule[[6]] <- "(A | B) & (C | D)"
test_rule[[7]] <- "!(C & D) | (E & F)"
test_rule[[8]] <- "(A | B) & (C | !D) & (E | F)"
parsed_rule <- list()
for (i in c(1:length(test_rule))){
  parsed_rule[[i]] <- CellNOptR:::build_sif_table_from_rule(test_rule[[i]], "T")
}

}

\name{indexFinder}
\alias{indexFinder}
\title{
Finds the indices, in the model fields, of the species that are measured/inhibited/stimulated
}
\description{
This function finds the indices, in the model fields, of the species that are measured/inhibited/stimulated. It looks for their position in model$namesSpecies which has the same order as the rows of interMat and notMat, and therefore these indexes can be used there as well.
}
\usage{
indexFinder(CNOlist, model,verbose=FALSE)
}
\arguments{
  \item{CNOlist}{
a CNOlist structure, as produced by makeCNOlist
}
  \item{model}{
a model structure, as produced by readSIF
}
  \item{verbose}{
do you want information about the cues and signals identities printed on the screen? Default if false but we would advise to set it to true when the function is called for the first time.
}
}
\value{
a list with fields:
\item{signals}{vector of indices of the measured species}
\item{stimulated}{vector of indices of the stimulated species}
\item{inhibited}{vector of indices of the inhibited species}
}
\author{
C. Terfve
}
\seealso{
\link{makeCNOlist}, \link{readSIF}
}
\note{For internal usage since version 1.3.28}
\examples{
data(CNOlistToy,package="CellNOptR")
data(ToyModel,package="CellNOptR")
indicesToy<-indexFinder(CNOlistToy,ToyModel,verbose=TRUE)
}

\name{simulateT1}
\alias{simulateT1}
\title{
deprecated since version 1.3.28. Cut and simulation of a boolean model at t1. 
Use simulateTN instead.
}
\description{
This function cuts a model according to a bitstring optimised at T1, and
simulates the model accordingly.
}
\usage{
simulateT1(CNOlist, model, bStringT1, simList, indexList)
}
\arguments{
  \item{CNOlist}{
a CNOlist object
}
  \item{model}{
a full model
}
  \item{bStringT1}{
a bitstring to cut the model, as output by \code{gaBinaryT1} (i.e. a vector of
1s and 0s, of length equal to the number of reactions in the model)
}
  \item{simList}{
a list of additional fields for simulation as created by \code{prep4sim},
corresponding to the full model
}
  \item{indexList}{
a list of indexes as created by indexFinder
}
}
\details{
This function is a wrapper for simulatorT1, that cuts the model before simulating it
}
\value{
a matrix of simulated values, including all species in the model, i.e. to be
used as input of gaBinaryTN (not implemented here) but not to be used directly
in \code{plotOptimResults.}
}
\author{
C.Terfve
}
\seealso{
cutAndPlotOptimResultsT1, simulatorT1
}
\examples{
# This will compute the output of a random model obtained by randomly selecting
# which gates of the initial models are included.

data(CNOlistToy,package="CellNOptR")
data(ToyModel,package="CellNOptR")

indicesToy<-indexFinder(CNOlistToy,ToyModel,verbose=FALSE)
ToyFields4Sim<-prep4sim(ToyModel)

simRes<-simulateT1(
	CNOlist=CNOlistToy, 
	model=ToyModel, 
	bStringT1=round(runif(length(ToyModel$reacID))), 
	simList=ToyFields4Sim, 
	indexList=indicesToy)
}

\keyword{internal}

\name{toSBML}
\alias{toSBML}
\title{
Export the network to SBML-qual format
}
\description{
Export a Boolean network <network> to an sbml-qual file <fileName>.
This file can then be read in using other software that supports SBMLqual standards.

The function also takes a bit string as input. 
It cuts the model according to the values in bitstrings and write the new model object to SBMLqual.

}
\usage{
toSBML(network, file, bitString = c(rep(1,length(network$reacID))),version=c("standard","cellnopt"))
}
\arguments{
	\item{network}{a valid CellNOptR network model created by e.g. readSIF()}
  \item{file}{a valid filename to save the SBMLqual model}
  \item{bitString}{optional vector of binary values (for example the resulted bitString from optimisation) to cut the unnecessary interactions from the network before exporting.}
   \item{version}{ defines the format of SBMLqual file, read details.}
}
\details{
	\code{version = "standard"} exports one transition block for each node of the network. This format is the SBMLqual standard, that can be imported then with other softwares
	
	\code{version = "cellnopt"} the exported file follows a simplified syntax, where each edge of the network is transformed to a transition block in the SBMLqual file. Can be later imported to CellNOptR again.
}

\author{
Francesco Ceccarelli
}


\examples{
  data(ToyModel,package="CellNOptR")
  toSBML(ToyModel,file = tempfile())
}
	

\name{CNORbool}
\alias{CNORbool}
\title{
Simple Boolean analysis standalone
}
\description{
This function performs the optimisation of a PKN model to a CNOlist data set. 
It optimises each time point found in the data and returns the processed model
as well as a list of optimised bitstring corresponding to each time points that
has been optimised.

This function does not create any plots or reports unlike \code{\link{CNORwrap}}.
}
\usage{
CNORbool(CNOlist, model, paramsList=defaultParameters(),
    compression=TRUE, expansion=TRUE, cutNONC=TRUE, verbose=FALSE,
    timeIndices = NULL)
}

\arguments{
\item{CNOlist}{
a CNOlist structure, as created by \link{makeCNOlist} or a MIDAS filename
}
  \item{model}{
a model structure, as created by \link{readSIF} or a SIF filename.
}

  \item{paramsList}{
   Parameters of the genetic algorithm. If not provided, it is populated with the \link{defaultParameters} function.
   }
\item{compression}{
compress the model (default TRUE)
}
\item{expansion}{
expand the gates (default TRUE)

}
\item{cutNONC}{
cut the NONC nodes off the model by (default TRUE)
}
\item{verbose}{
FALSE}
\item{timeIndices}{by default, optimise T1 and T2 assuming there are the 2 first
time points. However, with this argument you can change that behaviour to
arbitrary time points.}

}
\value{
This function returns 2 components. The first one is the processed model used in
the optimisation. The second is a list of optimised bitstrings found for each
time points available in the MIDAS data set.
}
\author{
T.Cokelaer, S.Schrier
}
\examples{

data(CNOlistToy,package="CellNOptR")
data(ToyModel,package="CellNOptR")
res = CNORbool(CNOlist=CNOlistToy, model=ToyModel)

}

\name{expandGates}
\alias{expandGates}
\title{
Expand the gates of a model
}
\description{
This function takes in a model and (1) splits all AND gates into ORs.  In addition,
(2) wherever there are more than one, it creates all possible ANDs combinations of
them, but considering only ANDs with 2, 3 or 4 inputs according to the user
argument (default is 2)
}
\usage{
expandGates(model, ignoreList=NA, maxInputsPerGate=2)
}
\arguments{
  \item{model}{a model structure}
  \item{ignoreList}{an index vector of states to ignore incoming edges during step (1), i.e. at the time of splitting up AND gates}
  \item{maxInputsPerGate}{maximum number of input per gates (Default is 2; up
to 4)}
}
\details{
This function returns a model with additional fields that help keep track of the processing done on the network.  I would advice not to overwrite on the initial model but rather to assign the result of this function to a variable with a different name.
}
\value{
returns a model, with additional fields:
\item{SplitANDs}{list that contains a named element for each AND reac that has been split, and each element contains a vector with the names of the of the reactions that result from the split if nothing was split, this element has the default value $initialReac  [1] "split1" "split2"}
\item{newANDs}{list that contains an element for each new '&' gate, named by the name of this new and reac, and containing a vector of the names of the reactions from which it was created (contains all the reacs in that pool, not the particular ones, this could be improved)}
}
\author{
C.Terfve. T. Cokelaer, A.MacNamara, Martin-Franz-Xaver Pirkl
}
\note{No need to call this function directly since version 0.99.24. Use 
 \link{preprocessing} instead.}


\examples{
#load data

data(CNOlistToy,package="CellNOptR")
data(ToyModel,package="CellNOptR")

#pre-process the model

indicesToy<-indexFinder(CNOlistToy,ToyModel,verbose=TRUE)
ToyNCNOindices<-findNONC(ToyModel,indicesToy,verbose=TRUE)
ToyNCNOcut<-cutNONC(ToyModel,ToyNCNOindices)
indicesToyNCNOcut<-indexFinder(CNOlistToy,ToyNCNOcut)
ToyNCNOcutComp<-compressModel(ToyNCNOcut,indicesToyNCNOcut)
indicesToyNCNOcutComp<-indexFinder(CNOlistToy,ToyNCNOcutComp)
ToyNCNOcutCompExp<-expandGates(ToyNCNOcutComp, maxInputsPerGate=4)
}

\name{ilpBinaryTN}
\alias{ilpBinaryTN}
\title{
ILP method used to optimise a model
}
\description{
This function is the ilp method to be used to optimise a model by fitting to data for time point 2, that should follow optimisation based on time point 1.
}
\usage{
ilpBinaryTN(cnolist, 
                        model, 
                        sizeFac = 0.0001, 
                        mipGap = 0, 
                        relGap = 0, 
                        timelimit = 3600, 
                        cplexPath, 
                        method = "quadratic",
                        numSolutions = 100, 
                        limitPop = 500, 
                        poolIntensity = 0, 
                        poolReplace = 2,
                        timeIndices = c(1, 2))
}
\arguments{
  \item{cnolist}{
a CNOlist on which the score is based (based on valueSignals[[2]], i.e. data at
time 1)
}
  \item{model}{
a model structure, as created by \code{readSIF}, normally pre-processed but that
is not a requirement of this function
}
  \item{sizeFac}{
the scaling factor for the size term in the objective function, default to 0.0001
}
  \item{mipGap}{
the absolute tolerance on the gap between the best integer objective and the objective of the best node remaining. When this difference falls below the value of this parameter, the linear integer optimization is stopped. Default set to 0
}
  \item{relGap}{
the relative tolerance on the objective value for the solutions in the solution pool. Solutions that are worse (either greater in the case of a minimization, or less in the case of a maximization) than the incumbent solution by this measure are not kept in the solution pool. Default set to 0
}
  \item{timelimit}{
the maximum optimisation time in seconds, default set to 3600
}
  \item{cplexPath}{
the path where the cplex solver is stored. Default set to "~/Documents/cplex"
}
  \item{method}{
the method of writing the objective function (quadratic/linear). Default set to "quadratic"
}
  \item{numSolutions}{
the number of solutions to save
}
  \item{limitPop}{
the number of solutions to be generated. Default set to 500
}
  \item{poolIntensity}{
the Intensity of solution searching. Default set to 4
}
  \item{timeIndices}{
the time indeces to optimize. Default set to timeIndices=c(1, 2)
}

  \item{poolReplace}{
 pool replacement strategy, consult CPLEX manual for details. 
}

}
\value{
This function returns a list with elements:
\item{bitstringILPAll}{the list of all optimal bitstrings identified}
\item{bScore}{the best score for each set of bitstrings}
\item{time_cplex_only}{the time it took for cplex to solve the problem}
\item{total_time}{the total time for the pipeline to run (writing problem + solving problem + retrieving solutions)}
\item{stringsTolScores}{the scores of the above-mentioned strings}
}
\author{
E Gjerga, H Koch
}
\references{
Alexander Mitsos, Ioannis N. Melas, Paraskeuas Siminelakis, Aikaterini D. Chairakaki, Julio Saez-Rodriguez, and Leonidas G. Alexopoulos. Identifying Drug Effects via Pathway Alterations using an Integer Linear Programming Optimization Formulation on Phosphoproteomic Data. PLoS Comput Biol. 2009 Dec; 5(12): e1000591.
}
\examples{


# Toy Exampple
data("ToyModel", package="CellNOptR")
data("CNOlistToy", package="CellNOptR")
pknmodel = ToyModel
cnolist = CNOlist(CNOlistToy)
model = preprocessing(data = cnolist, model = pknmodel, compression = TRUE, expansion = TRUE)
plotModel(model = model, CNOlist = cnolist)

# Training to data - ILP
\dontrun{
resILP = ilpBinaryTN(cnolist = cnolist, model = model)
}
}

\name{cutAndPlotResultsTN}
\alias{cutAndPlotResultsTN}
\title{
Plot the results of an optimisation at tN
}
\description{
This function takes a model and an optimised bitstring, it cuts the model according to the bitstring and plots the results of the simulation along with the experimental data. This function is designed to work on results of a 2 step optimisation.
}


\usage{
cutAndPlotResultsTN(CNOlist, model, bStrings, plotPDF = FALSE, tag=NULL, 
    plotParams = list(maxrow = 10))
}
\arguments{
  \item{CNOlist}{
a CNOlist, corresponding to the optimisation one
}
  \item{model}{
a model (the full one that was used for optimisation)
}
  \item{bStrings}{
a list of bitstring at different time points 
}
  \item{plotPDF}{
TRUE or FALSE, tells whether you want a pdf to be produced or not
}
  \item{tag}{
NULL or string; tells whether you want to prefix filenames with a tag (replaces
the default behaviour).
}
  \item{maxrow}{
maximum number of row in the plot. 
}
\item{plotParams}{
a list of option related to the PDF and plotting outputs. (1) maxrow
is the maximum number of row used to plot the results. See  \link{plotOptimResultsPan}
for other fields.}
}
\note{New in version 1.3.28}
\value{
This function returns plotted MSEs}
\author{
T. Cokelaer, A. MacNamara, Sarah Schrier, C. Terfve based on \link{cutAndPlotResultsT1}}

\seealso{
\link{gaBinaryT1}, \link{prep4sim}, \link{cutAndPlotResultsT1}
}
\examples{
#load data

data(CNOlistToy2,package="CellNOptR")
data(ToyModel2,package="CellNOptR")

#pre-process model
model = preprocessing(CNOlistToy2, ToyModel2)

#optimise t1
ToyT1<-gaBinaryT1(
	CNOlist=CNOlistToy2,
	model=model,
	maxGens=20, 
	popSize = 10,
	verbose=FALSE)

#Optimise T2
ToyT2<-gaBinaryTN(
	CNOlist=CNOlistToy2,
	model=model,
	bStrings=list(ToyT1$bString),
	maxGens=20, 
	popSize = 10,
	verbose=FALSE)

cutAndPlotResultsTN(
	CNOlist=CNOlistToy2,
	model=model,
	bStrings=list(ToyT1$bString, bStringT2=ToyT2$bString),
	plotPDF=FALSE)
}

\name{findNONC}
\alias{findNONC}
\title{
Find the indexes of the non-observable and non controllable species
}
\description{
This function finds the indexes of the non-observable and non controllable species and returns the indices, in the model, of the species to remove
}
\usage{
findNONC(model, indexes, verbose=FALSE)
}
\arguments{
    \item{model}{
        a model structure, as created by \link{readSIF}
    }
    \item{indexes}{
        a list of indexes of the species stimulated/inhibited/measured, as created by \link{indexFinder} from a model.        
    }
    \item{verbose}{
        verbose option (default to FALSE)
    }
}
\details{
This function uses the function floyd.warshall.all.pairs.sp from the package RBGL.  Non observable nodes are those that do not have a path to any measured species in the model, whereas non controllable nodes are those that do not receive any information from a species that is perturbed in the data.
}
\value{
a vector of indices of species to remove
}
\author{
C. Terfve
}
\seealso{
\link{cutNONC}, \link{indexFinder}, \link{readSIF}
}
\note{No need to call this function directly since version 0.99.24. Use 
 \link{preprocessing} instead.}

\examples{
    data(CNOlistToy, package="CellNOptR")
    data(ToyModel, package="CellNOptR")
    checkSignals(CNOlistToy, ToyModel)
    indicesToy <- indexFinder(CNOlistToy, ToyModel)
    ToyNCNOindices <- findNONC(ToyModel, indicesToy)
}

\name{ToyModel2}
\docType{data}
\alias{ToyModel2}
\title{Toy model}
\description{This data object contains the Toy model from the package vignette, already loaded and formatted as a Model object, and modified for the 2 time points version (a negative fedback between cJun and Jnk (!cJun=Jnk) is added).}
\usage{data(ToyModel)}
\format{ToyModel is a list with fields "reacID" (character vector), "namesSpecies" (character vector), "interMat" (numerical matrix), "notMat"(numerical matrix). }
\source{This data and model is extracted from the Matlab version of CellNOpt1.0 (\url{http://www.ebi.ac.uk/saezrodriguez/software.html#CellNetOptimizer}).}
\references{J. Saez-Rodriguez, L. G. Alexopoulos, J. Epperlein, R. Samaga, D. A. Lauffenburger, S. Klamt and P. K. Sorger. Discrete logic modeling as a means to link protein signaling networks with functional analysis of
mammalian signal transduction, Molecular Systems Biology, 5:331, 2009.}
\keyword{datasets}

\name{model2igraph}
\alias{model2igraph}

\title{
  Convert a model object to a igraph object
}

\description{
  This function receives as input a model object and converts it to a graph
object made by igraph. igraph provides lots of utilities especially to write the
file in different format such as GML.
}

\usage{
model2igraph(model)

}


\arguments{
  \item{model}{
the model as generated using \link{readSIF}
}
}



\value{
\item{g}{a igraph object}
}

\author{
    T. Cokelaer
}

\seealso{
\link{graph2sif}, \link{sif2graph} 
}

\examples{
    data(ToyModel,package="CellNOptR")
    model2igraph(model=ToyModel)
}

    

\name{invokeCPLEX}
\alias{invokeCPLEX}
\alias{invokeCPLEX}

\title{
Solving the ILP problem with CPLEX.
}
\description{
This function takes as an input the file name where we write the ILP formulation together with CPLEX parameters
and then solves the ILP problem. 
}
\usage{
invokeCPLEX(inputFileName, 
                        outputFileName, 
                        mipGap=mipGap, 
                        relGap = relGap, 
                        timelimit=timelimit, 
                        cplexPath = cplexPath, 
                        numSolutions = numSolutions, 
                        limitPop = limitPop, 
                        poolIntensity = poolIntensity, 
                        poolReplace = poolReplace)
}
\arguments{
  \item{inputFileName}{the file name where the cplex ilp problem is stored
  }
  \item{outputFileName}{the file name where to store the cplex result
  }
  \item{mipGap}{the mipgap
  }
  \item{relGap}{the relGap
  }
  \item{timelimit}{the timelimit
  }
  \item{cplexPath}{the cplex solver path
  }
  \item{numSolutions}{the number of desired solutions
  }
  \item{limitPop}{the limitPop
  }
  \item{poolIntensity}{the poolIntensity
  }
  \item{poolReplace}{the poolReplace
  }


}

\author{
E Gjerga, H Koch
}

\name{computeScoreT1}
\alias{computeScoreT1}

\title{
  Compute the score of a model/data set using a bitString to cut the model.
}

\description{
The bitString made of 0 and 1 allows to select a submodel from the model provided. 
Then, the simulator function are called to compute the objective function. The 
sizeFac and NAFac are penalties added to the final score as described in 
\code{gaBinaryT1}. The indexList and simList arguments can be provided to speed up the code otherwise, they are recomputed from the CNOlist and model.
}

\usage{
computeScoreT1(CNOlist, model, bString, simList=NULL, indexList=NULL,
    sizeFac=0.0001, NAFac=1, timeIndex=2)

}


\arguments{
    \item{CNOlist}{
        a CNOlist structure, as created by \code{makeCNOlist}.
    }
    \item{model}{
        a model structure, as created by code{readSIF}, normally pre-processed 
        but that is not a requirement of this function.
    }
    \item{bString}{
        a bitstring of the same size as the number of reactions in the model above
    }
    \item{simList}{
        If provided, simList should be created by \code{prep4sim}, that has also already been cut to 
        contain only the reactions to be evaluated. 
    }
    \item{indexList}{
        If provided, indexList should contain a list of indexes of the species stimulated/inhibited/measured in the 
        model, as created by \code{indexFinder}.
    }
    \item{sizeFac}{
    the scaling factor for the size term in the objective function, default to 0.0001
    }
    \item{NAFac}{
    the scaling factor for the NA term in the objective function, default to 1    
    }
  \item{timeIndex}{the index of the time point to optimize. Must be greater or
    equal to 2 (1 corresponds to time=0). Must be less than the number of time
    points. Default is 2. 
    }


}



\value{
    \item{score}{See \code{gaBinaryT1} for details}
}

\author{
    T. Cokelaer
}

\examples{
    data(CNOlistToy,package="CellNOptR")
    data(ToyModel,package="CellNOptR")
    model <- preprocessing(CNOlistToy,ToyModel)
    score = computeScoreT1(CNOlist(CNOlistToy), model, bString=rep(1,16))
}

\name{LiverDREAM}
\docType{data}
\alias{DreamModel}
\title{Model used for the DREAM3 challenge}
\description{This data object contains the model used in the package vignette, already loaded and formatted as a Model object. This is to be used with the data in "CNOListDREAM"}
\usage{data(DreamModel)}
\format{DreamModel is a list with fields "reacID" (character vector), "namesSpecies" (character vector), "interMat" (numerical matrix), "notMat"(numerical matrix). }
\source{This data and model is extracted from the Matlab version of CellNOpt1.0 (\url{http://www.ebi.ac.uk/saezrodriguez/software.html#CellNetOptimizer}).}
\references{
\enumerate{
    \item{J. Saez-Rodriguez, L. G. Alexopoulos, J. Epperlein, R. Samaga, D. A.
Lauffenburger, S. Klamt and P. K. Sorger. Discrete logic modeling as a means to
link protein signaling networks with functional analysis of mammalian signal
transduction, Molecular Systems Biology, 5:331, 2009.}
    \item{Prill RJ, Marbach D, Saez-Rodriguez J, Sorger PK, Alexopoulos LG, Xue X,
Clarke ND, Altan-Bonnet G, and Stolovitzky G. Towards a rigorous assessment of
systems biology models: the DREAM3 challenges. PLoS One, 5(2):e9202, 2010. }
}
}
\keyword{datasets}

\name{cutModel}
\alias{cutModel}
\title{
Cut a model structure according to a bitstring
}
\description{
This function is for developers only. 
}
\usage{
cutModel(model, bString)
}

\arguments{
  \item{model}{the model object to cut.}
  \item{bString}{the bitString to be used to cut the model object.}
}


\value{
  \item{cutModel}{the new model object}
}

\note{added in version 1.3.16}

\author{
 T. Cokelaer
}

\examples{
   
    data(ToyModel,package="CellNOptR")
    
    bString = rep(1,length(ToyModel$reacID))
    # remove some reactions by setting them to 0. 
    bString[c(2,5,6)] <- 0
    
    prepModel = cutModel(model = ToyModel, bString = bString )
 
    
}

\name{internals}
\alias{internals}
\title{
List of CellNOptR internal functions.
}
\description{
This is a list of functions that are part of CelNOptR but are not exposed to the
end-user. It may be of interest for developers. They may have a manual
associated to it. If so, you can get the documentation in a R console as usual
by tying preceding the name of the function with question tag.
}


 \details{
  \tabular{lll}{
  Function:         \tab deprecated since           \tab manual \cr
  ------------         \tab ---------------------          \tab ---------- \cr
  cSimulator:       \tab internal usage             \tab yes    \cr
  buildBitString:   \tab internal usage             \tab yes    \cr
  simulateT1:       \tab 1.3.28 (use simulateTN)   \tab yes     \cr
  }
}

\author{
T.Cokelaer
}

\name{plotOptimResultsPDF}
\alias{plotOptimResultsPDF}
\title{
Plot the data and simulated values in a pdf file
}
\description{
This is a wrapper for \code{plotOptimResults}
}
\usage{
plotOptimResultsPDF(simResults, expResults, times, namesCues, namesSignals,
valueCues, filename, formalism="new")
}
\arguments{
  \item{simResults}{
a list with a field for each time point, each containing a matrix of dimensions number of conditions * number of signals, with the first field being t0.  Typically produced by simulating a model and then extracting the columns that correspond to signals
}
  \item{expResults}{
same as above, but contains the experimental results, ie this is \code{CNOlist$valueSignals}
}
  \item{times}{
a vector of times, its length should be the same as the number of fields in simResults and ExpResults
}
  \item{namesCues}{
a vector of names, typically \code{CNOlist$namesCues}
}
  \item{namesSignals}{
a vector of names, typically \code{CNOlist$namesSignals}
}
  \item{valueCues}{
a matrix of dimensions (number of conditions) * (number of cues), typically \code{CNOlist$valueCues}
}
  \item{filename}{
a name for your file, eg. "plot.pdf"
}
 \item{formalism}{
 New convention is to take the time=0 data set into account to compute the MSE.
 you can use the previous convetion by setting this argument to something
 different from the default value.
 }

}
\details{
The coloring of the background is done as follows: the mean absolute difference between observed and simulated values are computed, and colours are chosen based on this value: red (above 0.9), indianred1 (between O.8 and 0.9), lightpink2 (between 0.7 and 0.8), lightpink (between 0.6 and 0.7), mistyrose (between 0.5 and 0.6), palegoldenrod (between 0.4 and 0.5), palegreen (between 0.3 and 0.4), darkolivegreen3 (between 0.2 and 0.3), chartreuse3 (between 0.1 and 0.2), forestgreen (between 0 and 0.1). This function is used inside \code{cutAndPlotResultsT1}.
}
\value{
This function doesn't return anything, it just produces a plot in a pdf document in your working directory.
}
\author{
C. Terfve
}
\seealso{
plotOptimResults, cutAndPlotResultsT1
}
\examples{

#We will plot the fit of the full initial model compared to the data, without any optimisation
#This is normally not done on a stand alone basis, but if you have a model and would like to visualise 
#its output compared to your data, then this is what you should do

#load and prepare data

data(CNOlistToy,package="CellNOptR")
data(ToyModel,package="CellNOptR")
indicesToy<-indexFinder(CNOlistToy,ToyModel,verbose=TRUE)
ToyFields4Sim<-prep4sim(ToyModel)

#simulate the model

simRes<-simulatorT1(CNOlist=CNOlistToy,model=ToyModel,simList=ToyFields4Sim,indexList=indicesToy)

#format the results and data as expected by plotOptimResults

simResults<-list(t0=matrix(data=0,nrow=dim(simRes)[1],ncol=dim(simRes)[2]),t1=simRes)
expResults<-list(t0=CNOlistToy$valueSignals[[1]],t1=CNOlistToy$valueSignals[[2]])

#plot

plotOptimResultsPDF(
	simResults=simResults,
	expResults=expResults,
	times=CNOlistToy$timeSignals[1:2],
	namesCues=CNOlistToy$namesCues,
	namesSignals=CNOlistToy$namesSignals,
	valueCues=CNOlistToy$valueCues, 
	filename="Toyfull.pdf")
}

\name{cutCNOlist}
\alias{cutCNOlist}
\title{
Cut a CNOlist structure according to a model
}
\description{
The MIDAS file may contain species that are not contained in the model. If you
want to remove cues and signals from your CNOlist that are not contained in the
model, you can use this function by using the parameter model. 
It can also be used to remove some time points by using the parameter
cutTimeIndices. Both parameters can be used at the same time and at least one of
them must be provided.
}
\usage{
    cutCNOlist(cnolist, model, cutTimeIndices, verbose=FALSE)
}

\arguments{
  \item{cnolist}{the CNOlist structure}
  \item{model}{the model}
  \item{cutTimeIndices}{the time indices to remove}
  \item{verbose}{Set it to True to get the signals and cues not found in the model}
}


\value{
  \item{cutCNOlist}{the new CNOlist object}
}

\seealso{
    \link{CNOlist-class}
}

\note{added in version 1.5.10}

\author{
 T. Cokelaer
}

\examples{
    data(CNOlistToy,package="CellNOptR")
    data(ToyModel,package="CellNOptR")
    
    cno_prep = cutCNOlist(cnolist = CNOlistToy,model = ToyModel,verbose = FALSE)
}

\name{writeScaffold}
\alias{writeScaffold}
\title{
Writes the scaffold network to a sif file (with attributes) and to a dot file
}
\description{
This function writes a cytoscape SIF file for the scaffold network, with an associated edge attribute file that holds whether the edge is present at t1,t2 or not present at all and another associated edge attribute file that holds the weights of the edges. This function also writes a dot file that contains the same information (see \code{writeDot} for more information about the dot file conventions).
}
\usage{
writeScaffold(modelComprExpanded, optimResT1, optimResT2, modelOriginal,
CNOlist, tag=NULL)
}
\arguments{
  \item{modelComprExpanded}{
The scaffold model (i.e. compressed and expanded)
}
  \item{optimResT1}{
The results of the optimisation process at t1
}
  \item{optimResT2}{
The results of the optimisation process at t2 (set this to NA if you have performed a one time point optimisation).
}
  \item{modelOriginal}{
The PKN model
}
  \item{CNOlist}{
The CNOlist on which the optimisation is based 
}
  \item{tag}{
NULL or string; tells whether you want to prefix filenames with a tag (replaces the default behaviour).
}
}
\details{
By scaffold network we mean the network that is used as a basis for optimisation (i.e. a compressed and expanded network), therefore no map back of the weights is necessary here.

The weights of the edges are computed as the mean across models within the relative tolerance limits, as output in the results from the optimisation \code{$stringsTol}. Strings that are in \code{$stringsTol} are the ones that are within the relative tolerance limits around the best solution in the population across all generations of the optimisation. 

!If there is no time 2, then the argument optimResT2 should be = NA.

}
\value{
This function does not return anything, it writes a sif file and 2 edge attributes files, and a dot file, in your working directory.
}
\author{
C.Terfve
}
\seealso{
writeNetwork, writeDot
}
\examples{


#load the data

data(CNOlistToy,package="CellNOptR")
data(ToyModel,package="CellNOptR")

#pre-process the model (partial)

indicesToy<-indexFinder(CNOlistToy,ToyModel,verbose=TRUE)
ToyNCNOcutComp<-compressModel(ToyModel,indicesToy)
indicesToyNCNOcutComp<-indexFinder(CNOlistToy,ToyNCNOcutComp)
ToyNCNOcutCompExp<-expandGates(ToyNCNOcutComp)

#optimise

ToyFields4Sim<-prep4sim(ToyNCNOcutCompExp)
initBstring<-rep(1,length(ToyNCNOcutCompExp$reacID))
ToyT1opt<-gaBinaryT1(
	CNOlist=CNOlistToy,
	model=ToyNCNOcutCompExp,
	initBstring=initBstring, 
	maxGens=3,
	popSize=5,
	verbose=TRUE)

#write the network

writeScaffold(
	modelOriginal=ToyModel,
	modelComprExpanded=ToyNCNOcutCompExp,
	optimResT1=ToyT1opt,
	optimResT2=NA,
	CNOlist=CNOlistToy)
}

\name{computeScoreTN}
\alias{computeScoreTN}

\title{
  Compute the score at TN of a model/data set using a bitString to cut the model.
}

\description{
The bitString made of 0 and 1 allows to select a submodel from the model provided. 
Then, the simulator function are called to compute the objective function. The 
sizeFac and NAFac are penalties added to the final score as described in 
\code{gaBinaryTN}.
}

\usage{
computeScoreTN(CNOlist, model, simList=NULL, indexList=NULL, simResPrev=NULL,
    bStringPrev=NULL, bStringNext=NULL, timeIndex=NULL, sizeFac=0.0001, NAFac=1, bStrings=NULL)
}


\arguments{
    \item{CNOlist}{
        a CNOlist structure, as created by \code{makeCNOlist}.
    }
    \item{model}{
        a model structure, as created by code{readSIF}, normally pre-processed 
        but that is not a requirement of this function.
    }
    \item{simList}{
        a simList as created by \code{prep4sim}, that has also already been cut to 
        contain only the reactions to be evaluated. If not provided, it is
        recomputed automatically.
    }
    \item{indexList}{
        a list of indexes of the species stimulated/inhibited/measured in the 
        model, as created by \code{indexFinder}. If not provided ,it is
        recomputed automatically.
    }
    \item{simResPrev}{
        Results of Previous simulation at time TN-1 step.
    }
    \item{bStringPrev}{
        the best bitString at time TN-1
    }
    \item{bStringNext}{
        the bitString to use to compute the score at time TN
    }
    \item{timeIndex}{
    Future feature will allows timeIndex to provide the exact list of indices to cut and plot. For now,
it is based on the bitStrings provided. 
    }
    \item{sizeFac}{
    the scaling factor for the size term in the objective function, default to 0.0001
    }
    \item{NAFac}{
    the scaling factor for the NA term in the objective function, default to 1    
    }
    \item{bStrings}{
    list of optimised bitstrings found at the previous time points
    }
}



\value{
    \item{score}{See \code{gaBinaryTN} for details}
}

\author{
    T. Cokelaer, S.Schrier
}

\examples{
    data(CNOlistToy2,package="CellNOptR")
    data(ToyModel2,package="CellNOptR")
    model <- preprocessing(CNOlistToy2, ToyModel2)
    bStringT1 = c(0,0,1,1,1,1,1,1,0,0,1,1,1,1,1,1)
    simT1<-simulateTN(CNOlist=CNOlistToy2, model=model, bStrings=list(bStringT1))

    score1 = computeScoreTN(CNOlistToy2, model, bStrings=list(bStringT1,c(1,0,1,0)))
}

\name{plot-method}
\docType{methods}
\alias{plot.CNOlist}
\alias{plot,CNOlist,ANY-method}



\title{plot a \code{"CNOlist"} object - methods}
\description{
 A plot method for \code{CNOlist}. 
}

\section{Usage}{
    \code{signature(x="CNOlist")}: Please see the help page for the \code{plot.CNOlist} method in the \code{CellNOptR} package
}
\section{arguments}{
    \describe{
	\item{x}{The \code{CNOlist} object to plot}
    }
}


\author{
T.Cokelaer
}
\seealso{
\code{\link{readMIDAS}}, \code{\link{makeCNOlist}}
}
\examples{
showClass("CNOlist")

files<-dir(system.file("ToyModel",package="CellNOptR"),full=TRUE)
cnolist  = CNOlist(files[[1]])
# accessors:
getCues(cnolist)
getInhibitors(cnolist)
getSignals(cnolist)
getTimepoints(cnolist)
getStimuli(cnolist)
# In version 1.3.30 and above, use the plot method instead of former plotCNOlist function.
plot(cnolist)
}
\keyword{methods}
\keyword{CNOlist}

\name{exhaustive}
\alias{exhaustive}
\title{Exhaustive search over the optimisation of a PKN model on MIDAS data.}

\description{
This function performs an exhaustive search of the parameter space tring all the
solutions. It is used internally by the genetic algorithm when a small model has
to be optimised and the number of solutions to try is smaller than the number of
iterations that the Genetic Algorithm will perform.
}
\usage{
exhaustive(CNOlist, model, shuffle=FALSE, Nmax=NULL, verbose=TRUE, sizeFac =
0.0001, NAFac = 1, relTol=0.1, timeIndex=2)
}
\arguments{
  \item{CNOlist}{
a CNOlist on which the score is based (based on valueSignals[[2]], i.e. data at
time 1)
}
  \item{model}{
a model structure, as created by \code{readSIF}, normally pre-processed but that
is not a requirement of this function
}
  \item{shuffle}{
The list of bitstrings is set up arbitrarely. You may want to shuffle it.
}
  \item{Nmax}{
The total number of computation will be 2 to the power N, where N is the size of
the model (ReacID field). The total number of computation can be large. You may
want to set a maximumn number of computation using Nmax. 
}
  \item{sizeFac}{
the scaling factor for the size term in the objective function, default to 0.0001
}
  \item{NAFac}{
the scaling factor for the NA term in the objective function, default to 1
}
  \item{relTol}{
the relative tolerance for the best bitstring reported by the genetic algorithm,
i.e., how different from the best solution, default set to 0.1
Not yet implemented.
}
  \item{verbose}{
logical (default to TRUE) do you want the statistics of each generation to be printed on the screen?
}

  \item{timeIndex}{the index of the time point to optimize. Must be greater or
equal to 2 (1 corresponds to time=0). Must be less than the number of time
points. Default is 2. 
    }


}
\value{
This function returns a list with elements:
\item{bString}{the best bitstring}
\item{bScore}{the best score}
\item{all_scores}{all scores that have been computed}
\item{results}{ a matrix with columns "Generation","Best_score","Best_bitString","Stall_Generation","Avg_Score_Gen","Best_score_Gen","Best_bit_Gen","Iter_time"}
\item{stringsTol}{the bitstrings whose scores are within the tolerance}
\item{stringsTolScores}{the scores of the above-mentioned strings}

Note that the field results, is not yet populated but maybe in the future.
}
\author{
T. Cokelaer
}
\seealso{
\link{gaBinaryT1}
}
\examples{
data(CNOlistToy,package="CellNOptR")
data(ToyModel,package="CellNOptR")

#pre-process model

model = preprocessing(CNOlistToy, ToyModel)

#optimise

results <-exhaustive(
	CNOlist=CNOlistToy,
	model=model,
    shuffle=TRUE,
    Nmax=1000,
	verbose=FALSE)
}

\name{normaliseCNOlist}
\alias{normaliseCNOlist}
\title{
Normalisation for boolean modelling.
}
\description{
This function takes in a CNOlist and does the normalisation of the data between
0 and 1, according to two different procedures (see details).
}
\usage{
normaliseCNOlist(CNOlist, EC50Data=0.5, HillCoef=2, EC50Noise=0., detection=0,
    saturation=Inf, changeTh=0, norm2TorCtrl=NULL, mode="time", 
    options=list(rescale_negative = TRUE), verbose=FALSE)
}


\arguments{
  \item{CNOlist}{
a CNOlist
}
  \item{EC50Data}{
parameter for the scaling of the data between 0 and 1, default=0.5
}
  \item{HillCoef}{
Hill coefficient for the scaling of the data, default to 2
}
  \item{EC50Noise}{
parameter for the computation of a penalty for data comparatively smaller than
other time points or conditions. No effect if set to zero (default).
}
  \item{detection}{
minimum detection level of the instrument, everything smaller will be treated as noise (NA), default to 0
}
  \item{saturation}{
saturation level of the instrument, everything over this will be treated as NA, default to Inf.
}
  \item{changeTh}{
threshold for relative change considered significant, default to 0
}
  \item{norm2TorCtrl}{
deprecated since 1.5.0. Use the mode argument instead.
}
  \item{mode}{
"time" or "ctrl" or "raw" (experimental): choice of a normalisation method:
"ctrl" computes the relative change compared to the control at the same time.
"time" computes the relative change compared to the same condition and
measurement at time 0. "raw" does not take into account time zero data; data are
relative to time 0 so can be positive or negative values.
}
\item{options}{
rescale column with negative values to be in 0-1 range (experimental)

}

  \item{verbose}{
prints some information if True (default is False)
}
}
\details{
The normalisation procedure works as follows: 

\enumerate{
    \item every value that is out of the dynamic range of the equipment (as
        specified by the parameters \code{detection} and \code{saturation} are set to
        NA,

    \item  values are transformed to fold changes relative to the same condition
        at t0 (if \code{mode="time"}) or the control condition (i.e. same
        inhibitors, no stimuli) at the same time (if \code{mode="ctrl"}),
    \item the fold changes are transformed with a Hill function
        \eqn{
        \frac{x^{HillCoef}}{ ((EC50Data^{HillCoef}) + (x^{HillCoef}))}
        }
    \item a penalty for "noisiness" is computed for each measurement as the value divided by the maximum
        value for that readout across all conditions and times (excluding values out of
        the dynamic range)
    \item  the noise penalty is transformed by a saturation function (for each
        measurement \eqn{\frac{x}{(EC50Noise+x}} where \eqn{x=\frac{x}{\max{x}}}),
    \item the noise penalty and Hilled fold changes are multiplied,
    \item if the fold change is negative and bigger than \code{ChangeTh}, the resulting product is multiplied by
        -1, if the fold change is smaller than \code{ChangeTh} (either positive or
        negative), it is set to 0.  

The normalisation procedure applied here is explained in details in saez-Rodriguez et al. (2009).

As the normalisation procedure works by computing a fold change relative to the
same condition at time 0 or the control condition, if the aforementioned
conditions have a value of zero (which is not expected with any common
biochemical technique), then the fold change calculation will return a "NaN"
value.  If this is a problem for your particular case then we would suggest
putting a dummy, very low value, instead of the zero, or setting that
measurement to "NA" in the MIDAS file.
    }
}
\value{
a normalised CNOlist
}
\references{
J. Saez-Rodriguez, L. G. Alexopoulos, J. Epperlein, R. Samaga, D. A. Lauffenburger, S. Klamt and P. K. Sorger. Discrete logic modeling as a means to link protein signaling networks with functional analysis of
mammalian signal transduction, Molecular Systems Biology, 5:331, 2009.
}
\author{
C. Terfve
}
\seealso{
\link{makeCNOlist}
}
\examples{
#Load a CNOlist

data(CNOlistToy,package="CellNOptR")

#Replace the values in the list by random values 
#(for demonstration purposes, when actually using this function you would simply load a non-normalised CNOlist)

CNOlistToy$valueSignals$t0<-matrix(
	data=runif(n=(dim(CNOlistToy$valueSignals$t0)[1]*dim(CNOlistToy$valueSignals$t0)[2]),min=0,max=400),
	nrow=dim(CNOlistToy$valueSignals$t0)[1],
	ncol=dim(CNOlistToy$valueSignals$t0)[2])
CNOlistToy$valueSignals[[2]]<-CNOlistToy$valueSignals[[1]]+matrix(
	data=runif(n=(dim(CNOlistToy$valueSignals$t0)[1]*dim(CNOlistToy$valueSignals$t0)[2]),min=0,max=100),
	nrow=dim(CNOlistToy$valueSignals$t0)[1],
	ncol=dim(CNOlistToy$valueSignals$t0)[2])

CNOlistToyN<-normaliseCNOlist(
	CNOlistToy, 
	EC50Data = 0.5, 
	HillCoef = 2, 
	EC50Noise = 0.1, 
	detection = 0, 
	saturation = Inf, 
	changeTh = 0, 
	mode = "time")
}

\name{createAndRunILP}
\alias{createAndRunILP}

\title{
Creating and running the ILP problem.
}
\description{
This function takes as an input the cno inputs (model + data) together with CPLEX parameters
and then solves the ILP problem. 
}
\usage{
createAndRunILP(model = model, 
                            midas = midas, 
                            cnolist = cnolist, 
                            accountForModelSize = accountForModelSize, 
                            sizeFac = sizeFac, 
                            source_path = source_path, 
                            mipGap = mipGap, 
                            relGap = relGap, 
                            timelimit = timelimit,
                            cplexPath = cplexPath, 
                            method = method, 
                            numSolutions = numSolutions, 
                            limitPop = limitPop, 
                            poolIntensity = poolIntensity, 
                            poolReplace = poolReplace)
}
\arguments{
  \item{model}{the model
  }
  \item{midas}{the midas table
  }
  \item{cnolist}{the cnolist object
  }
  \item{accountForModelSize}{the verbose parameter whether to account for model size
  }
  \item{sizeFac}{the size penalty factor
  }
  \item{source_path}{the source path
  }
  \item{mipGap}{the mipgap
  }
  \item{relGap}{the relGap
  }
  \item{timelimit}{the timelimit
  }
  \item{cplexPath}{the cplex solver path
  }
  \item{method}{the optimization method (quadratic/linear)
  }
  \item{numSolutions}{the number of desired solutions
  }
  \item{limitPop}{the limitPop
  }
  \item{poolIntensity}{the poolIntensity
  }
  \item{poolReplace}{the poolReplace
  }


}

\author{
E Gjerga, H Koch
}

\name{writeObjectiveFunction}
\alias{writeObjectiveFunction}
\alias{writeObjectiveFunction}

\title{
Writing the objective function for the ILP implementation of CellNOptR.
}
\description{
This function takes as input the integer variables assigned to each of the elements in the network and data in a CNOlist object
in order to produce the objective function needed to be optimized by the ILP solver. 
}
\usage{
writeObjectiveFunction(model, 
                                   midasExperimentPart, 
                                   y_vector=y_vector,
                                   accountForModelSize = TRUE, 
                                   sizeFac = .000001, 
                                   meansOfMeasurements_at_t0, 
                                   method = "quadratic" )
}
\arguments{
  \item{model}{the model
  }
  \item{midasExperimentPart}{the experimental part of CNOlist where data is stored
  }
  \item{y_vector}{the variables for each interaction in the PKN
  }
  \item{accountForModelSize}{the verbose variable if we wish to apply the size penalty
  }
  \item{sizeFac}{the size penalty factor
  }
  \item{meansOfMeasurements_at_t0}{the means of measurements at time-point 0
  }
  \item{method}{the method of defining the objective function ("quadratic/linear")
  }


}

\author{
E Gjerga, H Koch
}
\references{
Alexander Mitsos, Ioannis N. Melas, Paraskeuas Siminelakis, Aikaterini D. Chairakaki, Julio Saez-Rodriguez, and Leonidas G. Alexopoulos. Identifying Drug Effects via Pathway Alterations using an Integer Linear Programming Optimization Formulation on Phosphoproteomic Data. PLoS Comput Biol. 2009 Dec; 5(12): e1000591.
}

\name{plotCNOlistPDF}
\alias{plotCNOlistPDF}
\title{
Plots a CNOlist into a pdf file.
}
\description{
This function is a wrapper for plotCNOlist, that plots the output directly in a pdf file.
}
\usage{
plotCNOlistPDF(CNOlist, filename)
}
\arguments{
  \item{CNOlist}{
a CNOlist
}
  \item{filename}{
a name for your pdf file, eg. "plot.pdf"
}
}
\value{
This function doesn't return anything, it just produces a pdf file containing your plot, in your working directory.
}
\author{
C. Terfve
}
\seealso{
\link{plotCNOlist}, \link{plotCNOlistLarge}, \link{plotCNOlistLargePDF}
}
\examples{
data(CNOlistToy,package="CellNOptR")
plotCNOlistPDF(CNOlist=CNOlistToy,filename="ToyModelGraph.pdf")
}

\name{gaBinaryT1}
\alias{gaBinaryT1}
\title{
Genetic algorithm used to optimise a model
}
\description{
This function is the genetic algorithm to be used to optimise a model by fitting to data containing one time point.
}
\usage{
gaBinaryT1(CNOlist, model, initBstring=NULL, sizeFac = 1e-04, 
    NAFac = 1, popSize = 50, pMutation = 0.5, maxTime = 60, maxGens = 500,
    stallGenMax = 100, selPress = 1.2, elitism = 5, relTol = 0.1, verbose=TRUE,
    priorBitString=NULL, timeIndex=2)
}
\arguments{
  \item{CNOlist}{
a CNOlist on which the score is based (based on valueSignals[[2]], i.e. data at
time 1)
}
  \item{model}{
a model structure, as created by \code{readSIF}, normally pre-processed but that
is not a requirement of this function
}
  \item{initBstring}{
an initial bitstring to be tested, should be of the same size as the number of
reactions in the model above (model$reacID). Default is all ones.
}
  \item{sizeFac}{
the scaling factor for the size term in the objective function, default to 0.0001
}
  \item{NAFac}{
the scaling factor for the NA term in the objective function, default to 1
}
  \item{popSize}{
the population size for the genetic algorithm, default set to 50
}
  \item{pMutation}{
the mutation probability for the genetic algorithm, default set to 0.5
}
  \item{maxTime}{
the maximum optimisation time in seconds, default set to 60
}
  \item{maxGens}{
the maximum number of generations in the genetic algorithm, default set to 500
}
  \item{stallGenMax}{
the maximum number of stall generations in the genetic algorithm, default to 100
}
  \item{selPress}{
the selective pressure in the genetic algorithm, default set to 1.2
}
  \item{elitism}{
the number of best individuals that are propagated to the next generation in the genetic algorithm, default set to 5
}
  \item{relTol}{
the relative tolerance for the best bitstring reported by the genetic algorithm,
i.e., how different from the best solution, default set to 0.1
}
  \item{verbose}{
logical (default to TRUE) do you want the statistics of each generation to be printed on the screen?
}
  \item{priorBitString}{
At each generation, the GA algorithm creates a population of bitstrings that
will be used to perform the optimisation. If the user knows the values of some
bits, they can be used to overwrite bit values proposed by the GA algorithm. If
provided, the priorBitString must have the same length as the initial bitstring and
be made of 0, 1 or NA (by default, this bitstring is set to NULL, which is
equivalent to setting all bits to NA). Bits that are set
to 0 or 1 are used to replace the bits created by the GA itself (see example).
}

  \item{timeIndex}{the index of the time point to optimize. Must be greater or
equal to 2 (1 corresponds to time=0). Must be less than the number of time
points. Default is 2. 
    }


}
\details{
The whole procedure is described in details in Saez-Rodriguez et al. (2009). The basic principle is that at each generation, the algorithm evaluates a population of models based on excluding or including some gates in the initial pre-processed model (this is encoded in a bitstring with contains 0/1 entries for each gate).  The population is then evolved based on the results of the evaluation of these networks, where the evaluation is obtained by simulating the model (to steady state) under the various conditions present in the data, and then computing the squared deviation from the data, to which a penalty is added for size of the model and for species in the model that do not reach steady state.
}
\value{
This function returns a list with elements:
\item{bString}{the best bitstring}
\item{bScore}{the best score}
\item{results}{ a matrix with columns "Generation","Best_score","Best_bitString","Stall_Generation","Avg_Score_Gen","Best_score_Gen","Best_bit_Gen","Iter_time"}
\item{stringsTol}{the bitstrings whose scores are within the tolerance}
\item{stringsTolScores}{the scores of the above-mentioned strings}
}
\references{
J. Saez-Rodriguez, L. G. Alexopoulos, J. Epperlein, R. Samaga, D. A. Lauffenburger, S. Klamt and P. K. Sorger. Discrete logic modeling as a means to link protein signaling networks with functional analysis of
mammalian signal transduction, Molecular Systems Biology, 5:331, 2009.
}
\author{
C. Terfve. T. Cokelaer
}
\seealso{
\link{gaBinaryTN}, \link{simulatorT1}
}
\examples{
data(CNOlistToy,package="CellNOptR")
data(ToyModel,package="CellNOptR")

#pre-process model

model = preprocessing(CNOlistToy, ToyModel)

#optimise

initBstring<-rep(1,length(model$reacID))
ToyT1opt<-gaBinaryT1(
	CNOlist=CNOlistToy,
	model=model,
	initBstring=initBstring,
	maxGens=100, popSize=10, verbose=FALSE)

# During the optimisation, some bits can be overwritten by your prior knowledge 
# First, you need to create a priorBitString made of NA where known bit values
# are replaced by 0 or 1
priorBitString = rep(NA, length(model$reacID))
priorBitString[1] = 0
priorBitString[2] = 1

# Second, you call the gaBinaryT1 function by providing the priorBitString
# argument:
ToyT1opt<-gaBinaryT1(CNOlist=CNOlistToy, model=model,
     initBstring=initBstring,	maxGens=10, popSize=10, verbose=FALSE, 
    priorBitString=priorBitString)
}

\name{plotFit}
\alias{plotFit}
\title{
Plot the evolution of an optimisation
}
\description{
This function takes in the results of an optimisation by gaBinaryT1 and plots the evolution of best fit and average fit against generations.
}
\usage{
plotFit(optRes, filename = NULL)
}
\arguments{
  \item{optRes}{
an object created by the optimisation engine (gabinaryT1)
}
  \item{filename}{
NULL or string: if provided, the plot is save in PDF format in the filename.
}
}
\value{
This function doesn't return anything, it just produces a plot in your graphics window.
}
\author{
C. Terfve
}
\seealso{
\link{gaBinaryT1}
}
\examples{
data(CNOlistToy,package="CellNOptR")
data(ToyModel,package="CellNOptR")

#process the model
model = preprocessing(CNOlistToy,ToyModel)

#optimise

ToyT1opt<-gaBinaryT1(
	CNOlist=CNOlistToy,
	model=model,
	maxGens=10,
	popSize=10,
	verbose=FALSE)

plotFit(optRes=ToyT1opt)
}

\name{writeMIDAS}
\alias{writeMIDAS}
\title{Write a CNOlist structure into a MIDAS file}
\description{
This function takes a CNOlist structure (output of makeCNOlist and readMIDAS) and save it a MIDAS format.
}
\usage{
writeMIDAS(CNOlist, filename,  timeIndices=NULL, overwrite=FALSE)
}

\arguments{
    \item{CNOlist}{
        a CNOlist structure
    }
    \item{filename}{
        a filename. Not overwritten if it exists already    
    }
    \item{timeIndices}{
        select subset of the times to be saved. Works with indices (not time values)
    }
    \item{overwrite}{
        overwrite the file if it exists already (Default is FALSE)
    }



}

\author{
T. Cokelaer
}
\seealso{
\link{makeCNOlist}, \link{readMIDAS}, \link{CNOlist-class}
}
\examples{
   
   data(CNOlistToy)
   writeMIDAS(CNOlistToy, 'test.csv')
   readMIDAS('test.csv')
   writeMIDAS(CNOlistToy, 'test.csv', timeIndices=c(1,2), overwrite=TRUE)
}

\name{crossInhibitedData}
\alias{crossInhibitedData}
\title{
If an inhibitor is also a measured species, replace the data with NA (when inhibited)
}
\description{
If an inhibitor is also a measured species, replace the data with NA (when inhibited)
}
\usage{
crossInhibitedData(object)
}

\arguments{
  \item{object}{
the CNOlist that contains the data
}
}



\value{
  the new cnolist
}

\author{
 T. Cokelaer
}



\examples{
data(ToyModel,package="CellNOptR")
data(CNOlistToy,package="CellNOptR")
cnolist = crossInhibitedData(CNOlist(CNOlistToy))
}

\name{plotCNOlist2}
\alias{plotCNOlist2}
\title{
Another version of plotCNOlist that allows to plot 2 cnolist in the same layout
to compare them. This function uses ggplot2 library. It is recommended for small
data sets (about 15 species). 
}
\description{
This function plots the data in a CNOlist as a matrix of plots with a  row for
each condition and a column for each signal, Cues are simply represented by a
number.
}
\usage{
plotCNOlist2(cnolist, simulated_cnolist=NULL, ymin=0,ymax=1)
}

\arguments{
  \item{cnolist}{
a CNOlist
}
  \item{simulated_cnolist}{
another cnolist
}
\item{ymin}{
    Change the lower y-limit (default is 0)
}
\item{ymax}{
    Change the lower y-limit (default is 1)
}


}
\details{
This function can plot either a single CNOlist, or 2 on top of each other.
}
\author{
T. Cokelaer
}
\seealso{
plotCNOlistPDF, plotCNOlistLarge, plotCNOlistLargePDF, plotCNOlist
}
\examples{
# this data set is not an object so we need to convert it
data(CNOlistToy,package="CellNOptR")
cnolist = CNOlist(CNOlistToy)
plotCNOlist2(cnolist)
}

\name{CNORwrap}
\alias{CNORwrap}
\title{
CNOR analysis wrapper
}
\description{
This function is a wrapper around the whole CNOR analysis, it performs the following steps:

 \enumerate{
  \item Plot the CNOlist;
  \item Checks data to model compatibility;
  \item Call the preprocessing function (Cut the nonc off the model, compress the model and expand the gates);
  \item Compute the residual error;
  \item Prepare for simulation;
  \item Optimisation T1 and T2 (optional);
  \item Plot simulated and experimental results;
  \item Plot the evolution of fit;
  \item Write the scaffold and PKN;
  \item Write the report
 }
}
\usage{
CNORwrap(paramsList=NA, data=NA, model=NA, name, namesData=NA, time=1,
compression=TRUE, expansion=TRUE, cutNONC=TRUE)
}

\arguments{

  \item{paramsList}{
    A list of parameters related to the Genetic Algorithm parameters:
   \enumerate{
     \item sizeFac: default to 1e-04;
     \item NAFac: default to 1;popSize: default to 50;
     \item pMutation: default to 0.5;
     \item maxTime: default to 60;
     \item maxGens: default to 500;
     \item stallGenMax: default to 100;
     \item selPress: default to 1.2;
     \item elitism: default to 5;
     \item relTol: default to 0.1;
     \item verbose: default to FALSE (default to true in the functions used by CNORwrap but CNORwrap sets them to false by default).
   }
  }
  and the Data and Model structure.

  If paramsList is not provided (NA), it is filled internally with the \link{defaultParameters} function.

  If Data and Model are not provided in \code{paramList}, the function looks for Data and Model arguments.

  If Data and Model are provided, the function overwrites the field \code{data} and \code{model} in \code{paramsList}.


  \item{data}{
a CNOlist structure, as created by \link{makeCNOlist}
}
  \item{model}{
a model structure, as created by \link{readSIF}.
}
 \item{name}{
a string that will be used to name the project and all graphs produced
}
 \item{namesData}{
a list with two elements:CNOlist and Model, each containing a string that is a
reference for the user to know which model/data set was used (it will be
included in the report). If not provided, the list is built automatically using
the name arguments.
}
 \item{time}{
either 1 or 2: Do you want to perform a one time point steady state optimisation
or a 2 time points pseudo steady state optimisation. By default this is set to 1.
}
\item{compression}{
compress the model (default TRUE)
}
\item{expansion}{
expand the gates (default TRUE)

}
\item{cutNONC}{
cut the NONC nodes off the model by (default TRUE)

}

}
\details{
If you do not provide a parameters list, you can provide only essential elements,
and all other parameters will be set to their default values. In this case, you
should set paramsList=NA, and provide the following fields: data, model, name, time.
}
\value{
This function does not return anything, it does the analysis, produces all the
plots and puts them in a folder that is in your working directory, and is called "Name".
}
\author{
C. Terfve
}
\examples{
#version with paramslist

data(CNOlistToy,package="CellNOptR")
data(ToyModel,package="CellNOptR")

pList = defaultParameters(CNOlistToy, ToyModel)
pList$maxGens = 5
pList$popSize = 5

CNORwrap(paramsList=pList, name="Toy",
    namesData=list(CNOlist="ToyData", model="ToyModel"))

\dontrun{
#version with default parameters

data(CNOlistToy,package="CellNOptR")
data(ToyModel,package="CellNOptR")

CNORwrap(name="Toy",
    namesData=list(CNOlist="ToyData",model="ToyModel"),
	data=CNOlistToy, model=ToyModel)
}
}

\name{CNOdata}
\alias{CNOdata}
\title{
Get data from a CellNOpt data repository
}
\description{
This function fetch a file from the URL provided (default is
http://www.ebi.ac.uk/~cokelaer/cellnopt/data/_downloads)
and save it into a temporary file.

You will need Rcurl package to use this function.
}
\usage{
CNOdata(filename, verbose=FALSE, url=NULL)
}
\arguments{
  \item{filename}{a valid filename that can be found in the url}
  \item{verbose}{ FALSE by default, it prints the path of the temporary file
where data has been copied.}
   \item{url}{ You can overwrite the default URL
(http://www.ebi.ac.uk/~cokelaer/cellnopt/data/_downloads) with this argument.
}
}
\value{
    the path of the temporary file where data has been copied.
}

\author{
T. Cokelaer
}
\examples{
\dontrun{readSIF(CNOdata("PKN-ToyMMB.sif"))}

}

\alias{cnodata}

\name{checkSignals}
\alias{checkSignals}
\title{
Check the CNOlist and model matching
}
\description{
This function checks that all the signals in a CNOlist match to species in the 
model. It also checks that the CNOlist and Model lists have the right format and 
contain the right fields. It is called by the \link{preprocessing} function so there is 
no need to call it directly anymore if you use the \link{preprocessing} function.

In version 1.3.20, check of inhibitors and stimuli is also performed. 
}
\usage{
checkSignals(CNOlist, model)
}
\arguments{
    \item{CNOlist}{
        A CNOlist structure, as created by \link{makeCNOlist}.
    }
    \item{model}{
        A model structure, as created by \link{readSIF}.
    }
}
\details{
If the formats are wrong, this function produces an error. If the
signals/inhibitors/stimuli do not match the species, this function produces a
warning that explains which signals does no match any species.
}

\author{
C. Terfve, T. Cokelaer
}
\seealso{
\code{\link{makeCNOlist}}, \link{readSIF}, \link{preprocessing}
}
\examples{
data(CNOlistToy, package="CellNOptR")
data(ToyModel, package="CellNOptR")
checkSignals(CNOlistToy, ToyModel)
}

\name{gaBinaryTN}
\alias{gaBinaryTN}
\title{
Genetic algorithm for time point N
}
\description{
This is the genetic algorithm for time point N, that should follow optimisation based on time point 1.

Replaced gaBinaryT2.
}
\usage{
gaBinaryTN(CNOlist, model,  bStrings, sizeFac = 1e-04, NAFac = 1,
popSize = 50, pMutation = 0.5, maxTime = 60, maxGens = 500,
stallGenMax = 100, selPress = 1.2, elitism = 5, relTol = 0.1, verbose=TRUE,
priorBitString=NULL, timeIndex = NULL)
}
\arguments{
  \item{CNOlist}{
a CNOlist on which the score is based (based on valueSignals[[3]], i.e. data at  t2)
}
  \item{model}{
a model structure, as created by \code{readSIF}, normally pre-processed but that
is not a requirement of this function
}
  \item{bStrings}{
the optimal bitstring from optimisation at time 1 (i.e. a vector of 0s and 1s)
}
  \item{sizeFac}{
the scaling factor for the size term in the objective function, default to 0.0001
}
  \item{NAFac}{
the scaling factor for the NA term in the objective function, default to 1
}
  \item{popSize}{
the population size for the genetic algorithm, default set to 50
}
  \item{pMutation}{
the mutation probability for the genetic algorithm, default set to 0.5
}
  \item{maxTime}{
the maximum optimisation time in seconds, default set to 60
}
  \item{maxGens}{
the maximum number of generations in the genetic algorithm, default set to 500
}
  \item{stallGenMax}{
the maximum number of stall generations in the genetic algorithm, default to 100
}
  \item{selPress}{
the selective pressure in the genetic algorithm, default set to 1.2
}
  \item{elitism}{
the number of best individuals that are propagated to the next generation in the gen. al. default set to 5
}
  \item{relTol}{
the relative tolerance for the best bitstring reported by the genetic algorithm, i.e.how different from the best solution can solutions be to be reported as well, default set to 0.1
}
  \item{verbose}{
logical (default to TRUE) do you want the statistics of each generation to be printed on the screen
}
\item{priorBitString}{
A bitString of same length at the initial bitstring made of 0, 1 or NA. By
default, this bitstring is set to NULL (equivalent to setting all bits to NA).
If provided, all bitstring in a population will be changed to be in agreement
with the priorBitString list.
}
\item{timeIndex}{todo}

}
\details{
This function takes in the same input as the T1 ga, but in addition it takes in
the bitstring optimised for T1, and does not take an initial bitstring. Be aware that the bitString that this function returns is one that only includes the bits that it actually looks at, i.e. the bits that were 0 in the bStringT1
}
\value{
This function returns a list with elements:
\item{bString}{the best bitstring}
\item{results}{ a matrix with columns "Generation","Best_score","Best_bitString","Stall_Generation","Avg_Score_Gen","Best_score_Gen","Best_bit_Gen","Iter_time"}
\item{stringsTol}{the bitstrings whose scores are within the tolerance}
\item{stringsTolScores}{the scores of the above-mentionned strings}
}
\author{
C.Terfve, T. Cokelaer
}
\seealso{
\link{getFit}, \link{simulatorT1}, \link{simulatorT2}

}
\examples{
#load data

data(CNOlistToy2,package="CellNOptR")
data(ToyModel2,package="CellNOptR")

#pre-process model

checkSignals(CNOlistToy2,ToyModel2)
model = preprocessing(CNOlistToy2, ToyModel2)

#optimise t1
ToyT1<-gaBinaryT1(
	CNOlist=CNOlistToy2,
	model=model,
	maxGens=10,
	popSize = 10,
	verbose=FALSE)

#Optimise T2

ToyT2<-gaBinaryTN(
	CNOlist=CNOlistToy2,
	model=model,
	bStrings=list(ToyT1$bString),
	maxGens=10,
	popSize = 10,
	verbose=FALSE)
}

\name{createILPBitstringAll}
\alias{createILPBitstringAll}
\alias{createILPBitstringAll}

\title{
Reading the optimal solutions as bitstrings.
}
\description{
This function takes as the cplex optimization results and the variables assigned to each interaction in the ILP formulation
in order to read the optimal bitstring. 
}
\usage{
createILPBitstringAll(cplexSolutionFileName,
                                 y_vector,
                                 binary_variables)
}
\arguments{
  \item{cplexSolutionFileName}{the file name where the cplex results are stored
  }
  \item{y_vector}{the variables for each interaction in the PKN
  }
   \item{binary_variables}{the binary variables of the ILP formulation
  }


}

\author{
E Gjerga, H Koch
}

\name{compressModel}
\alias{compressModel}
\title{
Compress a model
}
\description{
This function compresses a model by compressing species that are not 
signals/inhibited/stimulated and that are not dead ends/in complex logic 
(i.e. only species with either one input or one output are compressed)/in self loops.

You can also use \link{preprocessing} function instead that calls \link{compressModel} and 
other preprocessing functions.  
}
\usage{
compressModel(model, indexes)
}
\arguments{
    \item{model}{
        a model structure as produced by \link{readSIF}.
    }
    \item{indexes}{
        list of indexes of the species stimulated/inhibited/measured in the model, as created by \link{indexFinder}.
    }
}
\details{
Be aware that in the multiple inputs/one output case, if one of the outputs is 
an '&' gate this function handles it fine as long as it is an '&' with 2 inputs 
and no more.
}
\value{
a compressed model list, with an additional field called 'speciesCompressed' 
that contains the names of the species that have been compressed
}
\author{
C. Terfve
}
\seealso{
\link{indexFinder}, \link{readSIF}, \link{preprocessing}
}
\note{No need to call this function directly since version 0.99.24. Use 
 \link{preprocessing} instead.}

\examples{
#load data

data(CNOlistToy,package="CellNOptR")
data(ToyModel,package="CellNOptR")

indicesToy<-indexFinder(CNOlistToy,ToyModel,verbose=FALSE)
toyComp<-compressModel(ToyModel,indicesToy)
}

\name{cutAndPlotResultsT1}
\alias{cutAndPlotResultsT1}
\title{
Plot the results of an optimisation at t1
}
\description{
This function takes a model and an optimised bitstring, it cuts the model according to the bitstring and plots the results of the simulation along with the experimental data.
}
\usage{
cutAndPlotResultsT1(model, bString, simList=NULL, CNOlist, indexList=NULL, plotPDF =
FALSE, tag = NULL, tPt=CNOlist@timepoints[2],  plotParams = list(maxrow=10))

}

\arguments{
  \item{model}{
a model (the full one that was used for optimisation)
}
  \item{bString}{
a bitstring for T1  as output by gaBinaryT1 (i.e. a vector of 1s and 0s)
}
  \item{simList}{
deprecated argument kept for back compatibility. 
a simlist corresponding to the model, as output by prep4sim
}
  \item{CNOlist}{
a CNOlist, corresponding to the optimisation one
}
  \item{indexList}{
deprecated argument kept for back compatibility. 
an indexList, produced by indexFinder ran on the model and the CNOlist above
}
  \item{plotPDF}{
TRUE or FALSE; tells whether you want a pdf to be produced or not
}
  \item{tag}{
NULL or string; tells whether you want to prefix filenames with a tag (replaces
the default behaviour).
}
  \item{tPt}{
The number of time points in the data.}

\item{plotParams}{
a list of option related to the PDF and plotting outputs. (1) maxrow
is the maximum number of row used to plot the results. See  \link{plotOptimResultsPan}
for other fields.
}
}
\value{
This function returns plotted MSEs and list of filenames generated (if any)
}
\author{
C.Terfve, T. Cokelaer,A. MacNamara
}
\seealso{
\link{gaBinaryT1}
}
\examples{
#load data

data(CNOlistToy,package="CellNOptR")
data(ToyModel,package="CellNOptR")

#pre-process model
model = preprocessing(CNOlistToy, ToyModel)

#optimise
ToyT1opt<-gaBinaryT1(
	CNOlist=CNOlistToy,
	model=model,
	maxGens=20, 
	popSize = 10,
	verbose=FALSE)

#plotting
cutAndPlotResultsT1(
	model=model,
	CNOlist=CNOlistToy,
	bString=ToyT1opt$bString,
	plotPDF=FALSE)
}

\name{CNOlist-methods}
\docType{methods}
\alias{CNOlist-methods}

\alias{getCues}
\alias{getCues,CNOlist}

\alias{getSignals}
\alias{getSignals,CNOlist}
%\alias{getSignals,CNOlist,CNOlist-method}

\alias{compatCNOlist}
\alias{compatCNOlist,CNOlist}
%\alias{compatCNOlist,CNOlist,CNOlist-method}

\alias{getStimuli}
\alias{getStimuli,CNOlist}
%\alias{getStimuli,CNOlist,CNOlist-method}

\alias{getTimepoints}
\alias{getTimepoints,CNOlist}
%\alias{getTimepoints,CNOlist,CNOlist-method}

\alias{getVariances}
\alias{getVariances,CNOlist}
\alias{getVariances,CNOlist,CNOlist-method}

\alias{setSignals<-}
\alias{setSignals<-,CNOlist}
\alias{setSignals<-,CNOlist,CNOlist-method}

\alias{getInhibitors}
\alias{getInhibitors,CNOlist}
%\alias{getInhibitors,CNOlist-method}

\alias{randomize}
\alias{randomize,CNOlist}
\alias{randomize,CNOlist,CNOlist-method}

\alias{length}
\alias{length,CNOlist}
\alias{length,CNOlist,ANY-method}

%\alias{plot}
%\alias{plot,CNOlist}
\alias{plot,CNOlist,CNOlist,CNOlist-method}
%\alias{plot,CNOlist,CNOlist,CNOlist-methods}
%\alias{plot,CNOlist,missing,CNOlist-method}

\alias{readErrors}
\alias{readErrors,CNOlist}
%\alias{readErrors,CNOlist,CNOlist-method}

\alias{writeErrors}
\alias{writeErrors,CNOlist}
%\alias{writeErrors,CNOlist,CNOlist-method}


\title{List of \link{CNOlist-class} methods}
\description{
 CNOlist is a class with a set of methods described here below.

}

\section{Usage}{
    \code{signature(x="CNOlist")}
}
\section{Getters}{
    \describe{
	\item{getCues}{Returns the cues (matrix) found in the \code{CNOlist}}
	\item{getSignals}{Returns the signals (list of matrices) found in the \code{CNOlist}}
	\item{getStimuli}{Returns the cues found in the \code{CNOlist}}
	\item{getInhibitors}{Returns the inhibitors found in the \code{CNOlist}}
	\item{getTimepoints}{Returns the timepoints found in the \code{CNOlist}}
	\item{getVariances}{Returns the Variances (list of matrices) found in the
        \code{CNOlist}. Will be different from zero only if replicates were found in the
        MIDAS data. See \code{makeCNOlist}}
    }
}
\section{Setters}{
    \describe{
    \item{setSignals}{Set the signals. No sanity check!}
    }
}

\section{Other methods}{
    \describe{
      \item{compatCNOlist}{convert the instance CNOlist into the old-style
            returned by \code{\link{makeCNOlist}} that is a list. Used in the ODE package. }

      \item{length}{returns length of CNOlist (number of time points)}

      \item{randomize}{randomizes the signals matrice in a CNOlist. See
            \code{\link{randomizeCNOlist}} for details}

      \item{show}{prints summary information }

      \item{plot}{plot the CNOlist instance using the \link{plotCNOlist} function.}

      \item{plot}{\code{signature(x="CNOlist", y="CNOlist")}: Please see the
          page of \code{\link{plotCNOlist2}} for more details.}
    
      \item{readErrors}{\code{signature(object="CNOlist", filename="MIDAS-file")}: reads measurement error corresponding to the data from the MIDAS file and updates the CNOlist object}
      
      \item{writeErrors}{\code{signature(object="CNOlist", filename="string",overwrite=F)}: writes measurement error corresponding to the data from the CNOlist to a MIDAS file}
      }
}



\author{
T.Cokelaer
}
\seealso{
\code{\link{CNOlist-class}}, \code{\link{randomizeCNOlist}} \code{\link{makeCNOlist}}
}
\examples{
showClass("CNOlist")

data(CNOlistToyMMB, package="CellNOptR")
cnolist = CNOlistToyMMB

# In version 1.3.30 and above, use the plot method instead of former plotCNOlist function.
plot(cnolist)
# In version 1.5.14 and above, use getters instead of the @ operator
getCues(cnolist)

# others:
new_cnolist = randomize(cnolist)

}
\keyword{methods}
\keyword{CNOlist}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/readRules.R
\name{readBNET}
\alias{readBNET}
\title{Read network from BNET file}
\usage{
readBNET(filename)
}
\arguments{
\item{filename}{BNET file. The file is a tab delimited file with two columns,
`targets` and `factors`. `factors` contains the logic rule and `targets` the node
activated by the logic rule.}
}
\value{
CellNOpt network
}
\description{
Read network from BNET file
}
\author{
Luis Tobalina
}

\name{sif2graph}
\alias{sif2graph}

\title{
  Convert sif to graph
}

\description{
  This function receives as input a network in form sif format and converts it to graph format.
}

\usage{
sif2graph(sif)
}


\arguments{
  \item{sif}{the name of a sif file or the equivalent table}
}

\details{
This function takes a network in sif format (tabel or file), i.e. sourceNode-tab-sign-tab-targetNode. If there are ANDs they should be introduced as dummy nodes called and# (don't forget the number after "and" otherwise this won't be recognised).  Please be aware that "and" nodes are not expected to be negated, i.e. there are not supposed to be !and1=xyz because that amounts to inverting the sign of all inputs of and1, which is more simply done at the inputs level.

In the resulting graph, the sign of each link is encodes as the weigth of the edge (-1 negative regulation, +1 positive regulation).
}

\value{
\item{g}{the corresponding graph}
}

\author{
    F.Eduati
}

\seealso{
\link{graph2sif}, \link{model2sif}, \link{readSIF}, 
}



\examples{
    
    data(ToyModel,package="CellNOptR")
    sif_file = tempfile(fileext = ".sif")
    writeSIF(model = ToyModel,filename = sif_file)
    g = sif2graph(sif_file)
    
}

\name{plotModel}
\alias{plotModel}
\title{
Plot a model
}
\description{
This function can be used to plot a prior model network before any
pre-processing step. However, additional information can be provided such as
a CNOlist (see makeCNOlist and readMIDAS) or information related to the 
pre-processing steps (compression, NONC nodes, expansion gates).
It can also be used to plot optimised model given the optimisation bitstring.}


\usage{
plotModel(model, CNOlist=NULL, bString=NULL, indexIntegr=NULL, signals=NULL, 
stimuli=NULL, inhibitors=NULL, NCNO=NULL, compressed=NULL, output="STDOUT",
filename=NULL, graphvizParams=list(), show=TRUE, remove_dot=TRUE, removeEmptyAnds=TRUE)
}

\arguments{
    \item{model}{
      a model as returned by \code{readSIF}. Alternatively, the filename can
      also be provided.
    }
    \item{CNOlist}{output of makeCNOlist}
    \item{bString}{
        a sequence made of numbers between 0 and 1 of same length as
        the one returned by the Genetic Algorithm (GA). This is a generalisation
        of the bitString returned by the GA function: several bit strings can be
        averaged and used.
    }
    \item{indexIntegr}{
        additional indices to highlight some edge (optional).
    }
    \item{signals}{
        a list of nodes belonging to the signals class
    }
    \item{stimuli}{
        a list of nodes belonging to the stimuli class
    }
    \item{inhibitors}{
        a list of nodes belonging to the inhibitors class
    }
    \item{NCNO}{
        a list of NCNO nodes.
    }
    \item{compressed}{
        a list of compressed nodes
    }
    \item{filename}{
        the filename (without extension) used to write the dot file
    }
    \item{output}{
        the type of output (PNG, PDF, SVG accepted)
    }
    \item{graphvizParams}{a list of optional arguments dedicated to Rgraphviz to
        tune the layout:
        \itemize{
        \item{arrowsize}{ default is 2 }
        \item{size}{a string for the size of the dot output; default is "15,15"}
        \item{fontsize}{ default is 22}
        \item{edgecolor}{ default is "black"}
        \item{nodeLabels}{ overwrit node label with a list of proper length.}
        \item{nodeWidth}{ default is 2}
        \item{nodeHeight}{ default is 1}
        \item{viewEmptyEdges}{  default is TRUE}
        \item{mode}{  can be 'classic' or 'sbgn' (default). The difference appears in the and gate.}
        \item{andWidth}{ = 0.2 in 'classic' mode and 0.5 in 'sbgn' mode}
        \item{andHeight}{ = 0.2 in 'classic' mode and 0.5 in 'sbgn' mode}
        }
    }

  \item{show}{ show the plot (default is True)}

  \item{remove_dot}{remove the dot file that has been created. Default is False}
 \item{removeEmptyAnds}{removes AND gates from plotted graph if the corresponding bit is 0 to give a compact view. Default is TRUE. }
}




\details{
This function plots the model and also saves it in a dot file that can be processed
later on. However, you can also save the plot in PNG or PDF or SVG format (one at
a time).

The CNOlist argument contains the signals/stimuli/inhibitors so if you provide
a CNOlist there is no need to use these arguments. If you decide to use them
they will overwrite the contents of the CNOlist argument.

optimRes is the output of gaBinary. One of its field is called bString and
contains a list of 0 and 1 (the optimisation is perfomed with a binary
procedure). This list of 0 and 1 is then used to plot or not the edges of the
model. However, you can provide a bitString made of floats (e.g., average of 
several bitStrings). In such case, edges will appear in gray light or dark
according to the bistring value (between 0 and 1).

}
\value{
a graph representation of the model
\item{graph$g}{A graph representation of the model}
\item{graph$attrs}{graph attributes}
\item{graph$nodeAttrs}{nodes attributes}
\item{graph$edgeAttrs}{edges attributes}
\item{graph$clusters}{clusters of nodes}


}
\author{
T. Cokelaer
}
\seealso{
\link{readMIDAS}, \link{readSIF}, \link{makeCNOlist}, \link{writeNetwork},
\link{writeDot}, \link{gaBinaryT1}
}

\note{This function depends on the Rgraphviz package.}

\examples{
    data(CNOlistToy,package="CellNOptR")
    data(ToyModel,package="CellNOptR")
    res<-plotModel(ToyModel, CNOlist=CNOlistToy, compressed=c("TRAF6", "p38"), 
        graphvizParams=list(mode="classic", fontsize=30))
    
}

\name{graph2sif}
\alias{graph2sif}

\title{
Convert graph to SIF
}

\description{
  This function converts a network form graph format to SIF format. The resulting table can also be saved in a SIF file.
}

\usage{
graph2sif(graph, writeSif=FALSE, filename="Graph")

}


\arguments{
  \item{graph}{
a graph, as generated using \link{sif2graph}
}
  \item{writeSif}{
if writeSif=FALSE (default) the SIF file is not saved. If writeSif=TRUE it is saved.
}
  \item{filename}{
the name of the SIF file saved if writeSif=TRUE. Default is Model.sif.
}

}


\details{
The sign of link is supposed to be encoded in the graph as the weigth of the edge (-1 negative regulation, +1 positive regulation).
}

\value{
\item{sifFile}{a table with all the links in the model in the format sourceNode-tab-sign-tab-targetNode}
}

\author{
    F. Eduati
}

\seealso{
\link{model2sif}, \link{sif2graph}, \link{readSIF}, 
}


\examples{
    
    data(ToyModel,package="CellNOptR")
    sif_file = tempfile(fileext = ".sif")
    writeSIF(model = ToyModel,filename = sif_file)
    g = sif2graph(sif_file)
    
    graph2sif(graph = g,writeSif = FALSE)
    
}

\name{plotOptimResultsPan}
\alias{plotOptimResultsPan}
\title{
Plots the data and simulated values from any CellNOptR formalism
}
\description{
This function plots the data and simulated values according to each experiment in CNOlist. The data is shown as black triangles and the simulation by a blue dashed line. The combination of cues is given by a panel where black denotes the presence and white the absence of the cue. The goodness-of-fit between model and data is color-coded on a continuous scale from white to red.
}
\usage{
plotOptimResultsPan(simResults, yInterpol=NULL, xCoords=NULL,
CNOlist=CNOlist, formalism=c("ss1","ss2","ssN","dt","ode"), pdf=FALSE,
pdfFileName="", tPt=NULL,
plotParams = list(margin = 0.1, width=15, height=12, cmap_scale=1, cex=1.6,
ymin=NULL, Fac=1, rotation=0))

}
\arguments{
\item{simResults}{
A list with a field for each time point, each containing a matrix of dimensions (number of conditions) * (number of signals), with the first field being t0.  Typically produced by simulating a model and then extracting the columns that correspond to signals.
}
\item{yInterpol}{
If using CNORdt, these are the interpolated experimental results from getFitTimeScale() that are needed to compare against the Boolean simulation.
}
\item{xCoords}{
These are the x-coordinates obtained from the optimized scaling factor in CNORdt that allow for comparison between time course experimental data and a Boolean model.
}
\item{CNOlist}{
A CNOlist.
}
\item{formalism}{
An abbreviation of the CellNOptR formalism being used.
}
\item{pdf}{
A Boolean argument denoting whether to print the figure produced by this function to file.
}
\item{pdfFileName}{
If printing to file, the filename to be used.
}
\item{tPt}{
The number of time points in the data.
}


\item{plotParams}{
a list of option related to the PDF and plotting outputs. Currently, the
following attributes are used: (1) margin of the boxes, (2) width and heigth used while creating the
PDF, (3) cmap_scale a value that scales the colors towards small errors (<1) or
large errors (>1); default is 1 (linear colormap) (4) cex is the fontsize used
in the header (5) ymin sets the minimum y axis limit; by default it is the
minimum value found over all data points and therefore can be negative.
}
}
\details{
Depending on the logic formalism, this function is generally called from cutAndPlotResults*(). As shown in the example below however, it can plot the fit of any data and corresponding compatible model. The color denotes the goodness-of-fit, where white shows no difference between simulation and data and red is the maximum error from all conditions and readouts.
}
\value{
This function does not return a value.
}
\references{
J. Saez-Rodriguez, L. G. Alexopoulos, J. Epperlein, R. Samaga, D. A. Lauffenburger, S. Klamt and P. K. Sorger. Discrete logic modeling as a means to link protein signaling networks with functional analysis of
mammalian signal transduction, Molecular Systems Biology, 5:331, 2009.
}
\author{
A. MacNamara
}
\seealso{
cutAndPlotResultsT1
}
\examples{

data(CNOlistToy,package="CellNOptR")
data(ToyModel,package="CellNOptR")
indicesToy <- indexFinder(CNOlistToy, ToyModel, verbose=TRUE)
ToyFields4Sim <- prep4sim(ToyModel)

# simulate model
simRes <- simulatorT1(CNOlist=CNOlistToy, model=ToyModel, simList=ToyFields4Sim, indexList=indicesToy)
simRes = simRes[, indicesToy$signals]

# format data and results

simResults <- list(t0=matrix(data=0, nrow=dim(simRes)[1], ncol=dim(simRes)[2]), t1=simRes)
# plot
plotOptimResultsPan(simResults,
  CNOlist=CNOlistToy,
  formalism="ss1",
  pdf=FALSE,
  tPt=10
)
}

\name{plotOptimResults}
\alias{plotOptimResults}
\title{
Plot the data and simulated values
}
\description{
This function is the equivalent of CNOPlotFits, it plots the data and the simulated values,  along with an image plot that tells which cues were present.  The plots are coloured according to the fit between data and simulated data.
}
\usage{
plotOptimResults(simResults, expResults, times, namesCues, namesSignals,
valueCues, formalism="new")
}
\arguments{
  \item{simResults}{
a list with a field for each time point, each containing a matrix of dimensions (number of conditions) * (number of signals), with the first field being t0.  Typically produced by simulating a model and then extracting the columns that correspond to signals
}
  \item{expResults}{
same as above, but contains the experimental results, ie this is \code{CNOlist$valueSignals}
}
  \item{times}{
a vector of times, its length should be the same as the number of fields in simResults and ExpResults
}
  \item{namesCues}{
a vector of names, typically \code{CNOlist$namesCues}
}
  \item{namesSignals}{
a vector of names, typically \code{CNOlist$namesSignals}
}
  \item{valueCues}{
a matrix of dimensions (number of conditions) * (number of cues), typically \code{CNOlist$valueCues}
}
  \item{formalism}{
New convention is to take the time=0 data set into account to compute the MSE.
you can use the previous convetion by setting this argument to something
different from the default value.
}
}
\details{
The colouring of the background is done as follows: the mean absolute difference between observed and simulated values are computed, and colours are chosen based on this value: red (above 0.9), indianred1 (between O.8 and 0.9), lightpink2 (between 0.7 and 0.8), lightpink (between 0.6 and 0.7), mistyrose (between 0.5 and 0.6), palegoldenrod (between 0.4 and 0.5), palegreen (between 0.3 and 0.4), darkolivegreen3 (between 0.2 and 0.3), chartreuse3 (between 0.1 and 0.2), forestgreen (between 0 and 0.1). This function is used inside \code{cutAndPlotResultsT1}.
}
\value{
This function doesn't return anything, it just produces a plot in your graphics window.
}
\references{
J. Saez-Rodriguez, L. G. Alexopoulos, J. Epperlein, R. Samaga, D. A. Lauffenburger, S. Klamt and P. K. Sorger. Discrete logic modeling as a means to link protein signaling networks with functional analysis of
mammalian signal transduction, Molecular Systems Biology, 5:331, 2009.
}
\author{
C. Terfve
}
\seealso{
cutAndPlotResultsT1
}
\examples{

#We will plot the fit of the full initial model compared to the data, without any optimisation
#This is normally not done on a stand alone basis, but if you have a model and would like to visualise its output compared to your data, then this is what you should do

#load and prepare data

data(CNOlistToy,package="CellNOptR")
data(ToyModel,package="CellNOptR")
indicesToy<-indexFinder(CNOlistToy,ToyModel,verbose=TRUE)
ToyFields4Sim<-prep4sim(ToyModel)

#simulate model

simRes<-simulatorT1(CNOlist=CNOlistToy,model=ToyModel, simList=ToyFields4Sim, indexList=indicesToy)

#format data and results

simResults<-list(t0=matrix(data=0,nrow=dim(simRes)[1],ncol=dim(simRes)[2]),t1=simRes)
expResults<-list(t0=CNOlistToy$valueSignals[[1]],t1=CNOlistToy$valueSignals[[2]])

#plot

plotOptimResults(
	simResults=simResults,
	expResults=expResults,
	times=CNOlistToy$timeSignals[1:2],
	namesCues=CNOlistToy$namesCues,
	namesSignals=CNOlistToy$namesSignals,
	valueCues=CNOlistToy$valueCues)
}

\name{makeCNOlist}
\alias{makeCNOlist}
\title{
Make a CNOlist structure
}
\description{
This function takes as input the output of readMIDAS and extracts the elements
that are needed in a CNO project. Instead, please use the \code{CNOlist} class to read a MIDAS
file that will be converted to a CNOlist.
}
\usage{
makeCNOlist(dataset, subfield, verbose=TRUE)
}
\arguments{
  \item{dataset}{
output of readMIDAS
}
  \item{subfield}{
TRUE or FALSE, specifies if the column headers contain subfields or not i.e. if
I should look for TR:sthg:sthg or just TR:sthg.
}
  \item{verbose}{
logical (default to TRUE) print information on the screen.
}
}
\details{
Be aware that most of the functions in this package, including this one, expect
the data to contain measurements at time 0, but these should all be equal to
zero according to the normalisation procedure that should be used. Therefore, if
you have one time point, the files valueSignals contains two matrices, one for
t0 and one for t1.

If there are replicate rows in the MIDAS file (i.e., identical cues and
identical time), this function averages the values of the measurements for these
replicates.

Columns with the following tags are ignored: NOINHIB, NO-INHIB, NO-LIG, NOCYTO.

}
\value{
a CNOlist with fields
\item{namesCues}{a vector of names of cues}
\item{namesStimuli}{a vector of names of stimuli}
\item{namesInhibitors}{a vector of names of inhibitors}
\item{namesSignals}{a vector of names of signals}
\item{timeSignals}{a vector of times}
\item{valueCues}{a matrix of dimensions nConditions x nCues, with 0 or 1 if the
    cue is present or absent in the particular condition}
\item{valueInhibitors}{a matrix of dimensions nConditions x nInhibitors, with 0
    or 1 if the inhibitor is present or absent in the particular condition}
\item{valueStimuli}{of dimensions nConditions x nStimuli, with 0 or 1 if the
    stimuli is present or absent in the particular condition}
\item{valueSignals}{a list of the same length as timeSignals, each element
    containing a matrix of dimensions nConditions x nsignals, with the measurements.}
\item{valueVariances}{a list of the same length as timeSignals, each element
    containing a matrix of dimensions nConditions x nsignals, with the standard
    deviation of the replicates.}
}
\references{
J. Saez-Rodriguez, L. G. Alexopoulos, J. Epperlein, R. Samaga, D. A.
Lauffenburger, S. Klamt and P. K. Sorger. Discrete logic modeling as a means to
link protein signaling networks with functional analysis of mammalian signal
transduction, Molecular Systems Biology, 5:331, 2009.
}
\author{
C. Terfve, T. Cokelaer
}
\seealso{
\link{readMIDAS}, \link{CNOlist-class}
}
\examples{
cpfile<-dir(system.file("ToyModel",package="CellNOptR"),full=TRUE)
file.copy(from=cpfile,to=getwd(),overwrite=TRUE)
dataToy<-readMIDAS(MIDASfile='ToyDataMMB.csv')
CNOlistToy<-makeCNOlist(dataset=dataToy,subfield=FALSE)
}

\name{CNOlistToy2}
\docType{data}
\alias{CNOlistToy2}
\title{Toy data with 2 time points}
\description{This data object contains the data associated with the Toy Model example from the package vignette, already loaded and formatted as a CNOlist object, and modified to contain 2 time points.  The second time point is such a way that all  of the signals stay as in time 1, except for cJun and Jnk which go to zero.}
\usage{data(CNOlistToy)}
\format{CNOlistToy is a list with the fields "namesCues" (character vector), "namesStimuli" (character vector), "namesInhibitors" (character vector), "namesSignals" (character vector), "timeSignals" (numerical vector), "valueCues" (numerical matrix), "valueInhibitors" (numerical matrix), "valueStimuli"(numerical matrix), "valueSignals"(numerical matrix). }
\source{This data and model is extracted from the Matlab version of CellNOpt1.0 (\url{http://www.ebi.ac.uk/saezrodriguez/software.html#CellNetOptimizer}).}
\references{J. Saez-Rodriguez, L. G. Alexopoulos, J. Epperlein, R. Samaga, D. A.
Lauffenburger, S. Klamt and P. K. Sorger. Discrete logic modeling as a means to
link protein signaling networks with functional analysis of mammalian signal transduction, Molecular Systems Biology, 5:331, 2009.}
\keyword{datasets}

\name{simulateTN}
\alias{simulateTN}
\title{
Cut and simulation of a boolean model at t1
}
\description{
This function cuts a model according to a bitstring optimised at T1, and
simulates the model accordingly.
}
\usage{
simulateTN(CNOlist, model, bStrings)
}
\arguments{
  \item{CNOlist}{
a CNOlist object
}
  \item{model}{
a full model
}
  \item{bStrings}{
a bitstring to cut the model, as output by \code{gaBinaryT1} (i.e. a vector of
1s and 0s, of length equal to the number of reactions in the model)
}
}
\details{
This function is a wrapper around the family of functions called simulatorT1 ,
T2 and TN.
}
\value{
a matrix of simulated values, including all species in the model, i.e. to be
used as input of gaBinaryTN.
}
\author{
T.Cokelaer, S.Schrier based on simulatorT1 (C.Terfve)
}
\seealso{
\link{cutAndPlotResultsT1}, \link{simulatorT1}
}
\examples{
# This will compute the output of a random model obtained by randomly selecting
# which gates of the initial models are included.

data(CNOlistToy,package="CellNOptR")
data(ToyModel,package="CellNOptR")

simRes<-simulateTN(
	CNOlist=CNOlistToy, 
	model=ToyModel, 
	bStrings=list(round(runif(length(ToyModel$reacID)))))
}

\name{readSBMLQual}
\alias{readSBMLQual}
\alias{readSBMLQual}
\title{
Read a SBMLQual document and returns a SIF object (as returned by readSIG
}
\description{
This function reads a SBMLQual XML file where a model is stored. The XML is
scanned and saved in SIF format in a temporary file. This file is read by
readSIF. The returned object is therefore the output of readSIF.
}
\usage{
readSBMLQual(filename)
}
\arguments{
  \item{filename}{
The name of a SBMLQual file.
}
}
\value{
a model list with fields:
    \item{interMat}{contains a matrix with column for each reaction and a row for each species, with a -1 where the species is the source node and a +1 where the species is a target node, and 0 otherwise}
    \item{notMat}{has the same format as interMat but just contains a 1 if the source node enters the reac with a negative effect, and 0 otherwise}
    \item{namesSpecies}{vector that contains the names of the species in the same order as the rows of the interMat and notMat matrices}
    \item{reacID}{vector that holds character strings specifying the reaction in full letters, in the same order as the columns of interMat and notMat}
}

\references{
SBMLQual: qualitative models. See SBML.org for details.
}
\author{
T. Cokelaer
}
\examples{

\dontrun{
sif = readSBMLQual("test.xml")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/readRules.R
\name{readBND}
\alias{readBND}
\title{Read network from BND file}
\usage{
readBND(filename)
}
\arguments{
\item{filename}{BND file.}
}
\value{
CellNOpt network
}
\description{
BND is a file format used by MaBoSS to store the boolean network definition.
The reader works if the logic for the activation of the node is stated withing the parameter `logic` of the node.
An example file can be found in `https://maboss.curie.fr/pub/example.bnd`.
}
\examples{
\dontrun{
model = readBND("https://maboss.curie.fr/pub/example.bnd")
}

}
\author{
Luis Tobalina
}
